Package: hyper2
Type: Package
Title: The Hyperdirichlet Distribution, Mark 2
Version: 3.2-3
Authors@R: person(given=c("Robin", "K. S."), family="Hankin", role = c("aut","cre"), email="hankin.robin@gmail.com", comment = c(ORCID = "0000-0001-5982-0415"))
Maintainer: Robin K. S. Hankin <hankin.robin@gmail.com>
Description: A suite of routines for the hyperdirichlet distribution
   and reified Bradley-Terry; supersedes the 'hyperdirichlet' package;
   uses 'disordR' discipline <doi:10.48550/ARXIV.2210.03856>.  To cite
   in publications please use Hankin 2017 <doi:10.32614/rj-2017-061>,
   and for Generalized Plackett-Luce likelihoods use Hankin 2024
   <doi:10.18637/jss.v109.i08>.
License: GPL (>= 2)
LazyData: yes
Depends: R (>= 4.1.0)
Suggests: abind, ape, bookdown, covr, igraph, knitr, markdown,
        pdftools, phangorn, PlackettLuce, pracma, prefmod, qcc,
        rmarkdown, rticles, stringr, Ternary, testthat
VignetteBuilder: knitr
Imports: alabama, calibrator, cubature, disordR (>= 0.0-9), frab,
        magrittr, methods, partitions, Rcpp (>= 1.0-7), Rdpack, crayon
LinkingTo: Rcpp
URL: https://github.com/RobinHankin/hyper2,
        https://robinhankin.github.io/hyper2/
BugReports: https://github.com/RobinHankin/hyper2/issues
RoxygenNote: 7.1.1
RdMacros: Rdpack
NeedsCompilation: yes
Packaged: 2026-06-23 07:02:58 UTC; rhankin
Author: Robin K. S. Hankin [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-5982-0415>)
Repository: CRAN
Date/Publication: 2026-06-23 10:00:09 UTC

useDynLib(hyper2, .registration=TRUE)

export("B")
export("GD")
export("GD_wong")
export("Ops.lsl")
export("accessor")
export("accessor3")
export("addL")
export("addL3")
export("advantage")
export("args2ordervec")
export("as.hyper3")
export("as.namedvectorlist")
export("as.ordertable")
export("as.suplist")
export("as.weight")
export("assign_lowlevel")
export("assign_lowlevel")
export("assigner")
export("assigner3")
export("attemptstable2supp3")
export("as.attemptstable")
export("rattemptstable")
export("balance")
export("brackets")
export("char2num")
export("character_to_number")
export("cheering3")
export("choose_losers")
export("choose_winners")
export("consistency")
export("dec")
export("den3")
export("dhyper2")
export("differentiate")
export("differentiate")
export("differentiate3")
export("dirichlet")
export("dirichlet3")
export("discard_flawed")
export("discard_flawed2")
export("e_to_p")
export("elimination")
export("equality")
export("equality3")
export("equalp")
export("equalp.test")
export("evaluate")
export("evaluate3")
export("fillup")
export("formula1_points_systems")
export("general_grouped_rank_likelihood")
export("ggrl")
export("goodbad")
export("gradient")
export("gradientn")
export("hessian")
export("hessian")
export("home_away")
export("home_away3")
export("home_draw_away3")
export("home_away_table")
export("hyper2")
export("hyper2")
export("hyper2")
export("hyper2")
export("hyper2")
export("hyper3")
export("hyper3_add")
export("hyper3_bw")
export("hyper3_equal")
export("hyper3_nv")
export("hyper3_m")
export("hyper3_prod")
export("hyper3_to_hyper2")
export("identityL")
export("identityL3")
export("inc")
export("indep")
export("is.dirichlet")
export("is.hyper2")
export("is.hyper3")
export("is_ok_hessian")
export("is_ok_weightedplayers")
export("is_valid_hyper2")
export("is_valid_hyper3")
export("keep_flawed")
export("knownp.test")
export("length")
export("like_series")
export("like_single_list")
export("list2nv")
export("loglik")
export("loglik")
export("loglik_lsl")
export("loglik_single")
export("loglik_single_redundant")
export("lsl")
export("lsl_add")
export("maxp")
export("maxp3")
export("maxp_lsl")
export("maxp_simplex")
export("maxp_single")
export("maxp_single2")
export("maxplist")
export("mean_hyper2")
export("mgf")
export("mult_grid")
export("multinom")
export("num3")
export("ordertable2points")
export("ordertable2supp")
export("ordertable2supp3")
export("ordertable_to_ranktable")
export("ordertrans")
export("ordertransplot")
export("ordervec2supp")
export("ordervec2supp3")
export("ordervec2supp3a")
export("overwrite")
export("overwrite3")
export("overwrite_lowlevel")
export("overwrite_lowlevel3")
export("p_to_e")
export("pair_grid")
export("pairwise")
export("pnames")
export("pnames<-")
export("pnames<-.hyper3")
export("pnv")
export("powers")
export("powers<-")
export("print")
export("probability")
export("profile_support_single")
export("profsupp")
export("psubs")
export("psubs_single")
export("pwa")
export("pwa3")
export("pwa3_single")
export("pwa23_single")
export("pwa23_multiple")
export("pwa23")
export("race")
export("ranktable_to_ordertable")
export("rankvec_likelihood")
export("rdirichlet")
export("rhyper2")
export("rhyper3")
export("rorder_single")
export("rp")
export("rpair3")
export("rp_unif")
export("rrace")
export("rrace3")
export("rracehyper3")
export("rrank")
export("rrankk")
export("rrank_single")
export("rwinner3")
export("saffy")
export("samep.test")
export("size")
export("setweight")
export("specificp.gt.test")
export("specificp.lt.test")
export("specificp.ne.test")
export("specificp.test")
export("sum.suplist")
export("suplist_add")
export("suplist_times_scalar")
export("swait")
export("tidy")
export("trial")
export("pick")
export("pass")
export("beats")
export("loses")
export("volley")
export("weights.hyper3")
export("white_draw3")
export("wikitable_to_ranktable")
export("zapweak")
export("zermelo")
export("zipf")
export(brackets)
export(hyper2_sum_hyper2)
export(pnames)
export(powers)
export(size)
export(as.ranktable)
export(is.ranktable)
export(suppfun)

export("ordertable")



import("Rdpack")
importFrom("Rdpack",reprompt)



importFrom(Rcpp, evalCpp)
importFrom("stats", "constrOptim","runif","rnorm","pchisq","rgamma","setNames")
importFrom("utils", "head")
importFrom("utils", "capture.output")
importFrom("partitions","perms")
importFrom("magrittr", "%>%", "%<>%","add","subtract")
importFrom("cubature", "adaptIntegrate")
importFrom("graphics", "abline", "par", "text","points")

importFrom("disordR","disord")
importFrom("disordR","hashcal")
importFrom("disordR","consistent")
importFrom("disordR","elements")
importFrom("disordR","is.disord")
importFrom("disordR","lapply")

importFrom("frab", "values")
importFrom("frab", "as.frab")
importFrom("frab", "as.namedvector")

importFrom("alabama","constrOptim.nl")

importFrom("calibrator","MH")

importFrom("crayon", "bold")

import(methods)

S3method(brackets,hyper2)
S3method(powers,hyper2)
S3method("powers<-",hyper2)
S3method(pnames,hyper2)
S3method(pnames,suplist)
S3method("pnames<-",hyper2)

S3method(print,ranktable)
S3method(print,ranktablesummary)
S3method(print,hyper2test)
S3method(print,attemptstable)
S3method(print,ordertable)
S3method(aperm,ordertable)

S3method("[", hyper2)
S3method("[<-", hyper2)

S3method(Ops,hyper2)
S3method(sum,hyper2)
S3method(print,hyper2)
S3method(head,hyper2)
S3method(length,hyper2)
S3method(summary,ranktable)

S3method(Ops,suplist)
S3method(sum,suplist)

S3method(Ops,lsl)

S3method("print",summary.hyper2)
S3method("summary",hyper2)

S3method(print,hyper3)
S3method("print",summary.hyper3)
S3method("summary",hyper3)

S3method("powers<-",hyper3)
S3method(Ops,hyper3)
S3method("sum",hyper3)
S3method("[", hyper3)
S3method("[<-", hyper3)
S3method("weights", hyper3)
S3method("pnames<-",hyper3)

S3method(as.vector, attemptstable)

S3method("as.ranktable",ordertable)
S3method("as.ranktable",ranktable)
S3method("as.ranktable",matrix)
S3method("[", ordertable)

S3method("suppfun", ranktable)
S3method("suppfun", ordertable)
S3method("suppfun", character)
S3method("suppfun", numeric)
S3method("suppfun", attemptstable)

The hyper2 package
================

<!-- README.md is generated from README.Rmd. Please edit that file -->

<img src="man/figures/hyper2.png" width = "150" align="right" />

<!-- badges: start -->

[![Total
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/hyper2)](https://CRAN.R-project.org/package=hyper2)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/hyper2)](https://cran.r-project.org/package=hyper2)
<!-- badges: end -->

# Overview

The `hyper2` package gives a suite of methods for creating and analysing
likelihood functions generated from a generalization of the
Bradley-Terry probability model. The package uses the `STL` map class of
`C++` for efficiency; it conforms to `disordR` discipline.

# Installation

The released version of `hyper2` on [CRAN](https://CRAN.R-project.org)
may be installed with:

``` r
# install.packages("hyper2")  # uncomment this to install the package
library("hyper2")
```

# Further information

For more detail, see the package vignette; type

`vignette("hyper2")`

at the R command line.

# Citation

To cite the `hyper2` package in publications, please use

Robin K. S. Hankin 2017. “Partial rank data with the `hyper2` package:
Likelihood functions for generalized Bradley-Terry models”. *The R
Journal*, 9(2):429–439

## ----setup, include=FALSE---------------------------------
set.seed(0)
library("hyper2")
library("magrittr")
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)
knit_print.function <- function(x, ...){dput(x)}
registerS3method(
  "knit_print", "function", knit_print.function,
  envir = asNamespace("knitr")
)

## ----out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE----
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))

## ----appidiom---------------------------------------------
chess <- hyper2()
chess["Topalov"] <- 30
chess["Anand"  ] <- 36
chess["Karpov" ] <- 22
chess[c("Topalov","Anand" )] <- 35  # bug!  should  be -35
chess[c("Anand","Karpov"  )] <- 35  # bug!  should  be -35
chess[c("Karpov","Topalov")] <- 18  # bug!  should  be -18

## ----showwarningunbalanced--------------------------------
chess

## ----correctid--------------------------------------------
chess[c("Topalov","Anand" )] <- -35
chess[c("Anand","Karpov"  )] <- -35
chess[c("Karpov","Topalov")] <- -18
chess
loglik(equalp(chess),chess)

## ----newinfo----------------------------------------------
chess["Anand"] %<>% inc
chess[c("Anand, Karpov")] %<>% dec(2)
chess

## ---------------------------------------------------------
H <- hyper2()
H['a'] %<>% inc(5)
H['b'] %<>% inc(2)
H['c'] %<>% inc(9)
H %<>% balance
H

## ----setup, include=FALSE---------------------------------
set.seed(0)
library("hyper2")
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width=6, fig.height=6)
knit_print.function <- function(x, ...){dput(x)}
registerS3method(
  "knit_print", "function", knit_print.function,
  envir = asNamespace("knitr")
)

## ----out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE----
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))

## ----label=showordertrans---------------------------------
ordertrans

## ----label=simpexample------------------------------------
x <- c(d=2,a=3,b=1,c=4)
x

## ---------------------------------------------------------
sort(x)

## ----label=useordertrans----------------------------------
o <- ordertrans(x) # by default, sorts names() into alphabetical order
o

## ----label=equalrearrangement-----------------------------
identical(x, o[c(4,1,2,3)])
identical(o, x[c(2,3,4,1)])

## ----showordervec-----------------------------------------
(Sx <- ordervec2supp(x))
(So <- ordervec2supp(o))

## ---------------------------------------------------------
Sx==So

## ---------------------------------------------------------
(x <- c(d=2, a=3, c=4, b=3, e=6))
(y <- c(e=3, c=2, a=4, b=5, d=1))
x+y

## ---------------------------------------------------------
ordertrans(x) + ordertrans(y)

## ---------------------------------------------------------
(z <- c(f=3, g=2, h=4, a=5, b=1))  # names NOT letters[1:5]
ordertrans(x) + ordertrans(z)  # arguably not well-defined

## ----label=lookatskating----------------------------------
skating_table

## ----makej1j2---------------------------------------------
j1 <- unclass(skating_table)[,1]  # column 1 is judge number 1
names(j1) <- rownames(skating_table)
j2 <- unclass(skating_table)[,2]  # column 2 is judge number 2
names(j2) <- rownames(skating_table)
j1
j2
cbind(j1,j2)

## ----j1vsj2,fig.cap="Judge 1 vs judge 2"------------------
par(pty='s')  # forces plot to be square
plot(j1,j2,asp=1,pty='s',xlim=c(0,25),ylim=c(0,25),pch=16,xlab='judge 1',ylab='judge 2')
abline(0,1)  # diagonal line
for(i in seq_along(j1)){text(j1[i],j2[i],names(j1)[i],pos=4,col='gray',cex=0.7)}

## ----label=maxlikeskating,cache=TRUE----------------------
mL <- skating_maxp  # predefined; use maxp(skating) to calculate ab initio
mL

## ----label=strengthstoranks-------------------------------
mL[] <- rank(-mL)  # minus because ranks orders from weak to strong
mL

## ----label=pointsskating,cache=TRUE-----------------------
mP <- rowSums(skating_table)  # 'P' for Points
mP[] <- rank(mP,ties='first') # positive sign here
mP

## ----label=ordertransexample------------------------------
ordertrans(mP,names(mL))  

## ----label=crapplot,fig.cap="points-based ranks vs likelihood ranks"----
plot(mL,ordertrans(mP,names(mL)))

## ----label=showoffordertransplot,fig.cap="points=based rank vs likelihood rank using `ordertransplot()`"----
ordertransplot(mL,mP,xlab="likelihood rank",ylab="Borda rank")

## ----transplotjudge1,fig.cap="Likelihood rank vs rank according to Judge 1"----
ordertransplot(mL,j1,xlab="likelihood rank",ylab="Judge 1 rank")

### R code from vignette source 'jss_hyper3_real.Rnw' [under version
### control at github, in directory hyper2/inst] with occasional
### additional documentation and instructions.  This file,
### jss_hyper3_real.R, is also under version control in the same
### directory.


### This file takes about an hour to run without cache.



###################################################
### code chunk number 1: preliminaries
###################################################
options(prompt = "R> ", continue = "+  ", width = 70, useFancyQuotes = FALSE)
library("hyper2")
library("magrittr")
howmany <- 23  # set to a smaller value to save time


###################################################
### code chunk number 2: defineoldfirm
###################################################

## Below we see hyper3 idiom for creating a hyper3 likelihood
## function, in this case using natural language and magrittr pipes.
## The function takes a single numeric argument, lambda, representing
## the home advantage.  We start with an empty likelihood function H
## with two Bradley-Terry entities [Rangers and Celtic] and populate
## it with two sets of results, each with three terms.  The first set
## of three corresponds to Rangers at home, the second to Celtic at
## home.  Looking at first set of three: these correspond to 81+42=123
## matches with Rangers at home, of which they won 81 and lost 42.  We
## see that the 81 won games are won with lambda helping (if indeed
## lambda is greater than 1).

oldfirm <- function(lambda){
    H <- hyper3(pnames = c("Rangers", "Celtic"))
    H[c(Rangers = lambda            )] %<>%      add(81     )
    H[c(                  Celtic = 1)] %<>%      add(     42)
    H[c(Rangers = lambda, Celtic = 1)] %<>% subtract(81 + 42)

    H[c(Celtic = lambda             )] %<>%      add(     76)
    H[c(                 Rangers = 1)] %<>%      add(46     )
    H[c(Celtic = lambda, Rangers = 1)] %<>% subtract(46 + 76)
    return(H)
}

###################################################
### code chunk number 3: useoldfirm
###################################################

## Below we see function oldfirm() being called with a lambda value of
## 1.88 [results pasted for convenience].  We see the bespoke print
## method using named vectors.  Observe that the order of the terms is
## implementation-specific, for the package follows disordR
## discipline.

oldfirm(1.88)

## Output reproduced below for convenience:

## log( (Celtic=1)^42 * (Celtic=1, Rangers=1.88)^-123 *
## (Celtic=1.88)^76 * (Celtic=1.88, Rangers=1)^-122 *
## (Rangers=1)^46 * (Rangers=1.88)^81)




###################################################
### code chunk number 4: profilelikeoldfirm
###################################################

## Below we define a function of multiplicative term lambda, which
## returns the log-likelihood, maximized over Bradley-Terry strengths,
## for the supplied value of lambda.  This furnishes a profile support
## function mapping lambda to support.  We apply this function to a
## range of lambda values, from 0.9 to 3.  


f <- function(lambda){maxp(oldfirm(lambda), give = 1)$`Log-likelihood`}
lam <- seq(from = 0.9, to = 3, len = howmany)
logLike <- sapply(lam, f)



###################################################
### code chunk number 5: plotproflikeoldfirm
###################################################

## Below we plot the log-likelihood function defined above.

plot(lam, logLike - max(logLike), type = 'b')
abline(v = 1, col = 'gray')
segments(x0 = 1.785, y0 = 0.1, y1 = -1, col = 'red')
abline(h = c(0, -2), col = 'gray')


###################################################
### code chunk number 6: optimoldfirm
###################################################

## Below we maximize the log-likelihood using base R function
## optimize(), for use in likelihood ratio tests.


(lambda_max <-  optimize(f, c(1.5, 2.0), maximum = TRUE)$maximum)


###################################################
### code chunk number 7: mleoldfirm
###################################################

## Below we explicitly display the evaluate at the maximized value of
## lambda.

maxp(oldfirm(lambda_max))


###################################################
### code chunk number 8: eg_RCLF3_lf
###################################################

## Below, we apply standard package function home_away3() to the
## Rangers-Celtic-Livingston-Falkirk results table, with arbitrary
## illustrative values for the multiplicative parameters lambda and D
## of 1.7 and 0.3 respectively.

home_draw_away3(RCLF3_table, lambda = 1.7, D = 0.3)


###################################################
### code chunk number 9: defineF
###################################################

## Below we define function f(), which takes a length-two numeric
## vector as argument v.  v[1] is lambda and v[2] is D; the function
## returns the profile log-likelihood at (lambda, D).

f <- function(v){
    H <- home_draw_away3(RCLF3_table, lambda = v[1], D = v[2])
    maxp(H, give = 1)$`Log-likelihood`
}


###################################################
### code chunk number 10: findmaxlambdaD
###################################################

## Below we optimize function f() [defined immediately above] to find
## the maximum likelihood estimate for (lambda, D)


maxv <- optim(par=c(2, 0.3), fn = f, control = list(fnscale = -1))$par
maxv


###################################################
### code chunk number 11: showevaluate
###################################################

## Below we find the maximum likelihood estimate for Bradley-Terry
## strengths conditional on the estimated values of (lambda, D)

maxp(home_draw_away3(RCLF3_table, lambda = maxv[1], D = maxv[2]))


###################################################
### code chunk number 12: makecontourdata
###################################################

## Below we calculate log-likelihoods as a function of lambda and D on
## a regular grid of values given by expand.grid() 

n <- howmany
lambda <- seq(from = 1, to = 3, len = n)
D <- seq(from = 0.1, to = 0.6, len = n)
V <- as.matrix(expand.grid(lambda, D))
LL <- apply(V, 1, f)
LL <- pmax(-20, LL - max(LL))


###################################################
### code chunk number 13: docontourplot
###################################################

## Below we plot a contour diagram showing log-likelihood as function
## of lambda on the x-axis and D on the y-axis

contour(lambda, D, matrix(LL, n, n), levels = -2*(1:9),
        xlab = expression(lambda), ylab = expression(D))
points(maxv[1], maxv[2], pch = 16, col = 'red')
abline(v=1, col = 'gray')


###################################################
### code chunk number 14: testlambdazero
###################################################

## Below we find the maximum likelihood estimate for the above
## problem, constrained so that lambda=1

jj <- optimize(f = function(D){f(c(lambda=1, D))}, interval = c(0.1, 0.4), maximum = TRUE)


###################################################
### code chunk number 15: defineredbuslf
###################################################

## Below we define bespoke function RB_BB_LF() which returns a hyper3
## log-likelihood function for our fictitious dataset on transport
## preferences, conditional on the supplied version of lambda.


`RB_BB_LF` <- function(lambda){
    ec <- c(C = 1, T = 2, RB = 3, BB = 3, W = 4) 
    h <- c(1, 1, lambda, 1)           
    (
        cheering3(v=c("RB", "BB", "C" , "T", "W"), e = ec, h = h) * 3 + 
        cheering3(v=c("BB", "RB", "T" , "C", "W"), e = ec, h = h) * 2 + 
        cheering3(v=c("T" , "BB", "RB", "C", "W"), e = ec, h = h) * 5 + 
        cheering3(v=c("W" , "BB", "RB", "T", "C"), e = ec, h = h) * 4 + 
        cheering3(v=c("C" , "RB", "BB", "W", "T"), e = ec, h = h) * 3 + 
        cheering3(v=c("BB", "C" , "RB", "T", "W"), e = ec, h = h) * 3
    )
}


###################################################
### code chunk number 16: definefunctiono
###################################################

## below we define function o() that takes a numerical value of lambda
## and returns the log-likelihood at the MLE of Bradley Terry
## strengths.

o <- function(lambda){maxp(RB_BB_LF(lambda), give = 1)$`Log-likelihood`} 


###################################################
### code chunk number 17: maxlikesim
###################################################

## Below we find the best-supported value of lambda for the fictitious
## red bus blue bus dataset.

(osup <- optimize(o, c(10, 40), maximum = TRUE))


###################################################
### code chunk number 18: LRT_RB
###################################################

## Below we find the difference in support between the support at the
## MLE for lambda and the null of lambda=1

(suppdiff <- osup$objective - o(lambda = 1))


###################################################
### code chunk number 19: plotproflike
###################################################

## Code below would plot a profile support curve for lambda (but was
## cut from the final version for reasons of brevity)

if(FALSE){
    lambda <- exp(seq(from = log(0.9), to = log(67), len = 19)) 
    L <- sapply(lambda, o)
    plot(log(lambda), L - max(L), type = "b")
    abline(h = c(0, -2))
    abline(v = 0)
}


###################################################
### code chunk number 20: showsushitable
###################################################

## Below we show part of the sushi preference dataset

noquote(head(sushi_table))


###################################################
### code chunk number 21: show_sushi_eq_classes
###################################################

## Below we show the equivalence classes of the different types of
## sushi

sushi_eq_classes


###################################################
### code chunk number 22: defcalculate_sushi_H
###################################################

## Below we define function make_sushi_H() which takes a particular
## numerical value of lambda and returns a hyper3 log-likelihood
## function.

make_sushi_H <- function(lambda){
    H <- hyper3()
    for(i in seq_len(nrow(sushi_table))){
        H <- H + cheering3(sushi_table[i,], e = sushi_eq_classes, h = c(1, lambda))
    }
    return(H)
}


###################################################
### code chunk number 23: calclikesushi
###################################################

## Below we define function f() which takes a particular value of
## lambda and returns the log-likelihood at the optimized value of
## Bradley-Terry strengths of the sushi


f <- function(lambda){
   H <- make_sushi_H(lambda)
   maxp(H, give = 1)$`Log-likelihood`
}
s <- exp(seq(from = log(0.9), to = log(10), len = howmany))
L <- sapply(s, f)


###################################################
### code chunk number 24: findstuffsushi
###################################################

## Below we find the maximum-likelihood estimate for lambda

osushi <- optimize(f, c(2, 4), maximum = TRUE)


###################################################
### code chunk number 25: tinysushi
###################################################

## Below we define the maximum likelihood estimate for lambda as
## 'sushimaxlambda' and also find the difference in support between
## this value of lambda and the null of lambda=1

sushimaxlambda <- osushi$maximum
sushisupport <- osushi$objective - f(1)


###################################################
### code chunk number 26: findothersushistuff
###################################################

## below we define function g() in such a way that g(x)=0 gives -2
## units of support relative to the evaluate.  This is useful to find
## the credible interval from sushi_lower to sushi_upper


g <- function(lambda){f(lambda) - osushi$objective + 2}
sushi_lower <- uniroot(g, interval = exp(c(0, 1)), tol = 0.01)$root
sushi_upper <- uniroot(g, interval = exp(c(1, 2)), tol = 0.01)$root


###################################################
### code chunk number 27: plotlikesushi
###################################################

## Below we plot the likelihood function L calculated in chunk
## calclikesushi and add some embellishments such as marking the
## credible interval, the null, and the evaluate

plot(log(s), L - max(L), type = 'b',
     xlab = expression(log(lambda)), ylab = 'Log-likelihood')
abline(v = 0, col = 'gray')
abline(h = c(0, -2), col = 'gray')
segments(x0 = log(sushimaxlambda), y0 =  0.1, y1 = -1.0, col = 'red' )
segments(x0 = log(sushi_lower    ), y0 = -2.2, y1 = -1.8, col = 'blue')
segments(x0 = log(sushi_upper    ), y0 = -2.2, y1 = -1.8, col = 'blue')


###################################################
### code chunk number 28: jss_hyper3_real.Rnw:1013-1015
###################################################

## Below we read the Boston 2018 dataset in and display some of it

a <- read.table("cloud9_boston_2018.txt", header = TRUE)
a[, -1]


###################################################
### code chunk number 29: defredbusfunc
###################################################

## Below we define function mk() which returns a hyper3 log-likelihood
## function for the counterstrike dataset given lambda, which
## quantifies the red bus-blue bus effect for the given equivalence
## classes of {{Skadoodle, Stewie2k}, {<everyone else>}}.  Function
## f() finds the support at the maximum likelihood BT strengths for
## the Cloud9 team


mk <- function(lam){
    e <- c(Skadoodle = 2, RUSH = 1, tarik = 1, autimatic = 1, Stewie2K = 2)
    H3 <- hyper3()
    for(i in seq_len(nrow(a))){
        d <- strsplit(a$deathorder[i],", ")[[1]]
        H3 <- H3 + cheering3(d, e = e, help = c(1, lam))
    }
    return(H3)
}
f <- function(l){maxp(mk(l), give=1)$`Log-likelihood`}


###################################################
### code chunk number 30: csgo_findopt
###################################################

## Below we  optimize f() defined in chunk defredbusfunc

csgo_optimize <- optimize(f, c(0.09, 0.14), maximum = TRUE, tol = 0.01)
csgo_optlam   <- csgo_optimize$maximum
csgo_support  <- csgo_optimize$objective - f(1)
csgo_pvalue   <- pchisq(csgo_support, df = 1, lower.tail = FALSE)


###################################################
### code chunk number 31: csgo_calcklikelambda
###################################################

## Below we evaluate function f() defined in defredbusfunc for a
## sequence of lambda values.

lam <- exp(seq(from = -4, to = log(1.1), len = howmany))
L <- sapply(lam, f)


###################################################
### code chunk number 32: cgso_findinterval
###################################################

## Below we find the evaluate and credible interval for the
## counterstrike dataset

g <- function(l){f(l) - csgo_optimize$objective  + 2}
csgo_lower <- uniroot(g, interval = exp(c(-4, -3)), tol = 0.01)$root
csgo_upper <- uniroot(g, interval = exp(c(-1, -0)), tol = 0.01)$root


###################################################
### code chunk number 33: csgo_plotlikelam
###################################################

## Below we plot the profile support function for lambda in the
## counterstrike dataset

plot(log(lam), L - max(L), type='b')
segments(x0 = log(csgo_optlam),y0 = -0.5, y1 =  0.1, col = 'red')
segments(x0 = log(csgo_lower), y0 = -2.2, y1 = -1.8, col = 'blue')
segments(x0 = log(csgo_upper), y0 = -2.2, y1 = -1.8, col = 'blue')
abline(v = 0, col = 'gray')
abline(h = c(0, -2), col = 'gray')

         
`tidy` <- function(H){
  b <- elements(brackets(H))
  p <- elements(powers(H))
  hyper2(b,p,pnames=sort(unique(c(b,recursive=TRUE))))
}

`keep_flawed` <- function(H, wanted){
  p <- pnames(H)    # might be NA
  bH <- elements(brackets(H))
  pH <- elements(powers(H))
  stopifnot(is.character(wanted))
  stopifnot(all(wanted %in% p))

  bracketout <- list()
  powerout <- NULL
  for(i in seq_along(H)){
    b <- bH[[i]]
    jj <- b[b %in% wanted]   # the meat
    if(length(jj) > 0){
      bracketout <- c(bracketout, list(jj))
      powerout <- c(powerout, pH[i])
    }
  }
  hyper2(L=bracketout, d=powerout, pnames=p)
}

`discard_flawed` <- function(H, unwanted){
    p <- pnames(H)
    stopifnot(is.character(unwanted))
    stopifnot(all(unwanted %in% p))
    return(keep_flawed(H, wanted=p[!(p %in% unwanted)]))
}

"summary.hyper2" <- function(object, ...){
    out <- list(
        size   = size(object),
        pnames = pnames(object),
        no.of.brackets = length(brackets(object)),
        sumofpowers = sum(powers(object)),
        tableoflengths = table(c(lapply(elements(brackets(object)), length), recursive=TRUE)),
        tableofpowers = table(powers(object))
  )
  class(out) <- "summary.hyper2"
  return(out)
}

"print.summary.hyper2" <- function(x, ...){
    cat(paste("A hyper2 object of size ", x[[1]], ".\n", sep=""))
    cat("pnames: ", x[[2]],"\n") 
    cat("Number of brackets:", x[[3]],"\n") 
    cat("Sum of powers:", x[[4]], "\n\n") 
    cat("Table of bracket lengths:")
    print(x[[5]])
    cat("\nTable of powers:")
    print(x[[6]])
}

"summary.hyper3" <- function(object, ...){
  out <- c(
      summary.hyper2(object), 
      list(tableofweights = table(unlist(weights(object))))
  )
  class(out) <- "summary.hyper3"
  return(out)
}

"print.summary.hyper3" <- function(x, ...){
    cat(paste("A hyper3 object of size ", x[[1]], ".\n",sep=""))
    cat("pnames: ", x[[2]],"\n") 
    cat("Number of brackets:", x[[3]], "\n") 
    cat("Sum of powers:", x[[4]], "\n\n") 
    cat("Table of bracket lengths:")
    print(x[[5]])
    cat("\nTable of powers:")
    print(x[[6]])
    cat("\nTable of weights:")
    print(x[[7]])
}

`profsupp` <- function(H, i, p, relative=TRUE, ...){
    out <- sapply(p,function(x){profile_support_single(H=H, i=i, p=x, evaluate=FALSE, ...)})
    if(relative){out <- out-max(out)}
    return(out)
}
    
`profile_support_single` <- function(H, i, p, evaluate=FALSE, ...){
    if(is.character(i)){
      i <- which(pnames(H) == i)
    }
    
    n <- size(H)
    delta <- 1e-4

    if(i<size(H)){  # regular, non-fillup value
        UI <- rep(0,n-1)
        UI[i] <- 1
        CI <- p
        start_min <- rep((1-p-delta)/(n-1),n-1)
        start_min[i] <- p+delta ## so p > p
        start_max <-  rep((1-p+delta)/(n-1),n-1)
        start_max[i] <- p-delta ## so p < p
    } else {   # fillup tested
        UI <- rep(-1,size(H)-1)
        CI <- p-1
        start_min <- rep(delta/n,n-1)         # all regular values small, fillup 1-delta
        start_max <- rep((1-delta)/(n-1),n-1) # all regular values big, fillup delta
    }
    
    m_min <- maxp(H,startp=start_min, fcm=+UI, fcv=+CI, ..., give=TRUE) # p_i >= p
    m_max <- maxp(H,startp=start_max, fcm=-UI, fcv=-CI, ..., give=TRUE) # p_i <= p
    ## in the above, 'm_min' interprets p as a minimum (that
    ## is, a lower bound) and 'm_max' interprets p as a
    ## maximum (that is, an upper bound).

    if(m_min$value < m_max$value){
        a <- m_min
    } else {
        a <- m_max
    }

    if(evaluate){
        jj <- fillup(a$par)
        names(jj) <- pnames(H)
        return(jj)
    } else {
        return(a$value)
    }
}

`hyper2` <-  function(L=list(), d=0, pnames){
  if(length(d) == 1){d <- rep(d,length(L))}
  if(missing(pnames)){pnames <- sort(unique(c(L,recursive=TRUE)))}
  stopifnot(is_valid_hyper2(L, d, pnames))
  out <- identityL(L, d)
  out$pnames <- pnames
  class(out) <- 'hyper2'  # This is the only class assignment in the package
  return(out)
}

## Following three functions are the only accessor methods in the package
setGeneric("brackets",function(x){standardGeneric("brackets")})
setGeneric("powers"  ,function(x){standardGeneric("powers"  )})
setGeneric("powers<-",function(x,value){standardGeneric("powers<-")})
setGeneric("pnames"  ,function(x){standardGeneric("pnames"  )})
`brackets` <- function(H){UseMethod("brackets")}
`powers`   <- function(H){UseMethod("powers"  )}
`powers<-` <- function(H,value){UseMethod("powers<-")}
`pnames`   <- function(H){UseMethod("pnames"  )}
`brackets.hyper2` <- function(H){disord(H$brackets,h=hashcal(H))}

`powers.hyper2` <- function(H){
  if(is_constant(H)){
    out <- 0
  } else {
    out <- H$powers
  }
  return(disord(out, h=hashcal(H)))
}

`powers<-.hyper2` <- function(H, value){
    stopifnot(consistent(powers(H), value))
    hyper2(elements(brackets(H)), elements(value))
}

`pnames.hyper2` <- function(H){ H$pnames }
`pnames.suplist` <- function(H){pnames(H[[1]])}
## accessor methods end

## Following function is the only setter method in the package
setGeneric("pnames<-",function(x, value){standardGeneric("pnames<-")})
`pnames<-` <- function(x, value){UseMethod("pnames<-")}
`pnames<-.hyper2` <- function(x, value){hyper2(elements(brackets(x)), elements(powers(x)), pnames=value)}

## setter methods end

`is.hyper2` <- function(H){inherits(H,"hyper2") && !inherits(H,"hyper3")}
`is.hyper3` <- function(H){inherits(H,"hyper2") &&  inherits(H,"hyper3")}

`length.hyper2` <- function(x){length(x$brackets)}

`is_constant` <- function(H){ length(H) == 0 }

`is_valid_hyper2` <- function(L,d,pnames){
  stopifnot(is.list(L))
  stopifnot(is.vector(d))
  stopifnot(is.numeric(d))
  stopifnot(length(L) == length(d))
  stopifnot(all(unique(c(L, recursive=TRUE)) %in% pnames))

  ## catch hyper2(list(c("a","a")),1):
  stopifnot(all(unlist(lapply(L, function(l){all(table(l) == 1)})))) 
  return(TRUE)
}

`size` <- function(H){
  if(is.null(H)){return(0)}
  if(inherits(H,"suplist")){
      return(max(sapply(seq_along(H),function(i){size(H[[i]])})))
  }
  if(identical(pnames(H),NA)){
    return(max(c(brackets(H),recursive=TRUE)))
  } else {
    return(length(pnames(H)))
  }
}

`as.hyper2` <- function(L, d, pnames){
    if(is.matrix(L)){
        L <- as.list(as.data.frame(t(L)))
    } else if(is.hyper3(L)){
        return(hyper3_to_hyper2(L))
    } else if(is.list(L)){
        ignore <- 0
    } else {
        stop("first argument must be a list or a matrix")
    }
    return(hyper2(L, d, pnames))
}

`.print.helper` <- function(cv){  # Character vector
  if(length(cv) == 1){return(cv)}
  out <- paste("(", cv[1], sep="")
  for(i in seq_along(cv)[-1]){ out <- paste(out, " + ", cv[i], sep="") }
  out <- paste(out, ")", sep="")
  return(out)
}
  
`print.hyper2` <- function(x, ...){
  if(!isFALSE(getOption("give_warning_on_nonzero_power_sum"))){
      if(sum(powers(x)) != 0){
          warning("powers have nonzero sum")
      }
  }
  b <- elements(brackets(x))
  powers <- elements(powers(x))
  if(length(b) == 0){  # not is.null(b)
      out <- paste(.print.helper(pnames(x)), "^0", sep="")
  }
  out <- "log("
  for(i in seq_along(b)){
    pn <- unlist(b[i])
    pp <- powers[i]
    if(pp == 1){
      out <- paste(out, .print.helper(pn))
    } else {
      out <- paste(out, .print.helper(pn), "^",pp,sep="")
    }
    if(i < length(b)){out <- paste(out," * ",sep="")}
  }
  
  out <- paste(out,")\n",sep="")
  
  for(i in strwrap(out)){
    cat(noquote(i))
    cat("\n")
  }
  return(invisible(x))
}

`hyper2_sum_hyper2` <- function(e1, e2){
  b1 <- elements(brackets(e1))
  b2 <- elements(brackets(e2))
  p1 <- elements(powers(e1))
  p2 <- elements(powers(e2))
  n1 <- pnames(e1)
  n2 <- pnames(e2)
  out <- addL(b1,p1,b2,p2)
  
  if(all(n2 %in% n1)){
      jj <- n1
  } else if(all(n1 %in% n2)){
      jj <- n2
  } else {
      jj <- sort(unique(c(b1, b2, recursive=TRUE)))
  }
  
  return(hyper2(out[[1]], out[[2]], pnames=jj))
}

`loglik` <- function(p, H, log=TRUE){
  if(is.matrix(p)){
    return(apply(p, 1, function(o){loglik_single(p=o, H, log=log)}))
  } else {
    return(loglik_single(p, H, log=log))
  }
}

`loglik_single` <- function(p, H, log=TRUE){
  stopifnot(sum(powers(H)) == 0)
  if(any(p<0)){
      stop(gettextf("All elements of p must be non-negative.  Value of p: %s", paste(p, collapse=", ")))
  }
  if(length(p) == size(H)-1){
    if(sum(p) > 1){
        stop(gettextf("length(p) == size(H)-1; elements of p must sum to <= 1. sum(p) = %s", sum(p)))
    }
    probs <- fillup(p)
  } else if(length(p) == size(H)){
    if(is.null(names(p))){stop("If length(p) == size(H), then p must be a named vector")}
    if(abs(sum(p)-1) > 1e-6){ # small numerical tolerance
        stop(gettextf("length(p) == size(H); elements of p must sum to 1. sum(p) = %s", sum(p)))
    }
    p <- ordertrans(p, H)  # no warning given if names not in correct order...
    stopifnot(identical(names(p), pnames(H))) #...but they must match up
    probs <- p
  } else {
    stop("length(p) must be either size(H) or size(H)-1")
  }
  if(is.hyper2(H)){
      out <- evaluate(brackets(H), powers(H), probs=probs, pnames=pnames(H))
  } else if(is.hyper3(H)){
      out <- evaluate3(brackets(H), weights(H), powers(H), probs=probs, pnames=pnames(H))
  } else {
      stop("H must be a hyper2 or hyper3 object")
  }

  if(log){
    return(out)
  } else {
    return(exp(out))
  }
}

`hyper2_equal` <- function(e1, e2){
  equality(
      brackets(e1), powers(e1),
      brackets(e2), powers(e2)
  )
}

`hyper2_sum_numeric` <- function(H, r){
    disord(powers(H), h=hashcal(H)) + r
}

`hyper2_prod` <- function(H, n){
    powers(H) <- powers(H) * n
    return(H)
}

`Ops.hyper2` <-
  function (e1, e2 = NULL) 
{
  f <- function(...){stop("odd---neither argument has class hyper2?")}
  unary <- nargs() == 1
  lclass <- inherits(e1, "hyper2")
  rclass <- !unary && inherits(e2, "hyper2")
  
  if(unary){
    stop("unary operator '", .Generic, "' is not implemented for hyper2 objects")
    }

  if (!is.element(.Generic, c("+", "-", "==", "!=", "*"))){
    stop("binary operator '", .Generic, "' is not implemented for hyper2 objects")
  }
  
  if(lclass && rclass){  
    if (.Generic == "+"){
      return(hyper2_sum_hyper2(e1, e2))
    } else if (.Generic == "-"){
      return(hyper2_sum_hyper2(e1, hyper2_prod(e2, -1)))
    } else if (.Generic == "==") {
      return(hyper2_equal(e1, e2))
    } else if (.Generic == "!=") {
      return(!hyper2_equal(e1, e2))
    } else {
      stop("H1 ", .Generic, " H2 not defined")
    }
  } else {  # one of lclass,rclass
    if (.Generic == "*"){    # H * n
      if(lclass && !rclass){
        return(hyper2_prod(e1, e2))
      } else if (!lclass && rclass){
        return(hyper2_prod(e2, e1))
      } else {
        stop("method not defined for hyper2")
      }
    } else if (.Generic == "+"){  # H + x
      if(lclass && !rclass){
        return(hyper2_sum_numeric(e1, e2))
      } else if (!lclass && rclass){
        return(hyper2_sum_numeric(e2, e1))
      } else {
        stop("this cannot happen")
      }
    } else if (.Generic == "-"){
      if(lclass && !rclass){
        return(hyper2_sum_numeric(e1, -e2))
      } else if (!lclass && rclass){
        return(hyper2_sum_numeric(e2, -e1))
      } else {
        stop("this cannot happen")
      }
    }
  }
}
`character_to_number` <- function(char, pnames){ # char = c("a","b","D")
    stopifnot(all(char %in% pnames))
    unlist(apply(outer(char, pnames, "=="), 1, which))
    }

`char2num` <- character_to_number

`[.hyper2` <- function(x, ...){
    dots <- list(...)
    first <- dots[[1]]

    if(nargs() > 2){  # something like H[3,4]
      wanted <- list(c(dots, recursive=TRUE))
    } else if(is.list(first)){
      wanted <- dots[[1]]
    } else if(is.matrix(first)){
        wanted <- as.list(as.data.frame(t(first)))
    } else if(is.disord(first)){
        return(hyper2(elements(brackets(x)[first]), elements(powers(x)[first])))
    } else if(is.vector(first)){
      wanted <- list(first)
    } else {
      wanted <- dots
    }

    out <- accessor(x[[1]], x[[2]], wanted)
    return(hyper2(out[[1]], out[[2]], pnames=pnames(x)))
}

`assign_lowlevel`<- function(x, index, value){ #H[index] <- value
    stopifnot(class(x) == 'hyper2')
    
    if(is.list(index)){
        ignore <- 3  # 'index' is supposed to be a list
    } else if(is.matrix(index)){
        index <- as.list(as.data.frame(t(index)))
    } else if(is.vector(index)){
        index <- list(index)
    } else {
        stop("replacement index must be a list, a matrix, or a vector")
    }
    value <- elements(value)
    stopifnot(is.numeric(value)) # coercion to integer is done in C
    stopifnot(is.vector(value))
    if(length(value) == 1){
        value <- rep(value, length(index))
    }
    return(assigner(brackets(x), powers(x), index,value))
}

`overwrite_lowlevel` <- function(x, value){
  stopifnot(class(x)     == 'hyper2')
  stopifnot(class(value) == 'hyper2')

  overwrite(brackets(x), powers(x), brackets(value), powers(value))
}

`[<-.hyper2` <- function(x, index, ..., value){
    if(inherits(value,"weight")){
        out <- setweight(as.hyper3(x),index,value)
    } else if(missing(index)){  # A[] <- B
        jj <- overwrite_lowlevel(x, value)
        if(all(pnames(value) %in% pnames(x))){
            out <- hyper2(jj[[1]], jj[[2]], pnames=pnames(x))
        } else {
            out <- hyper2(jj[[1]], jj[[2]])
        }
    } else { # index supplied
        stopifnot(consistent(index, value))
        jj <- assign_lowlevel(x, index, value)
        if(all(c(index,recursive=TRUE) %in% pnames(x))){
            out <- hyper2(jj[[1]], jj[[2]], pnames=pnames(x)) # do not change pnames
        } else { # index introduces a new pname
            out <- hyper2(jj[[1]], jj[[2]])
        }
    }
    return(out)
}

`gradient` <- function(H, probs=indep(maxp(H))){
    if(length(probs) == size(H)){
        jj <- probs
    } else if(length(probs) == size(H)-1){
        jj <- fillup(probs)
    } else {
        stop("probs is wrong length")
    }
    if(is.hyper2(H)){
        return(differentiate(brackets(H), powers(H), jj, pnames(H), size(H))$grad_comp)
    } else {
        return(differentiate3(brackets(H), weights(H), powers(H), jj, pnames(H), size(H))$grad_comp)
    }
}

`gradientn` <- function(H,probs=maxp(H)){
  stopifnot(length(probs) == size(H)) # not necessarily summing to 1!
  out <- differentiate_n(brackets(H), powers(H), probs, pnames(H),size(H))$grad_comp
  names(out) <- pnames(H)
  return(out)
}

`hessian` <- function(H, probs=indep(maxp(H)), border=TRUE){
    n <- size(H)
    stopifnot(length(probs) == n-1)
    out <- hessian_lowlevel(brackets(H), powers(H), fillup(probs), pnames(H), n)$block_hessian_components
    out <- matrix(out[seq_len((n-1)^2)], n-1, n-1)
    if(isFALSE(border)){return(out)}
    jj <- gradient(H,probs=probs)
    out <- cbind(rbind(out,jj), c(jj,0))
    rownames(out) <- pnames(H)
    colnames(out) <- pnames(H)
    return(out)
}

`is_ok_hessian` <- function(M, give=TRUE){
    stopifnot(is.matrix(M))
    stopifnot(nrow(M) == ncol(M))
    n <- nrow(M)
    M <- M[n:1, n:1]
    s <- seq(from=3, to=n)
    alt <- (s%%2) == 1  # T,F,T,F,T...
    jj <- sapply(s, function(n){det(M[seq_len(n), seq_len(n)])})
    if(give){return(jj)} else {return(all((jj>0) == alt))}
}

`fillup` <- function(x, H=NULL, total=1){
  if(is.matrix(x)){
    out <- cbind(x, total-rowSums(x))
    if(!is.null(H)){colnames(out) <- pnames(H)}
  } else {  # assumed to be a vector
    out <- c(x, total-sum(x))
    if(!is.null(H)){names(out) <- pnames(H)}
  }
  return(out)
}

`indep` <- function(x){
   if(is.matrix(x)){
     return(x[, -ncol(x)])
  } else {  # assumed to be a vector
    return(x[-length(x)])
  }
}

`maxp` <- function(H, startp=NULL, give=FALSE, fcm=NULL, fcv=NULL, SMALL=1e-6, n=1, show=FALSE, justlikes=FALSE, ...){
    if(isTRUE(getOption("use_alabama"))){ms <- maxp_single2} else {ms <- maxp_single}
    best_so_far <- -Inf # best (i.e. highest) likelihood found to date
    likes <- rep(NA, n)
    if(is.null(startp)){ startp <- indep(equalp(H)) }
    if(length(startp) == size(H)){startp <- indep(startp)}
    for(i in seq_len(n)){
        if(i>1){startp <- startp+runif(size(H)-1, max=SMALL/size(H))}
        jj <- ms(H, startp=startp, give=TRUE, fcm=fcm, fcv=fcv, SMALL=SMALL, ...)
        likes[i] <- jj$value
        if(show){cat(paste(i,"; ", best_so_far, "  " , jj$value,"\n", sep=""))}
        if(jj$value > best_so_far){ # that is, if we have found something better
            out <- jj
            best_so_far <- jj$value
        }
    }  # i loop closes
    if(justlikes){ return(likes) }
    if(isTRUE(give)){
        return(c(out,likes=list(likes),evaluate=list(fillup(out$par,H))))
    } else if(isFALSE(give)){
        return(fillup(out$par, H))
    } else {
        return(list(fillup(out$par, H),"Log-likelihood" = out$value))
    }
}  # maxp() closes

`maxp_single` <- function(H, startp=NULL, give=FALSE, fcm=NULL, fcv=NULL, SMALL=1e-6, maxtry=100, ...){
    if(inherits(H,"suplist")){return(maxplist(Hlist=H,startp=startp,give=give,fcm=fcm,fcv=fcv,...))}
    if(inherits(H,"lsl"    )){return(maxp_lsl(Hlist=H,startp=startp,give=give,fcm=fcm,fcv=fcv,...))}
    
    if(length(startp) == size(H)){startp <- indep(startp)}

    n <- size(H)
    if(is.null(startp)){
        startp <- rep(1/n,n-1)
    }

    objective <- function(p){ -loglik(p,H) }
    gradfun   <- function(p){ -(gradient(H,p))} #NB minus signs
    
    UI <- rbind(
        diag(nrow=n-1),  # regular: p1 >=0, p2 >= 0, ..., p_{n-1} >= 0
        -1,              # fillup: p_n = 1-(p1+p2+...+p_{n-1}) >= 0
        fcm)             # further problem-specific constraints
    CI <- c(
        rep(SMALL,n-1),  # regular
        -1+SMALL,        # fillup
        fcv)             # further contraint vector

    ## repeat until stupid wmmin bug does not occur...
    teenytiny <- 1e-15
    startp_try <- startp
    for(i in seq_len(maxtry)){
        out <- try(constrOptim(
            theta = startp_try,
            f     = objective,
            grad  = gradfun,
            ui    = UI,
            ci    = CI,
            ...))
        if(!inherits(out, "try-error")){  # worked!
            break
        } else { # stupid wmmin bug occurs
            cat("known R bug (bugzilla ID 17703; wmmin not finite).  Kludge:  maxp_single() will try  a slightly different start point\n")
            startp_try <- startp + teenytiny*(runif(length(startp))-0.5)
            teenytiny <- teenytiny * 1.1
        }
    }
    if(i==maxtry){  # stupid wmmin bug occurs every time...
        stop("known R bug (bugzilla ID 17703).  Try increasing maxtry.")
    }
        

    out$value <- -out$value # correct for -ve sign in objective()
    
    if(give){
      return(out)
    } else {
      jj <- fillup(out$par)
      if(!identical(pnames(H),NA)){names(jj) <- pnames(H)}
      return(jj)
    }
}

## Function maxp_single2() is like maxp_single() but uses the alabama
## package for constrained optimization instead of constrOptim() with
## its stupid wmmin finite error.




## above takes UI and CI [intended for constrOptim()] and returns a function

`maxp_single2` <- function(H, startp=NULL, give=FALSE, fcm=NULL, fcv=NULL, SMALL=1e-6, maxtry=100, ...){
    if(inherits(H,"suplist")){return(maxplist(Hlist=H,startp=startp,give=give,fcm=fcm,fcv=fcv,...))}
    
    n <- size(H)
    if(is.null(startp)){
        startp <- rep(1/n,n-1)
    }

    if(length(startp) == size(H)){startp <- indep(startp)}

    objective <- function(p){ -loglik(p,H) }
    gradfun   <- function(p){ -(gradient(H,p))} #NB minus signs
    
    UI <- rbind(
        diag(nrow=n-1),  # regular: p1 >=0, p2 >= 0, ..., p_{n-1} >= 0
        -1,              # fillup: p_n = 1-(p1+p2+...+p_{n-1}) >= 0
        fcm)             # further problem-specific constraints
    CI <- c(
        rep(SMALL,n-1),  # regular
        -1+SMALL,        # fillup
        fcv)             # further contraint vector

        out <- constrOptim.nl(
            par = startp,
            fn = objective,
            gr  = gradfun,
            hin = function(x){drop(UI%*%x - CI)},
            control.outer = list(trace=FALSE),
            ...)
   
    out$value <- -out$value # correct for -ve sign in objective()
    
    if(give){
      return(out)
    } else {
      jj <- fillup(out$par)
      if(!identical(pnames(H),NA)){names(jj) <- pnames(H)}
      return(jj)
    }
}

`maxp_simplex` <- function(H, n=100, show=FALSE, give=FALSE, ...){
    if(isTRUE(getOption("use_alabama"))){ms <- maxp_single2} else {ms <- maxp_single}
    best_so_far <- -Inf # best (i.e. highest) likelihood found to date
    likes <- rep(NA,n)
    for(i in seq_len(n)){
        jj <- ms(H, startp=indep(rp_unif(1,H)), give=TRUE, ...)
        likes[i] <- jj$value
        if(show){cat(paste(i,"; ", best_so_far, "  " , jj$value,"\n", sep=""))}
        if(jj$value > best_so_far){ # that is, if we have found something better
            out <- jj
            best_so_far <- jj$value
        }
    }  # i loop closes, stop searching
    if(give){
        return(c(out, likes=list(likes)))
    } else {
        out <- fillup(out$par)
        if(!identical(pnames(H), NA)){names(out) <- pnames(H)}
        return(out)
    }
}

`maxplist` <- function (Hlist, startp = NULL, give = FALSE, fcm = NULL, fcv = NULL, SMALL=1e-6, ...){
    n <- size(Hlist[[1]])
    if (is.null(startp)) {
        startp <- rep(1/n, n - 1)
    }
    objective <- function(p) {
        -like_single_list(p, Hlist)
    }
    
    UI <- rbind(diag(nrow = n - 1), -1, fcm)
    CI <- c(rep(SMALL, n - 1), -1 + SMALL, fcv)

    if(isTRUE(getOption("use_alabama"))){
        out <- constrOptim.nl(par = startp, fn = objective, gr = NULL, 
                              hin = function(x){drop(UI%*%x - CI)},
                              control.outer = list(trace=FALSE),...)
    } else {
        out <- constrOptim(theta = startp, f = objective, grad = NULL, 
                           ui = UI, ci = CI, ...)
    }
        out$value <- -out$value
        if (give) {
        return(out)
    }
    else {
        jj <- fillup(out$par)
        return(jj)
    }
}

`maxp_lsl` <- function (HLSL, startp = NULL, give = FALSE, fcm = NULL, fcv = NULL, SMALL=1e-6, ...){
    allpnames <- pnames(HLSL[[1]][[1]])
    n <- length(allpnames)

    if (is.null(startp)) {
        startp <- rep(1/n, n)
	names(startp) <- allpnames
        startp <- indep(startp)
    }
    
    objective <- function(p) { -loglik_lsl(p, HLSL) }
    
    UI <- rbind(diag(nrow = n - 1), -1, fcm)
    CI <- c(rep(SMALL, n - 1), -1 + SMALL, fcv)
    
    if(isTRUE(getOption("use_alabama"))){
        out <- constrOptim.nl(par = startp, fn = objective, gr = NULL, 
                              hin = function(x){drop(UI%*%x - CI)},
                              control.outer = list(trace=FALSE),...)
    } else {
        out <- constrOptim(theta=startp,f=objective,grad = NULL,ui=UI,ci=CI,...)
    }
    out$value <- -out$value
    if (give) {
        return(out)
    }
    else {
        jj <- fillup(out$par)
        names(jj) <- allpnames
        return(jj)
    }
}

`sum.hyper2` <- function(x, ..., na.rm=FALSE){
  if(nargs()==1){
    return(x)
  } else if (nargs()==2){
    return(hyper2_sum_hyper2(x, ...))
  } else {
    return(hyper2_sum_hyper2(x, Recall(...)))
  }
}

`head.hyper2` <- function(x, ...){ x[head(elements(brackets(x)), ...)] }

`rankvec_likelihood` <- function(v, nonfinishers = NULL){
  stopifnot(all(table(v)==1))
  out <- hyper2()
  v <- rev(v)   # first-placed competitor is the first element of v
  for(i in seq_along(v)){
      out[v[i]] <- out[v[i]] + 1  #  out[v[i]] %<>% inc
      still_running <- c(v[seq_len(i)], nonfinishers)
      out[still_running] <- out[still_running] - 1
  }
  return(out)
}

`race` <- rankvec_likelihood

#`addrank` <- function(H,ranks){
#     .Defunct(new = 'rank_likelihood',msg='use H <- H+rank_likelihood(...)')
#     ranks <- rbind(ranks)
# 
#     for(r in seq_len(nrow(ranks))){
#         rank <- rev(ranks[r,])
#         for(i in seq_along(rank)){
#             H[rank[i]] <- powers(H[rank[i]]) + 1
#             H[rank[seq_len(i)]] <- powers(H[rank[seq_len(i)]])-1
#         }
#     }
#     return(H)
# }

`rrank_single` <- function(p){
    ell <- length(p)
    r <- integer(ell)
    i <- 1   # 'i' means placing.
    while(any(p > 0)){  
        m <- sample(ell,1,prob=p) #randomly chosen competitor
        r[i] <- m   # place 'i' is competitor 'm'  
        p[m] <- 0   # competitor 'm' can't place again.
        i <- i+1    # consider the next place
    }
    return(r)
}

rorder_single <- function(p){
    o <- rrank_single(p)
    o[o] <- seq_along(o)
    return(o)
}

`rrank` <- function(n=7, p=(4:1)/10, pnames=NULL, fill=FALSE, rnames=NULL){
    if(fill){ p <- fillup(p) }
    if(is.null(pnames)){pnames <- names(p)}
    if(is.null(pnames)){pnames <- letters[seq_along(p)]}
    out <- t(replicate(n, rrank_single(p)))
    out[] <- pnames[out]
    rownames(out) <- rnames
    return(as.ranktable(out))
}

`rrankk` <- function(n = 37, p = (20:1)/210, pnames=NULL, fill=FALSE, rnames=NULL){
    rrank(n = n, p = p, pnames = pnames, fill = fill, rnames = rnames)
}

`.allorders` <- function(x){
  out <- perms(length(x))
  matrix(x[out],nrow=length(x))
}

`pair_grid` <- function(a,b){  ## sort of a  generalized expand.grid()
  rbind(
      kronecker(a,t(rep(1L,ncol(b)))),
      kronecker(t(rep(1L,ncol(a))),b))
}

`mult_grid` <- function(L){
  if(length(L) == 0){
    return(1)
  } else if(length(L) == 1){
    return(L)
  } else {
    return(Recall(c(list(pair_grid(L[[1]],L[[2]])),L[-(1:2)])))
  }
}

`general_grouped_rank_likelihood` <- function(H, ...){
  ## ggrl(1:3,4,5:9) means group(1:3) came first, 4 in the middle, 5:9
  ## last
    dotargs <- list(...)
    if(any(unlist(lapply(dotargs, is.character)))){
        dotargs <- lapply(dotargs, character_to_number, pnames=pnames(H))
    }

    if(any(table(c(dotargs,recursive=TRUE)) > 1)){
        stop('repeated competitor')
    }

    out <- mult_grid(lapply(dotargs, .allorders))[[1]]
    out[] <- pnames(H)[out]
    out <- apply(out,2,rankvec_likelihood)
    for(i in seq_along(out)){
        pnames(out[[i]]) <- pnames(H)
    }
    return(as.suplist(out))
}

`ggrl` <- general_grouped_rank_likelihood

`choose_losers` <- function(H,all_players,losers){
  stopifnot(losers %in% all_players)
  winners <- all_players[!all_players %in% losers]
  ggrl(H,winners,losers)
}

`choose_winners` <- function(H,all_players, winners){
  stopifnot(all(winners %in% all_players))
  losers <- all_players[!all_players %in% winners]
  ggrl(H,winners,losers)
}

`goodbad` <- function(winners,losers){
    stopifnot(!any(winners %in% losers))
    ggrl(hyper2(pnames=c(winners, losers)), winners, losers)
}

`elimination` <- function(all_players){
    if(is.numeric(all_players) && (length(all_players)==1)){
      all_players <- letters[seq_len(all_players)]
    }
    all_players <- rev(all_players)
    H <- ggrl(hyper2(pnames=sort(all_players)),all_players,all_players[length(all_players)])
    players <- all_players[-length(all_players)]
    while(length(players) > 1){
        for(i in seq_along(H)){
            H[[i]] <- ggrl(H[[i]],players,players[length(players)])
        }
        H <- unlist(H,recursive=FALSE)
        players <- players[-length(players)]
    }
    return(as.suplist(H))
}

`like_single_list` <- function(p,Lsub){;  # eg like_single(p,Lc)
  sum(unlist(lapply(Lsub,loglik,p=p,log=FALSE)))
  ## sum, because it can be any one of the orders specified in the
  ## elements of Lsub
}

`like_series` <- function(p,L,log=TRUE){
    out <- prod(unlist(lapply(L, function(Lsub){like_single_list(p,Lsub)})))
    if(log){ out <- log(out) }
    return(out)
}

`dirichlet` <-  function(powers, alpha){
    if(!xor(missing(powers),missing(alpha))){
        stop("supply exactly one of powers, alpha")
    }
  
    if(missing(powers)){
        powers <- alpha-1
    }

    np <- names(powers)
    if(is.null(np)){stop("supply a named vector")}
    hyper2(as.list(np),d=powers) - hyper2(list(np),d=sum(powers))
}

`GD` <- function(alpha, beta, beta0=0){
  k <- length(alpha)
  stopifnot(length(beta) == k-1)
  
  H <- dirichlet(powers=alpha-1)
  for(i in 2:(k-1)){
    H[names(alpha)[i:k]] <- beta[i-1] -(alpha[i]+beta[i])
  }

  H[names(alpha)[k]] <- beta[k-1]-1
  H[names(alpha)] <- beta0-(alpha[1]+beta[1])
  return(H)   
}

`GD_wong` <- function(alpha, beta){
  k <- length(alpha)
  stopifnot(length(beta) == k)

  gamma <- beta[-k]-alpha[-1]-beta[-1]
  gamma <- c(gamma, beta[k]-1)
  
  H <- dirichlet(powers=alpha-1)
  jj <- gamma[length(gamma)]
  H[names(jj)] <- jj
  for(i in seq_len(k-1)){
    H[names(alpha)[(i+1):(k)]] <- gamma[i]
  }
  return(H)   
}

`equalp` <- function(H){
    n <- size(H)
    out <- rep(1/n,n)
    names(out) <- pnames(H)
    return(out)
}

`zipf` <- function(n){
  nn <- NULL
  if(is.hyper2(n)){
    nn <- pnames(n)
    n <- size(n)
  }
  jj <- 1/seq_len(n)
  names(jj) <- nn
  jj/sum(jj)
}

`saffy` <- function(M){
    out <- hyper2(pnames=colnames(M))
    for(i in seq_len(nrow(M))){
        onerow <- M[i,]
        choices <- names(onerow[!is.na(onerow)])
        out[choices] %<>%  subtract(sum(onerow,na.rm=TRUE))
    }
    for(j in seq_len(ncol(M))){
        out[colnames(M)[j]] %<>% add(sum(M[,j],na.rm=TRUE))
    }
    return(out)
}

`volley` <- function(M){
    out <- hyper2()
    pn <- colnames(M)
    if(is.null(pn)){stop("colnames of matrix are NULL")}

    for(i in seq_len(nrow(M))){
        onerow <- M[i,]
        winning_side <- onerow == 1
        losing_side  <- onerow == 0
        playing <- !is.na(onerow)
        if(any(playing) && any(winning_side) && any(losing_side)){
          out[pn[which(winning_side)]] %<>% inc
          out[pn[which(playing)]] %<>% dec
        }
    }
    return(out)
}

`inc` <- function(H, val=1){ H %<>% add(val)}   # increment
`dec` <- function(H, val=1){ H %<>% subtract(val)}   # decrement

`trial` <- function(winners, players, val=1){
    stopifnot(all(winners %in% players))
    H <- hyper2()
    H[winners] %<>% add(val)
    H[players] %<>% subtract(val)
    return(H)
}

`pick` <-  function(winners, allplayers){
    stopifnot(all(winners %in% allplayers))
    hyper2(L = list(winners, allplayers), d=c(1,-1))
}

`pass` <-  function(losers,  allplayers){
    stopifnot(all(losers %in% allplayers))
    hyper2(L = list(setdiff(allplayers, losers), allplayers), d=c(1,-1))
}

`beats` <- function(winners, losers){
    stopifnot(length(intersect(winners, losers)) == 0)
    hyper2(L = list(winners, c(winners, losers)), d=c(1,-1))
}

`loses` <- function(losers, winners){ beats(winners,losers) }

`rhyper2` <- function(n=8, s=5,  pairs=TRUE, teams=TRUE, race=TRUE, pnames=letters){
  n <- n - n%%2  # Force 'n' to be even
  H <- hyper2()
  if(pairs){
      M <- replicate(s,sample(n,2,replace=FALSE),simplify=TRUE)
      M[] <- pnames[M]
      H <- hyper2(split(M,rep(seq_len(ncol(M)),each=nrow(M))),-1)
      H <- H + hyper2(as.list(M[1,]),1)
  }

  if(teams){
      for(i in seq_len(s)){
          players <- pnames[seq_len(n)]
          H <- H + trial(sample(players,n/2,replace=FALSE),players)
      }
  }

  if(race){  # Plackett-Luce
      for(i in seq_len(s)){
          H <- H + rankvec_likelihood(pnames[sample(n)])
    }
  }

  return(H)
}

`ordervec2supp` <- function(d){
    if(is.null(names(d))){stop("vector must be named")}
    wanted <- d != 0
    if(any(sort(d[wanted]) != seq_len(sum(wanted)))){
        stop("nonzero elements of d should be 1,2,3,4,...,n")
    }
    
    nd <- names(d)
    out <- hyper2()
    while(any(d>0)){
        eligible <- which(d>=0)  #NB inclusive inequality; zero is DNC etc who

        ## Increment numerator power of the first choice among eligible players:
        out[nd[d == 1]] %<>% inc
        
        ## Power of set of all eligible players decrements:
        out[nd[eligible]] %<>% dec
        
        ## once you've come first in the field, you are ineligible to be first again:
        d[d == 1] <- -1  # NB strictly <0
        
        ## Now, everyone moves down the list, so who *was* in
        ## second place becomes first place, who *was* third place
        ## becomes second, and so on:
        
        d[d > 0] %<>% dec
        
    } # while() loop closes
    return(out)
}

`ordertable2supp` <- function(x, noscore, noninformative, incomplete=TRUE){
    if(missing(noscore)){
        noscore <- c("Ret", "WD", "DNS", "DSQ", "DNP", "NC", "DNQ", "EX", "Sick")
    }
    if(missing(noninformative)){
        noninformative <- c("dead", "won")
    }
    venues <- colnames(x)

    ## Now create a numeric matrix, fmat.  Two steps: first, count
    ## any no-score as zero (and noninformative scores as -1):
  
    jj <- apply(x,2,function(y){
        if(any(y %in% noscore)){y[y%in%noscore] <- 0}
        if(any(y %in% noninformative)){y[y %in% noninformative] <- -1}
        return(y)
    })
    
    ## Second, convert to numeric and strip out names; transpose of
    ## x (because we want each row to be a venue):
    fmat <- matrix(as.numeric(jj),byrow=TRUE,ncol=nrow(x)) 
    colnames(fmat) <- rownames(x)
    rownames(fmat) <- venues
    
    out <- hyper2(d=ncol(fmat))
    
    ## Now cycle through the rows; each row is a venue [voter]
    for(i in seq_len(nrow(fmat))){
        o <- fmat[i,,drop=TRUE]
        if(incomplete){ o[o>0] <- rank(o[o>0]) }
        o <- o[o >= 0] # remove uninformative observations
        out %<>% add(ordervec2supp(o))
    } # i loop closes
    return(out)
} 

`zapweak` <- function(H, minstrength=1e-5, maxit, ...){
  if(missing(maxit)){maxit <- size(H)-1}
  for(i in seq_len(maxit)){
    cat(paste("iteration ",i,", size(H) = ",size(H),"\n",sep=""))
    m <- maxp(H,n=1,...)
    too_weak <- m < minstrength
    if(any(too_weak)){
      H %<>% discard_flawed(names(m[which.min(m)]))
    } else {
     break
    }
  }
  return(H)
}

`pwa` <- function(H, pwa, chameleon='S', hyper3=FALSE){
    if(hyper3){return(pwa23(H, pwa, chameleon='S'))}
    stopifnot(is.hyper2(H))
    if(length(pwa) > 1){stop("argument pwa must be of length 1")}

    `%notin%` <- Negate(`%in%`)  # Remove this for R >= 4.6-0
    if(any(pwa %notin% pnames(H))){
        stop(gettextf(
            "Argument pwa (%s) must be a member of pnames (%s).",
            pwa, paste(pnames(H), collapse = ", ")))
    }
    if(chameleon %in% pnames(H)){
        stop(gettextf(
            "Argument chameleon (%s) must not be a member of pnames (%s)",
            chameleon, paste(pnames(H), collapse = ", ")))
        }
  
    B <- elements(brackets(H))
    ## overwrite B in place:
    for(i in seq_along(B)){
        if(any(pwa %in% B[[i]])){ B[[i]] <- c(B[[i]],chameleon) }
    }
    return(hyper2(B,elements(powers(H))))
}

`advantage` <- function(H, ...){
    sb <- list(...)
    v <- names(sb)
    out <- H
    for (i in seq_along(sb)) {
        out <- pwa(out, v[i], sb[[i]])
    }
    return(out)
}

`wikitable_to_ranktable`  <- function(wikitable, strict=FALSE){
  f <- function(x){  # deal with DNF etc and zero
    suppressWarnings(out <- as.numeric(as.vector(x)))
    DNF <- is.na(out) | (out == 0)
    if(sum(DNF) > 1 && strict){
      warning("more than one competitor did not place.  EM algorithm used")
    }
    out[DNF] <- max(out[!DNF]) + seq_len(sum(DNF))
    return(out)
  }

  xx <- apply(wikitable,2,f)
  rownames(xx) <- rownames(wikitable)
  ordertable_to_ranktable(xx)
}

`is.dirichlet` <- function(H){
    len <- unlist(elements(lapply(brackets(H), length)))
    pow <- elements(powers(H))
    bra <- elements(brackets(H))
    if (sum(len > 1) > 1) {
        print("brackets must all be of length 1 [the numerators] except one [the denominator]")
        return(FALSE)
    }
    denominator_bracket <- unlist(bra[len > 1])
    numerator_brackets <- unlist(bra[len == 1])
    if (all(numerator_brackets %in% denominator_bracket)) {
        return(TRUE)
    }
    else {
        print("Dirichlet distribution requires denominator to be all competitors")
        return(FALSE)
    }
}

`rdirichlet` <- function(n, H){
    if(is.hyper2(H)){
        stopifnot(is.dirichlet(H))
        alpha <- setNames(sapply(seq_len(size(H)), function(i){powers(H[pnames(H)[i]])}), pnames(H)) + 1
        pn <- pnames(H)
        s <- size(H)
    } else {   # H is vector of alpha
        pn <- names(H)
        s <- length(H)
        alpha <- H
    }
    out <- t(apply(matrix(rgamma(n*s, shape=alpha), s, n), 2, function(x){x/sum(x)}))
    colnames(out) <- pn
    return(out)
}

`rp_unif` <-function(n,H){
    stopifnot(is.hyper2(H))
    alpha <- rep(1,size(H)) # '1' because rdirichlet() takes alpha, not powers
    names(alpha) <- pnames(H)
    return(rdirichlet(n=n,H=alpha))
}
    
`discard_flawed2` <- function(x, unwanted,...){
    if(is.character(unwanted)){
        wanted <- !which(rownames(x) == unwanted)
    }
    o <- wikitable_to_ranktable(x,...) # was   o <- ordertable_to_ranktable(x)
    class(o) <- "matrix"
    o <- t(apply(o[,unwanted],1,rank))
    colnames(o) <- rownames(x)[unwanted]
    class(o) <- "ranktable"
    return(ranktable_to_ordertable(o))
}

`ordertable` <- function(x){
    x <- as.matrix(x)
    if(is.null(rownames(x))){stop("rownames must be supplied")}
    ## This is the only place class ordertable is set:
    class(x) <- c("ordertable", "matrix")
    return(x)
}

`print.ordertable` <- function(x, ...){
    cat("An ordertable:\n")
    print(noquote(unclass(x)))
}

`aperm.ordertable` <- function(a, perm, ...) {
  a <- unclass(a)
  aperm(a, perm, ...)
}

`as.ordertable` <- function(w){
    if(is.ranktable(w)){return(ranktable_to_ordertable(w))}
    out <-
        apply(w,2,    
              function(x){ # 'x' is a column of w
                  out <- suppressWarnings(as.numeric(x))  # 'ret' etc -> NA
                  no <- is.na(out)
                  o <- out[!no]
                  o[o > 0] <- rank(o[o > 0])
                  out[!no] <- o
                  out[no] <- 0
                  return(out)
              } )
    rownames(out) <- rownames(w)
    return(ordertable(out))
}

`ordertable2points` <- function(o,points,totals=TRUE){
    points <- c(points,rep(0,1+max(o)-length(points)))
    o[o == 0] <- length(points)
    o[] <- points[o]
    if(totals){
        return(rowSums(o))
    } else {
        return(o)
    }
}

`ordertrans` <- function(x,players){

    if(missing(players)){
      return(x[order(names(x))])
    } else {
      if(is.hyper2(players) || is.hyper3(players)){
        players <- pnames(players)
      }
    }
      
    stopifnot(length(x) == length(players))
    stopifnot(all(sort(names(x)) == sort(players)))
    stopifnot(all(table(names(x)) == 1))
    x[apply(outer(players,names(x),`==`),1,which)]
}

`ordertransplot` <- function(ox,oy, plotlims, ...){
  stopifnot(all(sort(names(ox)) == sort(names(oy))))
  ox <- ordertrans(ox)
  oy <- ordertrans(oy)
  par(pty='s') # square plot
  if(missing(plotlims)){plotlims <- c(0,max(c(ox,oy)))}
  plot(ox,oy,asp=1,pty='s',xlim=plotlims,ylim=plotlims,pch=16,...)
  abline(0,1)
  for(i in seq_along(ox)){text(ox[i],oy[i],names(ox)[i],pos=4,col='gray',cex=0.7)}
}

`consistency` <- function(H,plot=TRUE,...){
  m1 <- maxp(H)
  pnames(H) <- rev(pnames(H))
  m2 <- rev(maxp(H))
  out <- rbind(orig=m1,rev=m2,diff=m1-m2)
  if(plot){
    par(pty="s")
    plot(m1,m2,xlab='',ylab='',log="xy",type="n")
    abline(0,1,col='gray')
    points(m1,m2,pch=16, ...)
    for (i in seq_along(m1)) {
      text(m1[i], m2[i], names(m1)[i], pos = 4, col = "gray", cex = 0.7)
    }
    return(invisible(out))
  } else {
    return(out)
  }
}

`zermelo` <- function(M, maxit=100, start, tol=0, give=FALSE){
    diag(M) <- 0  # usual convention is NA on the diagonal
    rM <- rowSums(M)  # only need to calculate this once
    M <- M + t(M)
    if(missing(start)){
        p <- rep(1/nrow(M), nrow(M))  # starting point for iteration
    } else {
        p <- start
    }
    if(give){
        pout <- matrix(0, maxit, ncol(M))
        colnames(pout) <- colnames(M)
        pout[1,] <- p
    }
    for(i in seq_len(maxit-1)){
        pnew <- rM/colSums(M/outer(p, p, `+`))  # the meat
        pnew[is.nan(pnew)] <- 0
        pnew <- pnew/sum(pnew)  # normalize
        if(give){pout[i+1,] <- pnew}
        if(all(abs(p-pnew) <= tol)){break} else {p <- pnew}  # maybe finish early
    }
    if(give){
        return(pout[seq_len(i+1),])
    } else {
        return(pnew)
    }
}

`pairwise` <- function(M){
    diag(M) <- 0
    if(is.null(rownames(M))){
        rownames(M) <- paste("p", seq_len(nrow(M)), sep="_")
        colnames(M) <- rownames(M)
    }
    out <- dirichlet(rowSums(M))
    out[rownames(M)] <- 0
    jj <- M + t(M)
    index <- which(upper.tri(M), arr.ind=TRUE)
    return(out - hyper2(apply(index, 1, function(x){rownames(M)[x]}, simplify=FALSE), jj[index]))
}

`home_away` <- function (home_games_won, away_games_won, monster="home"){
    if (is.complex(home_games_won)) {
        away_games_won <- Im(home_games_won)
        home_games_won <- Re(home_games_won)
    }
    
    teams <- rownames(home_games_won)
    stopifnot(identical(teams, colnames(home_games_won)))
    stopifnot(identical(teams, rownames(away_games_won)))
    stopifnot(identical(teams, colnames(away_games_won)))
    teams <- c(teams, monster)
    H <- hyper2(pnames = teams)
    for (i in seq_len(nrow(home_games_won))) {
        for (j in seq_len(ncol(home_games_won))) {
            if (i != j) {
                home_team <- teams[i]
                away_team <- teams[j]

		home_wins <- home_games_won[i,j]
		away_wins <- away_games_won[i,j] 
                no_of_matches <- home_wins + away_wins 
		
                H[c(home_team,           monster)] %<>% inc(home_wins)
                H[c(          away_team         )] %<>% inc(away_wins) 
                H[c(home_team,away_team, monster)] %<>% dec(no_of_matches)
            }
        }
    }
    return(H)
}

`balance` <- function(H){
    H[pnames(H)] <- 0
    H[pnames(H)] <- -sum(powers(H))
    return(H)
}

`rrace` <- function(strengths){
    out <- strengths
    for (i in seq_along(out)) {
        nextfinisher <- sample(names(strengths),size=1,prob=strengths)
        out[i] <- nextfinisher
        strengths <- strengths[names(strengths) != nextfinisher]
    }
    names(out) <- NULL
    return(out)
}

`home_away_table` <- function(a, give = FALSE, teams){
    if(missing(teams)){ teams <- unique(c(a$hometeam, a$awayteam)) }
    homewins <- matrix(0, length(teams), length(teams))
    dimnames(homewins) <- list(`@home` = teams, `@away` = teams)
    awaywins <- homewins
    draws    <- homewins
    for(i in seq_len(nrow(a))){
        if(a$homescore[i] > a$awayscore[i]){ # home win
            homewins[a$hometeam[i], a$awayteam[i]] %<>% add(1)
        } else if(a$homescore[i] < a$awayscore[i]){  # away win
            awaywins[a$hometeam[i], a$awayteam[i]] %<>% add(1)
        } else { # draw
            draws[a$hometeam[i], a$awayteam[i]] %<>% add(1)
        }
    }

    diag(homewins) <- NA
    diag(awaywins) <- NA
    diag(draws)    <- NA
    
    if(give){
        return(list(home_wins = homewins, away_wins = awaywins, draws = draws))
    } else {
        return(homewins + 1i*awaywins)
    }
}

`[.ordertable` <- function(x, i, j, ..., drop = FALSE) {
   as.ordertable(NextMethod("[", drop = drop))
}

test_that("dirichlet() and dirichlet3() tests", {

    f <- function(n){
        jj <- setNames(sample(n,n),sample(letters[seq_len(n)]))
        expect_true(dirichlet(alpha=jj+1) == dirichlet(powers=jj))
    }

    sapply(4:9,f)

    
    expect_true(
        dirichlet(powers=c(t=4,r=6,p=2)) ==
        hyper2(list("r","t",c("t","p","r"),"p") ,c(6,4,-12,2))
    )


    expect_error(dirichlet3(1:5))  # no names attribute
    
    jj <- setNames(4:6,c("o","a","w"))
    expect_error(dirichlet3(jj,setNames(0.8 + 1:3,c("o","w","a")))) # names different order
    
    expect_true(
        dirichlet3(jj,7) ==
        hyper3(list(c(a=1),c(a=1,o=7,w=1),c(o=7),c(w=1)), powers=c(5,-15,4,6))
    )
    
    expect_true(
        dirichlet3(jj,lambda=1:3) ==
        hyper3(list(c(a=2),c(a=2,o=1,w=3),c(o=1),c(w=3)),powers=c(5,-15,4,6))
    )

} )

test_that("suplist tests", {

W <- hyper2(pnames=letters[1:5])
W1 <- ggrl(W, 'a', letters[2:3],'d')  # 2-element list
W2 <- ggrl(W, 'e', letters[1:3],'d')  # 6-element list
W3 <- ggrl(W, 'c', letters[4:5],'a')  # 2-element list

W1+W2+W3

a <- lsl(list(W1,W2,W3),4:6)
loglik_lsl(equalp(W),a,log=TRUE)

} )

test_that("integrate() tests", {
    TOL <- 1e-7

    x <- c(a=1,b=2,c=3)
    expect_true(abs(B(dirichlet(alpha = x)) - prod(gamma(x))/gamma(sum(x))) < TOL)

    expect_true(all(abs(mean_hyper2(dirichlet(alpha=x)) - c(1/6, 1/3, 1/2)) < TOL))

    
} )

test_that("ranktable tests", {

    x <- rrank()
    expect_output(print(x))
    expect_output(print(summary(x)))

    expect_true(all(as.ranktable(as.ordertable(x)) == x))


    a <- skating_table
    o <- as.ordertable(unclass(a)[order(rownames(a)),])
    expect_true(all(as.ordertable(as.ranktable(o)) == o))
    


    expect_silent(ordertable_to_ranktable(pentathlon_table))

    x <- cbind(c(a=3, b=1, c=2), 1:3)
    colnames(x) <- c("j1", "j2")
    x <-  as.ordertable(x)
    y <- as.ranktable(x)
    expect_true(all(unclass(y) == letters[c(2,1,3,2,1,3)]))

    expect_true(suppfun(pentathlon_table) == pentathlon)
    
} )

## This file follows the structure of aaa.R in the freegroup package.

## Define some checker functions, and call them at the end.  They
## should all return TRUE if the package works, and stop with error if
## a test is failed.  Function checker1() has one argument, checker2()
## two, and checker3() has three.  

test_that("Test suite aaa.R",{

checker1 <- function(x){

    expect_error(!x)

    expect_error(x < x)
    expect_error(x > x)
    expect_error(x <=x)
    expect_error(x >=x)

    expect_error(x/x)
    expect_error(x^x)
  

    if(length(x) > 1){expect_true(is.disord(x+1))}
    if(length(x) > 1){expect_true(is.disord(x-1))}
    if(length(x) > 1){expect_true(is.disord(1+x))}
    if(length(x) > 1){expect_true(is.disord(1-x))}

    expect_true(x == x, info=x)

    expect_true(1*x == x, info=x)
    expect_true(2*x == x+x, info=x)
    expect_true(3*x == x+x+x, info=x)
    expect_true(4*x == x+x+x+x, info=x)
    expect_true(5*x == x+x+x+x+x, info=x)
    expect_true(6*x == x+x+x+x+x+x, info=x)

    expect_true(x+x-x == x, info=x)

  return(TRUE)
}  # checker1() closes


checker2 <- function(x,y){
  expect_true(x == y+x-y, info=list(x,y))
  expect_true(x+y == y+x, info=list(x,y))

  return(TRUE)
}

checker3 <- function(x,y,z){
  expect_true(x+(y+z) == (x+y)+z, info=list(x,y,z)) # additive associativity
  return(TRUE)
} # checker3() closes


for(i in 1:2){
    x <- rhyper2()
    y <- rhyper2()
    z <- rhyper2()

    checker1(x)
    checker2(x,y)
    checker3(x,y,z)

}

checker1(rhyper2()*0)
checker2(rhyper2(),rhyper2()*0)
checker2(rhyper2(),rhyper2()*0)
checker3(rhyper2(),rhyper2(),rhyper2()*0)
checker3(rhyper2(),rhyper2()*0,rhyper2()*0)

})

`equalp.test` <- function(H, startp=NULL, ...){
    n <- size(H)
    m_alternative <- maxp(H,startp=startp, give=TRUE,...)
    alternative_support <- m_alternative$value
    jj <- fillup(m_alternative$par)
    names(jj) <- pnames(H)
    alternative_estimate <- jj
    
    df <- size(H)-1

    jj <- equalp(H)
    names(jj) <- pnames(H)
    null_estimate <- jj
    null_support <- loglik(indep(jj),H)

    support_difference <- alternative_support-null_support

    if(identical(pnames(H),NA)){
        jj <- paste(paste("p_",seq_len(n)," = ",sep=""),collapse="")
        null_hypothesis <- substr(jj,1,nchar(jj)-3)
    } else {
        null_hypothesis = paste(pnames(H),collapse=" = ")
    }

    rval <- list(
        statistic = support_difference,
        p.value = pchisq(2*support_difference,df=df,lower.tail=FALSE),
        df = df,
        null_hypothesis = null_hypothesis,
        null_estimate = null_estimate,
        null_support = null_support,

        alternative_hypothesis = "sum p_i=1",
        alternative_estimate = alternative_estimate,
        alternative_support = alternative_support,
        method = "Constrained support maximization",
        data.name = deparse(substitute(H))
        )
    class(rval) <- "hyper2test"
    return(rval)
}   # equalp.test() closes

`knownp.test` <- function(H,p,...){

    n <- size(H)
    m_alternative <- maxp(H,give=TRUE,...)
    alternative_support <- m_alternative$value
    jj <- fillup(m_alternative$par)
    names(jj) <- pnames(H)
    alternative_estimate <- jj
    
    df <- size(H)-1

    if(missing(p)){stop("p must be supplied")}
    stopifnot(identical(names(p),pnames(H)))
    null_estimate <- p
    null_support <- loglik(indep(p),H)

    support_difference <- alternative_support-null_support
    null_hypothesis <- paste(paste(paste(names(p),"=",round(p,getOption("digits"))),collapse=", "))

    rval <- list(
        statistic = support_difference,
        p.value = pchisq(2*support_difference,df=df,lower.tail=FALSE),
        df = df,
        null_hypothesis = null_hypothesis,
        null_estimate = null_estimate,
        null_support = null_support,

        alternative_hypothesis = "sum p_i=1",
        alternative_estimate = alternative_estimate,
        alternative_support = alternative_support,
        method = "Constrained support maximization",
        data.name = deparse(substitute(H))
        )
    class(rval) <- "hyper2test"
    return(rval)
}   # knownp.test() closes



`specificp.test` <- function(H, i, specificp=1/size(H), alternative = c("two.sided","less","greater"), ...){
    alternative <- match.arg(alternative)

    return(switch(alternative,
                  "two.sided" = specificp.ne.test(H=H, i=i, specificp, ...),
                  "less"      = specificp.lt.test(H=H, i=i, specificp, ...),
                  "greater"   = specificp.gt.test(H=H, i=i, specificp, ...)
                  )
           )
}   # specificp.test() closes

`specificp.ne.test` <- function(H, i, specificp=1/size(H), ...){
    rsp <- round(specificp,getOption("digits"))  # for printing purposes
    if(is.character(i)){
      null_hypothesis <- paste("sum p_i=1, ",i, " = ", rsp, sep="")
      if(any(i %in% pnames(H))){
          i <- which(pnames(H)==i)
      } else {
          stop(gettextf("entity %s not in pnames(H)", dQuote("i")))
      }
    } else {
      null_hypothesis <- paste("sum p_i=1, p_",i, " = ", rsp, sep="")
    }
    delta <- 1e-5
    specificp <- max(delta,specificp)
    
    n <- size(H)
    # Do the null (restricted optimization) first:

    if(i<size(H)){  # regular, non-fillup value
        UI <- rep(0,n-1)
        UI[i] <- 1
        CI <- specificp
        start_min <- rep((1-specificp-delta)/(n-1),n-1)
        start_min[i] <- specificp+delta ## so p > specificp
        start_max <-  rep((1-specificp+delta)/(n-1),n-1)
        start_max[i] <- specificp-delta ## so p < specificp
    } else {   # fillup tested
        UI <- rep(-1,size(H)-1)
        CI <- specificp-1
        start_min <- rep(delta/n,n-1)         # all regular values small, fillup 1-delta
        start_max <- rep((1-delta)/(n-1),n-1) # all regular values big, fillup delta
    }

    m_min <- maxp(H,startp=start_min, fcm=+UI, fcv=+CI, ..., give=TRUE) # p_i >= specificp
    m_max <- maxp(H,startp=start_max, fcm=-UI, fcv=-CI, ..., give=TRUE) # p_i <= specificp
    ## in the above, 'm_min' interprets specificp as a minimum (that
    ## is, a lower bound) and 'm_max' interprets specificp as a
    ## maximum (that is, an upper bound).

    if(m_min$value < m_max$value){
        a <- m_min
    } else {
        a <- m_max
    }

    jj <- fillup(a$par)
    names(jj) <- pnames(H)
    null_estimate <- jj
    null_support <- a$value
        
    ## Now the alternative, free optimization
    m_alternative <- maxp(H, ..., give=TRUE)  # free optimization
    alternative_support <- m_alternative$value
    jj <- fillup(m_alternative$par)
    names(jj) <- pnames(H)
    alternative_estimate <- jj

    ## For debugging, this is a good place to uncomment the following line
    ## browser()

    df <- 1
    support_difference <- alternative_support - null_support

    rval <- list(
        statistic = support_difference,
        p.value = pchisq(2*support_difference,df=df,lower.tail=FALSE),
        df = df,
        null_hypothesis = null_hypothesis,
        null_estimate = null_estimate,
        null_support = null_support,
        alternative_hypothesis = "sum p_i=1",
        alternative_estimate = alternative_estimate,
        alternative_support = alternative_support,
        method = "Constrained support maximization",
        sidedness = "(two sided)",
        data.name = deparse(substitute(H))
        )
    class(rval) <- "hyper2test"
    return(rval)
}   # specificp.ne.test() closes

`specificp.gt.test` <- function(H, i, specificp=1/size(H), delta=1e-5, ...){  # alternative = "greater"
    ## NB here we treat specificp as a *lower* bound for the
    ## optimization, the constraint being 'specificp <= p_i' (or,
    ## operationally, p_i <= specificp); the sense of 'greater'---as
    ## in, why the function is ".gt." rather than ".lt."---is that we
    ## suspect p_i is _greater than_ specificp and want to find
    ## evidence for this; so, operationally, we try to reject the
    ## hypothesis that p_i is less than specificp (and infer that p_i
    ## is indeed greater than specificp).

    rsp <- round(specificp,getOption("digits"))  # for printing purposes
    if(is.character(i)){
      null_hypothesis <- paste("sum p_i=1, ", i, " <= ", rsp, sep="")
      if(any(i %in% pnames(H))){
          i <- which(pnames(H)==i)
      } else {
          stop(gettextf("entity %s not in pnames(H)", dQuote("i")))
      }
    } else {
      null_hypothesis <- paste("sum p_i=1, p_",i, " = ", rsp, sep="")
    }
    if(specificp==0){null_hypothesis <- c(null_hypothesis, " (notional)")}
    n <- size(H)
  
    # Do the null first: (restricted optimization, p <= specificp
    if(i<size(H)){  # regular, non-fillup value
        UI <- rep(0,n-1)
        UI[i] <- 1
        start_max <- rep((1-specificp+delta)/(n-1),n-1)
        if(specificp==0){
            CI <- delta
            start_max[i] <- delta/2 ## so p < specificp
        } else {
            CI <- specificp
            start_max[i] <- specificp-delta 
        }
    } else {   # fillup tested
        if(specificp==0){
        UI <- rep(-1,size(H)-1)
        CI <- delta-1
        start_max <- rep((1-delta*(n-1)/n)/(n-1),n-1)
        } else {
        UI <- rep(-1,size(H)-1)
        CI <- specificp-1
        start_max <- rep((1-specificp+delta)/(n-1),n-1)
        }
    }

    a <- maxp(H,startp=start_max, fcm=-UI, fcv=-CI, ..., give=TRUE) # p_i <= specificp
    ## in the above, maxp() interprets specificp as a
    ## maximum (that is, an upper bound).

    jj <- fillup(a$par)
    if(!identical(pnames(H),NA)){names(jj) <- pnames(H)}
    null_estimate <- jj
    null_support <- a$value
        
    ## Now the alternative, free optimization
    m_alternative <- maxp(H, ..., give=TRUE)  # free optimization
    alternative_support <- m_alternative$value
    jj <- fillup(m_alternative$par)
    if(!identical(pnames(H),NA)){names(jj) <- pnames(H)}
    alternative_estimate <- jj
  
    alternative_hypothesis <- "sum p_i=1"
    ## For debugging, this is a good place to uncomment the following line
    ## browser()

    df <- 1
    support_difference <- alternative_support - null_support

    rval <- list(
        statistic = support_difference,
        p.value = pchisq(2*support_difference,df=df,lower.tail=FALSE),
        df = df,
        null_hypothesis = null_hypothesis,
        null_estimate = null_estimate,
        null_support = null_support,
        alternative_estimate = alternative_estimate,
        alternative_support = alternative_support,
        alternative_hypothesis = alternative_hypothesis,
        method = "Constrained support maximization",
        sidedness = "(one-sided)",
        data.name = deparse(substitute(H))
        )
    class(rval) <- "hyper2test"
    return(rval)
}   # specificp.gt.test() closes

`specificp.lt.test` <- function(H, i, specificp=1/size(H), ...){  # alternative = "less"
    ## NB here we treat specificp as an *upper* bound for the
    ## optimization, the constraint being 'specificp >= p_i' (or,
    ## operationally, p_i >= specificp); the sense of 'less'---as
    ## in, why the function is ".lt." rather than ".gt."---is that we
    ## suspect p_i is _less than_ specificp and want to find
    ## evidence for this; so, operationally, we try to reject the
    ## hypothesis that p_i is greater than specificp (and infer that p_i
    ## is indeed less than specificp).


    rsp <- round(specificp,getOption("digits"))  # for printing purposes
    if(is.character(i)){
      null_hypothesis <- paste("sum p_i=1, ",i, " >= ", rsp, sep="")
      if(any(i %in% pnames(H))){
          i <- which(pnames(H)==i)
      } else {
          stop(gettextf("entity %s not in pnames(H)", dQuote("i")))
      }
    } else {
      null_hypothesis <- paste("sum p_i=1, p_",i, " = ", rsp, sep="")
    }

    delta <- 1e-4
    specificp <- min(max(delta,specificp),1-delta)
    n <- size(H)
  
    # Do the null first: (restricted optimization, p >= specificp
    if(i<size(H)){  # regular, non-fillup value
        UI <- rep(0,n-1)
        UI[i] <- 1
        CI <- specificp
        start_min <- rep((1-specificp-delta)/(n-1),n-1)
        start_min[i] <- specificp+delta ## so p > specificp
    } else {   # fillup tested
        UI <- rep(-1,size(H)-1)
        CI <- specificp-1
        start_min <- rep((1-specificp-delta)/(n-1),n-1) 
    }

    a <- maxp(H,startp=start_min, fcm=+UI, fcv=+CI, ..., give=TRUE) # p_i >= specificp
    ## in the above, maxp() interprets specificp as a
    ## minimum (that is, a lower bound).

    jj <- fillup(a$par)
    if(!identical(pnames(H),NA)){names(jj) <- pnames(H)}
    null_estimate <- jj
    null_support <- a$value
        
    ## Now the alternative, free optimization
    m_alternative <- maxp(H, ..., give=TRUE)  # free optimization
    alternative_support <- m_alternative$value
    jj <- fillup(m_alternative$par)
    if(!identical(pnames(H),NA)){names(jj) <- pnames(H)}
    alternative_estimate <- jj
  
    alternative_hypothesis <- "sum p_i=1"

    ## For debugging, this is a good place to uncomment the following line
    ## browser()

    df <- 1
    support_difference <- alternative_support - null_support

    rval <- list(
        statistic = support_difference,
        p.value = pchisq(2*support_difference,df=df,lower.tail=FALSE),
        df = df,
        null_hypothesis = null_hypothesis,
        null_estimate = null_estimate,
        null_support = null_support,
        alternative_estimate = alternative_estimate,
        alternative_support = alternative_support,
        alternative_hypothesis = alternative_hypothesis,
        method = "Constrained support maximization",
        sidedness = "(one-sided)",
        data.name = deparse(substitute(H))
        )
    class(rval) <- "hyper2test"
    return(rval)
}   # specificp.lt.test() closes

`samep.test` <- function(H, i, give=FALSE, startp=NULL, ...){
    n <- size(H)
    stopifnot(all(table(i)==1))
    stopifnot(length(i) > 1)
    stopifnot(length(i) < n)
    
    if(is.character(i)){
        stopifnot(all(i %in% pnames(H)))
        jj <- paste(paste(i," = ",sep=""),collapse="")
        null_hypothesis <- substr(jj,1,nchar(jj)-3)
        i <- which(pnames(H) %in% i)
    } else {  # numeric
        stopifnot(all(i<=n))
        jj <- paste(paste("p_",i," = ",sep=""),collapse="")
        null_hypothesis <- substr(jj,1,nchar(jj)-3)
    }
    SMALL <- 1e-6

    m_alternative <- maxp(H, startp=startp, ..., give=TRUE)  # free optimization
    alternative_support <- m_alternative$value
    jj <- fillup(m_alternative$par)
    names(jj) <- pnames(H)
    alternative_estimate <- jj
    
    o <- n-length(i) 

    ## Function q_to_p() takes a minimal vector V and returns a vector
    ## of strengths.  We would have fillup(q_to_p(jj)) summing to 1.
    ## Vector V is suitable for constrOptim() to work with.

    ## Browse[1]> icons_maxp
    ##         NB          L         PB        THC         OA       WAIS 
    ## 0.25230411 0.17364433 0.22458188 0.17011281 0.11068604 0.06867083 

    ## Browse[1]> startq
    ## [1] 0.1000000 0.1666567 0.1666567 0.1666567

    ## Browse[1]> q_to_p(startq)
    ## [1] 0.1666567 0.1000000 0.1666567 0.1666567 0.1000000

    ## Browse[1]> q_to_p(startq)
    ## [1] 0.1666567 0.1000000 0.1666567 0.1666567 0.1000000

    ## Browse[1]> fillup(q_to_p(startq))
    ## [1] 0.1666567 0.1000000 0.1666567 0.1666567 0.1000000 0.3000300

    ## optimization proceeds over R^(n-length(i))
    startq <- rep(-SMALL+1/n,n-length(i))  # old one


    if(any(i==n)){
        `q_to_p` <- function(qq){   # NB 'i' in scope
            ii <- i[i<n]  # ii is one shorter than i
            out <- rep(NA,n-1)
            out[ii] <- qq[1]
            r <- max(which(!(seq_len(n-1) %in% ii)))
            out[r] <- 1-qq[1]*length(i)-sum(qq[-1])  # ensures fillup correct
            out[-c(i,r)] <- qq[-1]
            return(out)
        }
    } else {
      startq[1] <- mean(alternative_estimate[i])
      startq[-1] <- alternative_estimate[-i][-(n-length(i))]
      startq <- startq*(1-SMALL)
        `q_to_p` <- function(qq){   # NB 'i' in scope
            out <- rep(0,n-1)
            out[i] <- qq[1]
            out[-i] <- qq[-1]
            return(out)
        }
    } # if(i==n) closes

    startq <- pmax(startq, SMALL*(1+SMALL))
    
    `objective` <- function(jj){ ## jj == c(v,p[-i])  (NB p[i]==v)
        -loglik(q_to_p(jj),H)
    }
    

    ## constraints on q:
    UI <- rbind(diag(nrow = o),-c(length(i),rep(1,o-1)))
    CI <- c(rep(SMALL,o),SMALL-1)


    constrained_opt <-
        constrOptim(theta = startq, f = objective, grad = NULL, ui = UI, ci = CI, ...)
    null_support <- -constrained_opt$value
    null_estimate <- fillup(q_to_p(constrained_opt$par))
    names(null_estimate) <- pnames(H)
    support_difference <- alternative_support - null_support
    df <- length(i)-1

    rval <- list(
        statistic = support_difference,
        p.value = pchisq(2*support_difference,df=df,lower.tail=FALSE),
        df = df,
        null_hypothesis = null_hypothesis,
        null_estimate = null_estimate,
        null_support = null_support,
        alternative_hypothesis = "sum p_i=1",
        alternative_estimate = alternative_estimate,
        alternative_support = alternative_support,
        method = "Constrained support maximization",
        sidedness = "",
        data.name = deparse(substitute(H))
        )
    class(rval) <- "hyper2test"
    return(rval)
}   # samep.test() closes

`print.hyper2test` <- function(x,...){
    cat("\n")
    cat(strwrap(x$method, prefix = "\t"), sep = "\n")
    cat("\n")
    cat("data:  ", x$data.name, "\n", sep = "")

    cat("null hypothesis: ", x$null_hypothesis, "\n", sep="")
    cat("null estimate:\n")
    print(x$null_estimate)
    cat("(argmax, constrained optimization)\n")
    cat("Support for null: ",x$null_support, "+ K\n\n")

    cat("alternative hypothesis: ",x$alternative_hypothesis, "\n")
    cat("alternative estimate:\n")
    print(x$alternative_estimate)
    cat("(argmax, free optimization)\n")
    cat("Support for alternative: ",x$alternative_support, "+ K\n\n")

    cat("degrees of freedom: ", x$df, "\n", sep = "")
    cat("support difference = ", x$statistic, "\n",sep="")
    cat("p-value: ", x$p.value, " ",x$sidedness, "\n", sep = "")
    cat("\n")
    return(invisible(x))
}   # print.hyper2test() closes

test_that("Miscellaneous hyper3 tests", {

    expect_output(print(equalp.test(icons)))
    expect_output(print(knownp.test(icons,setNames(zipf(6),pnames(icons)))))
    expect_output(print(samep.test(icons,c("NB","L"))))
    expect_output(print(specificp.test(icons,"NB",0.1)))
    expect_output(print(specificp.test(icons,"NB",0.1,alternative="greater")))
    expect_output(print(specificp.test(icons,"NB",0.1,alternative="less")))
    expect_output(print(specificp.test(icons,"NB",0.1,alternative="two.sided")))
} )

`rp` <- function(n, H, startp=NULL, fcm=NULL, fcv=NULL,SMALL=1e-6, l=loglik, fillup=TRUE, ...){

  mypi <- function(p){
    s <- size(H)
    UI <- rbind(diag(nrow = s - 1), -1, fcm)
    CI <- c(rep(SMALL, s - 1), -1 + SMALL, fcv)
    if(any(UI %*% p < CI)){
      return(0)
    } else {
      return(l(p,H,log=FALSE))
    }
  }
  if(is.null(startp)){startp <- indep(equalp(H))}
  out <- calibrator::MH(n,start=startp,sigma=diag(0.01,nrow=size(H)-1),pi=mypi)
  if(fillup){
    out <- cbind(out,1-rowSums(out))
    colnames(out) <- pnames(H)
  }

  return(out)
}

`lsl` <- function(suplists,powers){
    stopifnot(is.list(suplists))
    stopifnot(unlist(lapply(suplists,function(x){inherits(x,"suplist")})))
    stopifnot(is.numeric(powers))
    stopifnot(is.vector(powers))
    stopifnot(length(powers) == length(suplists))
    out <- list(suplists=suplists,powers=powers)
    class(out) <- "lsl"
    return(out)
}

`loglik_lsl` <- function(p, LSL, log = TRUE){
    out <- 0
    for(i in seq_along(LSL$powers)){
        out <- out + LSL$powers[i]*log(like_single_list(p,LSL$suplists[[i]]))
    }
    if(log){
        return(out)
    } else {
        return(exp(out))
    }
}
    
`Ops.lsl` <- function(e1,e2){
    f <- function(...){stop("odd---neither argument has class suplist?")}
    unary <- nargs() == 1
    lclass <- inherits(e1,"lsl")
    rclass <- !unary && inherits(e2,"lsl")
    
    if(unary){stop("Unary operators not implemented for lsl objects")}
    
    if(.Generic == "+"){
        stopifnot(lclass & rclass)
        return(lsl_add(e1,e2))
    } else {
        stop(gettextf("binary operator %s not implemented for lsl objects", dQuote(.Generic)))
    }
}

`lsl_add` <- function(e1,e2){  # terribly inefficient, do not use
    lsl(
        suplists = c(e1$suplists,e2$suplists),
        powers   = c(e1$powers, e2$powers)
    )
}

setGeneric("suppfun",function(x){standardGeneric("suppfun")})

`suppfun` <- function(x, ...){UseMethod("suppfun")}
`suppfun.ordertable` <- function(x, ...){ordertable2supp(x, ...)}
`suppfun.character` <- function(x, nonfinishers=NULL, ...){rankvec_likelihood(x, nonfinishers=nonfinishers, ...)}
`suppfun.numeric` <- function(x, ...){ordervec2supp(x)}

`cheering3` <- function(v,e,help,nonfinishers=NULL){

  ## competitors abcdef, with two groups of mutual friends: (abd) with
  ## mutual helpfulness 1.1, and (cf) with mutual helpfulness 1.3;
  ## competitors "e" and "g" are in groups of 1.

  ## use-case:
  ## cheering3(v=letters[1:7],e=c(a=1,b=1,c=2,d=1,f=2),help=c(1.88,1.66))


  stopifnot(all(table(names(e))==1))
  stopifnot(all(e>0))
  stopifnot(all(e==round(e)))

  no_group <- !(v %in% names(e))
  if(any(no_group)){
    jj <- seq(from=max(e)+1,len=sum(no_group))
    names(jj) <- v[no_group]   # a character vector
    e <- c(e,jj)
  }

  out <- hyper3()

  finished <- NULL # these are competitors who have finished and are cheering their friends
  still_running <- v
  while(length(still_running) > 0){
    winner <- still_running[1]  # winner is the first element of the field
    winners_friends <- names(e[e==e[winner]])
    jjn <- 1  # numerator
    names(jjn) <- winner

    winner_ec_number <- e[names(e)==winner]  # this would be an integer; "ec" means "equivalence class"
    finished_ec_number <- e[names(e) %in% finished]

    if(winner_ec_number %in% finished_ec_number){  # that is, if the winner has a friend who has finished...
      jjn[] <- help[winner_ec_number]   # ...then he gets extra strength.
    }

    jjd <- rep(1,length(c(still_running,nonfinishers))) # denominator
    names(jjd) <- c(still_running,nonfinishers)
    for(i in seq_along(jjd)){
      runner_ec_number <- e[names(e) == c(still_running,nonfinishers)[i]]
      if(length(finished_ec_number)>0){
        if(length(runner_ec_number) ==0){browser()}
        if(runner_ec_number %in% finished_ec_number){ jjd[i] <- help[runner_ec_number] }
      }
    }
    out[jjn] %<>% inc   # numerator
    out[jjd] %<>% dec   # denominator
    finished <- c(finished,winner)     # the winner is now finished ...
    still_running <- still_running[-1] # ... and not still running
  } # while(length(still_running)>0) loop closes
  return(out)
}

#' @export
multinom <- function(x){
    stopifnot(all(x >= 0))
    xf <- as.frab(x)
    v <- disordR::elements(frab::values(xf)) # vector of integers
    n <- disordR::elements(names(xf))        # vector of names

    if(all(x > 0)){  # NB strict
        L <- c(n, list(n))
    } else {
        L <- c(n, list(unique(names(x))))
    }
    return(hyper2(L, c(v, -sum(v))))
}

`as.suplist` <- function(L){
  if(inherits(L,"hyper2")){out <- list(L)}
  stopifnot(is.list(L))
  stopifnot(all(unlist(lapply(L,function(x){inherits(x,"hyper2")}))))
  class(L) <- c("list","suplist")
  return(L)
}

`Ops.suplist` <-
  function (e1, e2 = NULL) 
{
  f <- function(...){stop("odd---neither argument has class suplist?")}
  unary <- nargs() == 1
  lclass <- inherits(e1,"suplist")
  rclass <- !unary && inherits(e2,"suplist")
  
  if(unary){stop("Unary operators not implemented for suplist objects")}
  
  if(.Generic == "+"){
      stopifnot(lclass & rclass)    
      return(suplist_add(e1,e2))
    } else if (.Generic == "*"){
      if(lclass & rclass){
          stop("<suplist> * <suplist> not defined")
      } else if(lclass & !rclass){  # W*6
          return(suplist_times_scalar(e1,e2))
      } else if (!lclass & rclass){ # 6*W
          return(suplist_times_scalar(e2,e1))
      } else {
	  f()
      }
    } else {
      stop("Binary operator '", .Generic, "' is not implemented for hyper2 objects")
    }
}

`suplist_times_scalar` <- function(e1,e2){
   stopifnot(e2==round(e2))
   stopifnot(length(e2)==1)
   stopifnot(e2>=0)
   if(e2==0){
     return(as.suplist*lapply(e1,function(x){x*0}))
   } else if(e2==1){
     return(e1)
   } else {
     out <- e1
     for(i in seq_len(e2-1)){out <- out + e1}
     return(out)
   }
}

`suplist_add` <- function(e1,e2){
  e1 <- as.suplist(e1)
  e2 <- as.suplist(e2)
  n <- length(e1)
  out <- list()
  for(i in seq_along(e1)){
    for(j in seq_along(e2)){
      out[[i + (j-1)*n]] <- e1[[i]] + e2[[j]]
    }
  }
  return(as.suplist(out))
}

`sum.suplist` <- function(x, ..., na.rm=FALSE){
  if(nargs()==1){
    return(x)
  } else if (nargs()==2){
    return(suplist_add(x, ...))
  } else {
    return(suplist_add(x, Recall(...)))
  }
}

`p_to_e` <- function(p){c(sum(p), p[-length(p)]/rev(cumsum(rev(p))[-1]))}

`e_to_p` <- function(e){e[1]* cumprod(c(1,1-e[-1]))* c(e[-1],1)}

`B` <- function(H, disallowed=NULL, give=FALSE, ...){  # modelled on hyperdirichlet::calculate_B()
  n <- size(H)-1  # "-1" because this is the dimension of the integrand, not the number of p's.

  ## No special dispensation for the Dirichlet!

  if(is.null(disallowed)){
    f <- function(e){ dhyper2_e(e,H, include.Jacobian=TRUE) }  # integrate over the whole simplex
  } else {
    f <- function(e){as.numeric(!disallowed(e_to_p(c(1,e)))) * dhyper2_e(e, H, include.Jacobian=TRUE)}
  }

  out <- adaptIntegrate(f,lowerLimit=rep(0,n),upperLimit=rep(1,n), ...)
  if(give){
    return(out)
  } else {
    return(out$integral)
  }
}

`Jacobian` <-
function(e){
  n <- length(e)
  e1 <- e[1]
  e <- e[-c(1,length(e))]
  prod(cumprod(1-e))*e1^n
}

`dhyper2` <- function(ip,H,...){
  ip <- rbind(ip)
  loglik(ip,H,log=FALSE)/B(H,...)
}

`dhyper2_e` <-  # analogous to dhyperdirichlet_e()
function(e, H, include.Jacobian=TRUE){
  
  e <- c(1,e)
  p <- e_to_p(e)

  out <- loglik(indep(p),H,log=FALSE)
  if(include.Jacobian){out <- Jacobian(e)*out}
  return(out)
}

`mgf` <- function(H,powers,...){ B(H + dirichlet(powers = powers), ...)/B(H, ...) }

`mean_hyper2` <- function(H, normalize=TRUE, ...){
  f <- function(i){
    jj <- rep(0,size(H))
    jj[i] <- 1
    names(jj) <- pnames(H)
    return(mgf(H,jj, ...))
  }

  out <- sapply(seq_len(size(H)),f)
  if(!identical(pnames(H),NA)){names(out) <- pnames(H)}
  if(normalize){
    out <- out/sum(out)
  }
  return(out)
}

"probability" <- function(H , disallowed=NULL, ...){
  B(H , disallowed=disallowed , give=FALSE, ...) / B(H, ...)
}

`print.ranktablesummary` <- function(x, ...){
    x <- unclass(x)
    x <- cbind(x[,1:3], "...", x[,ncol(x) + c(-1,0)])
    x <- noquote(x)
    print(x)
    class(x) <- NULL
    return(invisible(x))
}

`summary.ranktable` <- function(object, ...){
    class(object) <- "ranktablesummary"
    return(object)
}

`ranktable_to_ordertable` <- function(xrank){
    out <- apply(xrank, 1, order)
    rownames(out) <- sort(unique(as.vector(xrank)))
    colnames(out) <- rownames(xrank)
    return(as.ordertable(out))
}

`ordertable_to_ranktable` <- function(xorder){
  stopifnot(all(apply(xorder, 2, function(x){all(sort(x) == seq_along(x))})))
  o <- t(apply(xorder, 2, function(x){seq_len(nrow(xorder))[order(x)]}))
  colnames(o) <- rownames(xorder)
  o <- rbind(o)
  if(is.null(colnames(o))){
    cn <- seq_len(ncol(o))
  } else {
    cn <- colnames(o)
  }
  out <- matrix(cn[o], ncol = ncol(o))
  colnames(out) <- paste("c", seq_len(ncol(o)), sep = "")
  rownames(out) <- rownames(o)
  return(as.ranktable(out))
}

setGeneric("as.ranktable", function(x){standardGeneric("as.ranktable")})

`is.ranktable` <- function(x){inherits(x, "ranktable")}
`as.ranktable` <- function(x){UseMethod("as.ranktable")}
`as.ranktable.ordertable` <- function(x){ordertable_to_ranktable(x)}
`as.ranktable.ranktable` <- function(x){x}
`as.ranktable.matrix` <- function(x){
    stopifnot(is.matrix(x))
    stopifnot(is.character(x))
    stopifnot(all(apply(x, 1, table) == 1))
    colnames(x) <- paste("c", seq_len(ncol(x)),sep="")
    class(x) <- "ranktable"  # this is the only place yadda yada ya
    return(x)
}

`print.ranktable` <- function(x,...){
    cat("A ranktable:\n")
    print(noquote(unclass(x)))
}

`suppfun.ranktable` <- function(x, times, ...){
  x <- rbind(x)  # deals with vectors
  if(missing(times)){times <- 1}
  times <- cbind(times, rep(1, nrow(x)))[,1]
  out <- hyper2()
  for(i in seq_len(nrow(x))){
      out <- out + suppfun(x[i, , drop=TRUE]) * times[i]
  } 
  return(out)
}

`psubs_single` <- function(H, from, to){
    stopifnot(length(from) == 1)
    stopifnot(length(to) == 1)
    b <- elements(brackets(H))
    p <- elements(powers(H))
    stopifnot(!(to %in% c(b,recursive=TRUE)))
    hyper2(lapply(b, function(x){x[x == from] <- to ; return(x)}), p)
}

`psubs` <- function(H, from, to){
    if(missing(to)){
        to <- from
        from <-pnames(H)
    }
    stopifnot(length(from) == length(to))
    wanted <- from %in% pnames(H)
    from <- from[wanted]
    to <- to[wanted]
        
    l <- length(from)
    if(l == 0){
        return(H)
    } else if(l == 1){
        return(psubs_single(H, from, to))
    } else {
        return(Recall(psubs_single(H, from[1], to[1]), from[-1], to[-1]))
    }
}

`as.attemptstable` <- function(x){
    x <- as.data.frame(x)
    class(x) <- c("attemptstable", "data.frame")  # the only place
    return(x)
}

`print.attemptstable` <- function(x, ...){
    cat("An attemptstable:\n")
    co <- capture.output(as.data.frame(x))
    
    cat(co[1]) # colnames
    cat("\n")
    for(i in 2:length(co)){
        string <- gsub("NA", " X", co[i])
        jj <- strsplit(string, " ")[[1]][-1]
        suppressWarnings(p <- as.numeric(jj[nchar(jj) > 0]))
        if(!isFALSE(getOption("bold_personal_best"))){
            if(any(!is.na(p))){
                string <- rowfiddle(string, which.max(p))
            }
        }
        cat(string)
        cat("\n")
    }
}

rowfiddle <- function(v, n){  # takes a string, eg v="Vesely     79.73   80.3  85.44      X  84.98      X"

    jj <- rle(strsplit(v, "")[[1]] == " ")
    start_position <- sum(jj$lengths[seq_len(n*2    )])
      end_position <- sum(jj$lengths[seq_len(n*2 + 1)])

    paste(
        substr(v, 1, start_position),
        crayon::bold(substr(v, start_position+1, end_position)),
        substr(v, end_position+1, nchar(v)),
        sep = ""
    )
}

`suppfun.attemptstable` <- function(x, ...){
    attemptstable2supp3(x, ...)
}

as.vector.attemptstable <- function(x, mode){
    sort(setNames(c(x,recursive=TRUE), rep(rownames(x),ncol(x))), decreasing=TRUE)
}

`attemptstable2supp3` <- function(x, decreasing=TRUE, give.supp=TRUE, nothrow_loses=FALSE){
    o <- c(matrix(suppressWarnings(as.numeric(c(x, recursive=TRUE))), nrow(x), ncol(x)))
    names(o) <- c(matrix(rownames(x), nrow(x), ncol(x)))
    o <- sort(o, na.last=TRUE, decreasing=decreasing)
    if(give.supp){
        if(nothrow_loses){
            nf <- names(o)[is.na(o)]
        } else {
            nf <- NULL
        }
        return(ordervec2supp3(v=names(o)[!is.na(o)], nonfinishers=nf))
    } else {
        return(o)
    }
}

`rattemptstable` <- function(ncompetitors=8, nthrows=6, prob=0.23){   
    n <- ncompetitors*nthrows
    out <- matrix(round(runif(n),3), ncol=nthrows)
    out[sample(seq_len(n), round(n*prob), replace=FALSE)] <- -Inf
    m <- apply(out, 1, max, na.rm=TRUE)
    out <- out[order(-m),]
    out[is.infinite(out)] <- NA
    rownames(out) <- letters[seq_len(ncompetitors)]
    colnames(out) <- paste("throw", seq_len(nthrows), sep="")
    return(as.attemptstable(out))
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

identityL <- function(L, powers) {
    .Call(`_hyper2_identityL`, L, powers)
}

addL <- function(L1, powers1, L2, powers2) {
    .Call(`_hyper2_addL`, L1, powers1, L2, powers2)
}

equality <- function(L1, powers1, L2, powers2) {
    .Call(`_hyper2_equality`, L1, powers1, L2, powers2)
}

accessor <- function(L, powers, Lwanted) {
    .Call(`_hyper2_accessor`, L, powers, Lwanted)
}

overwrite <- function(L1, powers1, L2, powers2) {
    .Call(`_hyper2_overwrite`, L1, powers1, L2, powers2)
}

assigner <- function(L, powers, L2, value) {
    .Call(`_hyper2_assigner`, L, powers, L2, value)
}

evaluate <- function(L, powers, probs, pnames) {
    .Call(`_hyper2_evaluate`, L, powers, probs, pnames)
}

hessian_lowlevel <- function(L, powers, probs, pnames, n) {
    .Call(`_hyper2_hessian_lowlevel`, L, powers, probs, pnames, n)
}

differentiate <- function(L, powers, probs, pnames, n) {
    .Call(`_hyper2_differentiate`, L, powers, probs, pnames, n)
}

differentiate_n <- function(L, powers, probs, pnames, n) {
    .Call(`_hyper2_differentiate_n`, L, powers, probs, pnames, n)
}

identityL3 <- function(L, W, p) {
    .Call(`_hyper2_identityL3`, L, W, p)
}

addL3 <- function(L1, W1, p1, L2, W2, p2) {
    .Call(`_hyper2_addL3`, L1, W1, p1, L2, W2, p2)
}

equality3 <- function(L1, W1, p1, L2, W2, p2) {
    .Call(`_hyper2_equality3`, L1, W1, p1, L2, W2, p2)
}

accessor3 <- function(L, W, powers, Lwanted, Wwanted) {
    .Call(`_hyper2_accessor3`, L, W, powers, Lwanted, Wwanted)
}

overwrite3 <- function(L1, W1, powers1, L2, W2, powers2) {
    .Call(`_hyper2_overwrite3`, L1, W1, powers1, L2, W2, powers2)
}

assigner3 <- function(L, W, p, L2, W2, value) {
    .Call(`_hyper2_assigner3`, L, W, p, L2, W2, value)
}

evaluate3 <- function(L, W, powers, probs, pnames) {
    .Call(`_hyper2_evaluate3`, L, W, powers, probs, pnames)
}

differentiate3 <- function(L, W, powers, probs, pnames, n) {
    .Call(`_hyper2_differentiate3`, L, W, powers, probs, pnames, n)
}

`hyper3` <- function(B=list(), W=list(), powers=0, pnames){
    if(is.matrix(B)){
      return(hyper3_m(B, W))
    } else if(all(unlist(lapply(B, is_ok_weightedplayers)))){
      return(hyper3_nv(B, powers, pnames))
    } else {
      return(hyper3_bw(B, W, powers, pnames))
    }
}

`hyper3_bw` <- function(B=list(), W=list(), powers=0, pnames){
    ## hyper3_bw(list(c("a","b"),c("a","b","d")),list(c(1,4),c(1,3,4)),c(1,-1),letters[1:5])
  
    stopifnot(length(B) == length(W))
    stopifnot(all(c(elements(W), recursive=TRUE) >= 0))
    stopifnot(all(unlist(lapply(B, length)) == unlist(lapply(W, length))))

    if(length(powers) == 1){powers <- rep(powers, length(B))}
    jj <- sort(unique(c(elements(B), recursive=TRUE)))
    if(missing(pnames)){
        pnames <- jj
    } else {  # pnames not missing; must be complete
        stopifnot(all(jj %in% pnames))
    }
    out <- identityL3(B, W, powers)  # the meat
    out$pnames <- pnames
    class(out) <- c('hyper3', 'hyper2')  # this is the only place class hyper3  is set
    return(out)
}

`hyper3_nv` <- function(L=list(), powers=0, pnames){ # "nv" = Named Vectors
    ## hyper3_nv(list(c(a=1,b=4),c(b=1,c=3,e=9)),1:2,letters[1:5])
    stopifnot(all(unlist(lapply(L, is_ok_weightedplayers))))
    hyper3_bw(lapply(L,names), lapply(L,as.vector), powers, pnames)
}

`hyper3_m` <- function(M, p, stripzeros=TRUE){
  if(stripzeros){
    wanted <- apply(M,1,function(x){any(x != 0)})
    M <- M[wanted,]
    p <- p[wanted]
  }

  hyper3(
      B      = rep(list(colnames(M)), nrow(M)),
      W      = unclass(as.data.frame(t(M))),
      powers = p[wanted],
      pnames = colnames(M)
  )
}

`is_ok_weightedplayers` <- function(x){ # x=c(a=33, b=4, c=65)
    if(is.null(names(x))){
        return(FALSE)
    }
    if(any(is.null(names(x)))){
        return(FALSE)
    }
    if(any(x < 0)){
        return(FALSE)
    }
    return(TRUE)
}

`is_valid_hyper3` <- function(L,d,pnames){
    if(length(L) != length(d)){stop("length of L must match length of d")}
    if(any(!(unique(c(lapply(L,names), recursive=TRUE)) %in% pnames))){
        stop("pnames must include all named elements")
    }
    return(TRUE)
}

`pnv` <-function(x){ # "pnv" == "Print Named Vector"
  f <- function(o){strsplit(capture.output(o), " ")[[1]][2]}
  paste(paste(paste(names(x), sapply(unname(x), f), sep = "=")), collapse = ", ")
}

setGeneric("weights", function(object, ...){standardGeneric("weights")})
`weights.hyper3` <- function(object, ...){disord(object$weights, h=hashcal(object))}

`hyper3_to_hyper2` <- function(x){
    if(any(c(weights(x), recursive=TRUE) != 1)){
        warning("not all weights are equal to one")
    }
    return(hyper2(L=elements(brackets(x)), d=elements(powers(x)), pnames=pnames(x)))
}

`as.hyper3` <- function(x){
    if(is.hyper3(x)){return(x)}
    if(is.hyper2(x)){
        return(hyper3_bw(
            B      = brackets(x),
            W      = lapply(brackets(x), function(x){rep(1,length(x))}),
            powers = powers(x),
            pnames = pnames(x)
        ))
    } else {
        stop("only hyper2 objects can be coerced at this time")
    }
}

`as.namedvectorlist` <- function(H3){
    b <- disordR::elements(brackets(H3))
    w <- disordR::elements(weights(H3))
    disord(Map(setNames, w, b), h=hashcal(H3))
}

`print.hyper3` <- function(x, ...){
    b <- elements(as.namedvectorlist(x))
    p <- elements(powers(x))
    if(!isFALSE(getOption("give_warning_on_nonzero_power_sum"))){
      if(sum(powers(x)) != 0){
          warning("powers have nonzero sum")
      }
    }
    out <- "log( "
    for(i in seq_along(b)){
        out <- paste(out, "(" , pnv(b[[i]]) , ")^" , p[i] , sep="")
        if(i < length(b)){out <- paste(out , " * " , sep = "")}
    }
    out <- paste(out, ")\n", sep = "")
    for (i in strwrap(out)) {
        cat(noquote(i))
        cat("\n")
    }
    return(invisible(x))
}

`is.hyper3` <- function(H){inherits(H, "hyper3")}

`powers<-.hyper3` <- function(H, value){
    stopifnot(consistent(powers(H), value))
    if(!is.disord(value) && length(value) > 1){stop("replacement not defined")}
    hyper3_nv(as.namedvectorlist(H), powers=value, pnames=pnames(H))
}

`hyper3_equal` <- function(e1, e2){
  equality3(
      elements(brackets(e1)), elements(weights(e1)), elements(powers(e1)),
      elements(brackets(e2)), elements(weights(e2)), elements(powers(e2))
      )
}

`hyper3_prod` <- function(e1, n){
    powers(e1) <- powers(e1)*n
    return(e1)
}

`hyper3_add` <- function(e1, e2){
  b1 <- elements(brackets(e1))
  b2 <- elements(brackets(e2))
  w1 <- elements(weights(e1))
  w2 <- elements(weights(e2))
  p1 <- elements(powers(e1))
  p2 <- elements(powers(e2))
  n1 <- pnames(e1)
  n2 <- pnames(e2)
  out <- addL3(b1, w1, p1, b2, w2, p2)
  
  if(all(n2 %in% n1)){
      jj <- n1
  } else if(all(n1 %in% n2)){
      jj <- n2
  } else {
      jj <- sort(unique(c(b1, b2, recursive=TRUE)))
  }
  
  return(hyper3(out[[1]], out[[2]], out[[3]], pnames=jj))
}

`Ops.hyper3` <-  function (e1, e2 = NULL) 
{
  f <- function(...){stop("odd---neither argument has class hyper3?")}
  unary <- nargs() == 1
  lclass <- inherits(e1, "hyper3")
  rclass <- !unary && inherits(e2, "hyper3")
  
  if(unary){
      if(.Generic == "+"){
          return(e1)
      } else if (.Generic == "-"){
          return(hyper3_prod(e1, -1))
      } else {
          stop(gettextf("Unary operator %s not defined for hyper3 objects", dQuote(.Generic)))
      }
  }  # if(unary) closes

  if (!is.element(.Generic, c("+", "-", "==", "!=", "*"))){
      stop(gettextf("binary operator %s not defined for hyper3 objects", dQuote(.Generic)))
  }
  
  if(lclass && rclass){  
    if (.Generic == "+"){
      return(hyper3_add(e1, e2))
    } else if (.Generic == "-"){
      return(hyper3_add(e1,hyper2_prod(e2, -1)))
    } else if (.Generic == "==") {
      return(hyper3_equal(e1, e2))
    } else if (.Generic == "!=") {
      return(!hyper3_equal(e1, e2))
    } else {
      stop(gettextf("H3 %s H3 not defined ", dQuote(.Generic)))
    }
  } else {  # one of lclass,rclass
    if (.Generic == "*"){    # H * n
      if(lclass && !rclass){
        return(hyper3_prod(e1, e2))
      } else if (!lclass && rclass){
        return(hyper3_prod(e2, e1))
      } else {
        stop("method not defined for hyper3")
      }
    } else if (.Generic == "+"){  # H + x
      if(lclass && !rclass){
        return(hyper2_sum_numeric(e1, e2))
      } else if (!lclass && rclass){
        return(hyper2_sum_numeric(e2, e1))
      } else {
        stop("this cannot happen")
      }
    } else if (.Generic == "-"){
      if(lclass && !rclass){
        return(hyper2_sum_numeric(e1, -e2))
      } else if (!lclass && rclass){
        return(hyper2_sum_numeric(e2, -e1))
      } else {
        stop("this cannot happen")
      }
    }
  }
}

# accessor3 <- function(L, W, powers, Lwanted, Wwanted) {}

char2nv <- function(x){
    jj <- table(x)
    out <- as.numeric(jj)
    names(out) <- names(jj)
    return(out)
}
    
`[.hyper3` <- function(x, ...){ # H3[list(c(a=1),c(a=1,b=2))]
    stopifnot(nargs() == 2)
    dots <- list(...)[[1]]
    if(!is.list(dots)){
        if(is.character(dots)){dots <- char2nv(dots)}
        dots <- list(dots)
    }

    out <- accessor3(
        L       = elements(brackets(x)),
        W       = elements(weights(x)),
        powers  = elements(powers(x)),
        Lwanted = lapply(dots,names),
        Wwanted = lapply(dots,as.vector)
                     )
    return(hyper3_bw(out[[1]], out[[2]], out[[3]], pnames=pnames(x)))
}

`assign_lowlevel3`<- function(x, index, value){ #H[index] <- value
    stopifnot('hyper3' %in% class(x))
    
    if(is_ok_weightedplayers(index)){
        index <- list(index)
    }
    value <- elements(value)
    stopifnot(is.numeric(value)) # coercion to integer is done in C
    stopifnot(is.vector(value))
    if(length(value) == 1){
        value <- rep(value, length(index))
    }
    ## assigner3 <- function(L, W, p, L2, W2, value)
    return(assigner3(elements(brackets(x)), elements(weights(x)), elements(powers(x)),
                     lapply(index,names), lapply(index,as.vector), value))
}

`overwrite_lowlevel3` <- function(x, value){
  stopifnot(class(x    ) == 'hyper3')
  stopifnot(class(value) == 'hyper3')

  overwrite3(
      elements(brackets(x    )), elements(weights(x    )), elements(powers(x    )), 
      elements(brackets(value)), elements(weights(value)), elements(powers(value))
  )
}

`[<-.hyper3` <- function(x, index, ..., value){  # index must be a named vector or a list of named vectors
    if(inherits(value,"weight")){
        out <- setweight(x, index, value)
    } else if(missing(index)){  # A[] <- B
        jj <- overwrite_lowlevel3(x, value)
        if(all(pnames(value) %in% pnames(x))){
            out <- hyper3_bw(jj[[1]], jj[[2]], jj[[3]], pnames=pnames(x))
        } else {
            out <- hyper3(jj[[1]], jj[[2]], jj[[3]])
        }
    } else { # index supplied
        if(is.character(index)){index <- char2nv(index)}
        jj <- assign_lowlevel3(x, index, value)
        if(all(c(index, recursive=TRUE) %in% pnames(x))){
            out <- hyper3_bw(jj[[1]], jj[[2]], jj[[3]], pnames=pnames(x)) # do not change pnames
        } else { # index introduces a new pname
            out <- hyper3_bw(jj[[1]], jj[[2]], jj[[3]])
            pnames(out) <- sort(unique(c(pnames(x), names(index))))
        }
    }
    return(out)
}

`loglik_single_redundant` <- function(p, H, log=TRUE){
  stopifnot(all(p >= 0))
  if(length(p) == size(H) - 1){
    stopifnot(sum(p) <= 1)
    probs <- fillup(p)
  } else if(length(p) == size(H)){
    if(is.null(names(p))){stop("p==size(H), p must be a named vector")}
    stopifnot(abs(sum(p)-1) < 1e-6)  # small numerical tolerance
    p <- ordertrans(p, H)  # no warning given if names not in correct order...
    stopifnot(identical(names(p), pnames(H))) #...but they must match up
    probs <- p
  } else {
    stop("length(p) must be either size(H) or size(H)-1")
  }
  out <- evaluate3(brackets(H), weights(H), powers(H), probs=probs, pnames=pnames(H))
  if(log){
    return(out)
  } else {
    return(exp(out))
  }
}

`rpair3` <- function(n=5, s=3, lambda=1.3){ # s = number of matches
    players <- letters[seq_len(n)]
    strengths <- zipf(n)
    names(strengths) <- players
    out <- hyper3()
    for(i in seq_len(s)){  # iterate through matches
        jj <- sample(players,2,replace=FALSE)
        white <- jj[1]
        black <- jj[2]
       
        prob_white_win <- strengths[white]*lambda / (strengths[white]*lambda + strengths[black])
        if(runif(1) < prob_white_win){ # white wins
            jj <- lambda
            names(jj) <- white
            out[jj] %<>% inc  # e.g. jj: c(a=1.1)
            jj <- c(jj, 1)
            names(jj)[2] <- black
            out[jj] %<>% dec # e.g. jj: c(a=1.1,b=1)
        } else { # black wins
            jj <- lambda
            names(jj) <- black
            out[jj] %<>% inc
            jj <- c(jj, 1)
            names(jj)[2] <- white
            out[jj] %<>% dec # e.g. jj: c(a=1.1,b=1)
        }
    }
    return(out)
}

       
`rhyper3` <- function(n=5, s=4, type='race', ...){
    bad <- TRUE
    while(bad){
        out <- 
            switch(type,
                   race = rracehyper3(n=n, size=s, ...),
                   pair = rpair3(n=n, s=s, ...)
                   )
        bad <- all(unlist(weights(out)) == 1)
    }
    return(out)
}

`maxp3` <- function(H3,startp,give=FALSE,fcm = NULL, fcv = NULL, 
                  SMALL = 1e-06, maxtry = 100, ...){

stop("not yet written")
}

`num3` <- function(v, helped=NULL, lambda=1){  # numerators
    out <- hyper3()
    tv <- table(v)
    for(i in seq_along(tv)){
        ntv <- names(tv)[i]
        if(ntv %in% helped){
          jj <- lambda
        } else {
          jj <- 1
        }
        names(jj) <- ntv
        out[jj] <- tv[i]
    }
    return(out)
}
    
`den3` <- function(v, helped=NULL, lambda=1){  # denominators
    tv <- table(v)
    out <- as.vector(tv)
    names(out) <- names(tv)
    out[names(tv) %in% helped] %<>% prod(lambda)
    return(out)
}
       
`ordervec2supp3` <- function(v, nonfinishers=NULL){ # v = c("a","b","a","a","c","a")
    out <- num3(v)
    for(i in seq_along(v)){
        out[den3(c(v[i:length(v)], nonfinishers))] %<>% dec
    }
    return(out)
}

`rankvec_likelihood3` <- `ordervec2supp3`

`ordervec2supp3a` <- function(v, nonfinishers=NULL, helped=NULL, lambda=1){
  out <- num3(v, helped=helped, lambda=lambda)  # numerator
    for(i in seq_along(v)){
      out[den3(c(v[i:length(v)], nonfinishers))] %<>% dec  # denominator
    }
  return(out)
}

`rwinner3` <- function(pn=c(a=2,b=4,c=2,d=1), ps=c(a=0.3,b=0.1,c=0.2,d=0.4)){
    stopifnot(all(table(names(pn)) <= 1))
    stopifnot(all(table(names(ps)) <= 1))
    stopifnot(identical(names(pn), names(ps)))

    sample(names(ps), 1, prob = (pn*ps)/sum(pn*ps))
}

`rrace3` <- function(pn=c(a=2,b=4,c=2,d=1), ps=c(a=0.3, b=0.1,c=0.2,d=0.4)){
    ## pn: c(a=2,b=4,c=2,d=1) # player numbers (two "a"s, four "b"s etc)
    ## ps: c(a=0.3, b=0.1,c=0.2,d=0.4)  # player strengths
    
    out <- NULL
    for(i in seq_len(sum(pn))){
        nextfinisher <- rwinner3(pn, ps)
        out <- c(out, nextfinisher)
        pn[nextfinisher] <- pn[nextfinisher] - 1
    }
    return(out)
}

`rracehyper3` <- function(n=4, size=9, ps=NULL, races=3){
    players <- letters[seq_len(n)]
    if(is.null(ps)){
        ps <- zipf(n)
        names(ps) <- players
    }
    pn <- as.vector(table(factor(sample(players,size,replace=TRUE), levels=players)))  
    names(pn) <- players

    out <- hyper3()
    for(i in seq_len(races)){
        out <- out + ordervec2supp3(rrace3(pn,ps))
    }
    return(out)
}

`pnames<-.hyper3` <- function(x, value){
    hyper3(
        elements(brackets(x)),
        elements(weights(x)),
        elements(powers(x)),
        pnames = value)
}

## a <- read.table("constructor_2021.txt",header=TRUE)
## constructor(a[,-ncol(a)])  # final column is points

`ordertable2supp3` <- function(a, noninformative=NULL){
    out <- hyper3()
    n <- a[,1]  # names of constructors
    for(i in seq(from=2, to=ncol(a)-1)){
        onecol <- suppressWarnings(as.numeric(a[,i]))
        wanted <- !(a[,i] %in% noninformative ) # %notin% not available to workflow
        jj <- n[wanted][order(onecol[wanted], na.last=TRUE)]
        n_finishers    <- sum(!is.na(jj))
        n_nonfinishers <- sum( is.na(jj))
        if(n_nonfinishers == 0){ # technically not necessary:  "else" clause works for == 0
            out <- out + ordervec2supp3(jj)
        } else { 
            out <- out + ordervec2supp3(jj[seq_len(n_finishers)], jj[n_finishers + seq_len(n_finishers)])
        }
    }
    return(out)
}

`args2ordervec` <- function(...){
    l <- list(...)
    if(any(names(l) == "") && !all(names(l) == "")){
        stop("either name all of the arguments, or none of them")
    }
    if(all(names(l) == "")){
        names(l) <- paste("X", seq_len(nargs()), sep="")
    }
    x <- unlist(l,use.names=FALSE)
    if(any(table(x) > 1)){stop("ties not implemented")}
    names(x) <- rep(names(l), lapply(l,length))
    return(names(sort(x)))
}

`home_away3` <- function(home_games_won, away_games_won, lambda){

    if(is.complex(home_games_won)){
        if(missing(lambda)){lambda <- away_games_won}
        away_games_won <- Im(home_games_won)
        home_games_won <- Re(home_games_won) 
    }

    teams <- rownames(home_games_won)
    stopifnot(identical(teams, colnames(home_games_won)))
    stopifnot(identical(teams, rownames(away_games_won)))
    stopifnot(identical(teams, colnames(away_games_won)))

    H <- hyper3(pnames=c(teams))

    for(i in seq_len(nrow(home_games_won))){
        for(j in seq_len(ncol(home_games_won))){
            if(i != j){  
                home_team <- teams[i]
                away_team <- teams[j]

		home_wins <- home_games_won[i,j]
		away_wins <- away_games_won[i,j] 
                no_of_matches <- home_wins + away_wins 

                ## home wins:
                jj <- lambda
                names(jj) <- home_team
                H[jj] %<>% inc(home_wins)
                
                ## away wins:
                jj <- 1
                names(jj) <- away_team
                H[jj] %<>% inc(away_wins)

                ## denominator
                jj <- c(lambda, 1)
                names(jj) <- c(home_team,away_team)
                H[jj] %<>% dec(no_of_matches)
            } # if(i != j) closes
        } # j loop closes
    } # i loop closes
    return(H)
}

`home_draw_away3` <- function(home_games_won, drawn_games, away_games_won, lambda, D){

    if(is.list(home_games_won)){
        drawn_games    <- home_games_won[[3]]
        away_games_won <- home_games_won[[2]]
        home_games_won <- home_games_won[[1]]
    }
        
    teams <- rownames(home_games_won)
    stopifnot(identical(teams, colnames(home_games_won)))
    stopifnot(identical(teams, rownames(away_games_won)))
    stopifnot(identical(teams, colnames(away_games_won)))
    stopifnot(identical(teams, rownames(drawn_games   )))
    stopifnot(identical(teams, colnames(drawn_games   )))

    H <- hyper3(pnames=c(teams))

    for(i in seq_len(nrow(home_games_won))){
        for(j in seq_len(ncol(home_games_won))){
            if(i != j){  
                home_team <- teams[i]
                away_team <- teams[j]

		home_wins <- home_games_won[i,j]
		away_wins <- away_games_won[i,j] 
                draws     <-    drawn_games[i,j]
                no_of_matches <- home_wins + away_wins + draws

                ## home wins:
                jj <- lambda
                names(jj) <- home_team
                H[jj] %<>% inc(home_wins)
                
                ## away wins:
                jj <- 1
                names(jj) <- away_team
                H[jj] %<>% inc(away_wins)

                ## draws:
                jj <- c(D, D)
                names(jj) <- c(home_team,away_team)
                H[jj] %<>% inc(draws)

                ## denominator
                jj <- c(D + lambda, D + 1)
                names(jj) <- c(home_team, away_team)
                H[jj] %<>% dec(no_of_matches)
            } # if(i != j) closes
        } # j loop closes
    } # i loop closes
    return(H)
}

## kka_array is arranged differently from the previous; it uses
## white_wins and white_losses, not white wins and white losses

`white_draw3` <- function(A, lambda, D){ # white_draw3(kka_array,1.1,0.2)
    stopifnot(identical(dimnames(A)[[1]], dimnames(A)[[2]]))
    players <- dimnames(A)[[1]] 
    H <- hyper3(pnames=c(players, "lambda", "D"))
    d <- dim(A)
    
    for(i in seq_len(d[1])){
        for(j in seq_len(d[2])[-i]){
            white_player <- players[i]
            black_player <- players[j]
            
            white_player_wins  <- A[i,j,1]
            white_player_draws <- A[i,j,2]
            white_player_loses <- A[i,j,3]
            
            total_matches <- white_player_wins + white_player_draws + white_player_loses 
            
            ## white player wins:
            jj <- lambda
            names(jj) <- white_player
            H[jj] %<>% inc(white_player_wins)
            
            ## white player loses (so the black player wins):
            jj <- 1
            names(jj) <- black_player
            H[jj] %<>% inc(white_player_loses)
            
            ## draw:
            jj <- c(D, D)
            names(jj) <- c(white_player, black_player)
            H[jj] %<>% inc(white_player_draws)
            
            ## lastly, change the denominator so the powers sum to zero:
            jj <- c(lambda+D , 1+D)
            names(jj) <- c(white_player, black_player)
            H[jj] %<>% dec(total_matches)
        } # j loop closes
    } # i loop closes
    return(H)
}

`list2nv` <- function(L, integers=TRUE){
  if(integers){
      jj <- seq_along(L)
  } else {
    jj <- names(L)
  }
  out <- rep(jj, times=unlist(lapply(L,length)))
  names(out) <- c(L, recursive=TRUE)
  return(out)
}

`sum.hyper3` <- function(x, ..., na.rm=FALSE){
  if(nargs() == 1){
    return(x)
  } else if (nargs() == 2){
    return(hyper3_add(x, ...))
  } else {
    return(hyper3_add(x, Recall(...)))
  }
}

`as.weight` <- function(x){
    stopifnot(is.numeric(x))
    stopifnot(x >= 0)
    class(x) <- "weight" # this is the only place that class is set to weight
    return(x)
}

`setweight` <- function(x, index, value){
    x <- as.hyper3(x)
    if(length(value)==1){value <- rep(value, length(index))}
    stopifnot(length(index) == length(value))
    for(i in seq_along(index)){
        x <- hyper3_nv(
            lapply(as.namedvectorlist(x),
                   function(p){
                       p[names(p) == index[i]] <- value[i] # The meat
                       return(p)
                   }),
            powers = powers(x),
            pnames = pnames(x)
        )
    }
    return(x)
}

`swait` <- function(x, ...){
    jj <- list(...)
    return(setweight(x, names(jj), unname(unlist(jj))))
}

`dirichlet3` <- function(powers, lambda=NULL){
    if(is.null(names(powers))){
        stop("dirichlet3d() requires powers to be a _named_ vector")
    }
    if(!is.null(names(lambda)) & !identical(names(lambda), names(powers))){
        stop("names(lambda) must match names(powers), if supplied")
    }
    if(length(lambda) < length(powers)){
        lambda <- c(lambda, rep(1,length(powers)-length(lambda)))
    }

    out <- hyper3()
    for(i in seq_along(powers)){
        out[setNames(lambda[i], names(powers)[i])] %<>% inc(powers[i])
    }
    out[setNames(lambda, names(powers))] %<>% dec(sum(powers))
    return(out)
}

"pwa3_single" <- function(H3, pwa, lambda){
    stopifnot(pwa %in% pnames(H3))
    stopifnot(length(pwa) == 1)
    H3 <- as.hyper3(H3)
    B <- elements(brackets(H3))
    W <- elements(weights(H3))
    for (i in seq_along(B)) {
        W[[i]][B[[i]] %in% pwa] <- lambda
    }
    return(hyper3_bw(B, W, powers(H3), pnames(H3)))
}

"pwa3" <- function(H3, ...){
    l <- list(...)
    if(length(l) == 1){
        x <- l[[1]]
        if(is.vector(x) && (!is.list(x)) && is.numeric(x) && (!is.null(names(x)))){
            l <- as.list(l[[1]])
        }
    }
    for(i in seq_along(l)){
        H3 <- pwa3_single(H3, names(l[i]), l[[i]])
    }
    return(H3)
}


"pwa23_single" <- function(H, pwa, chameleon="S"){ # takes H a hyper2 (or hyper3), returns a hyper3
    H <- as.hyper3(H)
    nvl <- elements(as.namedvectorlist(H))
    for(i in seq_along(nvl)){
        if(pwa %in% names(nvl[[i]])){nvl[[i]] <- frab::as.namedvector(as.frab(nvl[[i]]) + frab::as.frab(setNames(1, chameleon)))}
    }
    hyper3_nv(L=nvl, powers=elements(powers(H)), pnames=unique(c(pnames(H),chameleon)))
}

"pwa23_multiple" <- function(H, pwa, chameleon="S"){
    H <- as.hyper3(H)
    L <- strsplit(paste(pwa, chameleon), " ")
    for(i in seq_along(L)){
        jj <- L[[i]]
        H <- pwa23_single(H, pwa=jj[1], chameleon=jj[2])
    }
    return(H)
}

"pwa23" <- function(H, ...){
    H <- as.hyper3(H)
    l <- list(...)
    for(i in seq_along(l)){
        H <- pwa23_single(H, names(l[i]), l[[i]])
    }
    return(H)
}

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
\documentclass[nojss]{jss}
\usepackage{dsfont}
\usepackage{bbm}
\usepackage{amsfonts}
\usepackage{wasysym}
\usepackage{amssymb}
\usepackage{yfonts}
\usepackage{wrapfig}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% declarations for jss.cls %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% just as usual
\author{Robin K. S. Hankin\\University of Stirling}
\title{Integration in the \pkg{hyper2} package}
%\VignetteIndexEntry{Integration in hyper2}

%% for pretty printing and a nice hypersummary also set:
\Plainauthor{Robin K. S. Hankin}
\Plaintitle{Integration in the hyper2 package}
\Shorttitle{Integration in the hyper2 package}

%% an abstract and keywords

\Abstract{The \pkg{hyper2} package presented a new formulation of the
  \pkg{hyperdirichlet} package, offering speed advantages and the
  ability to deal with higher-dimensional datasets.  However,
  \pkg{hyper2} was based on likelihood methods and as originally
  uploaded did not have the ability to integrate over the unit-sum
  simplex.  This functionality has now been incorporated into the
  package which is documented here, by reproducing earlier analysis.}

\Keywords{Dirichlet distribution, hyperdirichlet, \pkg{hyper2},
  combinatorics, \proglang{R}, multinomial distribution, constrained
optimization, integration, simplex, unit-sum constraint}
\Plainkeywords{Dirichlet distribution, hyperdirichlet, hyper2,
  combinatorics, R, multinomial distribution, constrained
optimization, integration, simplex, unit-sum constraint}

%% publication information
%% NOTE: This needs to filled out ONLY IF THE PAPER WAS ACCEPTED.
%% If it was not (yet) accepted, leave them commented.
%% \Volume{13}
%% \Issue{9}
%% \Month{September}
%% \Year{2004}
%% \Submitdate{2004-09-29}
%% \Acceptdate{2004-09-29}

%% The address of (at least) one author should be given
%% in the following format:
\Address{
  Robin K. S. Hankin\\
  University of Stirling\\
  E-mail: \email{hankin.robin@gmail.com}
}
%% It is also possible to add a telephone and fax number
%% before the e-mail in the following format:
%% Telephone: +43/1/31336-5053
%% Fax: +43/1/31336-734

%% for those who use Sweave please include the following line (with % symbols):
%% need no \usepackage{Sweave.sty}

%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\newcommand{\bolda}{\mathbf a}
\newcommand{\boldp}{\mathbf p}
\newcommand{\boldV}{\mathbf V}
\newcommand{\boldalpha}{\mbox{\boldmath$\alpha$}}
\newcommand{\boldzero}{\mathbf 0}

\newcommand{\mcf}{\mathcal F}
\newcommand{\mcm}{\mathcal M}
\newcommand{\mcs}{\mathcal S}
\newcommand{\mcw}{\mathcal W}

\newcommand{\pG}{p_{\mbox{\frakfamily g}}}

\SweaveOpts{}
\begin{document}

<<echo=FALSE,print=FALSE>>=
ignore <- require(hyper2,quietly=TRUE)
ignore <- require(magrittr,quietly=TRUE)
@ 

\section{Introduction}

\setlength{\intextsep}{0pt}
\begin{wrapfigure}{r}{0.2\textwidth}
\begin{center}
\includegraphics[width=1in]{\Sexpr{system.file("help/figures/hyper2.png",package="hyper2")}}
\end{center}
\end{wrapfigure}
To cite this work in publications, please use~\citet{hankin2017_rnw}.
The \pkg{hyper2} package presented a new
formulation of the hyperdirichlet distribution~\citep{hankin2010}
which offered speed advantages over the original \pkg{hyperdirichlet}
package, and the ability to deal with higher-dimensional datasets.
However, \pkg{hyper2} was based on likelihood methods and as
originally uploaded did not have the ability to integrate over the
unit-sum simplex.  This functionality has now been incorporated into
the package which is documented here, by reproducing earlier analysis.

\section{Chess}
Consider Table~\ref{chess} in which matches between three chess
players are tabulated; this dataset was analysed by~\citet{hankin2010}.


\[C
\frac{p_1^{30}p_2^{36}p_3^{22}}{
  \left(p_1+p_2\right)^{35}
  \left(p_2+p_3\right)^{35}
  \left(p_1+p_3\right)^{18}
  }
\]

(the symbol `$C$' consistently stands for an undetermined constant).
This likelihood function is provided in the \pkg{hyper2} package
as the \code{chess} dataset:

<<chess_show>>=
chess
@ 


We can calculate the normalizing constant:

\begin{Sinput}
  B(chess)
\end{Sinput}
\begin{Soutput}
  [1] 1.442828e-28
\end{Soutput}

comparing well with the value given by the \pkg{hyperdirichlet}
package of $1.47\times 10^{-28}$.  \citet{hankin2010} went on to
calculate the $p$-value for $H_0\colon
p=\left(\frac{1}{3},\frac{1}{3},\frac{1}{3}\right)$ as 0.395, a
calculation which may be performed in the \pkg{hyper2} package as
follows:

\begin{Sinput}
f <- function(p){loglik(indep(p),chess) > loglik(c(1,1)/3,chess)}
probability(chess, disallowed=f,tol=0.1)
\end{Sinput}
\begin{Soutput}
  [1]  0.4099
\end{Soutput}

Again comparing well with the older result (smaller values of
\code{tol} give closer agreement at the expense of increased
computation time).  Finally, we can calculate the probability that
Topalov is a better player than Anand:

\begin{Sinput}
T.lt.A <- function(p){p[1]<p[2]}
probability(chess, disallowed=T.lt.A,tol=0.01) 
\end{Sinput}
\begin{Soutput}
  [1] 0.7123
\end{Soutput}

again showing reasonable agreement with the 2010 value of 0.701.

\begin{table}
\centering
\begin{tabular}{|ccc|c|}\hline
Topalov  & Anand & Karpov & total\\ \hline
22 & 13 & -  & 35\\ 
-  & 23 & 12 & 35\\ 
8  &  - & 10 & 18 \\ \hline
30 & 36 & 22 & 88 \\ \hline
\end{tabular}
\caption{Results of 88 chess matches \label{chess} (dataset
  \code{chess} in the \pkg{aylmer} package) between three
  Grandmasters; entries show number of games won up to 2001 (draws are
  discarded).  Topalov beats Anand 22-13; Anand beats Karpov 23-12;
  and Karpov beats Topalov 10-8}
\end{table}

\section{Verification}

In a breathtaking display of arrogance and/or incompetence,
\citet{hankin2010} did not actually provide any evidence that the
integration suite of \pkg{hyperdirichlet} was accurate.  Here I
compensate for that inexcusable lapse by comparing numerical results
with analytical formulae.  Consider the standard Dirichlet
distribution:

\begin{equation}
  \frac{
    p_1^{\alpha_1-1}\ldots p_k^{\alpha_k-1}
  }{
    B\left(\alpha_1,\ldots,\alpha_k\right)
  }
\end{equation}

where it is understood that the $p_i>0$ and $\sum p_i=1$; here
$B=\frac{\Gamma\sum\alpha_i}{\prod\Gamma\alpha_i}$ is the
normalization constant.  We can verify that \code{hyper2::B()} is
operating as expected for the case $\alpha= (1,2,3,4)$:

<<>>=
x <- c(a=1, b=2, c=3, d=4)  # needs a named vector
ans1 <- B(dirichlet(alpha = x),tol=0.1)
ans2 <- prod(gamma(x))/gamma(sum(x)) 
c(numerical=ans1,theoretical=ans2)   # should agree
@   

Further, consider a Dirichlet distribution with
$\alpha_1=\alpha_2=\alpha_3=\alpha_4=3$.  Then, by symmetry, the
probability that $p_1<p_2$ should be exactly $\frac{1}{2}$:

<<>>=
f <- function(p){p[1]<p[2]}
H <- dirichlet(alpha=c(a=3,b=3,c=3,d=3))
probability(H,f,tol=0.1)
@   

(compare exact value of \code{0.5}; note the loose tolerance of
\code{0.1}, needed to keep computational time short---the integrand
has a severe discontinuity which is computationally expensive to
integrate across).  Further, $\Prob(p_1<p_2<p_3)$ should be exactly
$\frac{1}{6}$:

<<>>=
g <- function(p){(p[1]<p[2]) & (p[2]<p[3])}
1-probability(H,disallowed=g,tol=0.1)
@   


(compare exact value of \code{0.1666}).

\section{More results: icons dataset}

Consider the \code{icons} dataset, shown in table~\ref{saffron}, and
the following hypotheses, again following \citet{hankin2010}, and
reproduced here for convenience. 

<<>>=
icons
maxp(icons)
@ 

\begin{table}
\centering
\begin{tabular}{|cccccc|c|}\hline
\multicolumn{6}{|c|}{icon}&\\ \hline
NB & L & PB & THC & OA & WAIS & total\\ \hline
5	&3	&-	&4	&-	&3	&15\\
3	&-	&5	&8	&-	&2	&18\\
-	&4	&9	&2	&-	&1	&16\\
1	&3	&-	&3	&4	&-	&11\\
4	&-	&5	&6	&3	&-	&18\\
-	&4	&3	&1	&3	&-	&11\\
5	&1	&-	&-	&1	&2	& 9\\
5	&-	&1	&-	&1	&1	& 8\\
-	&9	&7	&-	&2	&0	&18\\ \hline
23      &24     &30     &24     &14     &9      &124\\ \hline
\end{tabular} 
\caption{Experimental results\label{saffron} from \citet{oneill2008}
  (dataset \code{icons} in the package): respondents' choice of `most
  concerning' icon of those presented.  Thus the first row shows
  results from respondents presented with icons NB, L, THC, and WAIS;
  of the~15 respondents, 5 chose NB as the most concerning (see text
  for a key to the acronyms).  Note the ``0'' in row~9, column~6: this
  option was available to the~18 respondents of that row, but none of
  them actually chose WAIS}
\end{table}

For reference, the other hypotheses were:

\begin{itemize}
\item $H_1\colon p_1\geqslant\frac{1}{6}$
\item $H_2\colon p_1\geqslant\max\left\{p_2,\ldots p_6\right\}$
\item $H_3\colon p_5+p_6\geqslant\frac{1}{3}$
\item $H_4\colon\max\left\{p_5,p_6\right\}\geqslant\min\left\{p_1,p_2,p_3,p_4\right\}$
\end{itemize}

<<>>=
f1 <- function(p){p[1] > 1/6}
f2 <- function(p){p[1] > max(fillup(p)[-1])}
f3 <- function(p){sum(fillup(p)[5:6]) > 1/3}
f4 <- function(p){max(fillup(p)[1:2]) > min(fillup(p)[3:6])}
@                                                
         
Here I will analyse just the first hypothesis, that is $H_1\colon
p_1\leqslant\frac{1}{6}$ using the integration facilities of the
\pkg{hyper2} package, and compare with previous results.  Here we
perform a Bayesian analysis, made possible by the efficient coding of
\pkg{hyper2}:

\begin{Sinput}
probability(icons, disallowed=function(p){p[1] > 1/6}, tol=0.1)
\end{Sinput}
\begin{Soutput}
  [1]  0.01502
\end{Soutput}

See how the disallowed region is the {\em expected} bit of the
parameter space.  Thus the probability that the $p_i$ are unexpected
(that is, $p_1<1/6$) is about 1.5\% or conversely,
$P\left(H_1\right)\simeq 0.985$.  The likelihood ratio reported was
about 2.608, which would correspond to a $p$-value of about

<<>>=
pchisq(2*2.608,df=1,lower.tail=FALSE)
@ 

or just over 2\% under an asymptotic distribution; thus this
frequentist technique gives comparable strength of evidence for $H_1$
to the Bayesian approach.

\section{Incomplete survey data}
 
This section performs the analysis originally presented
in~\citet{altham2010}.  The data, given here in
table~\ref{Linetaldata} arises from 69 medical malpractice claims, and
are the two surgeons' answers to the question: was there a
communication breakdown in the hand-off between physicians caring for
the patient?


\begin{table}[!th]
\centering
\begin{tabular} {|l| c  c  c c c|}
\hline
Reviewer 1 &\multicolumn{5}{c|}{Reviewer 2}\\ \cline{2-6}
 $ \ $&Yes& No& Missing&\rule{2mm}{0mm}& Total\\
\hline
Yes& 26& 1&2&&29\\
No & 5&18&9&&32\\
Missing& 4&4&0&&8\\
&&&&&\rule{0mm}{0mm}\\
Total & 35 & 23 & 11 && 69\\  
\hline
\end{tabular}
\caption{Two surgeon reviews of malpractice claims data}
\label{Linetal}
\end{table}

\begin{table}[!th]
\centering
\begin{tabular} {|l| c  c  c c c|}
\hline
Reviewer 1 &\multicolumn{5}{c|}{Reviewer 2}\\ \cline{2-6}
 $ \ $&Yes& No& Missing&\rule{2mm}{0mm}& Total\\
\hline
Yes     & $y_{11}$ & $y_{10}$& $z_{1+}$ && $y_{1+} + z_{1+}$\\
No      & $y_{01}$ & $y_{00}$& $z_{0+}$ && $y_{0+} + z_{0+}$\\
Missing & $u_{+1}$ & $u_{+0}$& $0$     && $ u_{++}$        \\
&&&&&\rule{0mm}{0mm}\\
Total & $y_{+1} + u_{+1}$ & $y_{+0} +u_{+0} $&$z_{++}$&&$n$ \\
\hline
\end{tabular}
\caption{Notation for the data}
\label{Linetaldata}
\end{table}

We may implement an appropriate likelihood function as follows:

<<>>=
H <- hyper2()
H["t00"] <- 18
H["t10"] <- 01
H["t01"] <- 05
H["t11"] <- 26
H[c("t11","t10")] <- 2
H[c("t01","t00")] <- 9
H[c("t11","t01")] <- 4
H[c("t10","t00")] <- 4
H <- balance(H)
H
@ 

(object \code{H} is provided as \code{handover} in the package).  Then
we may estimate the probability that reviewer 2 is more likely to give
a `yes' than reviewer 1 as follows:

<<>>=
free <- maxp(H,give=TRUE)
m <- fillup(free$par)
names(m) <- pnames(H)
m
free$value
@ 

Then the constrained optimization:

<<>>=
obj <- function(p){-loglik(p,H)}   # objective func
gr  <- function(p){-gradient(H,p)} # gradient, needed for speed
UI <- rbind(diag(3),-1)           # UI and CI specify constraints
CI <- c(rep(0,3),-1)              # p_i >= 0 and sum p_i <= 1
@ 

                      
We will test $H_A\colon p_2<p_3$ using the method of support.

\begin{Sinput}
> constrained <- maxp(H,give=TRUE,fcm = rbind(c(0,-1,1)), fcv=0,maxtry=1e5)
> constrained 
\end{Sinput}
\begin{Soutput}
$par
[1] 0.42735779 0.06018069 0.06018069

$value
[1] -66.14478

$counts
function gradient 
     318       43 

$convergence
[1] 0

$message
NULL

$outer.iterations
[1] 2

$barrier.value
[1] 0.0001060435

$likes
 [1] -82.48451 -66.73119 -82.48454 -66.14478 -67.33553 -67.11853 -66.14607
 [8] -66.16411 -66.27162 -66.67668
\end{Soutput}


Thus the support for $H_A$ is about $66.14478-64.14538=1.9999$, or
almost exactly 2 units of support.
\bibliography{hyper2}
\end{document}

---
title: "The `ordertrans()` function in the `hyper2` package"
author: "Robin K. S. Hankin"
output:
  bookdown::html_document2:
      base_format: rmarkdown::html_vignette
bibliography: hyper2.bib
link-citations: true
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{ordertrans}
  %\usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
set.seed(0)
library("hyper2")
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)
knitr::opts_chunk$set(fig.width=6, fig.height=6)
knit_print.function <- function(x, ...){dput(x)}
registerS3method(
  "knit_print", "function", knit_print.function,
  envir = asNamespace("knitr")
)
```

```{r out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE}
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))
```

```{r, label=showordertrans}
ordertrans
```

To cite the `hyper2` package in publications, please use
@hankin2017_rmd.  The `ordertrans()` function can be difficult to
understand and this short document provides some sensible use-cases.
The manpage provides a very simple example, but here we are going to
use an even simpler example:


```{r,label=simpexample}
x <- c(d=2,a=3,b=1,c=4)
x
```

In the above, object `x` is a named vector with elements
`seq_along(x)` in some order.  It means that competitor `d` came
second, competitor `a` came third, `b` came first and `c` came fourth.
Technically `x` is an order vector because it answers the question
"where did a particular competitor come?"  However, it might equally
be a rank vector because it answers the question "who came first? who
came second?"  (see also the discussion at `rrank.Rd`).

But note that `x` is not helpfully structured to answer either of
these questions: you have to go searching through the names or the
ranks respectively---both of which may appear in any order---to find
the competitor or rank you are interested in.  To find the rank vector
(that is, who came first, second etc), one would use `sort()`:

```{r}
sort(x)
```

But this is suboptimal to find the order vector ["where did a
particular competitor come?"].  For the order vector, we want to
rearrange the elements of `x` so that the names are in alphabetical
order.  That way we can see straightaway where competitor `a` placed
(in this case, third).  This nontrivial task is accomplished by
function `ordertrans()`:


```{r,label=useordertrans}
o <- ordertrans(x) # by default, sorts names() into alphabetical order
o
```

Observe that objects `x` and `o` are equal in the sense that they are
a rearrangement of one another:

```{r,label=equalrearrangement}
identical(x, o[c(4,1,2,3)])
identical(o, x[c(2,3,4,1)])
```

One consequence of this is that the resulting Plackett-Luce support
functions will be the same:

```{r,showordervec}
(Sx <- ordervec2supp(x))
(So <- ordervec2supp(o))
```

Note carefully that the two support functions above can be
mathematically identical but not formally `identical()` because
neither the order of the brackets, nor the order of terms within a
bracket, is defined.  They look the same on my system but YMMV.  It is
possible that the two support functions might appear to be different
even though they are mathematically the same. We can verify that the
two are mathematically identical using package idiom `==`:

```{r}
Sx==So
```

## Another use-case

How about this:

```{r}
(x <- c(d=2, a=3, c=4, b=3, e=6))
(y <- c(e=3, c=2, a=4, b=5, d=1))
x+y
```

Above, observe that R is behaving as intended $\ldots$ but is
potentially confusing.  Take the first element.  This has value
`2+5=7` and name `d` from the name of element 1 of `x`.  But it would
be reasonable to ask for the first element to be the sum of element
`d` of `x` and element `d` of `y`, which would be `2+1=3`.
This can be done with `ordertrans()`:

```{r}
ordertrans(x) + ordertrans(y)
```

See how `ordertrans()` has rearranged both `x` and `y` so that the
names are in alphabetical order.

But this is not perfect.  Consider:

```{r}
(z <- c(f=3, g=2, h=4, a=5, b=1))  # names NOT letters[1:5]
ordertrans(x) + ordertrans(z)  # arguably not well-defined
```

In this case the result is arguably incorrect.




## Skating

Let us consider the skating dataset and use `ordertrans()` to study it
(note that the skating dataset is analysed in more depth in
`skating.Rmd`).

```{r,label=lookatskating}
skating_table
```

We might ask how judges `J1` and `J2` compare to one another?  We need to
create vectors like `x` and `y` above:

```{r,makej1j2}
j1 <- unclass(skating_table)[,1]  # column 1 is judge number 1
names(j1) <- rownames(skating_table)
j2 <- unclass(skating_table)[,2]  # column 2 is judge number 2
names(j2) <- rownames(skating_table)
j1
j2
cbind(j1,j2)
```

In the above, see how objects `j1` and `j2` have identical names, in
the same order.  Observe that `hughes` is ranked 1 (that is, first) by
`J1`, and 4 (that is, 4th) by `J2`.  This makes it sensible to plot
`j1` against `j2`:

```{r j1vsj2,fig.cap="Judge 1 vs judge 2"}
par(pty='s')  # forces plot to be square
plot(j1,j2,asp=1,pty='s',xlim=c(0,25),ylim=c(0,25),pch=16,xlab='judge 1',ylab='judge 2')
abline(0,1)  # diagonal line
for(i in seq_along(j1)){text(j1[i],j2[i],names(j1)[i],pos=4,col='gray',cex=0.7)}
```

In figure \@ref(fig:j1vsj2), we see general agreement but differences
in detail.  For example, `hughes` is ranked first by judge 1 and
fourth by judge 2.  However, other problems are not so easy.  Suppose
we wish to compare the ranks according to likelihood with the ranks
according to some points system.

```{r,label=maxlikeskating,cache=TRUE}
mL <- skating_maxp  # predefined; use maxp(skating) to calculate ab initio
mL
```

Note that in the above, the competitors' names are in alphabetical
order.  We first need to convert strengths to ranks:

```{r,label=strengthstoranks}
mL[] <- rank(-mL)  # minus because ranks orders from weak to strong
mL
```

(note that the names are in the same order as before, alphabetical).
In the above we see that `slutskya` ranks first, `hughes` second, and
so on.  Another way of ranking the skaters is to use a Borda-type
system: essentially the rowsums of the table (and, of course, the
_lowest_ score wins):


```{r,label=pointsskating,cache=TRUE}
mP <- rowSums(skating_table)  # 'P' for Points
mP[] <- rank(mP,ties='first') # positive sign here
mP
```

It is not at all obvious how to compare `mP` and `mL`.  For example,
we might be interested in `hegel`.  It takes some effort to find that
her likelihood rank is 20 and her Borda rank is 19.  Function
`ordertrans()` facilitates this:


```{r,label=ordertransexample}
ordertrans(mP,names(mL))  
```

See above how `ordertrans()` shows the points-based ranks but in
alphabetical order, to facilitate comparison with `mL`.  We can now
plot these against one another:

```{r,label=crapplot,fig.cap="points-based ranks vs likelihood ranks"}
plot(mL,ordertrans(mP,names(mL)))
```

However, figure \@ref(fig:crapplot) is a bit crude.  Function
`ordertransplot()` gives a more visually pleasing output, see figure
\@ref(fig:showoffordertransplot).


```{r,label=showoffordertransplot,fig.cap="points=based rank vs likelihood rank using `ordertransplot()`"}
ordertransplot(mL,mP,xlab="likelihood rank",ylab="Borda rank")
```

So now we may compare judge 1 against likelihood:


```{r transplotjudge1,fig.cap="Likelihood rank vs rank according to Judge 1"}
ordertransplot(mL,j1,xlab="likelihood rank",ylab="Judge 1 rank")
```

In figure \@ref(fig:transplotjudge1), looking at the lower-left
corner, we see (reading horizontally) that the likelihood method
placed Slutskya first, then Hughes second, then Kwan third; while
(reading vertically) judge 1 placed Hughes first, then Kwan, then
Slutskya.

# References

---
title: "Creating `hyper3` objects in the `hyper2` package"
author: "Robin K. S. Hankin"
output: html_vignette
bibliography: hyper2.bib
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{hyper3 creation methods}
  %\usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
set.seed(0)
library("hyper2")
library("magrittr")
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)
knit_print.function <- function(x, ...){dput(x)}
registerS3method(
  "knit_print", "function", knit_print.function,
  envir = asNamespace("knitr")
)
```

```{r out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE}
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))
```

To cite the `hyper2` package in publications, please use
@hankin2017_rmd.  In this vignette I give self-contained short
examples of each function that creates or modifies `hyper3` objects.


```{r hyper3_default}
hyper3() # default creation, returns a uniform support function
```

---


```{r hyper3_1}
hyper3(list(c(a=1.2),c(b=1),c(a=1.2,b=1)),powers=c(3,4,-7)) # dispatches to hyper3_nv()
```

---

```{r hyper3_2}
hyper3(B=list("a",c("a","b"),"b"),W=list(1.2,c(1.2,1),1),powers=c(3,4,-7)) # dispatches to hyper3_bw()
```

---

```{r hyper3_3}
M <- matrix(rpois(15,3),5,3)
colnames(M) <- letters[1:3]
M
hyper3(M,c(2,3,-1,-5,1))   # dispatches to hyper3_m()
```
---

```{r hyper3_add_terms}
H <- hyper3()
H[c(p1=1.3)] <- 5  # terms not present are created
H[c(p2=1  )] <- 2
H[c(p1=1.3,p2=1)] <- -7
H
```
---


```{r hyper3_extract}
H <- hyper3(list(c(a=1.2),c(b=1),c(a=1.2,b=1)),powers=c(3,4,-7))
H[c(a=1.2)] <- 100  # terms present may be modified...
H[c(a=1.2,x=1)] %<>% dec(97) # or incremented
H
```
---

```{r hyper3_ordervec2supp3}
ordervec2supp3(c("a","a","b","c","a","b","c"))
```
---

```{r hyper3_ordervec2supp3nonfinishers}
ordervec2supp3(c("a","b"),nonfinishers=c("a","b"))  # a > b >> {a,b}
```
---

```{r hyper3_attemptstable2supp3}
(jj <- data.frame(throw1=c(a=8,b=2,c=1),throw2=c(5,"X",3)))
attemptstable2supp3(jj)
```
---

```{r hyper3_dirichlet3}
dirichlet3(c(x=6,y=2,z=2),lambda=1.8)  
```
---


```{r hyper3_modifyweight}
(H <- rankvec_likelihood(letters[1:5]))
H["b"] <- as.weight(1.88)
H
setweight(H,"c",1000)
```
---

```{r hyper3_ordertable2supp3}
(o <- constructor_2020_table[1:6,1:4])
ordertable2supp3(o)
```
---

```{r hyper3_home_away3}
M <- matrix(c(NA,9+2i,7+2i,6+5i,NA,2+4i,2+2i,9+3i,NA),3,3)
teams <- letters[1:3]
dimnames(M) <- list("@home" = teams,"@away"=teams)
M
home_away3(M,lambda=1.88)
```
---

```{r hyper3_powersbecomes}
(H <- hyper3(list(c(a=1.2),c(b=1),c(a=1.2,b=1)),powers=c(3,4,-7)))
powers(H) <- powers(H) * 3
H
```





## References

---
title: "Function `cheering()` in the `hyper2` package"
author: "R. K. S. Hankin"
output: bookdown::html_document2
bibliography: hyper2.bib
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library("hyper2")
library("partitions")
library("pdftools")
```

```{r out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE}
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))
```

To cite the `hyper2` package in publications, please use @hankin2017_rmd.

This document is a distillation and clarification of
`inst/plackett_luce_monster.Rmd`.

In the following, I use the ''race'' metaphor: five runners take part
in a race and arrive in order: $a\succ b\succ c\succ d\succ e$.
Consider the following Plackett-Luce likelihood function for this
observation:


\begin{equation}
\frac{p_a}{p_a + p_b + p_c + p_d + p_e}\cdot
\frac{p_b}{      p_b + p_c + p_d + p_e}\cdot
\frac{p_c}{            p_c + p_d + p_e}\cdot
\frac{p_d}{                  p_d + p_e}\cdot
\frac{p_e}{                        p_e}
\end{equation}


Note the use of multiplication between the terms, indicating
conditional independence.  We may use the multiplicative
generalization of Bradley-Terry strengths to introduce an element of
non-independence between the terms.  There are many ways to do this,
but one simple case would be to define equivalence classes of the
competitors with each equivalence class comprising mutually supporting
runners.  Equivalence classes of size one correspond to unsupported
runners.  The metaphor would be that a runner who has finished the
race is able to support other members of his equivalence class by
cheering his teammates, boosting their better performance.  The
`hyper2` package includes function `cheering()` which implements this
functionality.

As an example, consider figure 0.1 This shows a partial probability
tree diagram for some of the $5!=120$ possible order statistics.  The
standard Plackett-Luce likelihoods have been modified to account for
two groups of mutually supporting runners: $\left\lbrace
a,b,c\right\rbrace$ with support term $\lambda$, and $\left\lbrace
d,e\right\rbrace$ with support term $\mu$.  From START, the first
runner to cross the finishing line is associated with standard
Plackett-Luce.  Taking the top path as an example, we see that the
likelihood function for $a\succ b\succ c\succ d\succ e$ would be


```{tikz, label=tikz3, fig.cap = "Partial probability\\label{tikzabcde} tree for five competitors $a$-$e$ with mutually supporting subsets $\\left\\lbrace a,b,c\\right\\rbrace$ [with support term $\\lambda$] and $(de)$ [with support term $\\mu$], hyper3 approach", fig.ext = 'png', echo=FALSE}
\usetikzlibrary{arrows}
\usetikzlibrary{patterns}
\begin{tikzpicture}[line cap=round,line join=round,>=triangle 45,x=1cm,y=1cm]
\fill (0,0) circle[radius=2pt]; % root; paths abcde
\draw (0,0) -- (2,3);
\draw (0,0) -- (2,2);
\draw (0,0) -- (2,1);
\draw (0,0) -- (2,0);
\draw (0,0) -- (2,-1);
\node at (0,2.5) (eq1) {$\underbrace{\left\lbrace a,b,c\right\rbrace}_{\lambda}\underbrace{\left\lbrace d,e\right\rbrace}_{\mu}$};

\node at (1.5,2.5) {$a$};
\node at (1.5,1.7) {$b$};
\node at (1.5,1.0) {$c$};
\node at (1.5,0.2) {$d$};
\node at (1.5,-0.5) {$e$};

\node at (3, 3)  {$\frac{a}{a+b+c+d+e}$};
\node at (3, 2)  {$\frac{b}{a+b+c+d+e}$};
\node at (3, 1)  {$\frac{c}{a+b+c+d+e}$};
\node at (3, 0)  {$\frac{d}{a+b+c+d+e}$};
\node at (3,-1)  {$\frac{e}{a+b+c+d+e}$};

\fill  (4, 3) circle[radius=2pt];  % a finishes; paths bcde
\fill  (4, 2) circle[radius=2pt];  % terminal node
\fill  (4, 1) circle[radius=2pt];  % terminal node
\fill  (4, 0) circle[radius=2pt];  % d finishes; paths abce
\fill  (4,-1) circle[radius=2pt];  % terminal node

\draw (4,3) -- (5,5);
\draw (4,3) -- (5,4);
\draw (4,3) -- (5,3);
\draw (4,3) -- (5,2);

\node at (4.5,4.6) {$b$};
\node at (4.5,3.7) {$c$};
\node at (4.5,3.2) {$d$};
\node at (4.5,2.7) {$e$};

\newcommand{\la}[1]{\lambda{#1}}
\newcommand{\ld}[1]{\mu    {#1}}
\node at (6, 5)  {$\frac{\la{b}}{\la{b}+\la{c}+d+e}$};
\node at (6, 4)  {$\frac{\la{c}}{\la{b}+\la{c}+d+e}$};
\node at (6, 3)  {$\frac{d     }{\la{b}+\la{c}+d+e}$};
\node at (6, 2)  {$\frac{e     }{\la{b}+\la{c}+d+e}$};

\fill (7.4, 5) circle[radius=2pt];  % ab finishes; pahts cde
\draw (7.4,5) -- (8.4,6);
\draw (7.4,5) -- (8.4,5);
\draw (7.4,5) -- (8.4,4);


\node at (7.9,5.7) {$c$};
\node at (7.9,5.2) {$d$};
\node at (7.9,4.7) {$e$};
\node at (9.3, 6) {$\frac{\la{c}}{\la{c}+d+e}$};
\node at (9.3, 5) {$\frac{d  }{\la{c}+d+e}$};
\node at (9.3, 4) {$\frac{e  }{\la{c}+d+e}$};

\fill (10.2, 6) circle[radius=2pt];  % abc finishes; paths de
\draw (10.2, 6) -- (11.2,7);
\node at (10.7,6.8) {$d$};

\node at (11.8, 7) {$\frac{d}{d+e}$};

\draw (10.2, 6) -- (11.2,6.2);
\node at (11.8, 6.2) {$\frac{e}{d+e}$};
\node at (10.7,6.3) {$e$};

\fill (10.2, 5) circle[radius=2pt];
\draw (10.2, 5) -- (11.2,5.3);
\node at (11.8, 5.3) {$\frac{\la{c}}{\la{c}+\ld{e}}$};
\node at (10.7,5.3) {$c$};

\draw (10.2, 5) -- (11.2,4.7);
\node at (11.8, 4.7) {$\frac{\ld{e}}{\la{c}+\ld{e}}$};
\node at (10.7,4.7) {$e$};

\fill (10.2, 4) circle[radius=2pt];
\draw (10.2, 4) -- (11.2,3.9);
\node at (11.8, 3.9) {$\frac{\la{c}}{\la{c}+\ld{e}}$};
\node at (10.7,4.1) {$c$};

\draw (10.2, 4) -- (11.2,3.1);
\node at (11.8, 3.1) {$\frac{\ld{e}}{\la{c}+\ld{e}}$};
\node at (10.7,3.7) {$e$};

\draw (4, 0) -- (5,1);
\node at (6, 1)  {$\frac{a}{a+b+c+\ld{e}}$};

\draw (4, 0) -- (5,0);
\node at (6, 0)  {$\frac{b}{a+b+c+\ld{e}}$};

\draw (4, 0) -- (5,-1);
\node at (6, -1)  {$\frac{c}{a+b+c+\ld{e}}$};

\draw (4, 0) -- (5,-2);
\node at (6, -2)  {$\frac{\ld{e}}{a+b+c+\ld{e}}$};

\node at (4.5,0.7) {$a$};
\node at (4.5,0.2) {$b$};
\node at (4.5,-0.3) {$c$};
\node at (4.5,-0.8) {$e$};


\fill (7.4, -2) circle[radius=2pt];  % de finishes; paths abc
\draw (7.4, -2) -- (8.4,-1);
\draw (7.4, -2) -- (8.4,-2);
\draw (7.4, -2) -- (8.4,-3);

\node at (7.9,-1.2) {$a$};
\node at (7.9,-1.8) {$b$};
\node at (7.9,-2.3) {$c$};

\node at (9.3, -1) {$\frac{a}{a+b+c}$};
\node at (9.3, -2) {$\frac{b}{a+b+c}$};
\node at (9.3, -3) {$\frac{c}{a+b+c}$};

\fill (10.2, -1) circle[radius=2pt];  % dea finishes; paths bc
\fill (10.2, -2) circle[radius=2pt];  % deb finishes; paths ac
\fill (10.2, -3) circle[radius=2pt];  % dec finishes; paths ab

\draw (10.2, -1) -- (11.2,-0);
\node at (10.6,-0.4) {$b$};
\node at (10.6,-0.8) {$c$};


\node at (11.8, 0) {$\frac{\la{b}}{\la{b}+\la{c}}$};
\draw (10.2, -1) -- (11.2,-1);

\node at (11.8, -1) {$\frac{\la{c}}{\la{b}+\la{c}}$};


\draw (10.2, -2) -- (11.2,-1.6);
\node at (11.8, -1.6) {$\frac{\la{a}}{\la{a}+\la{c}}$};

\node at (10.6,-1.7) {$a$};
\node at (10.6,-2) {$c$};

\draw (10.2, -2) -- (11.2,-2.2);
\node at (11.8, -2.2) {$\frac{\la{c}}{\la{a}+\la{c}}$};

\draw (10.2, -3) -- (11.2,-2.9);
\node at (11.8, -2.9) {$\frac{\la{a}}{\la{a}+\la{b}}$};

\draw (10.2, -3) -- (11.2,-3.4);
\node at (11.8, -3.4) {$\frac{\la{b}}{\la{a}+\la{b}}$};

\node at (10.6,-2.8) {$a$};
\node at (10.6,-3.3) {$b$};

\fill (7.4, 1) circle[radius=2pt];  % da finishes; paths bce

\draw (7.4, 1) -- (8.4,2);
\draw (7.4, 1) -- (8.4,1);
\draw (7.4, 1) -- (8.4,0);

\node at (7.9,1.7) {$b$};
\node at (7.9,1.2) {$c$};
\node at (7.9,0.7) {$e$};

\node at (9.3, 2) {$\frac{\la{b}}{\la{b}+\la{c}+e}$};
\node at (9.3, 1) {$\frac{\la{c}}{\la{b}+\la{c}+e}$};
\node at (9.3, 0) {$\frac{e     }{\la{b}+\la{c}+e}$};


\fill (10.2, 1) circle[radius=2pt];  % da finishes; paths bce
\draw (10.2, 1) -- (11.2,2);
\node at (11.8, 2) {$\frac{\la{b}}{\la{b}+\ld{e}}$};
\draw (10.2, 1) -- (11.2,1);
\node at (11.8, 1) {$\frac{\ld{e}}{\la{b}+e}$};

\end{tikzpicture}
```

\begin{equation}\label{asuccb}
\frac{p_a}{p_a + p_b + p_c + p_d + p_e}\cdot
\frac{\lambda p_b}{\lambda p_b + \lambda p_c + p_d + p_e}\cdot
\frac{\lambda p_c}{\lambda p_c + p_d + p_e}\cdot
\frac{p_d}{p_d + p_e}
\end{equation}

In equation \ref{asuccb}, the first term is standard Plackett-Luce: at
this point, nooone has finished and cheering effects are absent.  The
second term reflects the fact that competitors $b$ and $c$ are
supported by competitor $a$, who has by this point finished the race
and is supporting his teammates. 

By contrast, the likelihood function for observation $d\succ a\succ
c\succ b\succ e$ would be

\begin{equation}
\frac{p_d}{p_a + p_b + p_c + p_d + p_e}\cdot
\frac{p_a}{p_a + p_b + p_c + \mu p_e}\cdot
\frac{\lambda p_c}{\lambda p_b + \lambda p_c + p_e}\cdot
\frac{\lambda p_b}{\lambda p_b + \mu p_e}
\end{equation}

where this likelihood function reflects the mutual support for
equivalence class $\left\lbrace d,e\right\rbrace$.  Note that the
final term reflects the fact that competitors $b$ and $e$ have their
support active when vying for fourth place, as members of both their
teams have finished at this point.

This probability model could easily be modified to account for
specific circumstances.  The cheering effect be asymmetrical (with $a$
helping $b$ but not vice-versa, for example).  The effect could
operate only on specific ordered pairs.  Or perhaps the effect might
have a finite lifetime: if $a$ places $n^\mathrm{th}$, then the
cheering effect is active only for competitors placing
$(n+r)^\mathrm{th}$ or above.


# Package idiom

We can investigate red bus-blue bus phenomenon (as discussed, in a
slightly different context, in `inst/red_bus_blue_bus.Rmd`).  Here, we
consider a person who is given the choice of five transport methods:

* `C`, car
* `T`, train
* `RB` a red bus
* `BB` a blue bus
* `W` walking


Now, he does not really care what colour the bus is.  If we ask him to
rank his options, it is highly probable that he will put `RB` and `BB`
consecutively (because they are essentially indistinguishable).  Can
we quantify the strength of this effect?  To do this, we define a
bespoke function `RB_BB_LF()` which returns a `hyper3` log-likelihood
function corresponding to repeated observations of our commuter's
reported ranks for the five options:


```{r,defineredbuslf}
`RB_BB_LF` <- function(s){
    ec <- c(C=1,T=2,RB=3,BB=3,W=4) # equivalence classes
    h <- c(1,1,s,1)                # strength of support
    (
        cheering3(v=c("RB","BB","C" ,"T","W"), e=ec, h=h)*3 +
        cheering3(v=c("W" ,"BB","RB","C","T"), e=ec, h=h)*7 +
        cheering3(v=c("W" ,"RB","BB","C","T"), e=ec, h=h)*8 +
        cheering3(v=c("W" ,"BB","RB","T","C"), e=ec, h=h)*7 +
        cheering3(v=c("W" ,"RB","BB","C","T"), e=ec, h=h)*7 +
        cheering3(v=c("RB","BB","C" ,"T","W"), e=ec, h=h)*3 +
        cheering3(v=c("RB","BB","C" ,"T","W"), e=ec, h=h)*3 +
        cheering3(v=c("BB","RB","T" ,"C","W"), e=ec, h=h)*2 +
        cheering3(v=c("T" ,"BB","RB","C","W"), e=ec, h=h)*2 +
        cheering3(v=c("W" ,"BB","RB","T","C"), e=ec, h=h)*4 +
        cheering3(v=c("C" ,"RB","BB","W","T"), e=ec, h=h)*4 +
        cheering3(v=c("BB","C" ,"RB","T","W"), e=ec, h=h)*3
    )
}
```

Above, we see from the function body that he reported `RB,BB,C,T,W`
three times [first row], `BB,RB,T,C,W` twice [second row], and so on;
perhaps his ranking depends on the weather or how tired he is on any
given day.  Observe that in almost every case he ranks `RB` and `BB`
consecutively.  Function `RB_BB_LF()` takes argument `s` that
quantifies the perceived similarity between `RB` and `BB`.  For
example:

```{r,examplerbbblf,cache=TRUE}
(H <- RB_BB_LF(s=1.8888))
(mH <- maxp(H,n=1))
```

Now to find a profile likelihood function for `s`:

```{r,label=proflikeredbusbluebus,cache=TRUE}
o <- function(s){maxp(RB_BB_LF(s),give=TRUE,n=1)$likes} # optimand
s <- exp(seq(from=log(1.3),to=log(147),len=17)) # putative similarity measures
L <- sapply(s,o)
L <- L-max(L)
```

We can plot these:



```{r,plotproflike}
plot(s,L,type="b")
abline(h=c(0,-2))
abline(v=1)
plot(log(s),L,type="b")
abline(h=c(0,-2))
abline(v=0)
```

And formally maximize the likelihood:

```{r,label=maxlikesim,cache=TRUE}
(osup <- optimize(o,c(6,10),maximum=TRUE))
```

So a likelihood ratio test of the null that $S=1$ would be:

```{r,label=LRT_RB,cache=TRUE}
(suppdiff <- o(osup$maximum) - o(1))
```


Easily satisfying Edwards's two-units-of-support criterion; Wilks gives us an asymptotic $p$-value:

```{r,label=RBBBpval}
pchisq(suppdiff*2,df=1,lower.tail=FALSE)
```


Now use the evaluate for the likelihood function:

```{r calcmaxmax, cache=TRUE}
maxHmax <- maxp(RB_BB_LF(s = osup$maximum))
```


```{r showmaxmax}
maxHmax
```


# University ranking analysis

Here we use a dataset of university rankings, `timesData.csv`, taken
from
`https://github.com/arnaudbenard/university-ranking/blob/master/timesData.csv`.

```{r, label=readtableandcalc,cache=TRUE}
a <- read.table("timesData.csv",sep=",", header=TRUE)
wanted <- c("California Institute of Technology", "Harvard University",
"Massachusetts Institute of Technology", "Princeton University",
"Stanford University", "University of Cambridge", "University of Oxford")
names(wanted) <- c("cal","harv","mass","prin","stan","cam","ox")

a <- a[a$university_name %in% wanted,]
a <- cbind(a,"top7rank"=0)
for(y in unique(a$year)){
    a[a$year==y,"top7rank"] <- order(
     			  as.numeric(a[a$year==y,"world_rank"]) +
                                     a[a$year==y,"research"]/1e6,
decreasing=TRUE)}
a <- a[,c("top7rank","university_name","year")]
a <- reshape(a,idvar="university_name",timevar="year",direction="wide")
for(i in seq_len(nrow(a))){
   a$university_name[i] <- names(which(wanted == a$university_name[i]))
}
rownames(a) <- a$university_name
a <- a[,-1]
colnames(a) <- paste("Y",2011:2016,sep="")
a
H <- suppfun(ordertable(a))
```

```{r,label=testuniv,cache=TRUE}
equalp.test(H)
samep.test(H,c("ox","cam"))
```

Start to use `hyper3` idiom:

```{r,label=defineoxcam}
H3 <- function(oxcam){
  out <- hyper3()
  for(i in seq_len(ncol(a))){
    jj <- rep("",nrow(a))
    jj[a[,i]] <- rownames(a)
    out <- out + cheering3(v=jj,e=c(ox=1,cam=1,prin=2, stan=3, mass=4, harv=5, cal=6), help=c(oxcam,1,1,1,1,1))
  }
  return(out)
}
```

```{r,label=calculateoxcam,cache=TRUE}
o <- function(oxcam){maxp(H3(oxcam),give=TRUE,n=1)$likes}
oc <- exp(seq(from=log(0.5),to=log(5),len=15))
L <- sapply(oc,o)
L <- L - max(L)
```


```{r,label=plotoxcam}
plot(log(oc),L,type="b")
abline(v=0)
```


# Five nations championship

The five nations rugby championship was held from 1910 to 1999 and
file `five_nations.txt` shows the order statistic for England (E),
Scotland (S), Ireland (I), Wales (W), and France (F).

`https://en.wikipedia.org/wiki/Six_Nations_Championship`

Here is `hyper2` analysis:


```{r,load5nationsdata,cache=TRUE}
a <- as.matrix(read.table("five_nations.txt",header=FALSE))
head(a)
H <- hyper2()
for(i in seq_len(nrow(a))){
  H <- H + race(a[i,-1])
}
mH <- maxp(H)
pie(mH)
equalp.test(H)
```


Now use `hyper3` to see whether teams do better following a win:

```{r,rugby3,cache=TRUE}
rugby <- function(lambda){
  H3a <- hyper3()
  for(i in seq(from=2,to=nrow(a))){
        last_year_winner <- a[i-1,2] # e.g. "W" or "E"
        H3a <- H3a + ordervec2supp3a(a[i,-1],nonfinishers=NULL,helped=last_year_winner,lambda=lambda)
  }
  return(H3a)
}
rugby(1.888)
rugby(1.111111)
```


```{r,rugby3max, cache=TRUE}
maxp(rugby(1.8),n=1,give=TRUE)
maxp(rugby(1.9),n=1,give=TRUE)
```


## References {-}

---
title: "Sushi monster"
author: "R. K. S. Hankin"
output: bookdown::html_document2
bibliography: hyper2.bib
---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library("hyper2")
```

```{r out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE}
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))
```


<font size="1"> most of the time is taken calculating the
`calclikesushi` and `maxlike`chunks.  With a sample size of 50 in
chunk `tryH`, and 11 points on the graph, it takes about half an hour
to run without cache. </font>

To cite the `hyper2` package in publications, please use @hankin2017_rmd.

This short document applies `hyper3` formalism to a dataset
obtained from the `preflib` library [@MaWa13a] in which ten
different types of sushi were placed in order of preference by 5000
judges.  Three of the sushis (maguro, toro, tekka-maki) were made with
tuna, and it is reasonable to view these three types as being
exchangeable in much the same way as the red and blue buses.  To
proceed, we need to define equivalence classes of sushi---one for
tuna-based sushis and one for everything else.  Of course, one can
also imagine the opposite phenomenon whereby the judge decides he has
had enough tuna for the moment and this disincentivises further tuna
choices.  This mechanism would be modelled by having $\lambda<1$.


```{r}
a <- as.matrix(read.table("sushi.txt", skip=22))
set.seed(0)
wanted <- sample(seq_len(nrow(a)),50,replace=TRUE,prob=a[,1])
a <- a[wanted,-1]
colnames(a) <- c("first","second","third","fourth","fifth","sixth","seventh","eighth","ninth","tenth")
```

```{r}
sushi_types <- c("ebi", "anago", "maguro", "ika", "uni", "sake", "tamago", "toro", "tekka", "kappa")
a[] <- sushi_types[a]
noquote(a)
sushi_eq_classes <-
    c("ebi"  = 1, "anago" = 1, "maguro" = 2, "ika" = 1, "uni" = 1,
      "sake" = 1, "tamago" = 1, "toro" = 2, "tekka" = 2,
      "kappa" = 1)
```
	

```{r,defcalculate_sushi_H}
make_sushi_H <- function(a, lambda){
    H <- hyper3()

    for(i in seq_len(nrow(a))){
        H <- H + cheering3(a[i,], e=sushi_eq_classes, h=c(1,lambda))
    }
    return(H)
}
```

```{r tryH, cache=TRUE}
H1 <- make_sushi_H(a, lambda=1)
H2 <- make_sushi_H(a, lambda=2)
```


```{r maxpsushiH,cache=TRUE}
mH1 <- maxp(H1,give=1)
mH2 <- maxp(H2,give=1)
```


```{r showsushiH}
mH1
mH2
```

```{r calclikesushi, cache=TRUE}
f <- function(lambda){
   H <- make_sushi_H(a, lambda = lambda)
   maxp(H,give=1)$`Log-likelihood`
}

s <- exp(seq(from = log(0.9), to = log(6), len = 11))
L <- sapply(s,f)
```

```{r maxlike, cache=TRUE}
(jj <- optimize(f, 2:3, maximum=TRUE))
```

```{r plotsushilike}
plot(log(s), L-jj$objective, type='b')
```

```{r calc_sushi, cache=TRUE}
sushi <- make_sushi_H(a, lambda = jj$maximum)
```

```{r calc_sushimaxp, cache=TRUE}
(sushi_maxp <- maxp(sushi))
```

### Package dataset {-}

Following lines create `sushi.rda`, residing in the `data/` directory of the package.

```{r,label=savesushidataset}
sushi_table <- a
save(sushi_table,  sushi, sushi_maxp, sushi_eq_classes, file="sushi.rda")
```

## References {-}

---
title: "exponential Bradley-Terry"
author: "robin hankin"
date: "2025-04-02"
output: html_document

---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(hyper2)
library(stringr)
library("partitions")
library("magrittr")
set.seed(0)
```

Basic idea is to have $n$ competitors with BT strengths $\beta, \beta
x,\ldots \beta x^{n-1}$ with $x\in[0,1]$.  The unit sum constraint
gives $\beta=\frac{1-x}{1-x^n}$ but this usually cancels out.  One of
the competitors is the _focal_ competitor with BT strength $\beta
x^{r-1}$ for some $r$ with $1\leqslant r < n$.  Thus $r=1$ means that
the focal competitor is the strongest among the $n$, $r=2$ means he is
the second, and so on.

# Some simple cases


We consider the smallest non-trivial cases $n=3$ and $n=4$.  For $n=3$
we have Plackett-Luce strengths $\alpha x,\alpha x^2,\alpha x^3$ [the
  extra factor of $x$ allows us to identify the $r$ in ``$\alpha x^r$''
  with the rank of the competitor, the best being rank 1].  Suppose
the focal competitor came first, the likelihood function would be

\begin{equation}
  \mathcal{L}(x,r)=\frac{x^{r-1}}{1+x+x^2}\qquad r=1,2,3
\end{equation}

```{r plot123first}
worker_123first <- function(...){
f <- function(x,r){ x^r/(x + x^2 + x^3) }

x <- seq(from=0.01 , to=1, by=0.01)
plot(c(-3,0), c(-3,0), type='n',
     xlab="log x", ylab="log-likelihood",main="(a)")
points(log(x),log(f(x,1)), type="l", col="black")
points(log(x),log(f(x,2)), type="l", col="red"  )
points(log(x),log(f(x,3)), type="l", col="blue" )

abline(h=0, lty=2)
abline(h = -2)
e <- exp(1)
r2reject <- ((e^2-1)-sqrt(e^4-2*e^2-3))/2
segments(x0=log(r2reject), y0=-2.2,y1=-1.8)

r3reject <- (1+sqrt(1+4*(e^2-1)))/(2*(e^2-1))
segments(x0=log(r3reject), y0=-2.2,y1=-1.8)
legend(x= -3, y= - 0.5, lty=1, col=c("black","red","blue"),
       legend=c("r=1","r=2","r=3"))
}
e <- exp(1)
r2reject <- ((e^2-1)-sqrt(e^4-2*e^2-3))/2
print(r2reject)
r3reject <- (1+sqrt(1+4*(e^2-1)))/(2*(e^2-1))
print(r3reject)

worker_123first()
pdf(file="123first.pdf")
worker_123first()
dev.off()


```

If the focal competitor came second we would have two Plackett-Luce
probabilities to add, corresponding to different order statistics.
$\alpha x^i$ or $\alpha x^j$ coming first where $i,j\neq r$.  For
example, if $r=1$ we would have two finishing orders that are
consistent with the observation: $2\succ 1\succ 3$ and $3\succ 1\succ
2$.  These are disjoint so the likelihood will be proportional to

$$
\frac{x^2}{x+x^2+x^3}\cdot\frac{x}{x+x^3} + 
\frac{x^3}{x+x^2+x^3}\cdot\frac{x}{x+x^2}
$$

After simplification:

\begin{equation}
  \mathcal{L}(x,r)=
  \begin{cases}
    \frac{x  }{x+x^2+x^3}\left(\frac{x^2}{x+x^3}   + \frac{x^3}{x+x^2}\right)\qquad r=1\\
    \frac{x^2}{x+x^2+x^3}\left(\frac{x  }{x^2+x^3} + \frac{x^3}{x+x^2}\right)\qquad r=2\\
    \frac{x^3}{x+x^2+x^3}\left(\frac{x  }{x^2+x^3} + \frac{x^2}{x+x^3}\right)\qquad r=3
\end{cases}
\end{equation}


```{r plot123second}
worker_123second <- function(...){
like <- function(x,r){
  S <- x + x^2 + x^3
  switch(r,
         '1' = x^2/S*x^1/(x^1+x^3) + x^3/S*x^1/(x^1+x^2), # 213+312
         '2' = x^1/S*x^2/(x^2+x^3) + x^3/S*x^2/(x^1+x^2), # 123+321
         '3' = x^1/S*x^3/(x^2+x^3) + x^2/S*x^3/(x^1+x^3)  # 132+231
  )
}

x <- seq(from=0.01, to=1, by=0.01)
plot(c(-3,0),c(-3,0), type='n',
     xlab="log x", ylab="log-likelihood", main="(b)")
points(log(x),log(like(x,1)), type="l", col="black")
points(log(x),log(like(x,2)), type="l", col="red"  )
points(log(x),log(like(x,3)), type="l", col="blue" )

abline(h=0,lty=2)
abline(h=-2)

legend(x = -3, y = -0.5, lty=1, col=c("black","red","blue"),
       legend=c("r=1", "r=2", "r=3"))
}
worker_123second()
pdf(file="123second.pdf")
worker_123second()
dev.off()
```

Now the focal competitor comes last:


```{r plot123third}
worker_123third <- function(...){
like <- function(x,r){
  S <- x + x^2 + x^3
  switch(r,
         '1' = x^3/S*x^2/(x^1+x^2) + x^2/S*x^3/(x^1+x^3), # 213+312
         '2' = x^1/S*x^3/(x^2+x^3) + x^3/S*x^1/(x^1+x^2), # 123+321
         '3' = x^1/S*x^2/(x^2+x^3) + x^2/S*x^1/(x^1+x^3)  # 132+231
  )
}

x <- seq(from=0.01, to=1, by=0.01)
plot(c(-3,0),c(-3,0),type='n',
     xlab="log x", ylab="log-likelihood", main="(c)")
points(log(x),log(like(x,1)), type="l", col="black")
points(log(x),log(like(x,2)), type="l", col="red"  )
points(log(x),log(like(x,3)), type="l", col="blue" )

abline(h=0,lty=2)
abline(h=-2)

legend(x = -3, y = -0.5, lty=1, col=c("black", "red", "blue"),
       legend=c("r=1", "r=2", "r=3"))
}

worker_123third()
pdf(file="123third.pdf")
worker_123third()
dev.off()
```


```{r allthree}
worker_allthree <- function(...){
op <- par(mfrow=c(1,3))
worker_123first()
worker_123second()
worker_123third()
dev.off()
par(op)
}
worker_allthree()
pdf(file="123all.pdf",height=4)
worker_allthree()
dev.off()
```
The $n=4$ case is harder as we potentially have six consistent
orderings for each observation.  As an example, if the focal
competitor comes third we have

\begin{equation}
\newcommand{pl}[4]{
\frac{x^{#1}}{x^{#1} + x^{#2} + x^{#3} + x^{#4}}\cdot
\frac{x^{#2}}{         x^{#2} + x^{#3} + x^{#4}}\cdot
\frac{x^{#3}}{                  x^{#3} + x^{#4}}}

\end{equation}


SDAdsfDSFFDSAfdsafdsff asdf dfs dfs dfs dsff dsfds fds fds sfd s fs sd sdf sdf sdf sdf sdf sdf sdf sdfs dfsdf sdsdfsa asdf saf

\begin{eqnarray}
\mathcal{L}(x;r=1)
\quad &=& \quad\hphantom{+}
            \pl{2}{3}{1}{4}\\
&{}&\quad + \pl{2}{4}{1}{3}\\
&{}&\quad + \pl{3}{2}{1}{4}\\
&{}&\quad + \pl{3}{4}{1}{2}\\
&{}&\quad + \pl{4}{2}{1}{3}\\
&{}&\quad + \pl{4}{3}{1}{2}
\end{eqnarray}

\begin{eqnarray}
\mathcal{L}(x;r=2) 
\quad &=& \quad\hphantom{+}
            \pl{1}{3}{2}{4}\\
&{}&\quad + \pl{1}{4}{2}{3}\\
&{}&\quad + \pl{3}{1}{2}{4}\\
&{}&\quad + \pl{3}{4}{2}{1}\\
&{}&\quad + \pl{4}{1}{2}{3}\\
&{}&\quad + \pl{4}{3}{2}{1}
\end{eqnarray}

and so on.


```{r fourcompetitors}
PL <- function(v){
  stopifnot(length(v) == 4)
  v[1]/(v[1]+v[2]+v[3]+v[4]) *
  v[2]/(     v[2]+v[3]+v[4]) *
  v[3]/(          v[3]+v[4])
}


f1 <- function(v){  # focal competitor came third, r=1
  return(
  PL(v^c(2,3,1,4)) +
  PL(v^c(2,4,1,3)) +
  PL(v^c(3,2,1,4)) +
  PL(v^c(3,4,1,2)) +
  PL(v^c(4,2,1,3)) +
  PL(v^c(4,3,1,2))
  )
}
f2 <- function(v){  # focal competitor came third, r=2
  return(
  PL(v^c(1,3,2,4)) +
  PL(v^c(1,4,2,3)) +
  PL(v^c(3,1,2,4)) +
  PL(v^c(3,4,2,1)) +
  PL(v^c(4,1,2,3)) +
  PL(v^c(4,3,2,1))
  )
}
f3 <- function(v){  # focal competitor came third, r=2
  return(
  PL(v^c(1,2,3,4)) +
  PL(v^c(1,4,3,2)) +
  PL(v^c(2,1,3,4)) +
  PL(v^c(2,4,3,1)) +
  PL(v^c(4,1,3,2)) +
  PL(v^c(4,2,3,1))
  )
}
f4 <- function(v){  # focal competitor came third, r=4
  return(
  PL(v^c(1,2,4,3)) +
  PL(v^c(1,3,4,2)) +
  PL(v^c(2,1,4,3)) +
  PL(v^c(2,3,4,1)) +
  PL(v^c(3,1,4,2)) +
  PL(v^c(3,2,4,1))
  )
}


worker <- function(...){

o <- 1:4

plot(c(-3,0),c(-3,0), type='n',
     xlab="log x", ylab="log-likelihood", main="Focal competitor came third")
x <- seq(from=0.01,to=1,by=0.001)

points(log(x),log(sapply(x,f1)), type="l", col="black")
points(log(x),log(sapply(x,f2)), type="l", col="red")
points(log(x),log(sapply(x,f3)), type="l", col="blue")
points(log(x),log(sapply(x,f4)), type="l", col="green")

abline(h=0,lty=2)
abline(h = -2, lty=1, col="gray")
           
legend("topright",lty=1,col=c("black", "red", "blue", "green"),
       legend=c("r=1", "r=2", "r=3", "r=4"), bg="white")


f1_solve <- function(x){log(sapply(exp(x), f1)) + 2}
f2_solve <- function(x){log(sapply(exp(x), f2)) + 2}
f4_solve <- function(x){log(sapply(exp(x), f4)) + 2}

root1 <- uniroot(f1_solve, c(-3,-0.1))$root
root2 <- uniroot(f2_solve, c(-3,-0.1))$root
root4 <- uniroot(f4_solve, c(-3,-0.1))$root

segments(x0=root1, y0=-2.1, y1=-1.9, col="gray")
segments(x0=root2, y0=-2.1, y1=-1.9, col="gray")
segments(x0=root4, y0=-2.1, y1=-1.9, col="gray")

print(c(root1, root2, root4))
} # worker() def closes



worker()
pdf(file="1234third.pdf")
worker()
dev.off()


```


# Try 8 competitors, focal competitor comes third


```{r calconetoeight, cache=TRUE}
pl <- function(v){
  out <- 1
  n <- length(v)
  for(i in seq_len(n-1)){
    out <- out*v[i]
    out <- out/sum(v[seq(from=i, to=n)])
  }
  return(out)
}

f <- function(x, place, no_of_comp){
    M <- perms(no_of_comp)
    out <- rep(0, no_of_comp)
    for(i in seq_len(ncol(M))){
        out[M[place,i]] %<>% `+`(pl(x^M[,i]))
    }
    return(out)
}

n <- 8
place <- 3
x <- seq(from = 0.1, to=1, by=0.01)
L <- matrix(0, length(x), n)
for(i in seq_along(x)){
    L[i,] <- f(x[i], place, n)
}
```

```{r onetoeight}
worker <- function(...){
    matplot(log(x),log(L), type='l', lty=1,ylim=c(-4,0))
}
pdf(file = "onetoeight.pdf")
worker()
dev.off()
worker()
```

# More stuff






```{r defineBT}
BT <- function(n,x){
    out <- x^(0:(n-1))
    if(x != 1){
        out <- out*(1-x)/(1-x^n)
    } else {
        out <- out/n
    }
    names(out) <- paste0("p",str_pad(1:n,ceiling(log10(n))))
    out
}
BT(4,0.5)
sum(BT(4,0.5))
```

We will consider $n=9$, $x=0.5$, $r=4$ and make repeated observations:


```{r showtabulate}
(jj <- BT(9,0.5))
o <- tabulate(replicate(1e4,which(rrace(jj) == "p4")))
o
plot(o,type='h')
```


    
```{r definegetprob, caasdf=TRUE}
getprob <- function(n, r, x, N=1e4){
    jj <- BT(n, x)
    wanted <- paste0("p", str_pad(r ,ceiling(log10(n))))
    c(r=r, x=x, setNames(tabulate(replicate(N, which(rrace(jj) == wanted)),nbins=n), paste0("p", 1:n)))
}
rbind(
getprob(9,1,0.5,N=100),
getprob(9,2,0.5,N=100),
getprob(9,3,0.5,N=100),
getprob(9,4,0.5,N=100),
getprob(9,5,0.5,N=100),
getprob(9,6,0.5,N=100),
getprob(9,7,0.5,N=100),
getprob(9,8,0.5,N=100),
getprob(9,9,0.5,N=100)
)

rbind(
getprob(9,1,0.2,N=100),
getprob(9,2,0.2,N=100),
getprob(9,3,0.2,N=100),
getprob(9,4,0.2,N=100),
getprob(9,5,0.2,N=100),
getprob(9,6,0.2,N=100),
getprob(9,7,0.2,N=100),
getprob(9,8,0.2,N=100),
getprob(9,9,0.2,N=100)
)
rbind(
getprob(9,1,0.9,N=100),
getprob(9,2,0.9,N=100),
getprob(9,3,0.9,N=100),
getprob(9,4,0.9,N=100),
getprob(9,5,0.9,N=100),
getprob(9,6,0.9,N=100),
getprob(9,7,0.9,N=100),
getprob(9,8,0.9,N=100),
getprob(9,9,0.9,N=100)
)
```

```{r makeII, cache=TRUE}
r_try <- 1:9
x_try <- seq(from = 0.1, to = 1, by = 0.05)
jj <- as.matrix(expand.grid(r_try, x_try))
II <- t(apply(jj, 1, function(v){getprob(9, v[1], v[2], N=1e4)}))
head(II)
```


```{r makesupp, cache=TRUE}
r_true <- 3
x_true <- 0.85
set.seed(0)
o <- getprob(9, r_true, x_true, 100)  # observations
o[1:2]
o[-(1:2)]

support_func <- function(o, v){
    v <- v[-(1:2)]
    v <- v/sum(v)
    dmultinom(o[-(1:2)], prob=v, log=TRUE)
}
support <- rep(0,nrow(II))
for(i in seq_len(nrow(II))){
    support[i] <- support_func(o, II[i,])
}
```
    
```{r}
x_try
length(support)
support <- support-max(support)
support <- pmax(support, -10)
support  <- matrix(support,length(r_try),length(x_try))
contour(r_try, x_try, support, xlab="r", ylab="x", levels = -c(1,(1:5)*2), lwd=c(1,3,1,1,1,1))
abline(v=1:9, lwd=0.3, col='gray')
abline(v = r_true)
abline(v = 1:9, col = 'gray', lty = 3, lwd = 0.3)
filled.contour(r_try, x_try, support, xlab = "r", ylab = "x")
```

```{r contourLO}
x_try
wanted <- 8:18
LO <- function(p){log(p/(1-p))}
contour(r_try, LO(x_try[wanted]), support[,wanted], xlab="r", ylab="Log-odds of x", levels = -c(1,(1:5)*2), lwd=c(1,3,1,1,1,1))
```

```{R showjj}
jj <- cbind(II[,1:2],c(support))
jj[jj[,2] == 1,]
```

#  Analysis from a single observation


We translate $\alpha\in [0,1]$ to a number from 1 to $n$ inclusive
with $\lceil n\alpha\rceil$

Write a likelihood function for 9,12 [that is, came 9th in a class of
12] There are 8 better and 3 worse than the focal student.


```{r ninetwelve}
single_like_fun <- function(n, a,          # n competitors, places a [e.g a=3 -> third]
                            a_try, x_try,  # values to try
                            N=1e2){        # N is number of trials
    p_try <- ceiling(n * a_try)
    jj <- as.matrix(expand.grid(p_try, x_try))
    II <- t(apply(jj,1,function(v){getprob(n, v[1], v[2], N)}))
    matrix(II[, ceiling(a*n) + 2], length(a_try), length(x_try))
}
```


```{r makesupp912, cache=TRUE}
a_try <- seq(from = 0.01, to = 0.99, len = 10)
x_try <- seq(from = 0.2, to = 1, by = 0.1)
allsupp <- list(
    log(single_like_fun(12, a =  8/12, a_try, x_try)),  # pure
    log(single_like_fun(17, a = 12/17, a_try, x_try)),  # pure
    log(single_like_fun(23, a = 18/23, a_try, x_try)),  # pure
    log(single_like_fun(14, a =  5/14, a_try, x_try)),  # applied
    log(single_like_fun(13, a =  3/13, a_try, x_try)),  # applied
    log(single_like_fun(11, a =  2/11, a_try, x_try)),  # applied
    log(single_like_fun(13, a =  3/13, a_try, x_try))   # applied
)
```

```{r showallsupp}
S <- Reduce("+",allsupp)
S <- pmax(S-max(S),-10)
contour(a_try, x_try, S, levels = -1*(1:7))
jj <- which(S == max(S), arr.ind = TRUE)
points(a_try[jj[1]], x_try[jj[2]], pch=16, col = 'red')
```


```{r showbothsupp}
S_pure    <- Reduce("+", allsupp[1:3])
S_applied <- Reduce("+", allsupp[4:7])
S_all     <- Reduce("+", allsupp[1:7])
max(S_all)
max(S_pure)
max(S_applied)
max(S_pure) + max(S_applied)
max(S_pure) + max(S_applied) - max(S_all)
pchisq(max(S_pure) + max(S_applied) - max(S_all),df=2,lower.tail=FALSE)
```


# Parkrun

In the following chunks, each execution of
`single_like_fun(...,N=1000)` takes  about 5 minutes

```{r sethowmany}
howmany <- 1000
```

```{r parkrunlikefunobs1, cache = TRUE}
print(system.time(parkrun01 <- single_like_fun(259, 177/259, N = howmany, a_try, x_try)))
```

```{r}
parkrun01
```

```{r parkrunlikefunobs2, cache = TRUE}
 print(system.time(parkrun02 <- single_like_fun(305, 222/305, N = howmany, a_try, x_try)))
```

```{r}
parkrun02
```

```{r parkrunlikefunobs3, cache = TRUE}
print(system.time(parkrun03 <- single_like_fun(297, 206/297, N = howmany, a_try, x_try)))
```

```{r}
parkrun03
```

```{r parkrunlikefunobs4, cache = TRUE}
print(system.time(parkrun04 <- single_like_fun(241, 142/241, N = howmany, a_try, x_try)))
```

```{r}
parkrun04
```

```{r parkrunlikefunobs5, cache = TRUE}
print(system.time(parkrun05 <- single_like_fun(179, 118/179, N = howmany, a_try, x_try)))
```

```{r}
parkrun05
```
 
```{r parkrunlikefunobs6, cache = TRUE}
print(system.time(parkrun06 <- single_like_fun(338, 224/338, N = howmany, a_try, x_try)))
```

```{r}
parkrun06
```

```{r parkrunlikefunobs7, cache = TRUE}
print(system.time(parkrun07 <- single_like_fun(203, 128/203, N = howmany, a_try, x_try)))
```

```{r}
parkrun07
```


```{r}
c(
sum(parkrun01),
sum(parkrun02),
sum(parkrun03),
sum(parkrun04),
sum(parkrun05),
sum(parkrun06),
sum(parkrun07)
)
```

```{r}
contour(a_try,x_try,parkrun05)
contour(a_try,x_try,parkrun06)
parkrun01*parkrun02*parkrun03*parkrun04*parkrun05*parkrun06*parkrun07
```



## Formula 1


```{r checo}
n <- 20 # 20 drivers
multiple_like_fun <- function(n,           # n competitors
                            a_try, x_try,  # values to try
                            N=1e2){        # N is number of trials
    p_try <- ceiling(n * a_try)
    jj <- as.matrix(expand.grid(p_try, x_try))
    t(apply(jj,1,function(v){getprob(n, v[1], v[2], N)}))
}
```


```{r checo2023}

o <- c(5,5,2,6,6,4,2,3,8,3,4,4,6,6,3,5,2,8,7,9)  # Lewis Hamilton 2023
# o <- c(7,9,9,9,6,6,7,4,3,4,1,3,1,8,5,9,6,4,10,2,12,4) # Lewis Hamilton 2024
# o <- c(6,8,3,5,2,1,2,4,2,2,20,3,2,5,1,3,4,1,4,2,6,6,10) # lando norris 2024
```

```{r trymultx, cache=TRUE}
II <- multiple_like_fun(20, a_try,x_try,1e4)
head(II)
nrow(II)
length(a_try)
length(x_try)
```

```{r calcmultx, cache=TRUE}
like <- list()
dim(II)
o
for(i in seq_along(o)){
    like[[i]] <-  matrix(II[, o[i] + 2], length(a_try), length(x_try))
}
```

```{r showlike, cache=FALSE}
options(digits=5)
like
alllike <- Reduce("*",like)
alllike 
alllike/max(alllike)
S <- log(alllike)
S <- S-max(S)
S <- pmax(S,-15)
contour(S,levels = -(0:6)*2, lwd=c(1,4,1,1,1,1,1))
filled.contour(S)
```

---
title: "Generalized Plackett-Luce Likelihoods"
author: "Robin K. S. Hankin"
bibliography: hyper2.bib
link-citations: true
output: html_vignette
vignette: |
  %\VignetteIndexEntry{hyper3}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include=FALSE}
library("hyper2",quietly=TRUE)
set.seed(0)
knitr::opts_chunk$set(echo = TRUE)
```

```{r label="hexsticker",out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE}
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))
```

To cite the `hyper2` package in publications, please use
@hankin2017_rmd; to cite `hyper3` functionality, use
@hankin2024_hyper3.

The `hyper2` package provides functionality to work with extensions of
the Bradley-Terry probability model such as Plackett-Luce likelihood
including team strengths and reified entities (monsters).  The package
allows one to use relatively natural R idiom to manipulate such
likelihood functions.  Here, I present a generalization of `hyper2` in
which multiple entities are constrained to have identical
Bradley-Terry strengths.  A new `S3` class `hyper3`, along with
associated methods, is motivated and introduced.  Three datasets are
analysed, each analysis furnishing new insight, and each highlighting
different capabilities of the package.


# Introduction

The `hyper2` package [@hankin2010,@hankin2017_rmd] furnishes
computational support for generalized Plackett-Luce [@plackett1975]
likelihood functions.  The preferred interpretation is a race (as in
track and field athletics): given six competitors $1-6$, we ascribe to
them nonnegative strengths $p_1,\ldots, p_6$; the probability that $i$
beats $j$ is $p_i/(p_i+p_j)$.  It is conventional to normalise so that
the total strength is unity, and to identify a competitor with his
strength.  Given an order statistic, say $p_1\succ p_2\succ p_3\succ
p_4$, the Plackett-Luce likelihood function would be

\begin{equation}\label{PL_like}
  \frac{p_1}{p_1+p_2+p_3+p_4}\cdot
  \frac{p_2}{    p_2+p_3+p_4}\cdot
  \frac{p_3}{        p_3+p_4}\cdot
  \frac{p_4}{            p_4}
  \qquad\mbox{Plackett-Luce}
\end{equation}


@mollica2014 call this a forward ranking process on the grounds
that the best (preferred; fastest; chosen) entities are identified in
the same sequence as their rank.  

Computational support for Bradley-Terry likelihood functions is
available in a range of languages.  @hunter2004, for example, presents
results in `MATLAB` [although he works with a nonlinear extension to
account for ties]; @allison1994 present related work for ranking
statistics in `SAS` and @maystre2022 has released a `python` package
for Luce-type choice datasets.

However, the majority of software is written in the R computer
language [@rcore2023], which includes extensive functionality for
working with such likelihood functions: @turner2020 discuss several
implementations from a computational perspective.  The `BradleyTerry`
package [@firth2005] considers pairwise comparisons using
`glm` but cannot deal with ties or player-specific
predictors; the `BradleyTerry2` package [@turner2012] implements a
flexible user interface and wider range of models to be fitted to
pairwise comparison datasets, specifically simple random effects.  The
`PlackettLuce` package [@turner2020] generalizes this to
likelihood functions of the form of the Plackett-Luce equation above,
and applies the Poisson transformation of @baker1994 to express the
problem as a log-linear model.  The `hyper2` package, in contrast,
gives a consistent language interface to create and manipulate
likelihood functions over the simplex
${S}_n=\left\lbrace\left(p_1,\ldots,p_n\right)\left|p_i\geq 0,\sum
p_i=1\right.\right\rbrace$.  A further extension in the package
generalizes this likelihood function to functions of ${\mathbf
p}=(p_1,\ldots,p_n)$ with

\begin{equation}\label{hyper2likelihood}
\mathcal{L}\left(\mathbf{p}\right)=
\prod_{s\in \mathcal{O}}\left({\sum_{i\in s}}p_i\right)^{n_s}\qquad\mbox{Hyperdirichlet likelihood}
\end{equation}

where $\mathcal{O}$ is a set of observations and $s$ a subset of
$\left\{1,2,\ldots,n\right\}$; numbers $n_s$ are integers which may be
positive or negative and we usually require $\sum_s n_s=0$.  The
`hyper2` package has the ability to evaluate such likelihood functions
at any point in $S_n$, thereby admitting a wide range of non-standard
nulls such as order statistics on the $p_i$ [@hankin2017_rmd].  It
becomes possible to analyse a wider range of likelihood functions than
standard Plackett-Luce [@turner2020].  For example, results involving
incomplete order statistics or teams are tractable.  Further, the
introduction of reified entities (monsters) allows one to consider
draws [@hankin2010], noncompetitive tactics such as collusion
[@hankin2020], and the phenomenon of team cohesion wherein the team
becomes stronger than the sum of its parts [@hankin2010].  Recent
versions of the package include experimental functionality
`cheering()` to investigate the relaxing of the assumption of
conditional independence of the forward-ranking process.

Here I present a different generalization.  Consider a race in which
there are six runners 1-6 but we happen to know that three of the
runners (1,2,3) are clones of strength $p_a$, two of the runners (4,5)
have strength $p_b$, and the final runner (6) is of strength $p_c$.
We normalise so $p_a+p_b+p_c=1$.  The runners race and the finishing
order is:

$$a\succ c\succ b\succ a\succ a \succ b$$

Thus the winner was $a$, second place for $c$, third for $b$, and so
on.  Alternatively we might say that $a$ came first, fourth, and
fifth; $b$ came third and sixth, and $c$ came second.  The
Plackett-Luce likelihood function for $p_a,p_b,p_c$ would be

\begin{equation}\label{plackettluce}
\frac{p_a}{3p_a+2p_b+p_c}\cdot
\frac{p_c}{2p_a+2p_b+p_c}\cdot
\frac{p_b}{2p_a+2p_b    }\cdot
\frac{p_a}{2p_a+ p_b    }\cdot
\frac{p_a}{ p_a+ p_b\vphantom{x_{x_{x_{x_{x_{x_{x_{x_{x_{x_{x_{x_{x_{x_{x_{x_{x_{x}}}}}}}}}}}}}}}}}}
                        }\cdot
\frac{p_b}{      p_b    },\qquad p_a+p_b+p_c=1\\
\mbox{generalized plackett-Luce likelihood}
\end{equation}

Here I consider such generalized Plackett-Luce likelihood functions,
and give an exact analysis of several simple cases.  I then show how
this class of likelihood functions may be applied to a range of
inference problems involving order statistics.  Illustrative examples,
drawn from Formula 1 motor racing, and track-and-field athletics, are
given.

## Computational methodology for generalized Plackett-Luce likelihood functions

Existing `hyper2` formalism as described by @hankin2017_rmd cannot
represent the Plackett-Luce likelihood equation, because the
hyperdirichlet likelihood equation uses sets as the indexing elements,
and in this case we need multisets\footnote{Note that the version of
`hyper2` presented by @hankin2017_rmd and reviewed by [@turner2020]
used integer-valued sets together with a print method that used a
complicated mapping system from integers to competitor names.  Current
methodology [following commit `51a8b46`] is to use sets of character
strings which represent the competitors directly; this allows for
easier combination of observations including different competitors.}.
The declarations

```
typedef map<string, long double> weightedplayervector;
typedef map<weightedplayervector, long double> hyper3;
```

show how the `map` class of the Standard Template Library is
used with `weightedplayervector` objects mapping strings to
long doubles (specifically, mapping player names to their
multiplicities), and objects of class `hyper3` are maps from a
`weightedplayervector` object to long doubles.  One advantage
of this is efficiency: search, removal, and insertion operations have
logarithmic complexity.  As an example, the following `C++`
pseudo code would create a log-likelihood function for the first term
in the Plackett-Luce likelihood equation above:

```
weightedplayervector n,d;
n["a"] = 1; 
d["a"] = 3; 
d["b"] = 2;
d["c"] = 1;

hyper3 L;
L[n] = 1;
L[d] = -1;
```

Above, we understand `n` and `d` to represent numerator and
denominator respectively.  Object `L` is an object of class
`hyper3`; it may be evaluated at points in probability space
[that is, a vector `[a,b,c]` of nonnegative values with unit sum]
using standard R idiom wrapping C++ back end.

## Package implementation

The package includes an `S3` class `hyper3` for this type of object;
extraction and replacement methods use `disordR` discipline
[@hankin2022].  Package idiom for creating such objects uses named
vectors:

```{r showLL}
LL <- hyper3()
LL[c(a = 1)] <- 1
LL[c(a = 3, b = 2, c = 1)] <- -1
LL
```

Above, we see object `LL` is a log-likelihood function of the
players' strengths, which may be evaluated at specified points in
probability space.  A typical use-case would be to assess $H_1\colon
p_a=0.9,p_b=0.05,p_c=0.05$ and $H_2\colon p_a=0.01,p_b=0.01,p_c=0.98$,
and we may evaluate these hypotheses using generic function
`loglik()`:

```{r loglikabc}
loglik(c(a = 0.01, b = 0.01, c = 0.98), LL)
loglik(c(a = 0.90, b = 0.05, c = 0.05), LL)
```

Thus we prefer $H_1$ over $H_2$ with about 3.5 units of support,
satisfying the standard two units of support criterion [@edwards1972],
and we conclude that our observation [in this case, that one of the
three clones of player $a$ beat the $b$ twins and the singleton $c$]
furnishes strong support against $H_2$ in favour of $H_1$.

The package includes many helper functions to work with order
statistics of this type.  Function `ordervec2supp3()`, for
example, can be used to generate a Plackett-Luce log-likelihood
function:

```{r orderacbaab}
(H <- ordervec2supp3(c("a", "c", "b", "a", "a", "b")))
```

(the package gives extensive documentation at `ordervec2supp.Rd`).  We
may find a maximum likelihood estimate for the players' strengths,
using generic function `maxp()`, dispatching to a specialist
`hyper3` method:

```{r mhmaxp}
(mH <- maxp(H))
```

(function `maxp()` uses standard optimization techniques to
locate the evaluate; it has access to first derivatives of the
log-likelihood and as such has rapid convergence, if its objective
function is reasonably smooth).

The package provides a number of statistical tests on likelihood
functions, modified from @hankin2017_rmd to work with `hyper3`
objects.  For example, we may assess the hypothesis that all three
players are of equal strength [viz $H_0\colon
p_a=p_b=p_c=\frac{1}{3}$]:

```{r testequality}
equalp.test(H)
```

showing, perhaps unsurprisingly, that this small dataset is consistent
with $H_0$. 

## Package helper functions

Arithmetic operations are implemented for `hyper3` objects in much the
same way as for `hyper2` objects: independent observations may be
combined using the overloaded `+` operator; an example is given below.

The original motivation for `hyper3` was the analysis of Formula 1
motor racing, and the package accordingly includes wrappers for
`ordervec2supp()` such as `ordertable2supp3()` and
`attemptstable2supp3()` which facilitate the analysis of commonly
encountered result formats.  Package documentation for order tables is
given at `ordertable.Rd` and an example is given below.

# Exact analytical solution for some simple generalized Plackett-Luce likelihood functions

Here I consider some order statistics with nontrivial maximum
likelihood Bradley-Terry strengths.  The simplest nontrivial case
would be three competitors with strengths $a,a,b$ and finishing order
$a\succ b\succ a$.  The Plackett-Luce likelihood function would be

\begin{equation}\label{aba}
\frac{a}{2a+b}\cdot\frac{b}{a+b}
\end{equation}

and in this case we know that $a+b=1$ so this is equal to
$\mathcal{L}=\mathcal{L}(a)=\frac{a(1-a)}{1+a}$.  The score would be
given by

\begin{equation}\label{aba_mle}
\frac{d\mathcal{L}}{da}=\frac{(1+a)(1-2a)-a(1-a)}{(1+a)^2}=
\frac{1-2a-a^2}{(1+a)^2}
\end{equation}

and this will be zero at $\sqrt{2}-1$; we also note that
$d^2\mathcal{L}/da^2=-4(1+a)^{-3}$, manifestly strictly negative for
$0\leq a\leq 1$: the root is a maximum.


```{r maxpaba}
maxp(ordervec2supp3(c("a", "b", "a")))
```

Above, we see close agreement with the theoretical value of
$(\sqrt{2}-1,2-\sqrt{2})\simeq (0.414,0.586)$.  Observe that the
maximum likelihood estimate for $a$ is strictly less than 0.5, even
though the finishing order is symmetric.  Using
$\mathcal{L}(a)=\frac{a(1-a)}{1+a}$, we can show that
$\log\mathcal{L}(\hat{a})=\log\left(3-2\sqrt{2}\right)\simeq -1.76$,
where $\hat{a}=\sqrt{2}-1$ is the maximum likelihood estimate for $a$.
Defining $\mathcal{S}=\log\mathcal{L}$ as the support [log-likelihood]
we have

\begin{equation}
\mathcal{S}=\mathcal{S}(a)=\log\left(\frac{a(1-a)}{1+a}\right)-\log\left(3-2\sqrt{2}\right)
\end{equation}

as a standard support function which has a maximum value of zero when
evaluated at $\hat{a}=\sqrt{2}-1$.  For example, we can test the null
that $a=b=\frac{1}{2}$, the statement that the competitors have equal
Bradley-Terry strengths:


```{r ahalf}
a <- 1/2 # null
(S_delta <- log(a * (1 - a)/(1 + a)) - log(3 - 2 * sqrt(2)))
```

Thus the additional support gained in moving from $a=\frac{1}{2}$ to
the evaluate of $a=\sqrt{2}-1$ is 0.029, rather small [as might be
expected given that we have only one rather uninformative observation,
and also given that the maximum likelihood estimate ($\simeq 0.41$) is
quite close to the null of $0.5$].  Nevertheless we can follow
[@edwards1972} and apply Wilks's theorem for a $p$ value:

```{r usechisq}
pchisq(-2 * S_delta, df = 1, lower.tail = FALSE)
```

The $p$-value is about 0.81, exceeding 0.05; thus we have no strong
evidence to reject the null of $a=\frac{1}{2}$.  The observation is
informative, in the sense that we can find a credible interval for
$a$.  With an $n$-units of support criterion the analytical solution
to $\mathcal{S}(p)=-n$ is given by defining $X=\log(3-2\sqrt{2})-n$
and solving $p(1-p)/(1+p)=X$, or
$p_\pm=\left(1-X\pm\sqrt{1+4X+X^2}\right)/2$, the two roots being the
lower and upper limits of the credible interval; see the figure below.


```{r, fig.width=7, fig.height=7, fig.cap="A support function for a>b>a"}
a <- seq(from = 0, by = 0.005, to = 1)
S <- function(a){log(a * (1 - a) / ((1 + a) * (3 - 2 * sqrt(2))))}
plot(a, S(a), type = 'b',xlab=expression(p[a]),ylab="support")
abline(h = c(0, -2))
abline(v = c(0.02438102, 0.9524271), col = 'red')
abline(v = sqrt(2) - 1)
```

## Fisher information

If we have two clones of $a$ and a singleton $b$, then there are three
possible rank statistics: (i), $a\succ a\succ b$ with probability
$\frac{2a^2}{1+a}$; (ii), $a\succ b\succ a$, with
$\frac{2a(1-a)}{(1+a)}$, (iii), $b\succ a\succ a$ at
$\frac{1-a}{1+a}$.  Likelihood functions for these order statistics
are given in the figure below.  It can be shown that the
Fisher information for such observations is

\begin{equation}
\mathcal{I}(a)=2\frac{1+a+a^2}{a(1-a)(1+a)^2}
\end{equation}

which has a minimum of about $6.21$ at at about $a=0.522$.  We can
compare this with the Fisher information matrix ${\mathcal I}$, for
the case of three distinct runners $a,b,c$, evaluated at its minimum
of $p_a=p_b=p_c=\frac{1}{3}$.  If we observe the complete order
statistic, $\left|{\mathcal I}\right| =\frac{1323}{16}\simeq 82.7$; if
we observe just the winner, $\left|{\mathcal I}\right|=27$, and if we
observe just the loser we have $\left|{\mathcal
I}\right|=\frac{16875}{256}\simeq 65.9$.  A brief discussion is given
at `inst/fisher_inf_PL3.Rmd`.

```{r, fig.width=7, fig.height=7, fig.cap="Support functions for observations a>a>b, a>b>a and b>a>a.  Horizontal dotted line represents two units of support"}
f_aab <- function(a){a^2 / (1 + a)}
f_aba <- function(a){a * (1 - a) / (1 + a)}
f_baa <- function(a){(1 - a) / (1 + a)}
p <- function(f, ...){
  a <- seq(from = 0, by = 0.005, to = 1)
  points(a, f(a) / max(f(a)), ...)
  }
plot(0:1, 0:1, xlab = expression(p[a]), ylab = "Likelihood", type = "n")
p(f_aab, type = "l", col = "black")
p(f_aba, type = "l", col = "red")
p(f_baa, type = "l", col = "blue")
text(0.8,0.8,"AAB")
text(0.8,0.4,"ABA",col="red")
text(0.8,0.05,"BAA",col="blue")
abline(h = exp(-2), lty = 2)
```

## Nonfinishers

If we allow non-finishers---that is, a subset of competitors who are
beaten by all the ranked competitors ([@turner2020] call this a _top
$n$ ranking_), there is another nontrivial order statistic, viz
$a\succ b\succ\left\lbrace a,b\right\rbrace$ [thus one of the two
$a$'s won, one of the $b$'s came second, and one of each of $a$ and
$b$ failed to finish].  Now

\begin{equation}
\mathcal{L}(a)=
\frac{a}{2a+2b}\cdot
\frac{b}{ a+2b}\propto\frac{a(1-a)}{2-a}
\end{equation}

(see how the likelihood function is actually simpler than for the
complete order statistic).  The evaluate would be $2-\sqrt{2}\simeq
0.586$:

```{r maxoab}
maxp(ordervec2supp3(c("a", "b"), nonfinishers=c("a", "b")))
```

The Fisher information for such observations has a minimum of
$\frac{68}{9}\simeq 7.56$ at $a=\frac{1}{2}$.  An inference problem
for a dataset including nonfinishers will be given below.

# An alternative to the Mann-Whitney test using generalized Plackett-Luce likelihood

The ideas presented above can easily be extended to arbitrarily large
numbers of competitors, although the analytical expressions tend to be
intractable and numerical methods must be used.  All results and
datasets presented here are maintained under version control and
available at `https://github.com/RobinHankin/hyper2`.  Given an
order statistic of the type considered above, the
Mann-Whitney-Wilcoxon test [@mann1947,wilcoxon1945] assesses a
null of identity of underlying distributions [@ahmad1996].
Consider the chorioamnion dataset [@hollander2013], used in
`wilcox.test.Rd`:

```{r define_xy_wilcox}
x <- c(0.80, 0.83, 1.89, 1.04, 1.45, 1.38, 1.91, 1.64, 0.73, 1.46)
y <- c(1.15, 0.88, 0.90, 0.74, 1.21)
```

Here we see a measure of permeability of the human placenta at term
(`x`) and between 3 and 6 months' gestational age (`y`).  The order
statistic is straightforward to calculate:

```{r hyper3osdef}
names(x) <- rep("x", length(x))
names(y) <- rep("y", length(y))
(os <- names(sort(c(x, y))))
```

Then object `os` is converted to a `hyper3` object, again
with `ordervec2supp3()`, which may be assessed using the Method
of Support:

```{r hyper3xytest}
Hxy <- ordervec2supp3(os)
equalp.test(Hxy)
```

Above, we use generic function `equalp.test()` to test the null
that the permeability of the two groups both have Bradley-Terry
strength of $0.5$.  We see a $p$ value of about 0.09; compare 0.25 from
`wilcox.test()`.  However, observe that the `hyper3`
likelihood approach gives more information than Wilcoxon's analysis:
Firstly, we see that the maximum likelihood estimate for the
Bradley-Terry strength of `x` is about 0.24, considerably less
than the null of 0.5; further, we may plot a support curve for this
dataset, given in the figure below.

```{r, fig.width=7, fig.height=7, fig.cap="A support function for the Bradley-Terry strength of permeability at term.  The evaluate of 0.24 is shown; and the two-units-of support credible interval, which does not exclude the null of strength 0.5 (dotted line), is also shown"}
a <- seq(from = 0.02, to = 0.8, len = 40)
L <- sapply(a, function(p){loglik(p, Hxy)})
plot(a, L - max(L), type = 'b',xlab=expression(p[a]),ylab="likelihood")
abline(h = c(0, -2))
abline(v = c(0.24))
abline(v=c(0.5), lty=2)
```

## A generalization of the Mann-Whitney test using generalized Plackett-Luce likelihood

The ideas presented above may be extended to more than two types of
competitors.  Consider the following table, drawn from the men's
javelin, 2020 Olympics:

```{r javtab}
javelin_table
```

Thus Chopra threw 87.03m on his first throw, 87.58m on his second, and
so on.  No-throws, ignored here, are indicated with an `X`.  We
may convert this to a named vector with elements being the throw
distances, and names being the competitors, using
`attemptstable2supp3()`:

```{r converttosupp3}
javelin_vector <- attemptstable2supp3(javelin_table,
       decreasing = TRUE, give.supp = FALSE)
options(width = 60)
javelin_vector
```

Above we see that Chopra threw the longest and second-longest throws
of 87.58m and 87.03 respectively; Vadlejch threw the third-longest
throw of 86.67m, and so on (`NA` entries correspond to
no-throws.)  The attempts table may be converted to a `hyper3`
object, again using function `attemptstable2supp3()` but this
time passing `give.supp=TRUE`:

```{r javorder}
javelin <- ordervec2supp3(v = names(javelin_vector)[!is.na(javelin_vector)])
```

Above, object `javelin` is a `hyper3` likelihood function, so one has
access to the standard likelihood-based methods, such as finding and
displaying the maximum likelihood estimate, shown in the figure below.


```{r setoptsdig3,echo=FALSE}
options(digits = 3)
```

```{r, fig.width=7, fig.height=7, fig.cap="Maximum  likelihood estimate for javelin throwers' Bradley-Terry strengths"}
(mj <- maxp(javelin))
dotchart(mj, pch = 16,xlab="Estimated Bradley-Terry strength")
```

From this, we see that Vadlejch has the highest estimated
Bradley-Terry strength, but further analysis with `equalp.test()`
reveals that there is no strong evidence in the dataset to reject the
hypothesis of equal competitive strength ($p=0.26$), or that Vadlejch
has a strength higher than the null value of $\frac{1}{8}$ ($p=0.1$).

A particularly attractive feature of this analysis is that the
Bradley-Terry strengths have direct operational significance: If two
competitors, say Vadlejch and Vesely, were to throw a javelin, then we
would estimate the probability that Vadlejch would throw further than
Vesely at $\displaystyle p_{\mbox{Vad}}/\left(p_{\mbox{Vad}} +
p_{\mbox{Ves}}\right)\simeq 0.74$.  Indeed, from a training or
selection perspective we might follow @hankin2017_rmd and observe that
log-contrasts [@ohagan2004] appear to have approximately Gaussian
likelihood functions for observations of the type considered here.
Profile log-likelihood curves for log-contrasts are easily produced by
the package, below.

```{r, echo=FALSE, fig.width=7, fig.height=7, fig.cap="Profile likelihood for log-contrast as per the text. Null indicated with a dotted line, and two-units-of-support limit indicated with horizontal lines; thus the null is not rejected"}
M <-  structure(c(3.13462808462437, -3.49547133821576, 2.31958387909791, 
 -1.453478109001, 1.85404664879129, -0.591727993171389, 1.47038547592281, 
 -0.172268427320532, 1.17564017651567, -0.00849666385090586, 0.927885079194978, 
 0, 0.706501034186756, -0.0998759637247275, 0.536848604955139, 
 -0.281432246390324, 0.33818759910768, -0.521602929477154, 0.167613563612845, 
 -0.816172716636203, 0.0827382775966942, -1.20676576256501, -0.138220714577699, 
 -1.5361019338357, -0.273432638006251, -1.95101772253715, -0.409370691327909, 
 -2.40043826949947, -0.560827492984493, -2.88462938724336, -0.666618197447362, 
 -3.39664587496794), dim = c(2L, 16L), dimnames = list(c("logcontrast", 
 "support"), NULL))
plot(t(M), type = "b")
abline(h = c(0, -2))
abline(v = 0, lty = 2)
abline(v = log(0.32062833 / 0.11402735)) # these from mp
```

We see that the credible range for $\log\left(p_{\mbox{Vad}}/
p_{\mbox{Ves}}\right)$ includes zero and we have no strong evidence
for these athletes having different (Bradley-Terry) strengths.


# Formula 1 motor racing: The Constructors' Championship

Formula 1 motor racing is an important and prestigious motor sport
[@codling2017,jenkins2010].  In Formula 1 Grand Prix, the
constructors' championship takes place between _manufacturers_ of
racing cars (compare the drivers' championship, which is between
drivers).  In this analysis, the constructor is the object of
inference.  Each constructor typically fields two cars, each of which
separately accumulates ranking-based points at each venue.  Here we
use a generalized Plackett-Luce model to assess the constructors'
performance.  The following table, included in the `hyper2`
package as a dataset, shows rankings for the first 9 venues of the
2021 season:

```{r cons2021}
constructor_2021_table[, 1:9]
```

Above, we see that Mercedes ("`Merc`") came first and third at Bahrain
(`BHR`); and came second and retired at Emilia Romagna (`EMI`); full
details of the notation and conventions are given in the package at
`constructor.Rd`.  The identity of the driver is viewed as
inadmissible information and indeed may change during a season.
Alternatively, we may regard the driver and the constructor as a joint
entity, with the constructor's ability to attract and retain a skilled
driver being part of the object of inference.  The associated
generalized Plackett-Luce `hyper3` object is easily constructed using
package idiom, in this case `ordertable2supp3()`, and we may use this
to assess the Plackett-Luce strengths of the constructors:

```{r cheatmaxp2, echo=TRUE, eval=FALSE}
const2020 <- ordertable2supp3(constructor_2020_table)
const2021 <- ordertable2supp3(constructor_2021_table)
options(digits=4)
```

```
maxp(const2020)
##    ARRF     ATH Ferrari      HF    Merc      MR       R 
## 0.04530 0.06807 0.06063 0.02623 0.37783 0.10026 0.09767 
##    RBRH  RPBWTM      WM 
## 0.12072 0.08055 0.02273
```

```
maxp(const2021)
##     AMM      AR    ARRF     ATH Ferrari      HF    Merc 
## 0.05942 0.07543 0.06238 0.05611 0.16939 0.02023 0.19395 
##      MM    RBRH      WM 
## 0.14126 0.18334 0.03848
```


Above, we see the strength of Mercedes falling from about 0.38 in 2020
to less than 0.20 in 2021 and it is natural to wonder whether this can
be ascribed to random variation.  Observe that testing such a
hypothesis is complicated by the fact that constructors field multiple
cars, and also that constructors come and go, with two 2020 teams
dropping out between years and two joining.  We may test this
statistically by defining a combined likelihood function for both
years, keeping track of the year:

```{r defcomblike}
H <- (
      psubs(constructor_2020, "Merc", "Merc2020") +
      psubs(constructor_2021, "Merc", "Merc2021")
     )
```

Above, we use generic function `psubs()` to change the name of
Mercedes from `Merc` to `Merc2020` and `Merc2021`
respectively.  Note the use of `+` to represent addition of
log-likelihoods, corresponding to the assumption of conditional
independence of results.  The null would be simply that the strengths
of `Merc2020` and of `Merc2021` are identical.  Package
idiom would be to use generic function `samep.test()`:

```{r usesameptest,eval=FALSE}
options(digits = 4)
samep.test(H, c("Merc2020", "Merc2021"))
```

```
## 
##  Constrained support maximization
## 
## data:  H
## null hypothesis: Merc2020 = Merc2021
## null estimate:
##      AMM       AR     ARRF      ATH  Ferrari       HF 
##  0.04239  0.05413  0.04677  0.04374  0.07568  0.02323 
## Merc2020 Merc2021       MM       MR        R     RBRH 
##  0.13903  0.13903  0.09016  0.07944  0.07475  0.10024 
##   RPBWTM       WM 
##  0.06235  0.02905 
## (argmax, constrained optimization)
## Support for null:  -1189 + K
## 
## alternative hypothesis:  sum p_i=1 
## alternative estimate:
##      AMM       AR     ARRF      ATH  Ferrari       HF 
##  0.03766  0.04824  0.04333  0.04060  0.07036  0.02132 
## Merc2020 Merc2021       MM       MR        R     RBRH 
##  0.23135  0.09216  0.07893  0.07973  0.07455  0.09322 
##   RPBWTM       WM 
##  0.06177  0.02679 
## (argmax, free optimization)
## Support for alternative:  -1184 + K
## 
## degrees of freedom: 1
## support difference = 4.722
## p-value: 0.002119
```

Above, we see strong evidence for a real decrease in the strength of
the Mercedes team from 2020 to 2021, with $p=0.002$.

# Conclusions and further work

Plackett-Luce likelihood functions for rank datasets have been
generalized to impose identity of Bradley-Terry strengths for certain
groups; the preferred interpretation is a running race in which the
competitors are split into equivalence classes of clones.
Implementing this in `R` is accomplished via a `C++` back-end making
use of the `STL` "map" class which offers efficiency advantages,
especially for large objects.

New likelihood functions for simple cases with three or four
competitors were presented, and extending to larger numbers furnishes
a generalization of the Mann-Whitney-Wilcoxon test that offers a
specific alternative (Bradley-Terry strength) with a clear operational
definition.  Further generalizations allow the analysis of more than
two groups, here applied to Olympic javelin throw distances.
Generalized Plackett-Luce likelihood functions were used to assess the
Grand Prix constructors' championship and a reasonable null.
Specifically, the hypothesis that the strength of the Mercedes team
remained unchanged between 2020 and 2021 was tested and rejected.

Draws are not considered in the present work but in principle may be
accommodated, either using likelihoods comprising sums of
Plackett-Luce probabilities [@hankin2017_rmd]; or the introduction of
a reified draw entity [@hankin2010].

Further work might include a systematic comparison between
`hyper3` approach and the Mann-Whitney-Wilcoxon test, including
the characterisation of the power function of both tests.  The package
could easily be extended to allow non-integer multiplicities, which
might prove useful in the context of reified Bradley Terry
techniques [@hankin2020].


# References

---
title: "The zero-sum-power issue in the `hyper2` package"
author: "Robin K. S. Hankin"
output: html_vignette
bibliography: hyper2.bib
link-citations: true
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{zeropower}
  %\usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
set.seed(0)
library("hyper2")
library("magrittr")
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)
knit_print.function <- function(x, ...){dput(x)}
registerS3method(
  "knit_print", "function", knit_print.function,
  envir = asNamespace("knitr")
)
```

```{r out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE}
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))
```

To cite the `hyper2` package in publications, please use
@hankin2017_rmd.

The powers of a `hyper2` object generally sum to zero.  This is
implicit in Bradley-Terry type likelihood functions in which
probabilities are generally ratios of linear combinations of
strengths.  Here I discuss this observation in the context of `hyper2`
package idiom.  Usually, powers not summing to zero is an indication
of a programming bug.  For example, consider the following likelihood
function, drawn from the `chess` dataset:

\[
\frac{p_1^{30}p_2^{36}p_3^{22}}{
  \left(p_1+p_2\right)^{35}
  \left(p_2+p_3\right)^{35}
  \left(p_1+p_3\right)^{18}
  }
\]


For a time, one package documentation file contained the following
`hyper2` idiom purporting to create the `chess` likelihood function:

```{r appidiom}
chess <- hyper2()
chess["Topalov"] <- 30
chess["Anand"  ] <- 36
chess["Karpov" ] <- 22
chess[c("Topalov","Anand" )] <- 35  # bug!  should  be -35
chess[c("Anand","Karpov"  )] <- 35  # bug!  should  be -35
chess[c("Karpov","Topalov")] <- 18  # bug!  should  be -18
```

However, the above commands include an error [the sign of the
denominator is incorrect].  The package includes two mechanisms to
detect this type of error.  Firstly, the print method (by default)
detects such unbalanced likelihood functions and gives a warning:

```{r showwarningunbalanced}
chess
```

and secondly, function `loglik()` traps nonzero power sums:

```
loglik(equalp(chess),chess)
```

```
## Error in loglik_single(p, H, log = log): sum(powers(H)) == 0 is not TRUE
```

The correct idiom would be

```{r correctid}
chess[c("Topalov","Anand" )] <- -35
chess[c("Anand","Karpov"  )] <- -35
chess[c("Karpov","Topalov")] <- -18
chess
loglik(equalp(chess),chess)
```


See how the print method gives immediate assurance that its argument
is indeed balanced (and besides, unbalanced likelihood functions
cannot even be evaluated with `loglik()`).  It is natural to suggest
including a check in the creation method---`hyper2()`---which would
prevent the creation of a hyper2 object with unbalanced powers.
However, this approach is not consistent with the package; consider
the following situation.  Suppose we wish to incorporate a new
observation into `chess`, specifically that Anand played Karpov once,
with Anand winning.  We might proceed as follows:

```{r, newinfo}
chess["Anand"] %<>% inc
chess[c("Anand, Karpov")] %<>% dec(2)
chess
```

However, after the increment, but before the decrement,
`chess` has a nonzero power sum, pending addition of another term.  At
this point, `chess` is unbalanced; its nonzero power sum is an
indicator that it is a temporary object.  That's OK as long as we
remember to add the denominator (as carried out in the next line)
which would mean dividing by `(Anand+Karpov)^2`, thereby restoring the
zero power sum.  If we forget to do this, the print method gives us a
warning, and indeed `loglik()` returns an error, which should prompt
us to check the coding.

## The print method

The `hyper2` print method is sensitive to option
`give_warning_on_nonzero_power_sum`.  If `TRUE` (the default), a
warning is issued if the powers have nonzero sum.  This is usually
appropriate.  If the option is `FALSE`, the warning is suppressed.
Note that the intermediate likelihood functions in the `chess` example
are not printed (or indeed evaluated), so unbalanced likelihood
functions are permitted, but only ephemerally.

## Function `balance()`

Sometimes it is convenient to accommodate the numerator terms all
together and enforce the zero power sum at the end.  This is
accomplished by `balance()`:

```{r}
H <- hyper2()
H['a'] %<>% inc(5)
H['b'] %<>% inc(2)
H['c'] %<>% inc(9)
H %<>% balance
H
```

## References

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{The hyper2 package}
%\VignetteDepends{hyper2}
%\VignetteKeywords{hyper2,dirichlet,Bradley-Terry, reified Bradley=Terry}
%\VignettePackage{hyper2}


\documentclass[nojss]{jss}
\usepackage{dsfont}
\usepackage{bbm}
\usepackage{amsfonts}
\usepackage{amsmath}
\usepackage{wasysym}
\usepackage{amssymb}
\usepackage{yfonts}
\usepackage{wrapfig}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% declarations for jss.cls %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% just as usual
\author{Robin K. S. Hankin\\University of Stirling}
\title{Partial rank data with the \pkg{hyper2} package: likelihood
  functions for generalized Bradley-Terry models}



%% for pretty printing and a nice hypersummary also set:
\Plainauthor{Robin K. S. Hankin}
\Plaintitle{A Generalization of the Dirichlet Distribution}
\Shorttitle{A Generalization of the Dirichlet Distribution}

%% an abstract and keywords
\Abstract{

Here I present the \pkg{hyper2} package for generalized Bradley-Terry
models and analyze two situations: single scull rowing, and the
competitive cooking game show MasterChef Australia.  A number of
natural statistical hypotheses may be tested straightforwardly using
the software.  To cite the package in publications, use
\citet{hankin2017_rnw}, on which this work is based.
}


\Keywords{Dirichlet distribution, hyperdirichlet distribution,
  combinatorics, \proglang{R}, multinomial distribution, constrained
optimization, Bradley-Terry}
\Plainkeywords{Dirichlet distribution, hyperdirichlet distribution,
  combinatorics, R, multinomial distribution, constrained
optimization, Bradley-Terry}
  
  
  
%% publication information
%% NOTE: This needs to filled out ONLY IF THE PAPER WAS ACCEPTED.
%% If it was not (yet) accepted, leave them commented.
%% \Volume{13}
%% \Issue{9}
%% \Month{September}
%% \Year{2004}
%% \Submitdate{2004-09-29}
%% \Acceptdate{2004-09-29}

%% The address of (at least) one author should be given
%% in the following format:
\Address{
  Robin K. S. Hankin\\
  University of Stirling\\
  E-mail: \email{hankin.robin@gmail.com}\\
}
%% It is also possible to add a telephone and fax number
%% before the e-mail in the following format:
%% Telephone: +43/1/31336-5053
%% Fax: +43/1/31336-734

%% for those who use Sweave please include the following line (with % symbols):

%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\newcommand{\bolda}{\mathbf a}
\newcommand{\boldp}{\mathbf p}
\newcommand{\boldV}{\mathbf V}
\newcommand{\boldalpha}{\mbox{\boldmath$\alpha$}}
\newcommand{\boldzero}{\mathbf 0}

\newcommand{\mcf}{\mathcal F}
\newcommand{\mcm}{\mathcal M}
\newcommand{\mcs}{\mathcal S}
\newcommand{\mcw}{\mathcal W}

\newcommand{\pG}{p_{\mbox{\frakfamily g}}}

\SweaveOpts{}
\begin{document}
\SweaveOpts{concordance=TRUE}



\section{Introduction}

<<set_seed_chunk,echo=FALSE,print=FALSE>>=
library("hyper2")
set.seed(0)
@ 

<<time_saver,echo=FALSE,print=FALSE>>=
calc_from_scratch <- FALSE
@ 

<<echo=FALSE,print=FALSE>>=
ignore <- require(magrittr,quietly=TRUE)
@ 

\section{Introduction: the Bradley-Terry model}

\setlength{\intextsep}{0pt}
\begin{wrapfigure}{r}{0.2\textwidth}
\begin{center}
\includegraphics[width=1in]{\Sexpr{system.file("help/figures/hyper2.png",package="hyper2")}}
\end{center}
\end{wrapfigure}

The Bradley-Terry model for datasets involving paired comparisons has
wide uptake in the {R} community.  However, existing
functionality\footnote{In theory, the \pkg{hyperdirichlet}
package~\citep{hankin2010} provides similar functionality but it is
slow and inefficient.  It is limited to a small number of players and
cannot cope with the examples considered here.  Also, the name-based
system of \pkg{hyper2} is considerably more powerful and natural than
the number-based system of \pkg{hyperdirichlet}.} is restricted to
paired comparisons.  The canonical problem is to consider~$n$ players
who compete against one another; the basic inference problem is to
estimate numbers~$\mathbf{p}=\left({p_1,\ldots,p_n}\right)$,
$p_i\geqslant 0$, $\sum p_i=1$ which correspond to player
``strengths''.  Information about the~$p_i$ may be obtained from the
results of paired comparisons between the players.

Applications are legion.  The technique is widely used in a
competitive sport context~\citep{turner2012}, in which matches are
held between two opposing individuals or teams.  It can also be
applied to consumer choice experiments in which subjects are asked to
choose a favourite from among two choices~\citep{hatzinger2012}, in
which case the~$p_i$ are known as ``worth parameters''.

If player~$i$ competes against player~$j$, and wins with
probability~$P_{ij}$ then the likelihood function for~$p_1,\ldots p_n$
corresponding to a win for~$i$ is~$\frac{p_i}{p_i+p_j}$.
As~\cite{turner2012} point out, this may be expressed as

\[
\operatorname{logit}\left(P_{ij}\right)=\log p_i-\log p_j
\]

\noindent
and this fact may be used to estimate~$\mathbf{p}$ using generalized
linear models.  However, consider the case where three competitors,
$i$, $j$, and~$k$ compete.  The probability that~$i$ wins is
then~$\frac{p_i}{p_i+p_j+p_k}$~\citep{luce1959}; but there is no
simple way to translate this likelihood function into a GLM.  However,
working directly with the likelihood function for~$\mathbf{p}$ has
several advantages which are illustrated below.  The resulting
likelihood functions may readily be generalized to accommodate more
general order statistics, as in a race.  In addition, likelihood
functions may be specified for partial order statistics; also,
observations in which a \emph{loser} is identified may be given a
likelihood function using natural {R} idiom.

\subsection{Further generalizations}

Observing the winner~$w$ from a preselected set of
competitors~$\mathcal{C}$ has a likelihood function
of~$p_w/\sum_{i\in\mathcal{C}} p_i$.  But consider a more general
situation in which two disjoint teams $\mathcal{A}$ and~$\mathcal{B}$
compete; this would have likelihood~$\sum_{i\in\mathcal{A}}
p_i/\sum_{i\in\mathcal{A}\cup\mathcal{B}} p_i$.  Such datasets
motivate consideration of likelihood
functions~$\mathcal{L}\left(\cdot\right)$ with
\begin{equation}\label{hyper2likelihood}
\mathcal{L}\left(\mathbf{p}\right)=
\prod_{s\in \mathcal{O}}\left({\sum_{i\in s}}p_i\right)^{n_s}
\end{equation}

\noindent
where~$\mathcal{O}$ is a set of observations and~$s$ a subset
of~$\left[n\right]=\left\{1,2,\ldots,n\right\}$; numbers~$n_s$ are
integers which may be positive or negative.  The approach adopted by
the~\pkg{hyperdirichlet} package is to store each of the~$2^n$
possible subsets of~$\left[n\right]$ together with an exponent:

\begin{equation}\label{hyperdirichletlikelihood}
\prod_{s\in 2^{\left[n\right]}}\left({\sum_{i\in s}}p_i\right)^{n_s}.
\end{equation}

\noindent
but this was noted as being needlessly memory intensive and slow; it
is limited, in practice, to~$n\leqslant 9$.  Use of the more efficient
equation~\ref{hyper2likelihood} necessitates some mechansim of keeping
track of which subsets of~$\left[n\right]$ have nonzero powers.

\section{The {hyper2} package}

One such mechanism is furnished by the \code{C++} Standard Template
Library's ``map'' class \citep{musser2009} to store and retrieve
elements.  A \emph{map} is an associative container that stores values
indexed by a key, which is used to sort and uniquely identify the
values.  In the package, the key is a (STL) \code{set} of strictly
positive integers~$\leqslant n$.  The relevant \code{typedef}
statements are:

\begin{verbatim}
typedef set<unsigned int> bracket;
typedef map<bracket, double> hyper2;
\end{verbatim}


\noindent In the \code{STL}, a map object stores keys and associated
values in whatever order the software considers to be most propitious.
This allows faster access and modification times but the order in
which the maps, and indeed the elements of a set, are stored is not
defined.  In the case of likelihood functions such as
Equation~\ref{hyper2likelihood}, this does not affect the value of the
expressions, as multiplication and addition are associative and
commutative operations.  The \pkg{hyper2} package follows
\code{disordR} discipline \citep{hankin2022_disordR}.

\section{The package in use}

\begin{table}
\centering
\begin{tabular}{|ccc|c|}\hline
Topalov  & Anand & Karpov & total\\ \hline
22 & 13 & -  & 35\\ 
-  & 23 & 12 & 35\\ 
8  &  -  & 10 & 18 \\ \hline
30 & 36 & 22 & 88 \\ \hline
\end{tabular}
\caption{Results of 88 chess matches \label{chess} (dataset
  \code{chess} in the \pkg{aylmer} package) between three
  Grandmasters; entries show number of games won up to 2001 (draws are
  discarded).  Topalov beats Anand 22-13; Anand beats Karpov 23-12;
  and Karpov beats Topalov 10-8}
\end{table}

Consider the \code{Chess} dataset of the \pkg{hyperdirchlet} package,
in which matches between three chess players are tabulated
(table~\ref{chess}).  The Bradley-Terry model~\citep{bradley1952} is
appropriate here~\citep{caron2012}, and the \pkg{hyper2} package
provides a likelihood function for the strengths of the players,
$p_1,p_2,p_3$ with~$p_1+p_2+p_3=1$.  A likelihood function might be

\[
\frac{p_1^{30}p_2^{36}p_3^{22}}{
  \left(p_1+p_2\right)^{35}
  \left(p_2+p_3\right)^{35}
  \left(p_1+p_3\right)^{18}
  }
\]

Using the \pkg{hyper2} package, the \proglang{R} idiom to create this
likelihood function is as follows:


<<chess_setup>>=
chess <- hyper2()
chess["Topalov"] <- 30
chess["Anand"  ] <- 36
chess["Karpov" ] <- 22
chess[c("Topalov","Anand" )] <- -35
chess[c("Anand","Karpov"  )] <- -35
chess[c("Karpov","Topalov")] <- -18
chess
@ 

The internal representation of \code{hyper2} objects is that of a
\proglang{map}, an associative array implemented as part of the
standard template library (\proglang{STL}) of \proglang{C++}.  It is
thus an unordered collection of (\emph{key}, \emph{value}) pairs.  In
this case the \emph{value} is the power and the \emph{key} is the
content of the bracket, which is a subset of
$\left\lbrace\mbox{Topalov},\mbox{Anand},\mbox{Karpov}\right\rbrace$.
The \proglang{stl} provides a \proglang{set} class for efficient
manipulation of such objects, and this is used in the package.

One side-effect of using this system is that the order of the
bracket-power key-value pairs is not preserved; above, note how the
terms appear in an essentially random order.  Also note that the use
of the \proglang{set} class means that the \proglang{R} idiom is
insensitive to the order of the terms within a bracket:

<<use_chess>>=
chess[c("Karpov","Topalov")] 
chess[c("Topalov","Karpov")] 

@ 

The package can calculate log-likelihoods:

<<loglikelihoodchess>>=
loglik(c(1/3,1/3),chess)
@ 

[the first argument of function \code{loglik()} is a vector of length
  2, third element of $\boldp$ being the ``fillup''
  value~\citep{aitchison1986}]; the gradient of the log-likelihood is
given by function \code{gradient()}:

<<gradientloglikelihoodchess>>=
gradient(chess,c(1/3,1/3))
@ 

Such functionality allows the rapid location of the maximum likelihood
estimate for~$\boldp$:

<<maxlikechess>>=
maxp(chess)
@ 

\section{Men's single sculling in the 2016 Summer Olympic Games}%'

In this section, I will take results from the 2016 Summer Olympic
Games and create a likelihood function for the finishing order in
Men's single sculling.  In Olympic sculling, up to six individual
competitors race a small boat called a scull over a course of length
2\,km; the object is to cross the finishing line first.  Note that
actual timings are irrelevant, given the model, as the sufficient
statistic is the order in which competitors cross the finishing line.
The 2016 Summer Olympics is a case in point: the gold and silver
medallists finished less than~5 milliseconds apart, corresponding to a
lead of~$\sim 2.5\,\mathrm{cm}$.

We may use a generalization of the Bradley-Terry model due
to~\citet{luce1959}, in which the probability of competitor~$i$
winning in a field of~$j=1,\ldots, n$ is

\[
\frac{p_i}{p_1+\cdots +p_n}.\]

However, there is information in the whole of the finishing order, not
just the first across the line.  Once the winner has been identified,
then the runner-up may plausibly be considered to be the winner among
the remaining competitors; and so on down the finishing order.
Without loss of generality, if the order of finishing
were~$1,2,3,4,5,6$, then a suitable likelihood function would be,
following~\cite{plackett1975}:

\begin{equation}\label{competitors_1_to_6_likelihood}
\frac{p_1}{p_1+p_2+p_3+p_4+p_5+p_6}\cdot
\frac{p_2}{p_2+p_3+p_4+p_5+p_6}\cdot
\frac{p_3}{p_3+p_4+p_5+p_6}\cdot
\frac{p_4}{p_4+p_5+p_6}\cdot
\frac{p_5}{p_5+p_6}\cdot
\frac{p_6}{p_6}
\end{equation}


We may represent the result of
Heat~1 as
\[
\mathrm{fournier}\succ
\mathrm{cabrera}\succ
\mathrm{bhokanal}\succ
\mathrm{saensuk}\succ
\mathrm{kelmelis}\succ
\mathrm{teilemb}
\]

(a full list of the finishing order for all~25 events is given in the
package as \code{rowing.txt}).  Incorporating the information from
Heat~1 into a likelihood function corresponding to
equation~\ref{competitors_1_to_6_likelihood} is straightforward using
the \code{rankvec\_likelihood()} function:

<<2016_olympics_heat1_first>>=
heat1 <- c("fournier", "cabrera", "bhokanal", "saensuk", "kelmelis", "teilemb")
H <- rankvec_likelihood(heat1)
H
@ 

(variable \code{heat1} shows the finishing order for Heat~1: Fournier
came first, then Cabrera, etc).  We can add the information from
heat~2 easily:


<<2016_olympics_heat1_first>>=
heat2 <- c("drysdale", "molnar", "esquivel", "garcia", "khafaji", "monasterio")
H <- H + rankvec_likelihood(heat2)
head(H)
@ 

Again observe that object \code{H} includes its terms in no apparent
order.  Although it would be possible to incorporate information from
subsequent heats in a similar manner, the package includes a
ready-made dataset, \code{rowing}:

<<rowers>>=
head(rowing)   # see rowing.Rd
@ 
 
Finding the maximum likelihood estimate for the
parameter~$p_\mathrm{banna},\ldots,p_\mathrm{zambrano}$ is
straightforward using the \code{maxp()} function, provided with the
package~(Figure~\ref{maxp_sculls2016}).  The optimization routine has
access to derivatives which means that the estimate is found very
quickly.


\begin{figure}[htbp]
\begin{center}
<<rowing_maxp,echo=TRUE,fig=TRUE>>=
dotchart(rowing_maxp)
@
\caption{Maximum likelihood estimate for the strengths of the~32
  competitors\label{maxp_sculls2016} in the Men's singles sculls in
  the 2016 Summer Olympics
}
  \end{center}
\end{figure}

Figure~\ref{maxp_sculls2016} shows very directly that the competitor
with the highest strength is Drysdale, the gold medallist for this
event.  The bronze and silver medallists were Synek and Martin
respectively, whose estimated strengths were second and third highest
in the field.

 
\section{MasterChef Australia}

MasterChef Australia is a game show in which amateur cooks compete for
a title~\citep{wikipedia_masterchef2017}.  From a statistical
perspective the contest is interesting because the typical show format
is to identify the \emph{weakest} player, who is then eliminated from
the competition.  Here, results from MasterChef Australia
Series~6~\citep{wikipedia_masterchef_australia_series6} will be
analysed; an extended discussion of the data used is given in the
package at~\code{masterchef.Rd}.

We wish to make inferences about the contestants' generalized
Bradley-Terry strengths~$p_1,\ldots,p_n$, $\sum p_i=1$.  One
informative event was a team challenge in which the contestants were
split randomly into two teams, red and blue:
  
<<setupteams>>=
team_red <- c("Jamie","Tracy","Ben","Amy","Renae","Georgia")
team_blue <- c("Brent","Laura","Emelia","Colin","Kira","Tash")
@ 
 
We may represent the fact that the red team won as 
\begin{equation}\label{redteamwon}
\left\{
\mbox{Jamie}+\mbox{Tracy}+\mbox{Ben}+
\mbox{Amy}+\mbox{Renae}+\mbox{Georgia}
\right\}
\succ
\left\{
\mbox{Brent}+\mbox{Laura}+\mbox{Emelia}+
\mbox{Colin}+\mbox{Kira}+\mbox{Tash}
\right\}
\end{equation}

The likelihood function for the observation given in
equation~\ref{redteamwon} would be

\begin{equation}\label{redbluelikelihood}
\frac{p_\mathrm{Jamie}+p_\mathrm{Tracy}+p_\mathrm{Ben}+p_\mathrm{Amy}+p_\mathrm{Renae}+p_\mathrm{Georgia}}{
p_\mathrm{Jamie}+p_\mathrm{Tracy}+p_\mathrm{Ben}+p_\mathrm{Amy}+p_\mathrm{Renae}+p_\mathrm{Georgia}+p_\mathrm{Brent}+p_\mathrm{Laura}+p_\mathrm{Emelia}+p_\mathrm{Colin}+p_\mathrm{Kira}+p_\mathrm{Tash}}
\end{equation}
  
To generate a likelihood function in \proglang{R}, we need to set up a
\code{hyper2} object with appropriate contestants:

<<masterchef_example>>=
H <- hyper2(pnames = c(
                "Amy", "Ben", "Brent", "Colin", "Emelia",
                "Georgia", "Jamie", "Kira", "Laura", "Renae",
                "Sarah", "Tash", "Tracy"))
H
@ 

Object \code{H} is a uniform likelihood function.  The package \proglang{R} idiom
for incorporating likelihood from Equation~\ref{redbluelikelihood} is
straightforward and natural:
<<redteamwins>>=
H[team_red] <- +1
H[c(team_red,team_blue)] <- -1
H
@ 

(Sarah did not take part).  The above idiom makes it possible to
define likelihood functions for observations that have a peculiar
probability structure, and I give two examples below.

One event involved eight competitors who were split randomly into four
teams of two.  The show format was specified in advance as follows:
The teams were to be judged, and placed in order.  The two top teams
were to be declared safe, and the other two teams sent to an
elimination trial from which an individual winner and loser were
identified, the loser being obliged to leave the competition.  The
format for this event is also typical in MasterChef.

The observation was that Laura and Jamie's team won, followed by
Emelia and Amy, then Brent and Tracy.  Ben and Renae's team came last:

\begin{equation}
\left\{{\mathrm{Laura}  + \mathrm{Jamie}}\right\}\succ
\left\{{\mathrm{Emelia} + \mathrm{Amy}  }\right\}\succ
\left\{{\mathrm{Brent}   + \mathrm{Tracy}}\right\}\succ
\left\{{\mathrm{Ben}    + \mathrm{Renae}}\right\}
\end{equation}

A plausible likelihood function can be generated using the standard
assumption~\citep{hankin2010} that the competitive strength of a team
is the sum of the strengths of its members.  We can then combine this
assumption with the order statistic technique of~\cite{plackett1975}:

\begin{align}
\frac{p_\mathrm{Laura}+p_\mathrm{Jamie}
}{
  p_\mathrm{Laura}  + p_\mathrm{Jamie} + 
  p_\mathrm{Emelia} + p_\mathrm{Amy}   +
  p_\mathrm{Brent}  + p_\mathrm{Tracy} +
  p_\mathrm{Ben}    + p_\mathrm{Renae}
  }\cdot\nonumber\\
\frac{
  p_\mathrm{Emelia} + p_\mathrm{Amy} 
}{
  p_\mathrm{Emelia} + p_\mathrm{Amy}   +
  p_\mathrm{Brent}  + p_\mathrm{Tracy} +
  p_\mathrm{Ben}    + p_\mathrm{Renae}
  }\cdot
\frac{
  p_\mathrm{Brent}  + p_\mathrm{Tracy}
}{  
  p_\mathrm{Brent}  + p_\mathrm{Tracy} +
  p_\mathrm{Ben}    + p_\mathrm{Renae}
  }
\end{align}

We would like to incorporate information from this observation into
object~\code{H}, which is a likelihood function for the two-team
challenge discussed above.  The corresponding \proglang{R} idiom is
natural:

<<teamchallenge>>=
blue   <- c("Laura","Jamie")   # first
yellow <- c("Emelia","Amy")    # second
green  <- c("Brent","Tracy")   # third
red    <- c("Ben","Renae")     # fourth

H[blue] <- 1
H[c(blue,yellow,green,red)] <- -1
H[yellow] <- 1
H[c(yellow,green,red)] <- -1
H[green] <- 1
H[c(green,red)] <- -1
H
@

We may incorporate subsequent observations relating to the elimination
trial among the four competitors comprising the losing two teams.  The
observation was that Laura won, and Renae came last, being eliminated.
We might write

\begin{equation}\label{renae_eliminated}
\left\{
\mbox{Laura}
\right\}
\succ
\left\{{\mbox{Brent},\mbox{Tracey},\mbox{Ben}}\right\}
\succ
\left\{\mbox{Renae}\right\},
\end{equation}

which indicates that Laura came first, then Brent/Tracey/Ben in some
order, then Renae came last.  For this observation a likelihood
function, following~\cite{plackett1975}, might be

\begin{align}
  \mathcal{L}\left({p_1,p_2,p_3,p_4,p_5}\right) &=
  \Prob\left({
    p_1\succ p_2\succ p_3\succ p_4\succ p_5
    \cup
    p_1\succ p_2\succ p_4\succ p_3\succ p_5
    \cup\ldots
  }\right)\\ \label{ggrlmath}
  &=\Prob\left({\bigcup_{\left[{abc}\right]} p_1\succ p_a\succ p_b\succ p_c\succ p_5
  }\right)\\ \nonumber
  &=    \frac{p_1}{p_1+p_2+p_3+p_4+p_5}\cdot\frac{p_2}{p_2+p_3+p_4+p_5}\cdot\frac{p_3}{p_3+p_4+p_5}\cdot\frac{p_4}{p_4+p_5}\\ \nonumber
  &{\qquad+}\frac{p_1}{p_1+p_2+p_4+p_3+p_5}\cdot\frac{p_2}{p_2+p_4+p_3+p_5}\cdot\frac{p_4}{p_4+p_3+p_5}\cdot\frac{p_3}{p_3+p_5}\\ \nonumber
  &{\qquad{}+}\frac{p_1}{p_1+p_3+p_2+p_4+p_5}\cdot\frac{p_3}{p_3+p_2+p_4+p_5}\cdot\frac{p_2}{p_2+p_4+p_5}\cdot\frac{p_4}{p_4+p_5}\\ \nonumber
  &{\qquad{}+}\cdots
\end{align}

where Laura's strength is shown as~$p_1$ etc for brevity.  The
\proglang{R} idiom is as follows:

<<>>=
L <- ggrl(H, 
          winner     = "Laura",
          btm4       = c("Brent", "Tracy","Ben"),
          eliminated = "Renae")
@ 

Arguments to \code{ggrl()} are disjoint subsets of the players, the
subsets themselves being passed in competition order from best to
worst.  Object \code{L} includes information from the team challenge
(via first argument \code{H}) and the elimination results.  It is a
list of length~$3!=6$ objects of class \code{hyper2}, each of which
gives a \citeauthor{luce1959} likelihood function for a consistent
total ordering of the competitors.

A final example (taken from MasterChef series 8, week 10) is given as
a generalization of the \citeauthor{luce1959} likelihood.  The format
was as follows.  Eight contestants were split randomly into four teams
of two, the top two teams being declared safe.  Note that the
likelihood interpretation differs from the previous team challenge, in
which the observation was an unambiguous team ranking: here, there is
only a partial ranking of the teams and one might expect this
observation to be less informative.  Without loss of generality, the
result may be represented as

\begin{equation}
  \left\{{p_1+p_2,p_3+p_4}\right\}\succ
  \left\{{p_5+p_6,p_7+p_8}\right\}
  \end{equation}
      
and a likelihood function on~$p_1,\ldots p_8$ for this observation
might be

\begin{align}
  \mathcal{L}\left( {p_1,\ldots, p_8} \right) &=
  \Prob\left({
    \left\{ {p_1+p_2} \right\}\succ 
    \left\{ {p_3+p_4} \right\}\succ 
    \left\{ {p_5+p_6} \right\}\succ 
    \left\{ {p_7+p_8} \right\} \cup\vphantom{\bigcup}
  }\right. \nonumber\\
  &\qquad \left.{
    \left\{ {p_1+p_2} \right\}\succ 
    \left\{ {p_3+p_4} \right\}\succ 
    \left\{ {p_7+p_8} \right\}\succ 
    \left\{ {p_5+p_6} \right\}\cup
  }\right.\nonumber\\
  &\qquad \left.{
    \left\{ {p_3+p_4} \right\}\succ 
    \left\{ {p_1+p_2} \right\}\succ 
    \left\{ {p_5+p_6} \right\}\succ 
    \left\{ {p_7+p_8} \right\}\cup
  }\right.\nonumber\\
  &\qquad \left.{
    \left\{ {p_3+p_4} \right\}\succ 
    \left\{ {p_1+p_2} \right\}\succ 
    \left\{ {p_5+p_6} \right\}\succ 
    \left\{ {p_7+p_8} \right\}\vphantom{\bigcup}
  }\right)\\
&=\frac{p_1+p_2}{p_1+p_2+p_3+p_4+p_5+p_6+p_7+p_8}\cdot
  \frac{p_3+p_4}{p_3+p_4+p_5+p_6+p_7+p_8}\cdot\frac{p_5+p_6}{p_5+p_6+p_7+p_8}\nonumber\\
&{}\qquad+\cdots+\nonumber\\
&{}\frac{p_3+p_4}{p_3+p_4+p_1+p_2+p_7+p_8+p_5+p_6}\cdot
  \frac{p_1+p_2}{p_3+p_4+p_7+p_8+p_5+p_6}\cdot\frac{p_7+p_8}{p_7+p_8+p_5+p_6}\nonumber\\
\nonumber
\end{align}

% \end{align}

\subsection{Maximum likelihood estimation}


The package provides an overall likelihood function for all
informative judgements in the series on the final 13 players in object
\code{masterchef\_series6}, documented at \code{masterchef.Rd}.  We
may assess a number of related hypotheses using the package.  The
first step is to calculate the likelihood for the hypothesis that all
players are of equal strength:
 
<<like_series1>>=
data("masterchef")
n <- 13   # 13 players
equal_strengths <- rep(1/n,n-1)
like_series(equal_strengths, masterchef)  # see masterchef.Rd
@ 

The strengths of the 13 players may be estimated using standard
maximum likelihood techniques.  This requires constrained optimization
in order to prevent the search from passing through inadmissible
points in p-space:

<<unconstrained_optimization,eval=FALSE>>=
UI <- rbind(diag(n-1),-1)  # p_i >= 0
CI <- c(rep(0,n-1),-1)     # p_1+...+p_{n-1} <= 1

constrOptim(  # maxp() does not work for masterchef_series6
    theta = equal_strengths,  # starting point for optimization
    f = function(p){-like_series(p,masterchef)}, # see masterchef.Rd
    ui=UI, ci=CI,
    grad=NULL)
@ 

\begin{figure}[htbp]
\begin{center}
<<masterchef_maxp,echo=TRUE,fig=TRUE>>=
masterchef_maxp
dotchart(masterchef_maxp)
@
\caption{Maximum likelihood estimate for the strengths of the top~13
  competitors\label{masterchef6} in Series~6 of \emph{MasterChef Australia}
}
\end{center}
\end{figure}

The resulting maximum likelihood estimate, \code{masterchef_maxp} in
the package, is shown pictorially in Figure~\ref{masterchef6}.  The
support at the precalculated evaluate is

<<like_series2>>=
like_series(indep(masterchef_maxp), masterchef)
@ 

and this allows us to test the hypothesis of equal player strengths:
by Wilks's theorem, $-2\log\Lambda$ has an asymptotic null distribution
of~$\chi^2_{12}$.  This corresponds to a $p$-value of

<<pvalcalcequalstrengths>>=
pchisq(2*(78.7-66.2),df=12,lower.tail=FALSE)
@ 

showing that the observations do constitute evidence for differential
player strengths.  Figure~\ref{masterchef6} suggests that Laura, the
runner-up, is actually a stronger competitor than the winner, Brent.
We may assess this statistically by finding the maximum likelihood
for~${\mathbf p}$, subject to the constraint
that~$p_\mathrm{Laura}\leqslant p_\mathrm{Brent}$:

<<brent.gt.laura, eval=FALSE>>=
UI <- rbind(UI,c(0,0,1,0,0,0,0,0,-1,0,0,0,0))  # Brent >= Laura
CI <- c(CI,0)
ans2 <-
constrOptim(  # maxp() does not work for masterchef_series6
    theta = equal_strengths,
    f = function(p){-like_series(p,masterchef_series6)},  # see masterchef.Rd
    grad=NULL,
    ui = UI, ci=CI)
@ 

Object \code{maxp_masterchef6_constrained} in the package is the
result of the above optimization, at which point the likelihood is


<<>>=
like_series(indep(masterchef_constrained_maxp), masterchef)
@ 

The two estimates differ by about~$1.18$, less than the
two-units-of-support criterion of~\citet{edwards1992}; alternatively,
one may observe that the likelihood ratio is not in the tail region of
its asymptotic distribution ($\chi^2_1$) as the $p$-value is about 0.12.
This shows that there is no strong evidence for Laura's competitive
strength being higher than that of Brent.  Similar techniques can be
used to give a profile likelihood function; the resulting support interval
for Laura's strength is~$\left[{0.145,0.465}\right]$, which does not
include~$\frac{1}{13}\simeq 0.077$, the mean player strength.  

However, further work would be needed to make statistically robust
inferences from these findings.  Even given equal competitive ability,
one would expect the player with the highest estimated Bradley-Terry
strength to have an elevated support interval; and it is not clear to
what extent Wilks, being asymptotic, is relevant to the data at hand.


\section{Conclusions}

Several generalization of Bradley-Terry strengths are appropriate to
describe competitive situations in which order statistics are
sufficient.

The \code{hyper2} package is introduced, providing a suite of
functionality for generalizations of the partial rank analysis
of~\cite{critchlow1985}.  The software admits natural \proglang{R}
idiom for translating commonly occurring observations into a
likelihood function.

The package is used to calculate maximum likelihood estimates for
generalized Bradley-Terry strengths in two competitive situations:
Olympic rowing, and \emph{MasterChef Australia}.  The estimates for
the competitors' strengths are plausible; and several meaningful
statistical hypotheses are assessed quantitatively.





\bibliography{hyper2}
\end{document}
 

---
title: "Public perception of climate change: the `icons` dataset in the `hyper2` package"
author: "Robin K. S. Hankin"
bibliography: hyper2.bib
link-citations: true
output: html_vignette
vignette: |
  %\VignetteIndexEntry{icons}
  %\VignetteEncoding{UTF-8}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r setup, include=FALSE}
library("hyper2",quietly=TRUE)
set.seed(0)
knitr::opts_chunk$set(echo = TRUE)
```

```{r label="hexsticker",out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE}
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))
```

To cite the `hyper2` package in publications, please use @hankin2017_rmd.
This short document analyses the climate change dataset introduced by
@oneill2008 and discussed in the `hyperdirichlet` R package
[@hankin2010], but using the improved `hyper2` package instead.
@oneill2009 observe that lay perception of climate change is a
complex and interesting process, and here we assess the engagement of
non-experts by the use of "icons" (this word is standard in this
context.  An icon is a "representative symbol") that illustrate
different impacts of climate change.

Here we analyse results of one experiment [@oneill2008], in which
subjects were presented with a set of icons of climate change and
asked to identify which of them they found most concerning.  Six icons
were used: PB [polar bears, which face extinction through loss of ice
floe hunting grounds], NB [the Norfolk Broads, which flood due to
intense rainfall events], L [London flooding, as a result of sea level
rise], THC [the thermo-haline circulation, which may slow or stop as a
result of anthropogenic modification of the water cycle], OA [oceanic
acidification as a result of anthropogenic emissions of ${\mathrm
C}{\mathrm O}_2$], and WAIS [the West Antarctic Ice Sheet, which is
rapidly calving as a result of climate change].  Methodological
constraints dictated that each respondent could be presented with a
maximum of four icons.  The R idiom below (dataset `icons` in the
package) shows the experimental results.

```{r label=loadlib}
library("hyper2",quietly=TRUE)
M <- icons_table # saves typing
M
```

(M is called `icons_table` in the package).  Each row of `M`
corresponds to a particular cue given to respondents.  The first row,
for example, means that a total of $5+3+4+3=15$ people were shown
icons NB, L, THC, WAIS [column names of the the non-NA entries]; 5
people chose NB as most concerning, 3 chose L, and so on.  The dataset
is more fully described in the package.  The builtin `icons`
likelihood function in the `hyper2` package may be created by using
the `saffy()` function:

```{r showicons}
icons
icons == saffy(icons_table)  # should be TRUE
```

At this point, the `icons` object as created above is mathematically
identical to the `icons` object in the `hyperdirichlet` package (and
indeed the `hyper2` package), but the terms might appear in a
different order due to `disordR` discipline.

## Analysis of the icons dataset

The first step is to find the maximum
likelihood estimate for the `icons` likelihood:

```{r estmaxlike,cache=TRUE}
options("digits" = 4)
(mic <- maxp(icons))
dotchart(mic,pch=16)
```

We also need the log-likelihood at an unconstrained maximum:

```{r calclog}
L1 <- loglik(indep(mic),icons)
options(digits=9)
L1
```

We see agreement to 4 decimal places with the value given in the
`hyperdirichlet` package.  The next step is to assess a number of
hypotheses concerning the relative sizes of $p_1$ through $p_6$.

# Hypothesis testing.

Below, I investigate a number of inequality hypotheses about
$p_1,\ldots, p_6$.  The general analysis proceeds as follows; we can
use $H_0\colon p_1=1/6$ as an example but the method applies to any
observation.  I consider the general case first, then modifications
for one-sided hypotheses.

* We have an observation ${\mathcal O}$  and 
wish to quantify the strength of evidence to support this.
* Translate our observation into a hypothesis phrased in terms as a
 restriction on parameter space.
* State a null hypothesis and alternative.  For example, we might
  have $H_0\colon p_1=1/6$ and $H_A\colon p_1\neq 1/6$, and $H_A$ is the complement of $H_0$.
  We sometimes write $H_A\colon\sum p_i=1$, ignoring the (measure zero) $H_0$.
* We then attempt to reject $H_0$ and to do this we perform two
  likelihood maximizations: one unconstrained, corresponding
  to $H_A$, and one constrained, corresponding to $H_0$.
* We calculate ${\mathcal L}_{H_0} :=\operatorname{argmax}_{p\in
  H_0}{\mathcal L}(p)$ and ${\mathcal L}_{H_A} :=
  \operatorname{argmax}_{p\in H_A}{\mathcal L}(p)$ and observe that, in general,
  ${\mathcal O}$ implies the strict inequality
  ${\mathcal L}_{H_A} > {\mathcal L}_{H_0}$.
* Calculate the likelihood ratio
  $R={\mathcal L}_{H_A}/{\mathcal L}_{H_0}>1$ or equivalently the
  _support_ for $H_0$, $\Lambda = \log R>0$.
* We may either use Edwards's criterion of two units of support per
  degree of freedom, or Wilks's theorem:
  $H_0\longrightarrow 2\Lambda\sim\chi^2_d$.  If we are
  considering inequality hypotheses, we have $d=1$. 
* If the criterion is met we may reject $H_0$ and infer
  that $H_A$ is the case.

As another example, consider $H_0\colon p_1=p_2=p_3$.  The vector
${\mathbf p}\in \left\{\left.\left(p_1,\ldots,p_6\right)\right| \sum
p_i=1\right\}$ is a point in a 5-dimensional manifold, and $H_0$
asserts that ${\mathbf p}\in
\left\{\left.\left(p_1,p_1,p_1,p_4,p_5,p_6\right)\right|
3p_1+p_4+p_5+p_6=1\right\}$, that is, a 3-dimensional manifold.  The
null imposes a loss of two degrees of freedom.

The logic above operates for one-sided tests.  We might observe that
$\hat{p_1}$ is the largest and wish to test a null of $p_1\leqslant
1/6$ against an alternative of $p_1>1/6$.  Note that the one-sided
p-value and likelihood ratio statistic are the same as the two-sided
values.

Below I will rework some of the hypotheses tested in the
hyperdirichlet package, with consistent labelling of null and
alternative hypotheses, and renumbering

## Equality of strengths

The most straightforward null would be the hypothesis of player
equality, specifically:

\[
H_E\colon p_1=p_2=\cdots=p_n=\frac{1}{n}.
\]

("E" for equal).  This was not carried out in @hankin2010, because I
had not thought of it.  Testing $H_E$ is implemented in the package as
`test.equalp()`.  Note that because $H_E$ is a point hypothesis we
would have $n-1$ degrees of freedom (because $H_0$ has $n-1$ df).

```{r doequalp,cache=TRUE}
equalp.test(icons)
```	

## Hypothesis 1: $p_1 = 1/6$

Following the analysis in @hankin2010, and restated above, we first
observe that NB [the Norfolk Broads] is the icon with the largest
estimated probability with $\hat{p_1}\simeq 0.25$ (O'Neill gives a
number of theoretical reasons to expect $p_1$ to be large).  This
would suggest that $p_1$ is in fact large, in some sense, and here I
show how to assess this statement statistically.  Consider the
hypothesis $H_0\colon p_1=1/6$, and as per the protocol above we will
try to reject it.

To that end, we perform a constrained optimization, with (active)
constraint that $p_1\leqslant 1/6$ (note that the inequality
constraint allows us to use fast `maxp()`, which has access to
derivatives).  We note the support at the evaluate and then compare
this support with the support at the unconstrained evaluate; if the
difference in support is large then this constitutes strong evidence
for $H_A$ and then would conclude that $p_1 > 1/6$.  In package idiom,
the optimization is implemented by the `specificp.test()` suite of
functions; these work by imposing additional constraints to the
`maxp()` function via the `fcm` and `fcv` arguments.  Using the
defaults we have:

```{r dospecificp,cache=TRUE}
specificp.test(icons,1)
```	

which tests a null of $p_1\leqslant\frac{1}{6}$; see how the evaluate
under the null is on the boundary and we have $\hat{p_1}=\frac{1}{6}$.
Compare the support of 2.607 with 2.608181 from the `hyperdirichlet`
package.  This exceeds Edwards's two-units-of-support criterion; the
$p$-value is obtained by applying Wilks's theorem on the asymptotic
distribution of 2\log\Lambda.

Both these criteria indicate that we may reject that hypothesis that
$p_1\leqslant 1/6$ and thereby infer $p_1>\frac{1}{6}$.

## Hypothesis 2: $p_1\geqslant\max\left(p_2,\ldots,p_6\right)$

We observe that NB (the Norfolk Broads) has large strength, and
hypothesise that it is in fact stronger than any other icon.  This is
another constrained likelihood maximization, although this one is not
possible with convex constraints.  In the language of the generic
procedure given above, we would have

\[
H_0\colon 
(p_1\leqslant p_2)\cup
(p_1\leqslant p_3)\cup
(p_1\leqslant p_4)\cup
(p_1\leqslant p_5)\cup
(p_1\leqslant p_6)
\]

\[
\overline{H_0}\colon 
(p_1 > p_2)\cap
(p_1 > p_3)\cap
(p_1 > p_4)\cap
(p_1 > p_5)\cap
(p_1 > p_6)
\]

(although note that $H_0$ has nonzero measure).  In words, $H_0$
states that $p_1$ is smaller than $p_2$, or $p_1$ is smaller than
$p_3$, etc; while $\overline{H_0}$ states that $p_1$ is larger than
$p_2$, and is larger than $p_3$, and so on.  Considering the evaluate,
we see that ${\mathcal L}_{H_0} < {\mathcal L}_{\overline{H_0}}$, so
optimizing over $\overline{H_0}$ is equivalent to unconstrained
optimization [of course, the intrinsic constraints $p_i\geqslant 0,
\sum p_i\leqslant 1$ have to be respected].  Here, $\overline{H_0}$ is
regions of $(p_1,\ldots,p_6)$ with $p_1$ being greater than *at least
one* of $p_2,\ldots,p_6$.  The union of convex sets is not necessarily
convex (e.g. a two-way Venn diagram).  As far as I can see, the only
way to do it is to perform a sequence of five constrained
optimizations: $p_1\leqslant p_2, p_1\leqslant p_3, p_1\leqslant p_4,
p_1\leqslant p_5$.  The fillup constraint would be $p_1\leqslant
p_6\longrightarrow 2p_1+p_2+\cdots +p_5\leqslant 1$.  We then choose
the largest likelihood from the five.

```{r largestoffive,cache=TRUE}
o <- function(Ul,Cl,startp,give=FALSE){
    small <- 1e-4  #  ensure start at an interior point
    if(missing(startp)){startp <- small*(1:5)+rep(0.1,5)}			
    out <- maxp(icons, startp=small*(1:5)+rep(0.1,5), give=TRUE, fcm=Ul,fcv=Cl)
    if(give){
        return(out)
    }else{
        return(out$value)
    }
}

p2max <- o(c(-1, 1, 0, 0, 0), 0)  # p1 <= p2
p3max <- o(c(-1, 0, 1, 0, 0), 0)  # p1 <= p3
p4max <- o(c(-1, 0, 0, 1, 0), 0)  # p1 <= p4
p5max <- o(c(-1, 0, 0, 0, 1), 0)  # p1 <= p5
p6max <- o(c(-2,-1,-1,-1,-1),-1)  # p1 <= p6 (fillup)
```

(the final line is different because $p_6$ is the fillup value).

```{r pnmax}
likes <- c(p2max,p3max,p4max,p5max,p6max)
likes
ml <- max(likes) 
ml
```

Thus the first element of `likes` corresponds to the maximum
likelihood, constrained so that $p_1\leqslant p_2$; the second element
corresponds to the constraint that $p_1\leqslant p_3$, and so on.  The
largest likelihood is the easiest constraint to break, in this case
$p_1\leqslant p_3$: this makes sense because $p_3$ has the second
highest MLE after $p_1$.  The extra likelihood is given by

```{r extralike}
L1-ml
```

(the `hyperdirichlet` package gives 0.0853 here, a surprisingly small
discrepancy given the difficulties of optimizing over a nonconvex
region).  We conclude that there is no evidence for
$p_1\geqslant\max\left(p_2,\ldots,p_6\right)$.

It's worth looking at the evaluate too:

```{r evalworth,cache=TRUE}
o2 <- function(Ul,Cl){
  jj <-o(Ul,Cl,give=TRUE)
  out <- c(jj[[1]],1-sum(jj[[1]]),jj[[2]])
  names(out) <- c("p1","p2","p3","p4","p5","p6","support")
  return(out)
}
rbind(
o2(c(-1, 1, 0, 0, 0), 0),  # p1 <= p2
o2(c(-1, 0, 1, 0, 0), 0),  # p1 <= p3
o2(c(-1, 0, 0, 1, 0), 0),  # p1 <= p4
o2(c(-1, 0, 0, 0, 1), 0),  # p1 <= p5
o2(c(-2,-1,-1,-1,-1),-1)   # p1 <= p6
)
```

In the above, the evaluate is the first five columns ($p_6$ being the
fillup) and the final column is the log-likelihood at the evaluate.
See how the constraint is active in each line: `M[1,] == M[1:5,2:6]`.
Also note that the largest log-likelihood is the second row: if we
were to violate any of the constraints, it would be $p_1<p_3$,
consistent with the fact that $p_3$ (polar bears) has the second
highest strength, after $p_1$.

## Low frequency responses

The next hypothesis follows from the observed smallness of $\hat{p_5}$
(ocean acidification) and $\hat{p_6}$ (West Antarctic Ice Sheet) at
0.111 and 0.069 respectively.  These two strengths correspond to
"distant" concerns and O'Neill had reason to consider their sum (which
she argued would be small).  Thus we specify $H_0\colon
p_5+p_6\leqslant \frac{1}{3}$.

The optimizing constraint of $p_5+p_6\geqslant\frac{1}{3}$ translates
to an operational constraint of
$-p_1-p_2-p_3-p_4\geqslant-\frac{2}{3}$ (because
$p_5+p_6=1-p_1-p_2-p_3-p_4$):

```{r lowfreq,cache=TRUE} 
jj <- o(c(-1,-1,-1,-1,0) , -2/3, give=TRUE,start=indep((1:6)/21))$value
jj
```

then the extra support is 

```{r extralowfreq}
L1-jj
```

(compare 7.711396 in `hyperdirichlet`, not sure why the discrepancy is
so large).

## Final example

The final example is motivated by the fact that both the distant
icons $p_5$ and $p_6$ had lower strength than any of the local icons
$p_1,\ldots, p_4$.  Thus we would have
$H_0\colon\max\left\{p_5,p_6\right\}\geqslant\min\left\{p_1,p_2,p_3,p_4\right\}$.
This means the null optimization is constrained so that *at least one*
of $\left\{p_5,p_6\right\}$ exceeds *at least one* of
$\left\{p_1,p_2,p_3,p_4\right\}$.  So we have the union of the various
possibilities:

\begin{equation}\label{eq:Habar}H_0=
\overline{H_A}\colon
\bigcup_{j\in\left\{5,6\right\}\atop k\in\left\{1,2,3,4\right\}}
\left\{\left(p_1,p_2,p_3,p_4,p_5,p_6\right)\left|\sum p_i=1,
p_j\geqslant p_k\right.\right\}
\end{equation}

The fillup value $p_6$ behaves differently in this context and
$p_6\geqslant p_2$, say, translates to $-p_1-2p_2-p_3-p_4-p_5\geqslant
-1$.

```{r ofcourse,cache=TRUE}
small <- 1e-4
start <- indep(c(small,small,small,small,0.5-2*small,0.5-2*small))
jj <- c(
   o(c(-1, 0, 0, 0, 1), 0,start=start),  # p1 >= p5
   o(c( 0,-1, 0, 0, 1), 0,start=start),  # p2 >= p5
   o(c( 0, 0,-1, 0, 1), 0,start=start),  # p3 >= p5
   o(c( 0, 0, 0,-1, 1), 0,start=start),  # p4 >= p5

   o(c(-2,-1,-1,-1,-1),-1,start=start),  # p1 >= p6
   o(c(-1,-2,-1,-1,-1),-1,start=start),  # p2 >= p6
   o(c(-1,-1,-2,-1,-1),-1,start=start),  # p3 >= p6
   o(c(-1,-1,-1,-2,-1),-1,start=start)   # p4 >= p6
   )
jj
```

Above, the elements of vector `jj` are the maximum likelihoods for
each of the separate components of the parameter space allowed under
$H_0$.  Note that these components are not disjoint.  So the maximum
likelihood for the whole of allowable parameters under $H_0$ would be
the maximum of these maxima:

```{r maxjj}
max(jj)
```

This corresponds to the fourth component of $H_0$, viz $p_4=p_5$.  The
extra support is thus

```{r extraofcourse}
L1-max(jj)
```

(compare `hyperdirichlet` which gives 3.16, not sure why the
difference although if pressed I would point to `hyper2` using
multiple walkers in its optimization [defaulting to $n=10$] ).  We
should look at the maximum value:

```{r lookatmax,cache=TRUE}
o(c( 0, 0, 0,-1, 1), 0,give=TRUE,start=start)
```

So the evaluate is at the boundary, for $p_4=p_5$; `THC` and `OA` have
the same Bradley-Terry strength.  The small amount of extra support
given by allowing an unconstrained optimization would suggest that
there is no strong evidence for the contention investigated, viz "both
the distant icons $p_5$ and $p_6$ have lower strength than any of the
local icons $p_1,\ldots, p_4$".


## References

---
title: "The `dirichlet()` function in the `hyper2` package"
author: "Robin K. S. Hankin"
output: html_vignette
bibliography: hyper2.bib
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{dirichlet}
  %\usepackage[utf8]{inputenc}
---

```{r setup, include=FALSE}
set.seed(0)
library("hyper2")
options(rmarkdown.html_vignette.check_title = FALSE)
knitr::opts_chunk$set(echo = TRUE)
knit_print.function <- function(x, ...){dput(x)}
registerS3method(
  "knit_print", "function", knit_print.function,
  envir = asNamespace("knitr")
)
```

```{r out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE}
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))
```

```{r, label=showdirichlet,comment=""}
dirichlet
```

To cite the `hyper2` package in publications, please use
@hankin2017_rmd.  We say that non-negative $p_1,\ldots, p_n$
satisfying $\sum p_i=1$ follow a _Dirichlet distribution_, and write
$\mathcal{D}(p_1,\ldots,p_n)$, if their density function is

\begin{equation}
\frac{\Gamma(\alpha_1+\cdots +\alpha_n)}{\Gamma(\alpha_1)\cdots\Gamma(\alpha_n)}
\prod_{i=1}^n p_i^{\alpha_i-1}
\end{equation}

for some parameters $\alpha_i>0$.  It is interesting in the current
context because it is conjugate to the multinomial distribution;
specifically, given a Dirichlet prior
$\mathcal{D}(\alpha_1,\ldots,\alpha_n)$ and likelihood function
$\mathcal{L}(p_1,\ldots,p_n)\propto\prod_{i=1}^n p_i^{\alpha_i}$ then
the posterior is $\mathcal{D}(\alpha_1+a_1,\ldots,\alpha_n+a_n)$.

In the context of the `hyper2` package, function `dirichlet()`
presents some peculiarities, discussed here.

## A simple example

Consider a three-way trial between players `a`, `b`, and `c` with
eponymous Bradley-Terry strengths.  Suppose that, out of $4+5+3=12$
trials, `a` wins 4, `b` wins 5, and `c` wins 3.  Then an appropriate
likelihood function would be:

```{r dirichletcompleteFALSE}
dirichlet(c(a=4,b=5,c=3))
```

Note the `(a+b+c)^-11` term, which is not strictly necessary according
the Dirichlet density function above which has no such term.  However,
we see that the returned likelihood function is ${\propto}
p_{a\vphantom{abc}}^4p_{b\vphantom{abc}}^5p_{c\vphantom{abc}}^3$
(because $p_a+p_b+p_c=1$).

```{r dirichletabc}
(L1 <- dirichlet(c(a=4,b=5,c=3)))
```

Now suppose we observe three-way trials between `b`, `c`, and `d`:

```{r dirichletbcd}
(L2 <- dirichlet(c(b=1,c=1,d=8)))
```

The overall likelihood function would be $L_1+L_2$:

```{r sumofL1andL2,cache=TRUE}
L1+L2
maxp(L1+L2)
```

Observe the dominance of competitor `d`, reasonable on the grounds
that `d` won 8 of the 10 trials against `b` and `c`; and `a`, `b`, `c`
are more or less evenly matched [chi square test,
$p=`r round(chisq.test(c(4,5,3),sim=TRUE)$p.value,2)`$].
Observe further that we can include four-way observations easily:

```{r fourwayobs}
(L3 <- dirichlet(c(a=4,b=3,c=2,d=6)))
L1+L2+L3
```

Package idiom `L1+L2+L3` operates as expected because `dirichlet()`
includes the denominator.  Sometimes we see situations in which a
competitor does not win any trials.  Consider the following:

```{r}
(L4 <- dirichlet(c(a=5,b=3,c=0,d=3)))
```

Above, we see that `c` won no trials and is not present in the
numerator of the expression.  However, `L4` is informative about
competitor `c`:

```{r label=L4testspec,cache=TRUE}
maxp(L4)
```

We see that the maximum likelihood estimate for `c` is zero (to within
numerical tolerance).  Further, we may reject the hypothesis that
$p_c=\frac{1}{4}$ [which might be a reasonable consequence of the
assumption that all four competitors have the same strength]:


```{r label=testcquarter,cache=TRUE}
specificp.test(L4,'c',0.25)
```

However, observe that we cannot reject the equality hypothesis, that
is, $p_a=p_b=p_c=p_d=\frac{1}{4}$:

```{r label=testequal,cache=TRUE}
equalp.test(L4)
```

## References

\documentclass[article]{jss}

%% -- LaTeX packages and custom commands ---------------------------------------

%% recommended packages
\usepackage{orcidlink,thumbpdf,lmodern}

%% another package (only for this demo article)
\usepackage{framed}
\usepackage{booktabs}
\usepackage{units}
\usepackage{amsmath}
\usepackage{amssymb}
\usepackage{tikz}
\usepackage{wrapfig}


%% new custom commands
\newcommand{\class}[1]{`\code{#1}'}
\newcommand{\fct}[1]{\code{#1()}}
\newcommand\headercell[1]{%
   \smash[b]{\begin{tabular}[t]{@{}c@{}} #1 \end{tabular}}}

%% For Sweave-based articles about R packages:
%% need no \usepackage{Sweave}
\SweaveOpts{engine=R, eps=FALSE, keep.source = TRUE}
<<preliminaries, echo=FALSE, results=hide>>=
options(prompt = "R> ", continue = "+  ", width = 70, useFancyQuotes = FALSE)
library("hyper2")
library("magrittr")
howmany <- 23  # set to a smaller value to save time
@


%% -- Article metainformation (author, title, ...) -----------------------------

%% - \author{} with primary affiliation (and optionally ORCID link)
%% - \Plainauthor{} without affiliations
%% - Separate authors by \And or \AND (in \author) or by comma (in \Plainauthor).
%% - \AND starts a new line, \And does not.
\author{Robin K. S. Hankin~\orcidlink{0000-0001-5982-0415}\\University of Stirling}
\Plainauthor{Robin K. S Hankin}

%% - \title{} in title case
%% - \Plaintitle{} without LaTeX markup (if any)
%% - \Shorttitle{} with LaTeX markup (if any), used as running title

\title{Generalized Bradley-Terry models}

%% - \Abstract{} almost as usual
\Abstract{
  A generalization of the Bradley-Terry model is presented in which
  strengths may be modified using a multiplicative factor $\lambda$;
  Generally, ordinary Bradley-Terry is recovered if $\lambda=1$.  This
  approach offers multiple advantages over other generalizations such
  as reified Bradley-Terry.  It becomes possible to represent
  non-independence of successive observations in Plackett-Luce
  likelihood functions which allows a parametrization of the ``red
  bus, blue bus'' problem in preference analysis.  I present examples
  drawn from the fields of food preference and the esport {\em
    Counterstrike}.
}

%% - \Keywords{} with LaTeX markup, at least one required
%% - \Plainkeywords{} without LaTeX markup (if necessary)
%% - Should be comma-separated and in sentence case.
\Keywords{JSS, style guide, comma-separated, not capitalized, \proglang{R}}
\Plainkeywords{JSS, style guide, comma-separated, not capitalized, R}
\Keywords{Plackett-Luce, Bradley-Terry, \proglang{R}}
\Plainkeywords{Plackett-Luce, Bradley-Terry, R}


%% - \Address{} of at least one author
%% - May contain multiple affiliations for each author
%%   (in extra lines, separated by \emph{and}\\).
%% - May contain multiple authors for the same affiliation
%%   (in the same first line, separated by comma).
\Address{
  Robin K. S. Hankin\\
  University of Stirling\\
  E-mail: \email{hankin.robin@gmail.com}
}

\begin{document}


%% -- Introduction -------------------------------------------------------------

%% - In principle "as usual".
%% - But should typically have some discussion of both _software_ and _methods_.
%% - Use \proglang{}, \pkg{}, and \code{} markup throughout the manuscript.
%% - If such markup is in (sub)section titles, a plain text version has to be
%%   added as well.
%% - All software mentioned should be properly \cite-d.
%% - All abbreviations should be introduced.
%% - Unless the expansions of abbreviations are proper names (like "Journal
%%   of Statistical Software" above) they should be in sentence case (like
%%   "generalized linear models" below).

\section{Introduction}
\begin{wrapfigure}{r}{0.1\textwidth}
    \centering
    \includegraphics[width=0.1\textwidth]{hyper2.png}\\
\end{wrapfigure}

The Bradley-Terry model for datasets involving paired comparisons has
wide uptake in the {R} community [see, for example,
\citet{turner2012,turner2020} and references therein].  The canonical
case is to consider~$n$ players who compete against one another in
pairs; the basic inference problem is to estimate
numbers~$\left({p_1,\ldots,p_n}\right)$, $p_i\geqslant 0$, $\sum
p_i=1$ so that $\Prob(\mbox{$i$~beats~$j$})=\frac{p_i}{p_i+p_j}$.
Information about the~$p_i$ may be obtained from the results of paired
comparisons between the players.

One generalization of the Bradley-Terry model is due
to~\citet{luce1959_jss}, in which the probability of competitor~$i$
winning in a field of~$j=1,\ldots, n$ is $\frac{p_i}{p_1+\cdots
  +p_n}$.  It is then natural to quantify some ephemeral enhancement
to competitor~$i$ using a multiplicative factor $\lambda$.  Then
$\Prob(\mbox{$i$ wins})=\frac{\lambda p_i}{p_1+\cdots+\lambda
  p_i+\cdots+p_n}$.  Examples of such enhancements include the
home-ground advantage in football and rugby, or playing white in
chess~\citep{hankin2020_jss}.  Note that $\lambda < 1$ would correspond to
a decrease in competitive ability.

Further, one might follow~\cite{hankin2017_rnw_jss} and consider teams of
players.  In doubles tennis, for example, $\left\lbrace
p_1,p_2\right\rbrace$ might play $\left\lbrace p_3,p_4\right\rbrace$
and win with probability
$\frac{p_1+p_2}{p_1+p_2+p_3+p_4}$~\citep{hankin2010}.  It is natural
to combine this idea with that of a multiplicative factor.  For
example, suppose $p_1$ has just learned a new special technique, and
$p_4$ has just been given a performance-enhancing drug.  If so, we
might have likelihood function
%
\begin{equation}\label{like1234}
  \mathcal{L}{\left(p_1,p_2,p_3,p_4;\lambda,\mu\right)}\propto
  \frac{
    \left(\lambda p_1 + p_2\right)^{w_{12}}\cdot
    \left(p_3 + \mu p_4\right)^{w_{34}}
    }{
    \left(\lambda p_1+p_2+p_3+\mu p_4\right)^{w_{12} + w_{34}}
    }
\end{equation}
%
where $w_{12}$ and $w_{34}$ are the number of wins by each team
respectively.  Then $\lambda$ would represent the effectiveness of
$p_1$'s special technique, and $\mu$ the enhancement afforded to $p_4$
by the drug; values $>1$ would correspond to a positive effect.  In
this paper I generalize the \pkg{hyper2} R
package~\citep{hankin2017_rnw_jss}, available on
CRAN~\citep{rcore2025}, for Bradley-Terry likelihoods to accommodate
this class of multiplicative factors with new \proglang{S3} class
\class{hyper3} and associated methods.

The most general likelihood function would be
proportional to
%
\begin{equation}\label{mostgeneral}
  \prod_{i=1}^N
  \sum_{j=1}^n
 \left(\lambda_{ij}p_j\right)^{w_i}
\end{equation}
%
where the $p_j$ are traditional Bradley-Terry strengths, $p_j\geqslant
0$, $\sum p_j=1$, and the $\lambda_{ij}\geqslant 0$ are multiplicative
factors that account for ephemeral or temporary changes in competitive
ability.  The data $w_1,\ldots w_N$ correspond to the number of
``wins'' and number of matches, as per $w_{12},w_{34}$ in
Equation~\ref{like1234}.  In practice, almost all the $\lambda_{ij}$
are zero.  To fix ideas, consider the data presented in
Table~\ref{oldfirm}.  Ignoring draws for the moment, we have a
likelihood proportional to
%
\begin{table}[t!]
\centering
\begin{tabular}{@{} *{5}{c} @{}}
\headercell{\small{Plays at home,}\\ \small{wins}} &
\multicolumn{4}{c@{}}{\small{Plays away, loses}}\\
\cmidrule(l){2-5}
&   Rangers     &    Celtic & Falkirk & Livingston\\
\midrule
   Rangers      &   NA &    81  &    69   &      15\\
   Celtic       &   76 &    NA  &    61   &      15\\
   Falkirk      &   17 &    17  &    NA   &       6\\
   Livingston   &    2 &     2  &     3   &      NA\\
\end{tabular}
\\
\rule{0mm}{16mm}
\\
\begin{tabular}{@{} *{5}{c} @{}}
\headercell{\small{Plays at home,}\\ \small{draws}} &
\multicolumn{4}{c@{}}{\small{Plays away, draws}}\\
\cmidrule(l){2-5}
&  Rangers     &    Celtic & Falkirk & Livingston\\
\midrule
   Rangers     &       NA  &   44 &     14   &       2\\
   Celtic      &       47  &   NA &     10   &       3\\
   Falkirk     &       12  &   19 &     NA   &       4\\
   Livingston  &       4   &   4  &     4    &      NA\\
   \end{tabular}
\\
\rule{0mm}{16mm}
\\
\begin{tabular}{@{} *{5}{c} @{}}
\headercell{\small{Plays at home,}\\ \small{loses}} &
\multicolumn{4}{c@{}}{\small{Plays away, wins}}\\
\cmidrule(l){2-5}
&  Rangers     &    Celtic & Falkirk & Livingston\\
\midrule
   Rangers     &   NA  &  42  &    2  &       0\\
   Celtic      &   46  &  NA  &    8  &       0\\
   Falkirk     &   57  &  43  &   NA  &       8\\
   Livingston  &   12  &  11  &    9  &      NA\\
  \end{tabular}
\caption{The \label{oldfirm} dominance of the Old Firm: 769 matches
  between Rangers, Celtic, Falkirk and Livingston arranged by home
  wins, draws, and home losses.  Data from {\tt stats.football.co.uk}}
\end{table}
%
\begin{equation}\label{firstfewRCFL}
  \left(\frac{\lambda p_R}{\lambda p_R + p_C}\right)^{81}\cdot
  \left(\frac{\lambda p_R}{\lambda p_R + p_F}\right)^{69}\cdots
  \left(\frac{\lambda p_L}{\lambda p_L + p_F}\right)^{3}\cdots
  \left(\frac{p_C}{p_C + \lambda p_R}\right)^{42}\cdots
    \left(\frac{p_F}{p_F + \lambda p_L}\right)^{9}
\end{equation}
%
where the home-ground advantage is represented by~$\lambda$.  In
package idiom, a term such as $\lambda p_R + p_C$ is represented as a
named numeric vector.  For example, if the home strength were $\lambda
= 1.8$ then one bracket would be {\tt c(Rangers=1.8, Celtic=1)}
with an associated power of $-81$.  The computational methodology is a
generalization of that used in~\cite{hankin2024_hyper3_jss} in which the
multiplicative factors correspond to number of clones.  However,
in~\cite{hankin2024_hyper3_jss} the factors are known non-negative
integers, and here they are, in general, unknown non-negative real
numbers.  The \class{hyper3} class enforces \pkg{disordR}
discipline~\citep{hankin2022_disordR_jss}.

\subsection{The package in use}
Creating a likelihood function for this dataset is straightforward.
As an illustration of package idiom, one way of representing a
likelihood function for Rangers-Celtic matches (here, ignoring draws)
would be:
%
<<defineoldfirm, cache = TRUE>>=
oldfirm <- function(lambda){
    H <- hyper3(pnames = c("Rangers", "Celtic"))
    H[c(Rangers = lambda            )] %<>%      add(81     )
    H[c(                  Celtic = 1)] %<>%      add(     42)
    H[c(Rangers = lambda, Celtic = 1)] %<>% subtract(81 + 42)

    H[c(Celtic = lambda             )] %<>%      add(     76)
    H[c(                 Rangers = 1)] %<>%      add(46     )
    H[c(Celtic = lambda, Rangers = 1)] %<>% subtract(46 + 76)
    return(H)
}
@
%
Above we see package idiom for creating a log-likelihood function.  We
first create an empty {\tt hyper3} object with an appropriate names
attribute.  The next three lines, using \pkg{magrittr} assignment
pipes, give the results for Rangers at home, and the next three lines
give the result for Celtic at home.  It is then easy to create a
bespoke log-likelihood function for Rangers-Celtic matches:
%
<<useoldfirm, cache = TRUE>>=
oldfirm(1.88)
@
%
We see a log-likelihood function for all $82+42+76+46=245$
Rangers-Celtic matches that were not score draws, conditional on
assumed value $\lambda=1.88$.  This corresponds to the relevant terms
of Equation~\ref{firstfewRCFL}.  We may make inferences about the true
value of $\lambda$ using profile support:
%
<<profilelikeoldfirm, cache = TRUE>>=
f <- function(lambda){maxp(oldfirm(lambda), give = 1)$`Log-likelihood`}
lam <- seq(from = 0.9, to = 3, len = howmany)
logLike <- sapply(lam, f)
@
%
Thus \fct{f} returns the support at the evaluate, conditional on a
supplied value of $\lambda$.  We may use this to create a profile
likelihood curve (Figure~\ref{profilesupportlambda}), or to find the
maximum likelihood estimate for $\lambda$:

\begin{figure}[htbp]
  \begin{center}
<<plotproflikeoldfirm, echo=FALSE, fig = TRUE>>=
plot(lam, logLike - max(logLike), type = 'b')
abline(v = 1, col = 'gray')
segments(x0 = 1.785, y0 = 0.1, y1 = -1, col = 'red')
abline(h = c(0, -2), col = 'gray')
@
\caption{A\label{profilesupportlambda} profile support function for
  the home-team advantage $\lambda$.  Vertical gray line shows
  $H_0\colon\lambda =1$, horizontal lines show the
  two-units-of-support limits; small red bar shows
  $\hat{\lambda}\simeq 1.785$}
\end{center}
\end{figure}


<<optimoldfirm, cache=TRUE>>=
(lambda_max <-  optimize(f, c(1.5, 2.0), maximum = TRUE)$maximum)
@

Then using this value we may estimate the Bradley-Terry strengths of
Rangers and Celtic:
%
<<mleoldfirm, cache=TRUE>>=
maxp(oldfirm(lambda_max))
@
%
Compare with the theoretical value of $\widehat{p_{\mbox{\tiny
      C}}}=\frac{\sqrt{42\cdot 76}}{\sqrt{81\cdot 46}+\sqrt{42\cdot
    76}}\simeq 0.4807$, $\widehat{p_{\mbox{\tiny R}}}
=\frac{\sqrt{81\cdot 46}}{\sqrt{81\cdot 46}+\sqrt{42\cdot 76}}\simeq
0.5193$, and $\widehat{\lambda}=\frac{\sqrt{76\cdot 81}}{\sqrt{46\cdot
    42}}\simeq 1.785$.

\section{Score draws}

About 22\% of matches in Table~\ref{oldfirm} end in a draw and
\pkg{hyper2} formalism may be used to assess this.  Consider the
likelihood function described in Table~\ref{windrawloss}; this is not
easily represented as a generalized linear model using the techniques
of~\citet{turner2020}.  Related likelihood functions for drawn matches
were considered using the reified entity technique
of~\citet{hankin2020_jss} in which a drawn match was considered to be
won by a fictitious entity (a ``draw monster'') who possessed his own
BT strength.  However, the likelihood function of
table~\ref{windrawloss} has a number of desirable features not
present in the reified entity approach of~\citet{hankin2020_jss}.
Firstly, if we consider two players of identical strengths and nullify
the home ground effect (that is, $\lambda=1$) then the probability of
a draw is $\frac{2D}{2+2D}$; the odds of a draw is simply $D$,
regardless of team strength.  Secondly, considering only won (that is,
not score-drawn) games, the probability of a home win is
$\frac{\lambda}{1+\lambda}$, that is, the odds of a home win is simply
$\lambda$, independently of the player strength. Such arguments give
direct operational significance to $\lambda$ and $D$.

\begin{table}[t!]
\centering
\begin{tabular}{l|c|c}\\&
{$i$ plays at home} &
{$i$ plays away} \\
  \hline
  $\Prob{(\mbox{$i$ wins})}$                             &
  $\frac{\lambda p_i}{\lambda p_i + D(p_i + p_j) + p_j}$  &
  $\frac{p_i}{p_i + D(p_i + p_j) + p_j}$          \\
  $\Prob{(\mbox{draw})}$                                 &
  $\frac{D(p_i + p_j)}{\lambda p_i+D(p_i+p_j) + p_j}$     &
  $\frac{D(p_i + p_j)}{p_i + D(p_i+p_j) + \lambda p_j}$   \\
  $\Prob{(\mbox{$i$ loses})}$                            &
  $\frac{p_j}{\lambda p_i + D(p_i+p_j) + p_j}$            &
  $\frac{\lambda p_j}{p_i + D(p_i + p_j) + \lambda p_j}$  \\
\end{tabular}
\caption{Probabilities of a win, draw, loss for the two teams under a
  two-parameter multiplicative Bradley-Terry model\label{windrawloss}}
\end{table}
%
We are now in a position to consider the entirety of
Table~\ref{oldfirm}.  Bespoke package function
\fct{home\_draw\_away3} converts data in the form of
Table~\ref{oldfirm} to a \code{hyper3} likelihood function following
Table~\ref{windrawloss}; it takes numerical values for the home
advantage $\lambda$ and proclivity to draw $D$:
%
<<eg_RCLF3_lf, cache = TRUE>>=
home_draw_away3(RCLF3_table, lambda = 1.7, D = 0.3)
@
%
Observe that the resulting likelihood function has two distinct types
of parameters: the plain Bradley-Terry strengths $\mathbf{p}=(p_R,
p_C,p_L,p_F)$, with unit sum; but also the auxiliary parameters
$\lambda,D$.  To make inferences about these parameters we calculate a
profile likelihood function: conditional on $\lambda,D$ we maximize
the likelihood over possible values of $\mathbf{p}$.
%
<<defineF>>=
f <- function(v){
    H <- home_draw_away3(RCLF3_table, lambda = v[1], D = v[2])
    maxp(H, give = 1)$`Log-likelihood`
}
@
%
Thus \fct{f} gives us a likelihood function for
$\mathbf{v}=(\lambda,D)$.  We may find the evaluate using
\fct{optim}:
%
<<findmaxlambdaD, cache = TRUE>>=
maxv <- optim(par=c(2, 0.3), fn = f, control = list(fnscale = -1))$par
maxv
@
%
Then
%
<<showevaluate, cache = TRUE>>=
maxp(home_draw_away3(RCLF3_table, lambda = maxv[1], D = maxv[2]))
@
%
Such estimates have rich predictive power.  For example, if Rangers
were to play Celtic with both teams away [thus nullifying the home
  ground advantage: $\lambda=1$ in Table~\ref{windrawloss}], the
probability of win/draw/loss for Rangers would be
$\left(\frac{p_R}{(1+D)(p_R+p_C)};\frac{D}{1+D};\frac{p_C}{(1+D)(p_R+p_C)}\right)$,
estimated at $(0.39;0.27;0.34)$.
%
<<makecontourdata, echo = FALSE, cache = TRUE>>=
n <- howmany
lambda <- seq(from = 1, to = 3, len = n)
D <- seq(from = 0.1, to = 0.6, len = n)
V <- as.matrix(expand.grid(lambda, D))
LL <- apply(V, 1, f)
LL <- pmax(-20, LL - max(LL))
@ 

\begin{figure}[htbp]
  \begin{center}
<<docontourplot, cache = FALSE, fig = TRUE>>=
contour(lambda, D, matrix(LL, n, n), levels = -2*(1:9),
        xlab = expression(lambda), ylab = expression(D))
points(maxv[1], maxv[2], pch = 16, col = 'red')
abline(v=1, col = 'gray')
@
\caption{Contours of support\label{contourlambdaD} evaluated over
  a grid.  Maximum likelihood estimate for $(\lambda, D)$ shown as
  a red dot at the evaluate.  Vertical gray line shows $H_0\colon\lambda=1$}
  \end{center}
\end{figure}


<<testlambdazero, cache = TRUE, eval = TRUE, echo = FALSE>>=
jj <- optimize(f = function(D){f(c(lambda=1, D))}, interval = c(0.1, 0.4), maximum = TRUE)
@
%
Further, we may calculate a support function over the
$(\lambda,D)$-plane, shown in Figure~\ref{contourlambdaD}.  For
example, we may test the hypothesis that $\lambda=1$ [corresponding to
  the left hand side of Figure~\ref{contourlambdaD}] for which the
support is about \Sexpr{round(f(maxv)-jj$objective,2)}, far
exceeding the two units of support per degree of freedom
criteron~\citep{edwards1992}.  Alternatively, we can observe that this
lies in the tail region of its asymptotic null $\chi^2_2$ distribution
with a $p$~value of about \Sexpr{signif(pchisq(f(maxv)-jj$objective,lower.tail=FALSE,df=2),digits=2)}.

Equation~\ref{mostgeneral} is very general and may be applied to a
range of situations in which standard Bradley-Terry strengths are
modified in some way.  In the following section, I apply it to
Plackett-Luce \citep{plackett1975_jss} likelihood functions for
rank data.

\section{Non-independence in successive arrivals}

In the following, I use the ``race'' metaphor: five runners take part
in a race and arrive in order $a\succ b\succ c\succ d\succ e$.  Thus
competitor $a$ wins, $b$ comes second, and so on.  Consider the
following Plackett-Luce~\citep{plackett1975_jss} likelihood function for
this observation:
%
\begin{equation}
\frac{p_a}{p_a + p_b + p_c + p_d + p_e}\cdot
\frac{p_b}{      p_b + p_c + p_d + p_e}\cdot
\frac{p_c}{            p_c + p_d + p_e}\cdot
\frac{p_d}{                  p_d + p_e}\cdot
\frac{p_e}{                        p_e}
\end{equation}
%
Note the use of multiplication between the terms, indicating the
standard assumption~\citep{turner2020} of conditional independence.
Now we use the multiplicative generalization of Bradley-Terry
strengths to introduce an element of non-independence between the
terms.  There are many ways to do this, but one simple way would be to
define equivalence classes of the competitors with each equivalence
class comprising mutually supporting runners.  The metaphor would be
that a runner who has finished the race is able to support other
members of his equivalence class by cheering his still-running
teammates, boosting their performance.  The \pkg{hyper2} package
includes function \fct{cheering} which implements this functionality.

As an example, consider Figure~\ref{tikzabcde}.  This shows a partial
probability tree diagram for some of the $5!=120$ possible order
statistics for five competitors.  The standard Plackett-Luce
likelihoods have been modified to account for two groups of mutually
supporting runners: $\left\lbrace a,b,c\right\rbrace$ with support
term $\lambda$, and $\left\lbrace d,e\right\rbrace$ with support term
$\mu$.  From START, the first runner to cross the finishing line has
standard Plackett-Luce probability.  Taking the top path as an
example, we see that the likelihood function for $a\succ b\succ c\succ
d\succ e$ would be

\begin{figure}[t!]
  \begin{center}
\usetikzlibrary{arrows}
\usetikzlibrary{patterns}
\begin{tikzpicture}[line cap=round,line join=round,>=triangle 45,x=1cm,y=1cm]
\fill (0,0) circle[radius=2pt]; % root; paths abcde
\draw[thick] (0,0) -- (2, 3) -- (4.8, 3);
\draw[thick] (0,0) -- (2, 2) -- (4.8, 2);
\draw[thick] (0,0) -- (2, 1) -- (4.8, 1);
\draw[thick] (0,0) -- (2, 0) -- (4.8, 0);
\draw[thick] (0,0) -- (2,-1) -- (4.8,-1);
\node at (0,2.5) (eq1) {$\underbrace{\left\lbrace a,b,c\right\rbrace}_{\lambda}\underbrace{\left\lbrace d,e\right\rbrace}_{\mu}$};

\node at (4.8, 2.8) {$a$};
\node at (4.8, 1.8) {$b$};
\node at (4.8, 0.8) {$c$};
\node at (4.8,-0.2) {$d$};
\node at (4.8,-1.2) {$e$};

\node at (3.4, 3.4)  {$\frac{p_a}{p_a+p_b+p_c+p_d+p_e}$};
\node at (3.4, 2.4)  {$\frac{p_b}{p_a+p_b+p_c+p_d+p_e}$};
\node at (3.4, 1.4)  {$\frac{p_c}{p_a+p_b+p_c+p_d+p_e}$};
\node at (3.4, 0.4)  {$\frac{p_d}{p_a+p_b+p_c+p_d+p_e}$};
\node at (3.4,-0.6)  {$\frac{p_e}{p_a+p_b+p_c+p_d+p_e}$};

\fill (4.8, 3) circle[radius=2pt];  % a finishes; paths bcde
\fill (4.8, 2) circle[radius=2pt];  % terminal node
\fill (4.8, 1) circle[radius=2pt];  % terminal node
\fill (4.8, 0) circle[radius=2pt];  % d finishes; paths abce
\fill (4.8,-1) circle[radius=2pt];  % terminal node

\draw[thick] (4.8,3) -- (5.8,5) -- (8.2,5);
\draw[thick] (4.8,3) -- (5.8,4) -- (8.2,4);
\draw[thick] (4.8,3) -- (5.8,3) -- (8.2,3);
\draw[thick] (4.8,3) -- (5.8,2) -- (8.2,2);

\newcommand{\la}[1]{\lambda{#1}}
\newcommand{\ld}[1]{\mu    {#1}}
\node at (7, 5.4)  {$\frac{\la{p_b}}{\la{p_b}+\la{p_c}+p_d+p_e}$};
\node at (7, 4.4)  {$\frac{\la{p_c}}{\la{p_b}+\la{p_c}+p_d+p_e}$};
\node at (7, 3.4)  {$\frac{p_d     }{\la{p_b}+\la{p_c}+p_d+p_e}$};
\node at (7, 2.4)  {$\frac{p_e     }{\la{p_b}+\la{p_c}+p_d+p_e}$};

\fill (8.2,5) circle[radius=2pt];  % ab finishes; paths cde
\fill (8.2,4) circle[radius=2pt];  % ab finishes; paths cde
\fill (8.2,3) circle[radius=2pt];  % ab finishes; paths cde
\fill (8.2,2) circle[radius=2pt];  % ab finishes; paths cde

\node at (8.2,4.8) {$b$};
\node at (8.2,3.8) {$c$};
\node at (8.2,2.8) {$d$};
\node at (8.2,1.8) {$e$};

\draw[thick] (8.2,5) -- (8.9,6) -- (10.6,6);
\draw[thick] (8.2,5) -- (8.9,5) -- (10.6,5);
\draw[thick] (8.2,5) -- (8.9,4) -- (10.6,4);



\node at (9.5, 6.4) {$\frac{\la{p_c}}{\la{p_c}+p_d+p_e}$};
\node at (9.5, 5.4) {$\frac{p_d  }{\la{p_c}+p_d+p_e}$};
\node at (9.5, 4.4) {$\frac{p_e  }{\la{p_c}+p_d+p_e}$};

\fill (10.6, 6) circle[radius=2pt];  % abc finishes; paths de
\node at (10.6,5.8) {$c$};
\draw[thick] (10.6, 6) -- (11.2,7) -- (12.2,7);
\fill (12.2,7) circle[radius=2pt];
\node at (12.3,6.8) {$d$};
\node[anchor = west] at (12.5,7) {$a\succ b\succ c\succ d\succ e$};
\node at (11.8, 7.4) {$\frac{p_d}{p_d+p_e}$};

\draw[thick] (10.6, 6) -- (11.2,6.2) -- (12.2,6.2);
\fill (12.2,6.2) circle[radius=2pt];
\node at (12.3,6.0) {$e$};
\node[anchor = west] at (12.5,6.2) {$a\succ b\succ c\succ e\succ d$};
\node at (11.8, 6.6) {$\frac{p_e}{p_d+p_e}$};

\fill (10.6, 5) circle[radius=2pt];
\node at (10.6,4.8) {$d$};

\draw[thick] (10.6, 5) -- (11.2,5.3) -- (12.2, 5.3);
\fill (12.2,5.3) circle[radius=2pt];
\node at (12.3,5.1) {$c$};
\node[anchor = west] at (12.5,5.3) {$a\succ b\succ d\succ c\succ e$};
\node at (11.8, 5.7) {$\frac{\la{p_c}}{\la{p_c}+\ld{p_e}}$};

\draw[thick] (10.6, 5) -- (11.2,4.7) -- (12.2, 4.7);
\fill (12.2,4.7) circle[radius=2pt];
\node at (12.3,4.5) {$e$};
\node[anchor = west] at (12.5,4.7) {$a\succ b\succ d\succ e\succ c$};
\node at (11.8, 5.0) {$\frac{\ld{p_e}}{\la{p_c}+\ld{p_e}}$};

\fill (10.6, 4) circle[radius=2pt];
\node at (10.6,3.8) {$e$};
\draw[thick] (10.6, 4) -- (11.2,3.9) -- (12.2, 3.9);
\fill (12.2,3.9) circle[radius=2pt];
\node at (12.3,3.7) {$c$};
\node[anchor = west] at (12.5,3.9) {$a\succ b\succ e\succ c\succ d$};
\node at (11.8, 4.3) {$\frac{\la{p_c}}{\la{p_c}+\ld{p_d}}$};

\draw[thick] (10.6, 4) -- (11.2,3.1) -- (12.2,3.1);
\fill (12.2,3.1) circle[radius=2pt];
\node at (11.8, 3.5) {$\frac{\ld{p_d}}{\la{p_c}+\ld{p_d}}$};
\node at (12.3,2.9) {$d$};
\node[anchor = west] at (12.5,3.1) {$a\succ b\succ e\succ d\succ c$};


\draw[thick] (4.8, 0) -- (5.8,1) -- (8.2,1);
\fill (12.2,1) circle[radius=2pt];
\node at (12.3,0.8) {$e$};
\node[anchor = west] at (12.5,1.0) {$d\succ a\succ c\succ e\succ b$};
\node at (7, 1.4)  {$\frac{p_a}{p_a+p_b+p_c+\ld{p_e}}$};

\draw[thick] (4.8, 0) -- (5.8,0) -- (8.2,0);
\fill (12.2,0) circle[radius=2pt];
\node at (12.3, -0.2) {$b$};
\node[anchor = west] at (12.5, 0) {$d\succ e\succ a\succ b\succ c$};
\node at (7, 0.4)  {$\frac{p_b}{p_a+p_b+p_c+\ld{p_e}}$};

\draw[thick] (4.8, 0) -- (5.8,-1) -- (8.2,-1);
\node at (7, -0.6)  {$\frac{p_c}{p_a+p_b+p_c+\ld{p_e}}$};

\draw[thick] (4.8, 0) -- (5.8,-2) -- (8.2,-2);
\node at (7, -1.6)  {$\frac{\ld{p_e}}{p_a+p_b+p_c+\ld{p_e}}$};

\fill (8.2, -2) circle[radius=2pt];  % de finishes; paths abc
\fill (8.2, -1) circle[radius=2pt];  % de finishes; paths abc
\fill (8.2,  0) circle[radius=2pt];  % de finishes; paths abc

\node at (8.2, -2.2) {$e$};
\node at (8.2, -1.2) {$c$};
\node at (8.2, -0.2) {$b$};
\node at (8.2,  0.8) {$a$};

\draw[thick] (8.2, -2) -- (8.9,-1) -- (10.6,-1);
\draw[thick] (8.2, -2) -- (8.9,-2) -- (10.6,-2);
\draw[thick] (8.2, -2) -- (8.9,-3) -- (10.6,-3);

\fill (12.2,-1) circle[radius=2pt];
\node at (12.3,-1.2) {$c$};
\node[anchor = west] at (12.5,-1) {$d\succ e\succ a\succ c\succ b$};

\node at (9.5, -0.6) {$\frac{p_a}{p_a+p_b+p_c}$};
\node at (9.5, -1.6) {$\frac{p_b}{p_a+p_b+p_c}$};
\node at (9.5, -2.6) {$\frac{p_c}{p_a+p_b+p_c}$};

\fill (10.6, -1) circle[radius=2pt];  % dea finishes; paths bc
\fill (10.6, -2) circle[radius=2pt];  % deb finishes; paths ac
\fill (10.6, -3) circle[radius=2pt];  % dec finishes; paths ab
\node at (10.6,-1.2) {$a$};
\node at (10.6,-2.2) {$b$};
\node at (10.6,-3.2) {$c$};


\draw[thick] (10.6, -1) -- (11.2,-0) -- (12.2,0);

\node at (11.8, 0.4) {$\frac{\la{p_b}}{\la{p_b}+\la{p_c}}$};
\draw[thick] (10.6, -1) -- (11.2,-1) -- (12.2,-1);

\node at (11.8, -0.6) {$\frac{\la{p_c}}{\la{p_b}+\la{p_c}}$};

\draw[thick](10.6, -2) -- (11.2,-1.8) -- (12.2,-1.8);
\fill (12.2,-1.8) circle[radius=2pt];
\node at (12.3, -2.0) {$a$};
\node[anchor = west] at (12.5, -1.8) {$d\succ e\succ b\succ a\succ c$};
\node at (11.8, -1.4) {$\frac{\la{p_a}}{\la{p_a}+\la{p_c}}$};

\draw[thick] (10.6, -2) -- (11.2,-2.6) -- (12.2,-2.6);
\fill (12.2,-2.6) circle[radius=2pt];
\node at (12.3, -2.8) {$c$};
\node[anchor = west] at (12.5,-2.6) {$d\succ e\succ b\succ c\succ a$};
\node at (11.8, -2.2) {$\frac{\la{p_c}}{\la{p_a}+\la{p_c}}$};


%\draw[thick] (10.6, -3) -- (11.2,-2.9);
%\fill (12.2,-2.9) circle[radius=2pt];
%\node at (11.8, -2.9) {$\frac{\la{p_a}}{\la{p_a}+\la{p_b}}$};

\draw[thick] (10.6, -3) -- (11.2,-3.4) -- (12.2, -3.4);
\fill (12.2,-3.4) circle[radius=2pt];
\node at (12.3, -3.6) {$b$};
\node[anchor = west] at (12.5,-3.4) {$d\succ e\succ c\succ b\succ a$};

\node at (11.8, -3) {$\frac{\la{p_b}}{\la{p_a}+\la{p_b}}$};

\fill (8.2, 1) circle[radius=2pt];  % da finishes; paths bce

\draw[thick] (8.2,1) -- (8.9,2) -- (10.6,2);
\draw[thick] (8.2,1) -- (8.9,1) -- (10.6,1);
\draw[thick] (8.2,1) -- (8.9,0) -- (10.6,0);

\node at (9.5, 2.4) {$\frac{\la{p_b}}{\la{p_b}+\la{p_c}+p_e}$};
\node at (9.5, 1.4) {$\frac{\la{p_c}}{\la{p_b}+\la{p_c}+p_e}$};
\node at (9.5, 0.4) {$\frac{p_e     }{\la{p_b}+\la{p_c}+p_e}$};
\fill (10.6, 2) circle[radius=2pt];  % da finishes; paths bce
\fill (10.6, 1) circle[radius=2pt];  % da finishes; paths bce
\fill (10.6, 0) circle[radius=2pt];  % da finishes; paths bce

\node at (10.6, 1.8) {$b$};
\node at (10.6, 0.8) {$c$};
\node at (10.6,-0.2) {$e$};


\draw[thick] (10.6, 1) -- (11.2,2) -- (12.2,2);
\node at (11.8, 2.4) {$\frac{\la{p_b}}{\la{p_b}+\ld{p_e}}$};
\fill (12.2,2) circle[radius=2pt];
\node at (12.3, 1.8) {$b$};
\node[anchor = west] at (12.5, 2) {$d\succ a\succ c\succ b\succ e$};
\draw[thick] (10.6, 1) -- (11.2,1) -- (12.2,1);
\node at (11.8, 1.4) {$\frac{\ld{p_e}}{\la{p_b}+p_e}$};

\end{tikzpicture}
\caption{Partial probability\label{tikzabcde} tree for five
  competitors $a$-$e$ with two mutually supporting subsets
  $\left\lbrace a,b,c\right\rbrace$ and $\left\lbrace d,
  e\right\rbrace$ having support $\lambda$, $\mu$ respectively}
\end{center}
\end{figure}

\begin{equation}\label{asuccb}
\frac{p_a}{p_a + p_b + p_c + p_d + p_e}\cdot
\frac{\lambda p_b}{\lambda p_b + \lambda p_c + p_d + p_e}\cdot
\frac{\lambda p_c}{\lambda p_c + p_d + p_e}\cdot
\frac{p_d}{p_d + p_e}
\end{equation}
%
In Equation \ref{asuccb}, the first term is standard Plackett-Luce: at
this point, no competitor has finished and cheering effects are
absent.  The second term reflects the fact that competitors $b$ and
$c$ are supported by competitor $a$, who has by this point finished
the race and is supporting his teammates.

By contrast, the likelihood function for observation $d\succ a\succ
c\succ b\succ e$ would be
%
\begin{equation}
\frac{p_d}{p_a + p_b + p_c + p_d + p_e}\cdot
\frac{p_a}{p_a + p_b + p_c + \mu p_e}\cdot
\frac{\lambda p_c}{\lambda p_b + \lambda p_c + \mu p_e}\cdot
\frac{\lambda p_b}{\lambda p_b + \mu p_e}
\end{equation}
%
where this likelihood function reflects the mutual support for
equivalence class $\left\lbrace d,e\right\rbrace$.  Note that the
final term reflects the fact that competitors $b$ and $e$ have their
support active when vying for fourth place, as members of both their
teams have finished at this point.

This probability model could easily be modified to account for
specific circumstances.  The cheering effect could be asymmetrical
(with $a$ helping $b$ but not vice-versa, for example).  The effect
might operate only on certain ordered pairs, following a directed
graph (cyclic or acyclic) instead of equivalence classes.  Or perhaps
the effect might have a finite lifetime: if $a$ places
$n^\mathrm{th}$, then the cheering effect is active only for
competitors placing $(n+r)^\mathrm{th}$ or above, for some fixed $r$.
There is no reason that $\lambda$ could not be less than one, which
would indicate that some competitors inhibit, rather than help,
others.

The probability model of Figure~\ref{tikzabcde} is applied, in a
simple one-parameter form, to two datasets: one a set of preferences
of different types of sushi, and one in the field of esports.  First,
however, we discuss a synthetic example showing how generalizing
Placket-Luce to admit non-independence of successive arrivals can be
applied to an apparently different situation.

\subsection{Red bus, blue bus}

The classic red bus-blue bus problem was originally stated by
\cite{mcfadden1980} although different metaphors have been employed
for the same phenomenon by \cite{tverskey1972_jss}.  The canonical
problem is as follows.  A commuter has a choice of going to work by
different transport methods.  Here, we can assume that he has possibly
strong---but unknown---views on different forms of transport which
might incorporate a random component.  If considering whether to use a
car or a bus, the colour of the bus is of no importance to him.  Such
preferences are problematic for Bradley-Terry: If our subject is
indifferent between cars and buses, then $p_{C} = p_{RB}$.  If he is
also indifferent to colour of bus, so $p_{RB}=p_{BB}$, the following
paradox occurs.  Bradley-Terry strengths must be equal so $p_{C} =
p_{RB} = p_{BB}=\frac{1}{3}$.  Thus $\Prob(C| \left\lbrace
C,RB\right\rbrace)=\frac{1}{2}$.  Now we introduce a blue bus as an
option.  Any realistic choice model will have $\Prob(C | \left\lbrace
C,RB,BB\right\rbrace)=\frac{1}{2}$ but Bradley-Terry gives us
$\frac{1}{3}$.  Various solutions to this problem have been proposed.
\cite{tverskey1972_jss}, for example, presents an approach involving
overlapping ``aspects'' of the choice set; \cite{mcfadden1980}
suggests using a generalized extreme-value model.

Here I suggest an alternative in the form of a one-parameter
generalization of Plackett-Luce likelihood functions for ranking
options, using the multiplicative generalization to Bradley-Terry
strengths presented above.  As a synthetic example, we specify options
RB and BB for red bus and blue bus together with options C, T and W
for car, train, and walking.

Specifically, it seems reasonable to assume that if we ask our
respondent to rank his options, it is highly probable that he will put
RB and BB consecutively (because they are essentially
indistinguishable).  Can we quantify the strength of this effect?  To
do this, we define a bespoke function \fct{RB\_BB\_LF} which
returns a \code{hyper3} log-likelihood function corresponding to
repeated observations of our commuter's reported ranks for the five
options.
%
<<defineredbuslf>>=
`RB_BB_LF` <- function(lambda){
    ec <- c(C = 1, T = 2, RB = 3, BB = 3, W = 4) 
    h <- c(1, 1, lambda, 1)           
    (
        cheering3(v=c("RB", "BB", "C" , "T", "W"), e = ec, h = h) * 3 + 
        cheering3(v=c("BB", "RB", "T" , "C", "W"), e = ec, h = h) * 2 + 
        cheering3(v=c("T" , "BB", "RB", "C", "W"), e = ec, h = h) * 5 + 
        cheering3(v=c("W" , "BB", "RB", "T", "C"), e = ec, h = h) * 4 + 
        cheering3(v=c("C" , "RB", "BB", "W", "T"), e = ec, h = h) * 3 + 
        cheering3(v=c("BB", "C" , "RB", "T", "W"), e = ec, h = h) * 3
    )
}
@
%
Above, we see from the function body that he reported $RB\succ BB\succ
C\succ T\succ W$ three times [first row], and so on; perhaps his
ranking depends on the weather or how tired he is on any given day.
Vector \code{ec} places the options into equivalence classes;
equivalence class 3 comprises the two buses and the other classes are
unsupported as they are of size one.  Vector \code{h} encodes the
strength of the nonindependence.  Observe that in almost every case he
ranks RB and BB consecutively.  Function \fct{RB\_BB\_LF} takes
argument \code{lambda} that quantifies the perceived similarity
between RB and BB in the same way as function \fct{oldfirm} above.
%
<<definefunctiono>>=
o <- function(lambda){maxp(RB_BB_LF(lambda), give = 1)$`Log-likelihood`} 
@ 
%
Above, we define function \fct{o} which returns the support for a
given value of $\lambda$ at the evaluate [over permissible Bradley
  Terry strengths for the options].  We can treat \fct{o} as a
support function for $\lambda$, and then maximize it:
%
<<label = maxlikesim, cache = TRUE>>=
(osup <- optimize(o, c(10, 40), maximum = TRUE))
@ 
%
So a likelihood ratio test of the null that $\lambda=1$ would be:
%
<<label = LRT_RB, cache = TRUE>>=
(suppdiff <- osup$objective - o(lambda = 1))
@
%
We can thus improve the hypothesis that $\lambda=1$, corresponding to
an unaltered Plackett-Luce system, by over 5.6 units of support by
dint of choosing instead $\lambda = 10.49$.  This easily satisfies
\citeauthor{edwards1992}'s two-units-of-support criterion;
alternatively, we could observe that this observation lies in the tail
region of its asymptotic $\chi^2_1$ distribution (Wilks's theorem),
with a $p$~value of \Sexpr{signif(pchisq(suppdiff*2,df=1,lower.tail=FALSE),3)}.
%
%It is straightforward to find a profile
%support function for $\lambda$, Figure~\ref{profilesupportlambdaRBBB}.
%We see that support of $\log\lambda$ is closely quadratic over a broad
%range.
%
%\begin{figure}[htbp]
%\begin{center}
%<<plotproflike, echo = FALSE>>=
%if(FALSE){
%    lambda <- exp(seq(from = log(0.9), to = log(67), len = 19)) 
%    L <- sapply(lambda, o)
%    plot(log(lambda), L - max(L), type = "b")
%    abline(h = c(0, -2))
%    abline(v = 0)
%}
%@ 
%\caption{A\label{profilesupportlambdaRBBB} profile support function
%  for the red bus, blue bus parameter $\lambda$, synthetic dataset.
%  Vertical line indicates null of $\lambda=1$, horizontal lines show
%  maximum support and $-2$}
%\end{center}
%\end{figure}

\subsection{Sushi}

We now apply the ideas above to a dataset obtained from the {\tt
  preflib} library~\citep{MaWa13a} in which ten different types of
sushi were placed in order of preference by 5000 judges.  Three of the
sushis (maguro, toro, tekka-maki) were made with tuna, and it is
reasonable to view these three types as being exchangeable in much the
same way as the red and blue buses.  To proceed, we need to define
equivalence classes of sushi---one for tuna-based sushis and one for
everything else.  Of course, one can also imagine the opposite
phenomenon whereby the judge decides he has had enough tuna for the
moment and this disincentivises further tuna choices.  This mechanism
would be modelled by having $\lambda<1$.  Implementation in
\code{hyper3} formalism is straightforward.  The dataset used is
\code{sushi\_table}, part of the \pkg{hyper2} package:
%
<<showsushitable>>=
noquote(head(sushi_table))
@
%
Thus, from the top line, we see that the first judge placed tamago
first, followed by anago and ika.  The equivalence classes are given
by
%
<<show_sushi_eq_classes>>=
sushi_eq_classes
@
%
so class 2 is the tuna group and class 1 everything else.  We can now
define a bespoke likelihood function for these observations, using
$\lambda$ as the argument:
%
<<defcalculate_sushi_H>>=
make_sushi_H <- function(lambda){
    H <- hyper3()
    for(i in seq_len(nrow(sushi_table))){
        H <- H + cheering3(sushi_table[i,], e = sushi_eq_classes, h = c(1, lambda))
    }
    return(H)
}
@
%
<<calclikesushi, echo = FALSE, cache = TRUE>>=
f <- function(lambda){
   H <- make_sushi_H(lambda)
   maxp(H, give = 1)$`Log-likelihood`
}
s <- exp(seq(from = log(0.9), to = log(10), len = howmany))
L <- sapply(s, f)
@ 
%
% chunk 'findstuffsushi' takes about 10-20 minutes to run
%
<<findstuffsushi, echo = FALSE, cache = TRUE>>=
osushi <- optimize(f, c(1, 4), maximum = TRUE)
@ 
%
% chunk tinysushi' takes about 5 minutes to run
%
<<tinysushi, echo = FALSE, cache = TRUE>>=
sushimaxlambda <- osushi$maximum
sushisupport <- osushi$objective - f(1)
@
%
% chunk 'findotherstuffsushi' takes about 10 minutes to run
%
<<findothersushistuff, echo = FALSE, cache = TRUE>>=
g <- function(lambda){f(lambda) - osushi$objective + 2}
sushi_lower <- uniroot(g, interval = exp(c(0, 1)), tol = 0.01)$root
sushi_upper <- uniroot(g, interval = exp(c(1, 2)), tol = 0.01)$root
@ 
%
(above, we note that the return value {\tt H} is the sum of successive
log-likelihood functions of each row of the table, assumed to be
independent).  Then, as before, we may define a profile likelihood
function of $\lambda$, Figure~\ref{profilesupportsushi}.  We
reject~$H_0\colon\lambda=1$ in favour of
$H_A\colon\lambda=\Sexpr{round(sushimaxlambda,2)}$ with about
$\Sexpr{round(sushisupport,2)}$ units of support.  Alternatively, we
may observe that this lies in the tail region of its asymptotic
$\chi^2_1$ distribution with a $p$~value of about
$\Sexpr{round(pchisq(2*sushisupport,df=1,lower.tail=FALSE),4)}$.  We can
therefore be reasonably sure that tuna sushi are somehow analagous to
red and blue buses, in the sense that all three tuna items tend to
occupy closely separated places in judges' preference ranks.

\begin{figure}[htbp]
  \begin{center}
<<plotlikesushi, echo = FALSE, fig = TRUE>>=
plot(log(s), L - max(L), type = 'b',
     xlab = expression(log(lambda)), ylab = 'Log-likelihood')
abline(v = 0, col = 'gray')
abline(h = c(0, -2), col = 'gray')
segments(x0 = log(sushimaxlambda), y0 =  0.1, y1 = -1.0, col = 'red' )
segments(x0 = log(sushi_lower    ), y0 = -2.2, y1 = -1.8, col = 'blue')
segments(x0 = log(sushi_upper    ), y0 = -2.2, y1 = -1.8, col = 'blue')
@ 
\caption{A\label{profilesupportsushi} profile support function for the
  exchangeability term $\lambda$ in the case of sushi preferences.
  Vertical gray line shows $H_0\colon\lambda =1$, horizontal lines
  show the two-units-of-support limits with the credible interval of
  $(\Sexpr{round(sushi_lower,2)},\Sexpr{round(sushi_upper,2)})$,
  marked in blue.  Red bar shows evaluate at
  $\lambda\simeq\Sexpr{round(sushimaxlambda,2)}$}
\end{center}
\end{figure}


\subsection{Counterstrike}

\newcommand{\skad}{{\sc Skadoodle}}
\newcommand{\rush}{{\sc RUSH}}
\newcommand{\tarik}{{\sc tarik}}
\newcommand{\stew}{{\sc Stewie2K}}

{\em Counter Strike: Global Offensive (CS:GO)} is a multiplayer
first-person shooter game in which two teams of five compete in an
immersive virtual reality combat environment.  Gameplay is typically
complex, including features such as cooperative teamwork and real-time
strategic fluidity.

Here I analyse a dataset taken from recent esports tournament {\em
  ELEAGUE Major: Boston 2018}.  This Valve-sponsored Major featured 24
elite teams competing for a million dollar prize pool; peak concurrent
viewership exceeded $1.3\times 10^6$, the highest in {\em CS:GO}
history at the time.  I study the performance of tournament champions
{\em Cloud9} who, as underdogs, narrowly defeated aggressive European
rivals {\em FaZe Clan} in the final.

Typically when a player is killed, the team suffers a number of
strategic disadvantages; player deaths are thus a quantifiable proxy
for both individual ability and also both teams' tactical execution.
Here I analyse the order statistic for number of deaths of each of the
five {\em Cloud9} players.
%
<<echo = FALSE>>=
a <- read.table("cloud9_boston_2018.txt", header = TRUE)
a[, -1]
@
%
From the first line we see that {\em Cloud9} played {\em FaZe Clan} on
the {\em inferno} map, won the match, and \skad\ had the least number
of deaths, followed by \rush, then \tarik, and so on.  Consider two
{\em Cloud9} players: \skad\ and \stew.  These teammates, although
comparable in skill [we fail to reject $H_0\colon p_{\mbox{\tiny\sc
      Skadoodle}} = p_{\mbox{\tiny\sc Stewie2k}}$, Plackett-Luce,
  $p=0.28$] have playing styles that stand in dramatic contrast.
Commentators characterise \skad\ as emphasising reactive map control
while \stew\ is more of a disruptive presence engaging in
high-variance play.

Given that these two players offer equally attractive and accessible
targets to the opposing team, it is possible that some tactical
situations might result in a systematic preference for targetting
\skad, and some a preference for killing \stew.  If this is so, the
deaths will be overdispersed and one would have a red bus-blue bus
situation, but with $\lambda<1$.
%
<<defredbusfunc, echo = FALSE>>=
mk <- function(lam){
    e <- c(Skadoodle = 2, RUSH = 1, tarik = 1, autimatic = 1, Stewie2K = 2)
    H3 <- hyper3()
    for(i in seq_len(nrow(a))){
        d <- strsplit(a$deathorder[i],", ")[[1]]
        H3 <- H3 + cheering3(d, e = e, help = c(1, lam))
    }
    return(H3)
}
f <- function(l){maxp(mk(l), give=1)$`Log-likelihood`}
@
%
<<csgo_findopt, cache = TRUE, echo = FALSE>>=
csgo_optimize <- optimize(f, c(0.09, 0.14), maximum = TRUE, tol = 0.01)
csgo_optlam   <- csgo_optimize$maximum
csgo_support  <- csgo_optimize$objective - f(1)
csgo_pvalue   <- pchisq(csgo_support, df = 1, lower.tail = FALSE)
@
%
<<csgo_calcklikelambda, cache=TRUE, echo=FALSE>>=
lam <- exp(seq(from = -4, to = log(1.1), len = howmany))
L <- sapply(lam, f)
@
%
<<cgso_findinterval, cache=TRUE, echo=FALSE>>=
g <- function(l){f(l) - csgo_optimize$objective  + 2}
csgo_lower <- uniroot(g, interval = exp(c(-4, -3)), tol = 0.01)$root
csgo_upper <- uniroot(g, interval = exp(c(-1, -0)), tol = 0.01)$root
@ 

\begin{figure}[htbp]
  \begin{center}
<<csgo_plotlikelam, echo = FALSE, fig = TRUE>>=
plot(log(lam), L - max(L), type='b')
segments(x0 = log(csgo_optlam),y0 = -0.5, y1 =  0.1, col = 'red')
segments(x0 = log(csgo_lower), y0 = -2.2, y1 = -1.8, col = 'blue')
segments(x0 = log(csgo_upper), y0 = -2.2, y1 = -1.8, col = 'blue')
abline(v = 0, col = 'gray')
abline(h = c(0, -2), col = 'gray')
@ 
\caption{A\label{profsupcounterstrike} profile support function for
  the red bus-blue bus parameter $\log\lambda$ applied to mutually
  supporting equivalence class
  $\left\lbrace\mbox{\skad},\mbox{\stew}\right\rbrace$.  Vertical gray
  line shows $H_0\colon\lambda =1$, for which the support is about
  $\Sexpr{round(csgo_support,2)}$ relative to the evaluate
  $\hat{\lambda}\simeq\Sexpr{round(csgo_optlam,3)}$ (small red bar),
  corresponding to a $p$~value of about
  $\Sexpr{round(csgo_pvalue,3)}$.  Horizontal lines show the
  two-units-of-support credible interval of
  $(\Sexpr{round(csgo_lower,3)},\Sexpr{round(csgo_upper,3)})$, marked
  in blue}
\end{center}
\end{figure}

To assess this, we follow exactly the same protocol as for the sushi
preference dataset but using the deathorder statistic from the dataset
above.  We specify two equivalence classes: one comprising \skad\ and
\stew, and one everyone else.  Figure~\ref{profsupcounterstrike} gives
a support function for $\lambda$ showing, among other things, that we
may reject the hypothesis that $\lambda=1$ in favour of $\lambda<1$ by
a likelihood ratio of about $\Sexpr{round(csgo_support,2)}$.  We
observe that this exceeds the two units of support criterion of
\cite{edwards1992} [asymptotic $p$~value
  $\simeq\Sexpr{round(csgo_pvalue,3)}$] so we may infer that there is
indeed some systematic mechanism---perhaps a positive feedback
loop---which tends to magnify differences in death statistic.

\section{Conclusions and further work}

Bradley-Terry strengths $p_1,\ldots,p_n$, $p_i\geqslant 0$, $\sum
p_i=1$ are generalized with non-negative multiplicative $\lambda$-
factors that represent temporary or ephemeral changes in competitive
strength.  The \pkg{hyper2} R package implements the resulting
likelihood functions using \proglang{S3} class {\tt hyper3}, and
furnishes a wide range of functionality geared towards assessing
meaningful nulls using likelihood methods.

The standard Plackett-Luce likelihood function
$\prod_{i=1}^{n}p_{(i)}\left[\sum_{j=i}^n p_{(j)}\right]^{-1}$ over
Bradley-Terry strengths assumes conditional independence of successive
rank observations $p_{(1)},\ldots p_{(n)}$, an assumption which is
broken in the current work.  As indicated above, multiplicative
$\lambda$- factors may be used to implement many different
non-independence schemes.

Further work might include an individualised multiplicative draw
factor, giving rise to likelihood functions such as
$\frac{p_1^{w_1};(\lambda_1p_1+\lambda_2p_2)^{d_{12}};
  p_2^{w_2}}{\left(p_1(1+\lambda_1)+p_2(1+\lambda_2)\right)^{w_1+d_{12}+w_2}}$.
One might also consider multiplicative factors for the strengths of
{\em reified} entities.  These modifications would be directly
representable with existing {\tt hyper3} idiom.

\bibliography{hyper2}
\end{document}

---
title: "exponential Bradley-Terry: likelihood-based methods on a grid of values"
author: "robin hankin"
date: "2025-04-02"
output: html_document

---

```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library(hyper2)
library(stringr)
library("partitions")
library("magrittr")
```

<font size="1">
(takes maybe twenty minutes to run without cach at $N=9$)
</font>

First we define `N`, the number of competitors.  This is used
consistently in the whole Rmd file, except for the first few bits
where I hard-code five or 11 competitors for ease of exposition.  In
principle, the Rmd file should process whatever the value of `N`.

```{r defineN}
N <- 9  # number of competitors
```


Now we define `BT()` which specifies Bradley-Terry strengths $\alpha x^i$:

```{r defineBT}
BT <- function(n, x){
    out <- x^(0:(n-1))
    if(x != 1){
        out <- out*(1-x)/(1-x^n)
    } else {
        out <- out/n
    }
    names(out) <- str_pad(1:n, width=ceiling(log10(n)), pad = "0")
    out
}
BT(4,0.5)
sum(BT(4,0.5))
```

We will consider $N=9$, $x=0.5$, $r=4$ and make repeated observations:


```{r showtabulate}
makeracetable <- function(no_of_races, no_of_competitors, x){
    t(replicate(no_of_races,
                as.numeric(rrace(BT(no_of_competitors, x)))))
}

makeracetable(no_of_races=10, no_of_competitors=5, x=0.7)
```


```{r, label=definegetonecompetitor}
get_one_competitor <- function(racetable, competitor){
    jj <- apply(racetable, 1, function(x){which(x==competitor)})
    tabulate(jj, ncol(racetable))
}
```


```{r, label=exampleracetable, cache=TRUE}
X <- makeracetable(1e4,11,0.7)  # using N=11 here
get_one_competitor(X,4)
get_one_competitor(X,11)

rbind(
get_one_competitor(X,1) ,
get_one_competitor(X,2) ,
get_one_competitor(X,9) ,
get_one_competitor(X,10)
)

t(sapply(seq_len(11),function(i){get_one_competitor(X,i)}))
```


OK now to put above code in to functional form:

```{r funcformdef}
get_all_competitors <- function(no_of_races, no_of_competitors, x){
    X <- makeracetable(no_of_races=no_of_races, no_of_competitors=no_of_competitors, x=x)
    out <- t(sapply(seq_len(no_of_competitors),function(i){get_one_competitor(X,i)}))
    dimnames(out) <- list(
        true_rank = paste("r_", seq_len(no_of_competitors), sep=""),
        observed_rank = paste("c_", seq_len(no_of_competitors), sep=""))
    return(out/no_of_races)
}
```

Now use it:

```{r useit, cache=TRUE}
get_all_competitors(100, N, 0.5)
```

Above, we see from the first line that, with a sample of 100 races,
the top competitor comes first with (estimated) probability about
0.XXXX, second with probability abot 0.XXX, and so on, coming fifth with
probability 0.XXXXX

```{r useit09, cache=TRUE}
get_all_competitors(100, N, 0.99)
```

Above, all competitors are about the same.  We see a more uniform
spread of observations.

```{r useit01, cache=TRUE}
get_all_competitors(100, N, 0.1)
```

Above, each competitor is ten times as good as the next one.  We see
underdispersed results.  So what we need to do now is a "meaningful
grid".  This is somewhat computationally intensive.


```{r definegrid}
no_of_races <- 1000
no_of_competitors <- N

true_x <- seq(from=0.1, by=0.1, to=0.9)
```

First, we combine a whole bunch of X matrices together:

```{r defineallX}
allX <- function(no_of_competitors, no_of_races, true_x){
    out <- NULL
    for(i in seq_along(true_x)){
        out <- rbind(out,
                     cbind(
                         true_x[i],
                         seq_len(no_of_competitors),
                         get_all_competitors(
			 no_of_races = no_of_races,
			 no_of_competitors = no_of_competitors,
			 true_x[i])
                     )
                     )
    }
    rownames(out) <- NULL
    jj <- colnames(out)
    jj[1:2] <- c("x","r")
    colnames(out) <- jj
    return(out)
}
```

```{r makeallX9, cache=TRUE}
allX9 <- allX(
          no_of_competitors = N,
          no_of_races = 50000,
          true_x = seq(from=0.1, to=0.9, by=0.1)
	  )
```

```{r}
head(allX9)
allX9[seq(from=round(nrow(allX9)/2 - 5), to=round(nrow(allX9)/2 + 5)),]
tail(allX9)
```

Now some observations:


```{r defmakeobs}
make_obs <- function(no_of_observations, no_of_competitors, r_true, x_true){
    kk <- makeracetable(no_of_races = no_of_observations, no_of_competitors = no_of_competitors, x = x_true)
    obs <- tabulate(apply(kk,1,function(o){which(o == r_true)}), nbins=no_of_competitors)
    return(obs)
}
```

```{r tableev, cache=TRUE}
make_obs(no_of_observations=20, no_of_competitors=N, x_true = 0.7, r_true = 6)
make_obs(no_of_observations=20, no_of_competitors=N, x_true = 0.7, r_true = 9)
make_obs(no_of_observations=20, no_of_competitors=N, x_true = 0.1, r_true = 6)


makemaxliketable <- function(n, nobs, no_of_competitors, allX, r_true, x_true){

    maxlike <- function(obs){
        support <- function(i){dmultinom(obs, prob=allX[i, -(1:2)], log=TRUE)}
        p <- sapply(seq_len(nrow(allX)), support)
        allX[which.max(p), 1:2]
    }

    evaluate <- matrix(NA,n,2)
    for(i in seq_len(n)){
        obs <- make_obs(no_of_observations=nobs, no_of_competitors=no_of_competitors, x_true=x_true, r_true=r_true)
        evaluate[i,] <- maxlike(obs)
    }
    return(table(x=evaluate[,1], r=evaluate[,2]))
}
```


```{r label=definemean}
biasx <- function(liketable, x_true){
    values <- as.numeric(rownames(liketable))
    probs <- rowSums(liketable) / sum(liketable) # sum(probs) == 1
    sum(probs*(values - x_true))
}

biasr <- function(liketable, r_true){
    values <- as.numeric(colnames(liketable))
    probs <- colSums(liketable) / sum(liketable) # sum(probs) == 1
    sum(probs*(values - r_true))

}

msex <- function(liketable, x_true){
    values <- as.numeric(rownames(liketable))
    probs <- rowSums(liketable) / sum(liketable) # sum(probs) == 1
    sum(probs*(values - x_true)^2)
}

mser <- function(liketable, r_true){
    values <- as.numeric(colnames(liketable))
    probs <- colSums(liketable) / sum(liketable) # sum(probs) == 1
    sum(probs*(values - r_true)^2)
}
```

```{r label=usebias, cache=TRUE}
(jj <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.8))
biasx(jj, x_true = 0.8)
biasr(jj, r_true = 4)
msex(jj, x_true = 0.8)
mser(jj, r_true = 4)
```

### Vary nothing

Just do the same thing a few times to gauge the variability in the system:

```{r varynothingH, cache=TRUE}
jj1 <- makemaxliketable(n=100, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj2 <- makemaxliketable(n=100, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj3 <- makemaxliketable(n=100, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj4 <- makemaxliketable(n=100, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj5 <- makemaxliketable(n=100, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj6 <- makemaxliketable(n=100, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj7 <- makemaxliketable(n=100, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj8 <- makemaxliketable(n=100, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj9 <- makemaxliketable(n=100, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj1
jj2

```{r varynothingbiasmseH, cache=TRUE}
fish <-
rbind(
c(biasr(jj1, 4), mser(jj1, 3.8), biasx(jj1, 0.7), msex(jj1, 0.7)),
c(biasr(jj2, 4), mser(jj2, 3.8), biasx(jj2, 0.7), msex(jj2, 0.7)),
c(biasr(jj3, 4), mser(jj3, 3.8), biasx(jj3, 0.7), msex(jj3, 0.7)),
c(biasr(jj4, 4), mser(jj4, 3.8), biasx(jj4, 0.7), msex(jj4, 0.7)),
c(biasr(jj5, 4), mser(jj5, 3.8), biasx(jj5, 0.7), msex(jj5, 0.7)),
c(biasr(jj6, 4), mser(jj6, 3.8), biasx(jj6, 0.7), msex(jj6, 0.7)),
c(biasr(jj7, 4), mser(jj7, 3.8), biasx(jj7, 0.7), msex(jj7, 0.7)),
c(biasr(jj8, 4), mser(jj8, 3.8), biasx(jj8, 0.7), msex(jj8, 0.7)),
c(biasr(jj9, 4), mser(jj9, 3.8), biasx(jj9, 0.7), msex(jj9, 0.7))
)
colnames(fish) <- c("bias(r)", "mse(r)", "bias(x)", "mse(x)")
```

```{r}
fish
mean(fish[,1])
mean(fish[,2])
sd(fish[,1])
mean(fish[,2])/sqrt(100)

```
```{r varynothingTH, cache=TRUE}
jj1 <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj2 <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj3 <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj4 <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj5 <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj6 <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj7 <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj8 <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj9 <- makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
jj1
jj2

```{r varynothingbiasmseTH, cache=TRUE}
fish <-
rbind(
c(biasr(jj1, 4), mser(jj1, 3.8), biasx(jj1, 0.7), msex(jj1, 0.7)),
c(biasr(jj2, 4), mser(jj2, 3.8), biasx(jj2, 0.7), msex(jj2, 0.7)),
c(biasr(jj3, 4), mser(jj3, 3.8), biasx(jj3, 0.7), msex(jj3, 0.7)),
c(biasr(jj4, 4), mser(jj4, 3.8), biasx(jj4, 0.7), msex(jj4, 0.7)),
c(biasr(jj5, 4), mser(jj5, 3.8), biasx(jj5, 0.7), msex(jj5, 0.7)),
c(biasr(jj6, 4), mser(jj6, 3.8), biasx(jj6, 0.7), msex(jj6, 0.7)),
c(biasr(jj7, 4), mser(jj7, 3.8), biasx(jj7, 0.7), msex(jj7, 0.7)),
c(biasr(jj8, 4), mser(jj8, 3.8), biasx(jj8, 0.7), msex(jj8, 0.7)),
c(biasr(jj9, 4), mser(jj9, 3.8), biasx(jj9, 0.7), msex(jj9, 0.7))
)
colnames(fish) <- c("bias(r)", "mse(r)", "bias(x)", "mse(x)")
```

```{r}
fish
mean(fish[,1])
mean(fish[,2])
sd(fish[,1])
mean(fish[,2])/sqrt(1000)

```

Above we have that the bias of $r$ has a standard deviation, given
(approximately at least) by the standard error of the mean.



### Vary `x_true`

```{r execuA, cache=TRUE}
vxt <- list(
makemaxliketable(n=4000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.1),
makemaxliketable(n=4000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.2),
makemaxliketable(n=4000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.3),
makemaxliketable(n=4000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.4),
makemaxliketable(n=4000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.5),
makemaxliketable(n=4000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.6),
makemaxliketable(n=4000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7),
makemaxliketable(n=4000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.8),
makemaxliketable(n=4000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.9))
```


```{r}
jj <- rbind(
c(biasr(vxt[[1]], 4), biasx(vxt[[1]], 0.1), mser(vxt[[1]], 4), msex(vxt[[1]], 0.1)),
c(biasr(vxt[[2]], 4), biasx(vxt[[2]], 0.2), mser(vxt[[2]], 4), msex(vxt[[2]], 0.2)),
c(biasr(vxt[[3]], 4), biasx(vxt[[3]], 0.3), mser(vxt[[3]], 4), msex(vxt[[3]], 0.3)),
c(biasr(vxt[[4]], 4), biasx(vxt[[4]], 0.4), mser(vxt[[4]], 4), msex(vxt[[4]], 0.4)),
c(biasr(vxt[[5]], 4), biasx(vxt[[5]], 0.5), mser(vxt[[5]], 4), msex(vxt[[5]], 0.5)),
c(biasr(vxt[[6]], 4), biasx(vxt[[6]], 0.6), mser(vxt[[6]], 4), msex(vxt[[6]], 0.6)),
c(biasr(vxt[[7]], 4), biasx(vxt[[7]], 0.7), mser(vxt[[7]], 4), msex(vxt[[7]], 0.7)),
c(biasr(vxt[[8]], 4), biasx(vxt[[8]], 0.8), mser(vxt[[8]], 4), msex(vxt[[8]], 0.8)),
c(biasr(vxt[[9]], 4), biasx(vxt[[9]], 0.9), mser(vxt[[9]], 4), msex(vxt[[9]], 0.9))
)
colnames(jj) <- c("bias(r)","bias(x)", "mse(r)", "mse(x)")
rownames(jj) <- seq(from=0.1,to=0.9,by=0.1)
jj
matplot(jj,type="b")
cbind(`bias(r)`=jj[,1], `SE(bias)` = jj[,3]/sqrt(4000))
cbind(`bias(x)`=jj[,2], `SE(bias)` = jj[,4]/sqrt(4000))
```


Above we see  that varying `x_true` from 0.1 to 0.9 increases the
variance of $r_\text{estimated}$

### Vary `r_true`

```{r execuB, cache=TRUE}
VR <- list(
makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 1, x_true = 0.7),
makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 2, x_true = 0.7),
makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 3, x_true = 0.7),
makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7),
makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 5, x_true = 0.7),
makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 6, x_true = 0.7),
makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 7, x_true = 0.7),
makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 8, x_true = 0.7),
makemaxliketable(n=1000, nobs=10, no_of_competitors=N, allX=allX9, r_true = 9, x_true = 0.7)
)
```

```{r showrvary, cache=TRUE}
jj <- rbind(
c(biasr(VR[[1]], 1), biasx(VR[[1]], 0.7), mser(VR[[1]], 1), msex(VR[[1]], 0.7)),
c(biasr(VR[[2]], 2), biasx(VR[[2]], 0.7), mser(VR[[2]], 2), msex(VR[[2]], 0.7)),
c(biasr(VR[[3]], 3), biasx(VR[[3]], 0.7), mser(VR[[3]], 3), msex(VR[[3]], 0.7)),
c(biasr(VR[[4]], 4), biasx(VR[[4]], 0.7), mser(VR[[4]], 4), msex(VR[[4]], 0.7)),
c(biasr(VR[[5]], 5), biasx(VR[[5]], 0.7), mser(VR[[5]], 5), msex(VR[[5]], 0.7)),
c(biasr(VR[[6]], 6), biasx(VR[[6]], 0.7), mser(VR[[6]], 6), msex(VR[[6]], 0.7)),
c(biasr(VR[[7]], 7), biasx(VR[[7]], 0.7), mser(VR[[7]], 7), msex(VR[[7]], 0.7)),
c(biasr(VR[[8]], 8), biasx(VR[[8]], 0.7), mser(VR[[8]], 8), msex(VR[[8]], 0.7)),
c(biasr(VR[[9]], 9), biasx(VR[[9]], 0.7), mser(VR[[9]], 9), msex(VR[[9]], 0.7))
)
colnames(jj) <- c("bias(r)","bias(x)", "mse(r)", "mse(x)")
rownames(jj) <- seq(from=0.1,  to=0.9, by=0.1)
jj
matplot(jj,type="b")
cbind(`bias(r)`=jj[,1], `SE(bias)` = jj[,3]/sqrt(1000))
cbind(`bias(x)`=jj[,2], `SE(bias)` = jj[,4]/sqrt(1000))
```

### Vary `nobs`


```{r execuC, cache=TRUE}
VN <- list(
makemaxliketable(n=1000, nobs= 10, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7),
makemaxliketable(n=1000, nobs= 20, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7),
makemaxliketable(n=1000, nobs= 50, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7),
makemaxliketable(n=1000, nobs=100, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7),
makemaxliketable(n=1000, nobs=200, no_of_competitors=N, allX=allX9, r_true = 4, x_true = 0.7)
)
```


```{r execvarynobs, cache=TRUE}
jj <- rbind(
c(biasr(VN[[1]], 4), biasx(VN[[1]], 0.7), mser(VN[[1]], 4), msex(VN[[1]], 0.7)),
c(biasr(VN[[2]], 4), biasx(VN[[2]], 0.7), mser(VN[[2]], 4), msex(VN[[2]], 0.7)),
c(biasr(VN[[3]], 4), biasx(VN[[3]], 0.7), mser(VN[[3]], 4), msex(VN[[3]], 0.7)),
c(biasr(VN[[4]], 4), biasx(VN[[4]], 0.7), mser(VN[[4]], 4), msex(VN[[4]], 0.7)),
c(biasr(VN[[5]], 4), biasx(VN[[5]], 0.7), mser(VN[[5]], 4), msex(VN[[5]], 0.7))
)
colnames(jj) <- c("bias(r)","bias(x)", "mse(r)", "mse(x)")
rownames(jj) <- c(10,20,50,100,200)
jj
matplot(jj,type="b")
cbind(`bias(r)`=jj[,1], `SE(bias)` = jj[,3]/sqrt(1000))
cbind(`bias(x)`=jj[,2], `SE(bias)` = jj[,4]/sqrt(1000))
```

---
title: "Eurovision Song Contest: collusion 2022 USING HYPER3"
output: bookdown::html_document2
author: "R. K. S. Hankin"
bibliography: hyper2.bib
---


```{r setup, include=FALSE}
knitr::opts_chunk$set(echo = TRUE)
library("hyper2")
library("magrittr")
options("digits" = 5)
abbreviated <- TRUE  # change to FALSE for full names
```

```{r out.width='20%', out.extra='style="float:right; padding:10px"',echo=FALSE}
knitr::include_graphics(system.file("help/figures/hyper2.png", package = "hyper2"))
```

<font size="1"> (takes five minute to run without cache; the time is spent in chunk `dotest`)
</font>


To cite the `hyper2` package in publications, please use
@hankin2017_rmd.  The file `eurovision2022_jury_pre_null.txt`, used
below, refers to the jury vote for the 2022 Eurovision Song contest,
pre-nullification; data supplied by Riley Uttley.
First we specify the matrix as appearing in the Wikipedia page:

```{r, specifymatrix}
eurovision_table <- as.matrix(read.table("eurovision2022_jury_pre_null.txt", header = TRUE))
excluded <- c(2, 8, 12, 14,15,16)

contestants <- c(
"AU", # Australia        1
"AZ", # Azerbaijan       2 excluded
"BE", # Belgijm          3
"CY", # Cyprus           4
"CZ", # Czech republic   5
"EE", # Estonia          6
"FI", # Finland          7
"GE", # Georgia          8 excluded
"IE", # Ireland          9 
"IL", # Israel          10 
"MT", # Malta           11 
"ME", # Montenegro      12 excluded 
"MK", # North Macedonia 13
"PL", # Poland          14 excluded
"RO", # Romania         15 excluded
"SM", # San Marino      16 excluded
"RS", # Serbia          17
"SE"  # Sweden          18
)

(points <- sort(unique(c(eurovision_table)),decreasing=TRUE))
preference <- eurovision_table*0  
for(i in seq_along(points)){ preference[eurovision_table == points[i]] <- i }
preference
```

```{r}
contestants[excluded]
```


```{r, morestuff,cache=TRUE}
H_excluded <- hyper3()
H_included <- hyper2()
```
        
```{r define_notin}
text <- "remove this for R >= 4.6-0"
print(text)
`%notin%` <- Negate(`%in%`)
```

```{r makeH, cache=TRUE}
for(i in excluded){
    d <- preference[,i,drop=TRUE]
    d[d==11] <- 0
    newH <- suppfun(d[!is.na(d)])
    newH <- newH |> pwa23_multiple(contestants[excluded], "collusion")
    H_excluded <- H_excluded + newH
} # i loop closes

for(i in seq_len(ncol(preference))){
    if(i %notin% excluded){
        d <- preference[,i,drop=TRUE]
        d[d==11] <- 0
        d <- d[!is.na(d)]
        newH <- suppfun(d)
        H_included <- H_included + newH
    }
} # i loop closes
```

```{r}
H_excluded
```

```{r findmaxboth, cache=TRUE}
mHex <- maxp(H_excluded)
mHin <- maxp(H_included)
```

```{r plotboth}
dotchart(mHex)
dotchart(mHin)
```

```{r calcH, cache=TRUE}
H <- H_excluded + as.hyper3(H_included)
```
```{r dotest, cache=TRUE}
result <- specificp.gt.test(H, "collusion",0)
```

```{r}
result
```

```{r}
H_excluded
summary(H_excluded)
summary(H)
```

% -*- mode: noweb; noweb-default-code-mode: R-mode; -*-
%\VignetteIndexEntry{The hyper2 package}
%\VignetteDepends{hyper2}
%\VignetteKeywords{hyper2,dirichlet,Bradley-Terry, reified Bradley=Terry}
%\VignettePackage{hyper2}


\documentclass[nojss]{jss}
\usepackage{dsfont}
\usepackage{bbm}
\usepackage{amsfonts}
\usepackage{amsmath}
\usepackage{wasysym}
\usepackage{amssymb}
\usepackage{yfonts}
\usepackage{wrapfig}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% declarations for jss.cls %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%% just as usual
\author{Robin K. S. Hankin\\University of Stirling}
\title{Partial rank data with the \pkg{hyper2} package: likelihood
  functions for generalized Bradley-Terry models}



%% for pretty printing and a nice hypersummary also set:
\Plainauthor{Robin K. S. Hankin}
\Plaintitle{A Generalization of the Dirichlet Distribution}
\Shorttitle{A Generalization of the Dirichlet Distribution}

%% an abstract and keywords
\Abstract{

Here I present the \pkg{hyper2} package for generalized Bradley-Terry
models and analyze two situations: single scull rowing, and the
competitive cooking game show MasterChef Australia.  A number of
natural statistical hypotheses may be tested straightforwardly using
the software.  To cite the package in publications, use
\citet{hankin2017_rnw}, on which this work is based.
}


\Keywords{Dirichlet distribution, hyperdirichlet distribution,
  combinatorics, \proglang{R}, multinomial distribution, constrained
optimization, Bradley-Terry}
\Plainkeywords{Dirichlet distribution, hyperdirichlet distribution,
  combinatorics, R, multinomial distribution, constrained
optimization, Bradley-Terry}
  
  
  
%% publication information
%% NOTE: This needs to filled out ONLY IF THE PAPER WAS ACCEPTED.
%% If it was not (yet) accepted, leave them commented.
%% \Volume{13}
%% \Issue{9}
%% \Month{September}
%% \Year{2004}
%% \Submitdate{2004-09-29}
%% \Acceptdate{2004-09-29}

%% The address of (at least) one author should be given
%% in the following format:
\Address{
  Robin K. S. Hankin\\
  University of Stirling\\
  E-mail: \email{hankin.robin@gmail.com}\\
}
%% It is also possible to add a telephone and fax number
%% before the e-mail in the following format:
%% Telephone: +43/1/31336-5053
%% Fax: +43/1/31336-734

%% for those who use Sweave please include the following line (with % symbols):

%% end of declarations %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\newcommand{\bolda}{\mathbf a}
\newcommand{\boldp}{\mathbf p}
\newcommand{\boldV}{\mathbf V}
\newcommand{\boldalpha}{\mbox{\boldmath$\alpha$}}
\newcommand{\boldzero}{\mathbf 0}

\newcommand{\mcf}{\mathcal F}
\newcommand{\mcm}{\mathcal M}
\newcommand{\mcs}{\mathcal S}
\newcommand{\mcw}{\mathcal W}

\newcommand{\pG}{p_{\mbox{\frakfamily g}}}

\SweaveOpts{}
\begin{document}
\SweaveOpts{concordance=TRUE}



<<set_seed_chunk,echo=FALSE,print=FALSE>>=
library("hyper2")
set.seed(0)
@ 

<<time_saver,echo=FALSE,print=FALSE>>=
calc_from_scratch <- FALSE
@ 

<<echo=FALSE,print=FALSE>>=
ignore <- require(magrittr,quietly=TRUE)
@ 

\section{Introduction: the Bradley-Terry model}

\setlength{\intextsep}{0pt}
\begin{wrapfigure}{r}{0.2\textwidth}
\begin{center}
\includegraphics[width=1in]{\Sexpr{system.file("help/figures/hyper2.png",package="hyper2")}}
\end{center}
\end{wrapfigure}

The Bradley-Terry model for datasets involving paired comparisons has
wide uptake in the {R} community.  However, existing
functionality\footnote{In theory, the \pkg{hyperdirichlet}
package~\citep{hankin2010} provides similar functionality but it is
slow and inefficient.  It is limited to a small number of players and
cannot cope with the examples considered here.  Also, the name-based
system of \pkg{hyper2} is considerably more powerful and natural than
the number-based system of \pkg{hyperdirichlet}.} is restricted to
paired comparisons.  The canonical problem is to consider~$n$ players
who compete against one another; the basic inference problem is to
estimate numbers~$\mathbf{p}=\left({p_1,\ldots,p_n}\right)$,
$p_i\geqslant 0$, $\sum p_i=1$ which correspond to player
``strengths''.  Information about the~$p_i$ may be obtained from the
results of paired comparisons between the players.

Applications are legion.  The technique is widely used in a
competitive sport context~\citep{turner2012}, in which matches are
held between two opposing individuals or teams.  It can also be
applied to consumer choice experiments in which subjects are asked to
choose a favourite from among two choices~\citep{hatzinger2012}, in
which case the~$p_i$ are known as ``worth parameters''.

If player~$i$ competes against player~$j$, and wins with
probability~$P_{ij}$ then the likelihood function for~$p_1,\ldots p_n$
corresponding to a win for~$i$ is~$\frac{p_i}{p_i+p_j}$.
As~\cite{turner2012} point out, this may be expressed as

\[
\operatorname{logit}\left(P_{ij}\right)=\log p_i-\log p_j
\]

\noindent
and this fact may be used to estimate~$\mathbf{p}$ using generalized
linear models.  However, consider the case where three competitors,
$i$, $j$, and~$k$ compete.  The probability that~$i$ wins is
then~$\frac{p_i}{p_i+p_j+p_k}$~\citep{luce1959}; but there is no
simple way to translate this likelihood function into a GLM.  However,
working directly with the likelihood function for~$\mathbf{p}$ has
several advantages which are illustrated below.  The resulting
likelihood functions may readily be generalized to accommodate more
general order statistics, as in a race.  In addition, likelihood
functions may be specified for partial order statistics; also,
observations in which a \emph{loser} is identified may be given a
likelihood function using natural {R} idiom.

\subsection{Further generalizations}

Observing the winner~$w$ from a preselected set of
competitors~$\mathcal{C}$ has a likelihood function
of~$p_w/\sum_{i\in\mathcal{C}} p_i$.  But consider a more general
situation in which two disjoint teams $\mathcal{A}$ and~$\mathcal{B}$
compete; this would have likelihood~$\sum_{i\in\mathcal{A}}
p_i/\sum_{i\in\mathcal{A}\cup\mathcal{B}} p_i$.  Such datasets
motivate consideration of likelihood
functions~$\mathcal{L}\left(\cdot\right)$ with
\begin{equation}\label{hyper2likelihood}
\mathcal{L}\left(\mathbf{p}\right)=
\prod_{s\in \mathcal{O}}\left({\sum_{i\in s}}p_i\right)^{n_s}
\end{equation}

\noindent
where~$\mathcal{O}$ is a set of observations and~$s$ a subset
of~$\left[n\right]=\left\{1,2,\ldots,n\right\}$; numbers~$n_s$ are
integers which may be positive or negative.  The approach adopted by
the~\pkg{hyperdirichlet} package is to store each of the~$2^n$
possible subsets of~$\left[n\right]$ together with an exponent:

\begin{equation}\label{hyperdirichletlikelihood}
\prod_{s\in 2^{\left[n\right]}}\left({\sum_{i\in s}}p_i\right)^{n_s}.
\end{equation}

\noindent
but this was noted as being needlessly memory intensive and slow; it
is limited, in practice, to~$n\leqslant 9$.  Use of the more efficient
equation~\ref{hyper2likelihood} necessitates some mechansim of keeping
track of which subsets of~$\left[n\right]$ have nonzero powers.

\section{The {hyper2} package}

One such mechanism is furnished by the \code{C++} Standard Template
Library's ``map'' class \citep{musser2009} to store and retrieve
elements.  A \emph{map} is an associative container that stores values
indexed by a key, which is used to sort and uniquely identify the
values.  In the package, the key is a (STL) \code{set} of strictly
positive integers~$\leqslant n$.  The relevant \code{typedef}
statements are:

\begin{verbatim}
typedef set<unsigned int> bracket;
typedef map<bracket, double> hyper2;
\end{verbatim}


\noindent In the \code{STL}, a map object stores keys and associated
values in whatever order the software considers to be most propitious.
This allows faster access and modification times but the order in
which the maps, and indeed the elements of a set, are stored is not
defined.  In the case of likelihood functions such as
Equation~\ref{hyper2likelihood}, this does not affect the value of the
expressions, as multiplication and addition are associative and
commutative operations.  The \pkg{hyper2} package follows
\code{disordR} discipline \citep{hankin2022_disordR}.

\section{The package in use}

\begin{table}
\centering
\begin{tabular}{|ccc|c|}\hline
Topalov  & Anand & Karpov & total\\ \hline
22 & 13 & -  & 35\\ 
-  & 23 & 12 & 35\\ 
8  &  -  & 10 & 18 \\ \hline
30 & 36 & 22 & 88 \\ \hline
\end{tabular}
\caption{Results of 88 chess matches \label{chess} (dataset
  \code{chess} in the \pkg{aylmer} package) between three
  Grandmasters; entries show number of games won up to 2001 (draws are
  discarded).  Topalov beats Anand 22-13; Anand beats Karpov 23-12;
  and Karpov beats Topalov 10-8}
\end{table}

Consider the \code{Chess} dataset of the \pkg{hyperdirchlet} package,
in which matches between three chess players are tabulated
(table~\ref{chess}).  The Bradley-Terry model~\citep{bradley1952} is
appropriate here~\citep{caron2012}, and the \pkg{hyper2} package
provides a likelihood function for the strengths of the players,
$p_1,p_2,p_3$ with~$p_1+p_2+p_3=1$.  A likelihood function might be

\[
\frac{p_1^{30}p_2^{36}p_3^{22}}{
  \left(p_1+p_2\right)^{35}
  \left(p_2+p_3\right)^{35}
  \left(p_1+p_3\right)^{18}
  }
\]

Using the \pkg{hyper2} package, the \proglang{R} idiom to create this
likelihood function is as follows:


<<chess_setup>>=
chess <- hyper2()
chess["Topalov"] <- 30
chess["Anand"  ] <- 36
chess["Karpov" ] <- 22
chess[c("Topalov","Anand" )] <- -35
chess[c("Anand","Karpov"  )] <- -35
chess[c("Karpov","Topalov")] <- -18
chess
@ 

The internal representation of \code{hyper2} objects is that of a
\proglang{map}, an associative array implemented as part of the
standard template library (\proglang{STL}) of \proglang{C++}.  It is
thus an unordered collection of (\emph{key}, \emph{value}) pairs.  In
this case the \emph{value} is the power and the \emph{key} is the
content of the bracket, which is a subset of
$\left\lbrace\mbox{Topalov},\mbox{Anand},\mbox{Karpov}\right\rbrace$.
The \proglang{stl} provides a \proglang{set} class for efficient
manipulation of such objects, and this is used in the package.

One side-effect of using this system is that the order of the
bracket-power key-value pairs is not preserved; above, note how the
terms appear in an essentially random order.  Also note that the use
of the \proglang{set} class means that the \proglang{R} idiom is
insensitive to the order of the terms within a bracket:

<<use_chess>>=
chess[c("Karpov","Topalov")] 
chess[c("Topalov","Karpov")] 

@ 

The package can calculate log-likelihoods:

<<loglikelihoodchess>>=
loglik(c(1/3,1/3),chess)
@ 

[the first argument of function \code{loglik()} is a vector of length
  2, third element of $\boldp$ being the ``fillup''
  value~\citep{aitchison1986}]; the gradient of the log-likelihood is
given by function \code{gradient()}:

<<gradientloglikelihoodchess>>=
gradient(chess,c(1/3,1/3))
@ 

Such functionality allows the rapid location of the maximum likelihood
estimate for~$\boldp$:

<<maxlikechess>>=
maxp(chess)
@ 

\section{Men's single sculling in the 2016 Summer Olympic Games}%'

In this section, I will take results from the 2016 Summer Olympic
Games and create a likelihood function for the finishing order in
Men's single sculling.  In Olympic sculling, up to six individual
competitors race a small boat called a scull over a course of length
2\,km; the object is to cross the finishing line first.  Note that
actual timings are irrelevant, given the model, as the sufficient
statistic is the order in which competitors cross the finishing line.
The 2016 Summer Olympics is a case in point: the gold and silver
medallists finished less than~5 milliseconds apart, corresponding to a
lead of~$\sim 2.5\,\mathrm{cm}$.

We may use a generalization of the Bradley-Terry model due
to~\citet{luce1959}, in which the probability of competitor~$i$
winning in a field of~$j=1,\ldots, n$ is

\[
\frac{p_i}{p_1+\cdots +p_n}.\]

However, there is information in the whole of the finishing order, not
just the first across the line.  Once the winner has been identified,
then the runner-up may plausibly be considered to be the winner among
the remaining competitors; and so on down the finishing order.
Without loss of generality, if the order of finishing
were~$1,2,3,4,5,6$, then a suitable likelihood function would be,
following~\cite{plackett1975}:

\begin{equation}\label{competitors_1_to_6_likelihood}
\frac{p_1}{p_1+p_2+p_3+p_4+p_5+p_6}\cdot
\frac{p_2}{p_2+p_3+p_4+p_5+p_6}\cdot
\frac{p_3}{p_3+p_4+p_5+p_6}\cdot
\frac{p_4}{p_4+p_5+p_6}\cdot
\frac{p_5}{p_5+p_6}\cdot
\frac{p_6}{p_6}
\end{equation}


We may represent the result of
Heat~1 as
\[
\mathrm{fournier}\succ
\mathrm{cabrera}\succ
\mathrm{bhokanal}\succ
\mathrm{saensuk}\succ
\mathrm{kelmelis}\succ
\mathrm{teilemb}
\]

(a full list of the finishing order for all~25 events is given in the
package as \code{rowing.txt}).  Incorporating the information from
Heat~1 into a likelihood function corresponding to
equation~\ref{competitors_1_to_6_likelihood} is straightforward using
the \code{rankvec\_likelihood()} function:

<<2016_olympics_heat1_first>>=
heat1 <- c("fournier", "cabrera", "bhokanal", "saensuk", "kelmelis", "teilemb")
H <- rankvec_likelihood(heat1)
H
@ 

(variable \code{heat1} shows the finishing order for Heat~1: Fournier
came first, then Cabrera, etc).  We can add the information from
heat~2 easily:


<<2016_olympics_heat1_first>>=
heat2 <- c("drysdale", "molnar", "esquivel", "garcia", "khafaji", "monasterio")
H <- H + rankvec_likelihood(heat2)
head(H)
@ 

Again observe that object \code{H} includes its terms in no apparent
order.  Although it would be possible to incorporate information from
subsequent heats in a similar manner, the package includes a
ready-made dataset, \code{rowing}:

<<rowers>>=
head(rowing)   # see rowing.Rd
@ 
 
Finding the maximum likelihood estimate for the
parameter~$p_\mathrm{banna},\ldots,p_\mathrm{zambrano}$ is
straightforward using the \code{maxp()} function, provided with the
package~(Figure~\ref{maxp_sculls2016}).  The optimization routine has
access to derivatives which means that the estimate is found very
quickly.


\begin{figure}[htbp]
\begin{center}
<<rowing_maxp,echo=TRUE,fig=TRUE>>=
dotchart(rowing_maxp)
@
\caption{Maximum likelihood estimate for the strengths of the~32
  competitors\label{maxp_sculls2016} in the Men's singles sculls in
  the 2016 Summer Olympics
}
  \end{center}
\end{figure}

Figure~\ref{maxp_sculls2016} shows very directly that the competitor
with the highest strength is Drysdale, the gold medallist for this
event.  The bronze and silver medallists were Synek and Martin
respectively, whose estimated strengths were second and third highest
in the field.

 
\section{MasterChef Australia}

MasterChef Australia is a game show in which amateur cooks compete for
a title~\citep{wikipedia_masterchef2017}.  From a statistical
perspective the contest is interesting because the typical show format
is to identify the \emph{weakest} player, who is then eliminated from
the competition.  Here, results from MasterChef Australia
Series~6~\citep{wikipedia_masterchef_australia_series6} will be
analysed; an extended discussion of the data used is given in the
package at~\code{masterchef.Rd}.

We wish to make inferences about the contestants' generalized
Bradley-Terry strengths~$p_1,\ldots,p_n$, $\sum p_i=1$.  One
informative event was a team challenge in which the contestants were
split randomly into two teams, red and blue:
  
<<setupteams>>=
team_red <- c("Jamie","Tracy","Ben","Amy","Renae","Georgia")
team_blue <- c("Brent","Laura","Emelia","Colin","Kira","Tash")
@ 
 
We may represent the fact that the red team won as 
\begin{equation}\label{redteamwon}
\left\{
\mbox{Jamie}+\mbox{Tracy}+\mbox{Ben}+
\mbox{Amy}+\mbox{Renae}+\mbox{Georgia}
\right\}
\succ
\left\{
\mbox{Brent}+\mbox{Laura}+\mbox{Emelia}+
\mbox{Colin}+\mbox{Kira}+\mbox{Tash}
\right\}
\end{equation}

The likelihood function for the observation given in
equation~\ref{redteamwon} would be

\begin{equation}\label{redbluelikelihood}
\frac{p_\mathrm{Jamie}+p_\mathrm{Tracy}+p_\mathrm{Ben}+p_\mathrm{Amy}+p_\mathrm{Renae}+p_\mathrm{Georgia}}{
p_\mathrm{Jamie}+p_\mathrm{Tracy}+p_\mathrm{Ben}+p_\mathrm{Amy}+p_\mathrm{Renae}+p_\mathrm{Georgia}+p_\mathrm{Brent}+p_\mathrm{Laura}+p_\mathrm{Emelia}+p_\mathrm{Colin}+p_\mathrm{Kira}+p_\mathrm{Tash}}
\end{equation}
  
To generate a likelihood function in \proglang{R}, we need to set up a
\code{hyper2} object with appropriate contestants:

<<masterchef_example>>=
H <- hyper2(pnames = c(
                "Amy", "Ben", "Brent", "Colin", "Emelia",
                "Georgia", "Jamie", "Kira", "Laura", "Renae",
                "Sarah", "Tash", "Tracy"))
H
@ 

Object \code{H} is a uniform likelihood function.  The package \proglang{R} idiom
for incorporating likelihood from Equation~\ref{redbluelikelihood} is
straightforward and natural:
<<redteamwins>>=
H[team_red] <- +1
H[c(team_red,team_blue)] <- -1
H
@ 

(Sarah did not take part).  The above idiom makes it possible to
define likelihood functions for observations that have a peculiar
probability structure, and I give two examples below.

One event involved eight competitors who were split randomly into four
teams of two.  The show format was specified in advance as follows:
The teams were to be judged, and placed in order.  The two top teams
were to be declared safe, and the other two teams sent to an
elimination trial from which an individual winner and loser were
identified, the loser being obliged to leave the competition.  The
format for this event is also typical in MasterChef.

The observation was that Laura and Jamie's team won, followed by
Emelia and Amy, then Brent and Tracy.  Ben and Renae's team came last:

\begin{equation}
\left\{{\mathrm{Laura}  + \mathrm{Jamie}}\right\}\succ
\left\{{\mathrm{Emelia} + \mathrm{Amy}  }\right\}\succ
\left\{{\mathrm{Brent}   + \mathrm{Tracy}}\right\}\succ
\left\{{\mathrm{Ben}    + \mathrm{Renae}}\right\}
\end{equation}

A plausible likelihood function can be generated using the standard
assumption~\citep{hankin2010} that the competitive strength of a team
is the sum of the strengths of its members.  We can then combine this
assumption with the order statistic technique of~\cite{plackett1975}:

\begin{align}
\frac{p_\mathrm{Laura}+p_\mathrm{Jamie}
}{
  p_\mathrm{Laura}  + p_\mathrm{Jamie} + 
  p_\mathrm{Emelia} + p_\mathrm{Amy}   +
  p_\mathrm{Brent}  + p_\mathrm{Tracy} +
  p_\mathrm{Ben}    + p_\mathrm{Renae}
  }\cdot\nonumber\\
\frac{
  p_\mathrm{Emelia} + p_\mathrm{Amy} 
}{
  p_\mathrm{Emelia} + p_\mathrm{Amy}   +
  p_\mathrm{Brent}  + p_\mathrm{Tracy} +
  p_\mathrm{Ben}    + p_\mathrm{Renae}
  }\cdot
\frac{
  p_\mathrm{Brent}  + p_\mathrm{Tracy}
}{  
  p_\mathrm{Brent}  + p_\mathrm{Tracy} +
  p_\mathrm{Ben}    + p_\mathrm{Renae}
  }
\end{align}

We would like to incorporate information from this observation into
object~\code{H}, which is a likelihood function for the two-team
challenge discussed above.  The corresponding \proglang{R} idiom is
natural:

<<teamchallenge>>=
blue   <- c("Laura","Jamie")   # first
yellow <- c("Emelia","Amy")    # second
green  <- c("Brent","Tracy")   # third
red    <- c("Ben","Renae")     # fourth

H[blue] <- 1
H[c(blue,yellow,green,red)] <- -1
H[yellow] <- 1
H[c(yellow,green,red)] <- -1
H[green] <- 1
H[c(green,red)] <- -1
H
@

We may incorporate subsequent observations relating to the elimination
trial among the four competitors comprising the losing two teams.  The
observation was that Laura won, and Renae came last, being eliminated.
We might write

\begin{equation}\label{renae_eliminated}
\left\{
\mbox{Laura}
\right\}
\succ
\left\{{\mbox{Brent},\mbox{Tracey},\mbox{Ben}}\right\}
\succ
\left\{\mbox{Renae}\right\},
\end{equation}

which indicates that Laura came first, then Brent/Tracey/Ben in some
order, then Renae came last.  For this observation a likelihood
function, following~\cite{plackett1975}, might be

\begin{align}
  \mathcal{L}\left({p_1,p_2,p_3,p_4,p_5}\right) &=
  \Prob\left({
    p_1\succ p_2\succ p_3\succ p_4\succ p_5
    \cup
    p_1\succ p_2\succ p_4\succ p_3\succ p_5
    \cup\ldots
  }\right)\\ \label{ggrlmath}
  &=\Prob\left({\bigcup_{\left[{abc}\right]} p_1\succ p_a\succ p_b\succ p_c\succ p_5
  }\right)\\ \nonumber
  &=    \frac{p_1}{p_1+p_2+p_3+p_4+p_5}\cdot\frac{p_2}{p_2+p_3+p_4+p_5}\cdot\frac{p_3}{p_3+p_4+p_5}\cdot\frac{p_4}{p_4+p_5}\\ \nonumber
  &{\qquad+}\frac{p_1}{p_1+p_2+p_4+p_3+p_5}\cdot\frac{p_2}{p_2+p_4+p_3+p_5}\cdot\frac{p_4}{p_4+p_3+p_5}\cdot\frac{p_3}{p_3+p_5}\\ \nonumber
  &{\qquad{}+}\frac{p_1}{p_1+p_3+p_2+p_4+p_5}\cdot\frac{p_3}{p_3+p_2+p_4+p_5}\cdot\frac{p_2}{p_2+p_4+p_5}\cdot\frac{p_4}{p_4+p_5}\\ \nonumber
  &{\qquad{}+}\cdots
\end{align}

where Laura's strength is shown as~$p_1$ etc for brevity.  The
\proglang{R} idiom is as follows:

<<>>=
L <- ggrl(H, 
          winner     = "Laura",
          btm4       = c("Brent", "Tracy","Ben"),
          eliminated = "Renae")
@ 

Arguments to \code{ggrl()} are disjoint subsets of the players, the
subsets themselves being passed in competition order from best to
worst.  Object \code{L} includes information from the team challenge
(via first argument \code{H}) and the elimination results.  It is a
list of length~$3!=6$ objects of class \code{hyper2}, each of which
gives a \citeauthor{luce1959} likelihood function for a consistent
total ordering of the competitors.

A final example (taken from MasterChef series 8, week 10) is given as
a generalization of the \citeauthor{luce1959} likelihood.  The format
was as follows.  Eight contestants were split randomly into four teams
of two, the top two teams being declared safe.  Note that the
likelihood interpretation differs from the previous team challenge, in
which the observation was an unambiguous team ranking: here, there is
only a partial ranking of the teams and one might expect this
observation to be less informative.  Without loss of generality, the
result may be represented as

\begin{equation}
  \left\{{p_1+p_2,p_3+p_4}\right\}\succ
  \left\{{p_5+p_6,p_7+p_8}\right\}
  \end{equation}
      
and a likelihood function on~$p_1,\ldots p_8$ for this observation
might be

\begin{align}
  \mathcal{L}\left( {p_1,\ldots, p_8} \right) &=
  \Prob\left({
    \left\{ {p_1+p_2} \right\}\succ 
    \left\{ {p_3+p_4} \right\}\succ 
    \left\{ {p_5+p_6} \right\}\succ 
    \left\{ {p_7+p_8} \right\} \cup\vphantom{\bigcup}
  }\right. \nonumber\\
  &\qquad \left.{
    \left\{ {p_1+p_2} \right\}\succ 
    \left\{ {p_3+p_4} \right\}\succ 
    \left\{ {p_7+p_8} \right\}\succ 
    \left\{ {p_5+p_6} \right\}\cup
  }\right.\nonumber\\
  &\qquad \left.{
    \left\{ {p_3+p_4} \right\}\succ 
    \left\{ {p_1+p_2} \right\}\succ 
    \left\{ {p_5+p_6} \right\}\succ 
    \left\{ {p_7+p_8} \right\}\cup
  }\right.\nonumber\\
  &\qquad \left.{
    \left\{ {p_3+p_4} \right\}\succ 
    \left\{ {p_1+p_2} \right\}\succ 
    \left\{ {p_5+p_6} \right\}\succ 
    \left\{ {p_7+p_8} \right\}\vphantom{\bigcup}
  }\right)\\
&=\frac{p_1+p_2}{p_1+p_2+p_3+p_4+p_5+p_6+p_7+p_8}\cdot
  \frac{p_3+p_4}{p_3+p_4+p_5+p_6+p_7+p_8}\cdot\frac{p_5+p_6}{p_5+p_6+p_7+p_8}\nonumber\\
&{}\qquad+\cdots+\nonumber\\
&{}\frac{p_3+p_4}{p_3+p_4+p_1+p_2+p_7+p_8+p_5+p_6}\cdot
  \frac{p_1+p_2}{p_3+p_4+p_7+p_8+p_5+p_6}\cdot\frac{p_7+p_8}{p_7+p_8+p_5+p_6}\nonumber\\
\nonumber
\end{align}

% \end{align}

\subsection{Maximum likelihood estimation}


The package provides an overall likelihood function for all
informative judgements in the series on the final 13 players in object
\code{masterchef\_series6}, documented at \code{masterchef.Rd}.  We
may assess a number of related hypotheses using the package.  The
first step is to calculate the likelihood for the hypothesis that all
players are of equal strength:
 
<<like_series1>>=
data("masterchef")
n <- 13   # 13 players
equal_strengths <- rep(1/n,n-1)
like_series(equal_strengths, masterchef)  # see masterchef.Rd
@ 

The strengths of the 13 players may be estimated using standard
maximum likelihood techniques.  This requires constrained optimization
in order to prevent the search from passing through inadmissible
points in p-space:

<<unconstrained_optimization,eval=FALSE>>=
UI <- rbind(diag(n-1),-1)  # p_i >= 0
CI <- c(rep(0,n-1),-1)     # p_1+...+p_{n-1} <= 1

constrOptim(  # maxp() does not work for masterchef_series6
    theta = equal_strengths,  # starting point for optimization
    f = function(p){-like_series(p,masterchef)}, # see masterchef.Rd
    ui=UI, ci=CI,
    grad=NULL)
@ 

\begin{figure}[htbp]
\begin{center}
<<masterchef_maxp,echo=TRUE,fig=TRUE>>=
masterchef_maxp
dotchart(masterchef_maxp)
@
\caption{Maximum likelihood estimate for the strengths of the top~13
  competitors\label{masterchef6} in Series~6 of \emph{MasterChef Australia}
}
\end{center}
\end{figure}

The resulting maximum likelihood estimate, \code{masterchef_maxp} in
the package, is shown pictorially in Figure~\ref{masterchef6}.  The
support at the precalculated evaluate is

<<like_series2>>=
like_series(indep(masterchef_maxp), masterchef)
@ 

and this allows us to test the hypothesis of equal player strengths:
by Wilks's theorem, $-2\log\Lambda$ has an asymptotic null distribution
of~$\chi^2_{12}$.  This corresponds to a $p$-value of

<<pvalcalcequalstrengths>>=
pchisq(2*(78.7-66.2),df=12,lower.tail=FALSE)
@ 

showing that the observations do constitute evidence for differential
player strengths.  Figure~\ref{masterchef6} suggests that Laura, the
runner-up, is actually a stronger competitor than the winner, Brent.
We may assess this statistically by finding the maximum likelihood
for~${\mathbf p}$, subject to the constraint
that~$p_\mathrm{Laura}\leqslant p_\mathrm{Brent}$:

<<brent.gt.laura, eval=FALSE>>=
UI <- rbind(UI,c(0,0,1,0,0,0,0,0,-1,0,0,0,0))  # Brent >= Laura
CI <- c(CI,0)
ans2 <-
constrOptim(  # maxp() does not work for masterchef_series6
    theta = equal_strengths,
    f = function(p){-like_series(p,masterchef_series6)},  # see masterchef.Rd
    grad=NULL,
    ui = UI, ci=CI)
@ 

Object \code{maxp_masterchef6_constrained} in the package is the
result of the above optimization, at which point the likelihood is


<<>>=
like_series(indep(masterchef_constrained_maxp), masterchef)
@ 

The two estimates differ by about~$1.18$, less than the
two-units-of-support criterion of~\citet{edwards1992}; alternatively,
one may observe that the likelihood ratio is not in the tail region of
its asymptotic distribution ($\chi^2_1$) as the $p$-value is about 0.12.
This shows that there is no strong evidence for Laura's competitive
strength being higher than that of Brent.  Similar techniques can be
used to give a profile likelihood function; the resulting support interval
for Laura's strength is~$\left[{0.145,0.465}\right]$, which does not
include~$\frac{1}{13}\simeq 0.077$, the mean player strength.  

However, further work would be needed to make statistically robust
inferences from these findings.  Even given equal competitive ability,
one would expect the player with the highest estimated Bradley-Terry
strength to have an elevated support interval; and it is not clear to
what extent Wilks, being asymptotic, is relevant to the data at hand.


\section{Conclusions}

Several generalization of Bradley-Terry strengths are appropriate to
describe competitive situations in which order statistics are
sufficient.

The \code{hyper2} package is introduced, providing a suite of
functionality for generalizations of the partial rank analysis
of~\cite{critchlow1985}.  The software admits natural \proglang{R}
idiom for translating commonly occurring observations into a
likelihood function.

The package is used to calculate maximum likelihood estimates for
generalized Bradley-Terry strengths in two competitive situations:
Olympic rowing, and \emph{MasterChef Australia}.  The estimates for
the competitors' strengths are plausible; and several meaningful
statistical hypotheses are assessed quantitatively.





\bibliography{hyper2}
\end{document}
 

\name{rhyper3}
\alias{rracehyper3}
\alias{rrace3}
\alias{rwinner3}
\alias{rhyper3}
\alias{rpair3}
\title{Random hyper3 objects}
\description{

  Various random \code{hyper3} objects, in the context of the race
  metaphor.  They return \dQuote{get you going} examples of
  \code{hyper3} objects.  The defaults are simple but non-trivial and
  have straightforward interpretations.

}
\usage{
rwinner3(pn = c(a=2, b=4, c=2, d=1), ps = c(a=0.3, b=0.1, c=0.2, d=0.4))
rpair3(n=5, s=3, lambda=1.3)
rrace3(pn = c(a=2, b=4, c=2, d=1), ps=c(a=0.3, b=0.1, c=0.2, d=0.4))
rracehyper3(n=4, size=9, ps=NULL, races=3)
rhyper3(n=5, s=4, type='race', ...)
}
\arguments{
  \item{pn}{A named integer vector showing numbers of each type of player}
  \item{ps}{A named vector showing strengths of each type of player}
  \item{n, size, races, s, type}{Arguments specifying the complexity 
    of the random \code{hyper3} object returned.  See details}
  \item{lambda}{Parameter}
  \item{...}{Further arguments passed to \code{rracehyper3()} or
    \code{rpair3()}}
}
\details{

  These functions return \code{hyper3} objects, as indicated by the
  \code{3} in their names.

  \itemize{

    \item Function \code{rwinner3()} is a low-level helper function that
    takes a player number argument \code{pn}, and a player strength
    argument \code{ps}.  It performs an \emph{in silico} race,  and
    returns the (name of) the winner, chosen randomly from a field of
    runners with appropriate strengths.  It is used repeatedly by
    \code{rrace3()} to select a winner from the diminishing pool of
    still-running players.

    \item Function \code{rpair3()} returns a \code{hyper3} object
    corresponding to repeated pairwise comparisons including a
    white-player advantage represented by \code{lambda}.
    
   \item Function \code{rrace3()} returns a rank statistic corresponding
   to finishing order for a Plackett-Luce race.  The output can be
   passed to \code{ordervec2supp3()}.

   \item Function \code{rracehyper3()} returns a more complicated
   \code{hyper3} object corresponding to repeated races.

   \item Function \code{rhyper3()} returns an even more complicated
   \code{hyper3} object corresponding to repeated races and pairwise
   comparisons.  The defaults are

  \preformatted{

   pn: c(a=2,   b=4,   c=2,   d=1  )  # numbers (two "a"s, four "b"s etc)
   ps: c(a=0.3, b=0.1, c=0.2, d=0.4)  # strengths
   }
}
  Argument \code{n} generally specifies the number of distinct types of
  players.  Files \code{inst/mann_whitney_wilcoxon.Rmd} and
  \code{inst/javelin.Rmd} show some use-cases for these functions.
}
\author{Robin K. S. Hankin}
\note{
  In function \code{rracehyper3()} [and by extension \code{rhyper3()}],
  if argument \code{n} exceeds 26 and argument \code{pn} takes its
  default value of \code{NULL}, then an error will be returned because
  there are only 26 players, one for each letter \code{a}-\code{z}.
}

\seealso{\code{\link{rrank}},\code{\link{ordertable2supp}},\code{\link{ordertrans}}}
\examples{

rracehyper3()
rrace3()
rwinner3()
rhyper3()
rpair3()
ordervec2supp3(rrace3())

table(replicate(100, which(rrace3(pn=c(a=1,b=10),ps=c(a=0.9,b=0.1)) == 'a')))

}

\name{Extract}
\alias{Extract.hyper2}
\alias{extract}
\alias{extractor}
\alias{assign_lowlevel}
\alias{overwrite_lowlevel}
\alias{[.hyper2}
\alias{[<-.hyper2}
\title{
Extract or replace parts of a hyper2 object
}
\description{
Extract or replace parts of a \code{hyper2} object, in accordance with
\CRANpkg{disordR} discipline.
}
\usage{
\method{[}{hyper2}(x, \dots)
\method{[}{hyper2}(x, index, \dots) <- value
assign_lowlevel(x, index, value)
overwrite_lowlevel(x, value)

}
\arguments{
  \item{x}{An object of class \code{hyper2}}
  \item{\dots}{Further arguments, currently ignored}
  \item{index}{A list with integer vector elements corresponding to the
      brackets whose power is to be replaced}
  \item{value}{Numeric vector of powers}
}
\details{
  These methods should work as expected, although the off-by-one issue
  might be a gotcha.

 For the extract method, \code{H[L]}, a \code{hyper2} object is
 returned.  The replace method, \code{H[L] <- value}, the index
 specifies the brackets whose powers are to be overwritten; standard
 \code{disordR} protocol is used.

 If the index argument is missing, viz \code{H1[] <- H2}, this is a
 special case.  Argument \code{H2} must be a \code{hyper2} object, and
 the idiom effectively executes \code{H1[brackets(H2)] <- powers(H2)},
 but more efficiently; note that this operation is well-defined even
 though (because of \CRANpkg{disordR} discipline) the order of the
 brackets is arbitrary.  This special case is included in the package
 because it has a very natural \code{C++} expression [function
 \code{overwrite()} in the \code{src/} directory] that was too neat to
 omit.

 Altering (incrementing or decrementing) the power of specified brackets
 is possible using idiom like \code{H[x] <- H[x] + 1}; this is
 documented at \code{\link{Ops.hyper2}}, specifically
 \code{hyper2_sum_numeric()} and a discussion is given at
 \code{\link{increment}}.
 
 Functions \code{assign_lowlevel()} and \code{overwrite_lowlevel()} are
 low-level helper functions and not really intended for the end-user.

}

\value{
The extractor method returns a \code{hyper2} object, restricted to the
elements specified
}
\author{Robin K. S. Hankin}
\note{
Use \code{powers()} and \code{brackets()} to extract a numeric vector of
 powers or a list of integer vectors respectively.

Replacement idiom \code{H[x] <- val} cannot use non-trivial recycling.
This is because the elements of \code{H} are stored in an arbitrary
order, but the elements of \code{val} are stored in a particular order.
Also see function \code{hyper2_sum_numeric()}.

}
\seealso{\code{\link{hyper2}},\code{\link{Ops.hyper2}}}
\examples{
data(chess)

chess["Topalov"]
chess[c("Topalov","Anand")]
chess[c("Anand","Topalov")]

# Topalov plays Anand and wins:

chess["Topalov"] <- chess["Topalov"]+1 
chess[c("Topalov","Anand")] <- chess[c("Topalov","Anand")]-1


# Topalov plays *Kasparov* and wins:
chess["Topalov"] <- chess["Topalov"] + 1
chess[c("Topalov","Kasparov")] <- chess[c("Topalov","Kasparov")] -1

# magrittr idiom:
# chess["Topalov"] \%<>\% inc
# chess[c("Topalov","Kasparov")] \%<>\% dec  

# overwriting idiom:
H <- hyper2(list("Topalov","X"),6)
chess[] <- H

H <- icons

}

\name{profile}
\alias{profile}
\alias{profile_support}
\alias{profile_likelihood}
\alias{profile_support_single}
\alias{profile_likelihood_single}
\alias{proflike}
\alias{profsupp}
\alias{profsup}
\title{Profile likelihood and support}
\description{
Given a support function, return a profile likelihood curve
}
\usage{
profsupp(H, i, p, relative=TRUE, ...)
profile_support_single(H, i, p, evaluate=FALSE, ...)
}
\arguments{
  \item{H}{hyper2 object}
  \item{i}{Name of player for which profile support is to be calculated}
  \item{p}{Strength of element \code{i}}
  \item{evaluate}{Boolean, with default \code{FALSE} meaning to return
    the maximal support for \code{p_i=p} and \code{TRUE} meaning to
    return the evaluate}
  \item{relative}{Boolean; if \code{TRUE} (default), return the support
    relative to the maximum support attained; if false, return the
    support as returned by \code{profile_support_single()}.}
  \item{...}{Arguments passed to \code{maxp()}}
}

\value{Returns the support at a particular value of \eqn{p_i}, or the
  evaluate conditional on \eqn{p_i}.}
\author{Robin K. S. Hankin}
\seealso{\code{\link{loglik}}}
\examples{

\dontrun{  # takes too long
p <- seq(from=0.5,to=0.4,len=10)
u <- profsupp(icons,"NB",p)
plot(p,u-max(u))
abline(h=c(0,-2))
}
}

\name{javelin}
\alias{javelin}
\alias{javelin2}
\alias{javelin_vector}
\alias{javelin_maxp}
\alias{javelin2_maxp}
\alias{javelin_table}
\docType{data}
\title{Javelin dataset}
\description{

  An \code{attemptstable} object showing results from the men's javelin,
  2020 Summer Olympics.
  
}
\usage{data(javelin)}
\details{

  \itemize{
    \item \code{javelin_table}, an \code{attemptstable} object detailing
    the throw distances of eight competitors (diacritics have been
    removed) for each of six throws
    \item \code{javelin1} and \code{javelin2}, support functions
    corresponding to the weighted Plackett-Luce likelihood.  The suffix
    \dQuote{2} means that no-throws are counted as losing attempts;
    otherwise, no-throws are ignored.
    \item \code{javelin_maxp} and \code{javelin2_maxp} are the
    corresponding maximum likelihood estimates for the players'
    strengths
    \item{javelin_vector} is a named vector with elements being the
    throw distances and names being the thrower
  }

These objects can be generated by running script
\code{inst/javelin.Rmd}, which includes some further discussion and
technical documentation, and creates file \code{javelin.rda} which
resides in the \code{data/} directory.

}
\author{Robin K. S. Hankin}
\note{

The format for Olympic javelin results is common to six of the field
events: long jump, triple jump, shot put, discus throw, hammer throw,
and javelin (high jump and long jump have a different format).  The
\code{inst/} directory on github includes some other datasets such as
\code{olympics_2020_womens_shotput.txt} in the same form as
\code{inst/javelin.txt}.

  }
\seealso{\code{\link{attemptstable2supp3}}}
\examples{

attemptstable2supp3(javelin_table)

pie(javelin_maxp)

}

\name{zipf}
\alias{zipf}
\title{Zipf's law}
\description{

A very short function that reproduces Zipf's law: a harmonic rank-probability
distribution, formally

\deqn{p(i)=\frac{i^{-1}}{\sum_{i=1}^{N} i^{-1}},\qquad i=1,\ldots,N}{omitted}

The volleyball dataset might reasonably be assumed to be zipf, but one
can reject this hypothesis at \eqn{5\%}, see the examples.


}
\usage{zipf(n)}
\arguments{\item{n}{Integer; if a hyper2 object is supplied this is
    interpreted as \code{size(n)}}}
\value{Returns a numeric vector summing to one}
\author{Robin K. S. Hankin}
\seealso{\code{\link{knownp.test}}}
\examples{
zipf(icons)
knownp.test(volleyball,zipf(volleyball))
}

\name{B}
\alias{B}
\alias{probability}
\alias{mgf}
\alias{mean}
\alias{Jacobian}
\alias{dhyper2}
\alias{dhyper2_e}
\alias{e_to_p}
\alias{p_to_e}
\alias{mean_hyper2}
\title{Normalizing constant for the hyperdirichlet distribution}
\description{
Numerical techniques for calculating the normalizing constant for the hyperdirichlet distribution
}
\usage{
B(H, disallowed=NULL, give=FALSE, ...)
probability(H, disallowed=NULL, ...)
mgf(H, powers, ...) 
dhyper2(ip, H, ...)
dhyper2_e(e, H, include.Jacobian=TRUE)
mean_hyper2(H, normalize=TRUE, ...)
Jacobian(e)
e_to_p(e)
p_to_e(p)
}
\arguments{
  \item{H}{Object of class hyper2}
  \item{powers}{Vector of length \code{dim(x)} whose elements are the
    powers of the expectation; see details section}
  \item{disallowed}{Function specifying a subset of the simplex
    over which to integrate; default \code{NULL} means to integrate over
    the whole simplex.  The integration proceeds over \code{p} with
    \code{disallowed(p)} evaluating to \code{FALSE}}
  \item{e,p}{A vector; see details}
  \item{ip}{A vector of probabilities corresponding to \code{indep(p)}
    where \code{p} is vector with unit sum}
  \item{include.Jacobian}{Boolean, with default \code{TRUE} meaning to
    include the Jacobian transformation in the evaluation, and
    \code{FALSE} meaning to ignore it; use \code{FALSE} for likelihood
    work and \code{TRUE} for probability densities}
  \item{give}{Boolean, with default \code{FALSE} meaning to return the
    value of the integral and \code{TRUE} meaning to return the full
    output of \code{adaptIntegrate()}}
  \item{normalize}{Boolean, indicates whether return value of
    \code{mean_hyper2()} is normalized to have unit sum}
  \item{\dots}{Further arguments passed to \code{adaptIntegrate()}}
}
\details{
  \itemize{

    \item Function \code{B()} returns the normalizing constant of a
    hyperdirichlet likelihood function.  Internally, \eqn{p} is
    converted to \code{e} (by \code{e_to_p()}) and the integral proceeds
    over a hypercube.  This function can be very slow, especially if
    \code{disallowed} is used.

    \item Function \code{dhyper2(ip,H)} is a probability density
    function on the independent components of a unit-sum vector, that
    is, \code{ip=indep(p)}.  This function calls \code{B()} each time so
    might be a performance bottleneck.

    \item Function \code{probability()} gives the probability of an
    observation from a hyperdirichlet distribution satisfying
    \code{!disallowed(p)}.
    
    \item Function \code{mgf()} is the moment generating function,
    taking an argument that specifies the powers of \code{p} needed: the
    expectation of \eqn{\prod_{i=1}^n {p_i}^{{\rm powers}[i]}}{prod
    p^powers} is returned.

    \item Function \code{mean_hyper2()} returns the mean value of the
    hyperdirichlet distribution.  This is computationally slow (consider
    \code{maxp()} for a measure of central tendency).  The function
    takes a \code{normalize} argument, not passed to
    \code{adaptIntegrate()}: this is Boolean with \code{FALSE} meaning
    to return the value found by integration directly, and default
    \code{TRUE} meaning to normalize so the sum is exactly 1
  }
}
\value{
  \itemize{
    \item Function \code{B()} returns a scalar: the normalization
    constant
    \item Function \code{dhyper2()} is a probability density function
    over \code{indep(p)}
    \item Function \code{mean()} returns a \eqn{k}-tuple with unit sum
    \item Function \code{mgf()} returns a scalar equal to the expectation of
    \code{p^power}
    \item Functions \code{is.proper()} and \code{validated()} return a Boolean
    \item Function \code{probability()} returns a scalar, a (Bayesian)
    probability
    }
}
\author{Robin K. S. Hankin}

\note{

Vignette \code{integration} gives some further discussion and examples.
  
The \pkg{adapt} package is no longer available on CRAN; from 1.4-3, the
\pkg{hyper2} package uses \code{adaptIntegrate()} of the
\CRANpkg{cubature} package.

}

\seealso{\code{\link{loglik}}}
\examples{

# Two different measures of central tendency:
# mean_hyper2(chess,tol=0.1)   # takes ~10s to run
maxp(chess)                    # faster

# Using the 'disallowed' argument typically results in slow run times;
# use high tol for speed:

# probability(chess,disallowed=function(p){p[1]>p[2]},tol=0.5)
# probability(chess,disallowed=function(p){p[1]<p[2]},tol=0.5)

# Above should sum to 1 [they are exclusive and exhaustive events]


}

\name{rrank}
\alias{rrank}
\alias{rrankk}
\alias{rrank_single}
\alias{rorder_single}
\alias{print.ranktable}
\title{Random ranks}
\description{
A function for producing ranks randomly, consistent with a specified
strength vector
}
\usage{
rrank(n = 7, p = (4:1)/10, pnames=NULL, fill = FALSE, rnames=NULL)
rrankk(n = 37, p = (20:1)/210, pnames=NULL, fill=FALSE, rnames=NULL)
\method{print}{ranktable}(x, ...)
rrank_single(p)
rorder_single(p)
}
\arguments{
  \item{n}{Number of observations}
  \item{p}{Strength vector}
  \item{pnames}{Character vector (\dQuote{player names}) specifying
    names of the columns}
  \item{rnames}{Character vector (\dQuote{row names} or \dQuote{race
      names}) specifying names of the rows}
  \item{fill}{Boolean, with default \code{FALSE} meaning to interpret
    the elements of \code{p} as strengths, notionally summing to one;
    and \code{TRUE} meaning to augment \code{p} with a fillup value}
  \item{x, ...}{Arguments passed to the print method}
}

\value{

Random rank observations.  Function \code{rrank()}, with no arguments,
returns a small get-you-going example of a ranktable object [see
\code{ranktable.Rd}].  Function \code{rrankk()} returns a bigger and
more complicated object.

If \code{n=1}, \code{rrank()} returns a vector; if \code{n>1} it returns
a matrix with \code{n} rows, each corresponding to a ranking.  The
canonical example is a race in which the probability of competitor
\eqn{i} coming first is \eqn{p_i/\sum p_j}{p_i/sum p}, where the
summation is over the competitors who have not already finished.

If, say, the first row of \code{rrank()} is \code{c(e, d, b, a, c)},
then competitor \code{e} came first, competitor \code{d} came second,
competitor \code{b} came third, and so on.

Note that function \code{rrank()} returns an object of class
\code{ranktable}.  The column names appear as \dQuote{\code{c1, c2,
...}} which is intended to be read \dQuote{came first}, \dQuote{came
second}, and so on.  The difference between \emph{rank} and \emph{order}
can be confusing.

\preformatted{
> x <- c(a=3.01, b=1.04, c=1.99, d=4.1)
> x
   a    b    c    d 
3.01 1.04 1.99 4.10 
> rank(x)
a b c d 
3 1 2 4 
> order(x)
[1] 2 3 1 4
}

In the above, \code{rank()} shows us that element \code{a} of \code{x}
(viz 3.01) is the third largest, element \code{b} (viz 1.04) is the
smallest, and so on; \code{order(x)} shows us that the smallest element
\code{x} is \code{x[2]}, the next smallest is \code{x[3]}, and so on.
Thus \code{x[order(x)] == sort(x)}, and \code{rank(x)[order(x)] ==
seq_along(x)}.  In the current context we want ranks not orders; we want
to know who came first, who came second, and so on:

\preformatted{
R> rrank(2,(4:1)/10)
     c1 c2 c3 c4
[1,]  2  3  1  4
[2,]  1  3  2  4
R> 
}

In the above, each row is a race; we have four runners and two races.
In the first race (the top row), runner number 2 came first, runner 3
came second, runner 1 came third, and so on.  In the second race (bottom
row), runner 1 came first, etc.  Taking the first race as an example:

\strong{Rank:} who came first? runner 2.  Who came second? runner 3.
Who came third? runner 1.  Who came fourth? runner 4.  Recall that the
Placket-Luce likelihood for a race in which the rank statistic was
\code{2314} (the first race) would be \eqn{
\frac{p_2}{p_2+p_3+p_1+p_4}\cdot
\frac{p_3}{p_3+p_1+p_4}\cdot
\frac{p_1}{p_1+p_4}\cdot
\frac{p_4}{p_4}}{omitted}.

\strong{Order:} where did runner 1 come?  third.  Where did runner 2
come?  first.  Where did runner 3 come?  second.  Where did runner 4
come?  fourth.  Thus the order statistic would be \code{3124}.

Vignette \dQuote{\code{skating_analysis}} gives another discussion.

Note that function \code{rrank()} returns an object of class
\dQuote{\code{rrank}}, which has its own print method.  This can be
confusing.  Further details are given at \code{ranktable.Rd}.

You can coerce a rank table to an ordertable with
\code{as.ordertable()}.

Function \code{rrank_single()} is a low-level helper function:

\preformatted{
> p <- c(0.02,0.02,0.9,0.02,0.02,0.02)  # competitor 3 the strongest
> rrank_single(p)
[1] 3 2 4 6 4 1
}

Above, we see from \code{p} that competitor 3 is the strongest, coming
first with probability 0.9.  And indeed the resulting rank statistic
given by \code{rorder_single()} shows competitor 3 coming first, 2
coming second, and so on.  Compare \code{rorder_single()}:

\preformatted{
> rorder_single(p)
[1] 6 3 1 4 5 2
> 
}

Above we see, reading elements of \code{rorder_single(p)} sequentially,
that competitor 1 came sixth, competitor 2 came third, and competitor 3
came first (as you might expect, as competitor 3 is the strongest).

Note that the R idiom for \code{rorder_single()} is the same as that
used in the \CRANpkg{permutations} package for inverting a permutation:
\code{o[o] <- seq_along(o)}.

}
\author{Robin K. S. Hankin}
\note{Similar functionality is given by \code{rrace3()}, documented at
  \link{rhyper3}.
}
\seealso{
  \code{\link{ordertrans}},
  \code{\link{ranktable}},
  \code{\link{suppfun}},
  \code{\link{skating}},
  \code{\link{rhyper3}}
}
\examples{

rrank_single(zipf(9))

ptrue <- (4:1)/10
names(ptrue) <- letters[1:4]

rrank()
as.ordertable(rrank())

rrank(10,p=ptrue)

H <- suppfun(rrank(10,p=ptrue))

## Following code commented out because they take too long:

# mH <- maxp(H)   # should be close to ptrue
# H <- H + rank_likelihood(rrank(30,mH)) # run some more races
# maxp(H)  # revised estimate with additional data


}

\name{moto}
\alias{moto}
\alias{motoGP}
\alias{motoGP_2019}
\alias{moto_table}
\alias{moto}
\alias{moto_maxp}
\docType{data}
\title{MotoGP dataset}
\description{Race results from the 2019 Grand Prix motorcycling season}
\usage{data(moto)}

\details{

Object \code{moto_table} is a dataframe of results showing ranks of 28
drivers (riders?) in the 2019 FIM MotoGP World Championship.  The format
is standard, that is, can be interpreted by function
\code{ordertable2supp()} if the final points column is removed.  The
corresponding support function is \code{motoGP_2019}.

These objects can be generated by running script \code{inst/moto.Rmd},
which includes some further discussion and technical documentation and
creates file \code{moto.rda} which resides in the \code{data/}
directory.

}
\references{

Wikipedia contributors. (2020, February 8). 2019 MotoGP season. In
\emph{Wikipedia, The Free Encyclopedia.} Retrieved 08:16, February 20,
2020, from \url{https://en.wikipedia.org/w/index.php?title=2019_MotoGP_season&oldid=939711064}
}
\note{
  Many drivers have names with diacritics, which have been removed from
  the dataframe.
}
\seealso{\code{\link{ordertable2supp}}}
\examples{
pie(moto_maxp)
}

\name{ordertable2supp}
\alias{ordertable2supp}
\alias{ordervec2supp}
\alias{ordervec2supp}
\alias{order_obs}
\alias{salad}
\title{Translate order tables to support functions}
\description{

  Translate order statistics to support functions using Plackett-Luce
  likelihoods.
  
}
\usage{
ordertable2supp(x, noscore, noninformative, incomplete=TRUE)
ordervec2supp(d)
}
\arguments{
  \item{x}{Data frame, see details}
  \item{d}{A named numeric vector giving order; zero entries are
    interpreted as that competitor coming last (due to, e.g., not
    finishing)}
  \item{incomplete}{Boolean, with \code{FALSE} meaning to insist that
    each rank \eqn{1,2,...,n} is present [zero entries mean \dQuote{did
    not place} irregardless].  The default \code{TRUE} allows for gaps.
    This is useful if we are considering the first few lines of an
    ordertable because there might be missing ranks.}

  \item{noscore}{Character vector giving the abbreviations for a
    non-finishing status such as \dQuote{did not finish} or
    \dQuote{disqualified}.  A missing argument is interpreted as
    \code{c("Ret", "WD", "DNS", "DSQ", "DNP", "NC", "DNQ", "EX",
      "Sick")}.  A non-finishing status is equivalent to coming (joint)
    last}
  \item{noninformative}{As for \code{noscore}, a character vector giving
    abbreviations for noninformative statuses.  For example, in the 2025
    MotoGP world championship, Marc Marquez declined to compete in the
    last four races on the grounds that he already had enough points to
    guarantee victory}
}
\details{

Function \code{ordertable2supp()} is intended for use on order tables
such as found at \url{https://en.wikipedia.org/wiki/2019_Moto3_season}.
This is a common format, used for Formula 1, motoGP, and other racing
sports.  Prepared text versions are available in the package in the
\code{inst/} directory, for example \code{inst/motoGP_2019.txt}.  Use
\code{read.table()} to create a data frame which can be interpreted by
\code{ordertable2supp()}.

Function \code{ordertable2supp()} in day-to-day use is discouraged: use
[\code{S3} generic] \code{suppfun()} instead.

Function \code{ordervec2supp()} takes an order vector \code{d} and
returns the corresponding Plackett-Luce loglikelihood function as a
\code{hyper2} object.  It requires a named vector; names of the elements
are interpreted as names of the players.  Use argument \code{pnames} to
supply the players' names (see the examples).

\preformatted{
> x <- c(b=2,c=3,a=1,d=4,e=5) # a: 1st, b: 2nd, c: 3rd etc
> ordervec2supp(x)
log( a * (a + b + c + d + e)^-1 * (a + b + d + e)^-1 * b * (b + d +
e)^-1 * c * (d + e)^-1 * e)
}

\deqn{
\frac{a}{a+b+c+d+e}\cdot
\frac{b}{b+c+d+e}\cdot
\frac{c}{c+d+e}\cdot
\frac{d}{d+e}\cdot
\frac{e}{e}
}{
  omitted
}

Zero entries mean \dQuote{did not finish}:

\preformatted{
> ordervec2supp(c(b=1,a=0,c=2))  # b: 1st, a: DNF, c: second
log((a + b + c)^-1 * (a + c)^-1 * b * c)
  }

\deqn{
\frac{b}{a+b+c}\cdot
\frac{c}{a+c}
}{
  omitted
}

Note carefully the difference between \code{ordervec2supp()} and
\code{rankvec_likelihood()}, which takes a character vector:

\preformatted{
>  names(sort(x))
[1] "a" "b" "c" "d" "e"
> rankvec_likelihood(names(sort(x)))
log( a * (a + b + c + d + e)^-1 * b * (b + c + d + e)^-1 * c * (c + d +
e)^-1 * d * (d + e)^-1)
> rankvec_likelihood(names(sort(x))) == ordervec2supp(x)
[1] TRUE
> 
}

Function \code{order_obs()} was used in the integer-indexed paradigm but
is obsolete in the name paradigm.  A short vignette applying
\code{ordervec2supp()} and \code{ordertable2supp()} to the \link{salad}
dataset of the \CRANpkg{prefmod} package [and further analysed in the
\CRANpkg{PlackettLuce} package] is presented at \code{inst/salad.Rmd}.

The \code{noninformative} argument is discussed at
\code{inst/curling.Rmd} and \code{inst/noninformative.Rmd}.

}

\value{Returns a \code{hyper2} object}
\author{Robin K. S. Hankin}
\seealso{\code{\link{ordertable}},\code{\link{ordertable2supp3}}}
\examples{

ordertable2supp(soling_table)    # discouraged
suppfun(soling_table)            # use this instead

# competitors a-f, racing at two venues:
x <- data.frame(
    venue1=c(1:5,"Ret"),venue2=c("Ret",4,"Ret",1,3,2),
    row.names=letters[1:6])

## First consider all competitors; incomplete=FALSE checks that all
## finishing competitors have ranks 1-n in some order for some n:

ordertable2supp(x, incomplete=FALSE)   # discouraged
suppfun(ordertable(x), incomplete=FALSE)           # use this instead


## Now consider just a-d; must use default incomplete=TRUE as at venue2
## the second and third ranked competitors are not present in x[1:4,]:

ordertable2supp(x[1:4,])   


## The "noninformative" argument [defaulting to c("dead", "won")] means
## to disregard that competitor:

x1 <- data.frame(x=c("won",1:3), row.names=letters[1:4])
ordertable2supp(x1)

## Compare
x2 <- data.frame(x=c("DNS",1:3), row.names=letters[1:4])
ordertable2supp(x2)


## Function ordervec2supp() is lower-level, used for order vectors:

a1 <- c(a=2,b=3,c=1,d=5,e=4) # a: 2nd, b: 3rd, c: 1st, d: 5th, e: 4th
a2 <- c(a=1,b=0,c=0,d=2,e=3) # a: 2nd, b: DNF, c: DNF, d: 2nd, e: 3rd
a3 <- c(a=1,b=3,c=2)         # a: 1st, b: 3rd, c: 2nd. NB only a,b,c competed
a4 <- c(a=1,b=3,c=2,d=0,e=0) # a: 1st, b: 3rd, c: 2nd, d,e: DNF


## results of ordervec2supp() may be added with "+" [if the observations
## are independent]:

H1 <- ordervec2supp(a1) + ordervec2supp(a2) + ordervec2supp(a3)
H2 <- ordervec2supp(a1) + ordervec2supp(a2) + ordervec2supp(a4)

## Thus H1 and H2 are identical except for the third race.  In H1, 'd'
## and 'e' did not compete, but in H2, 'd' and 'e' did not finish (and
## notionally came last):

# pmax(H1)
# pmax(H2)   # d,e not finishing affects their estimated strength

}

\name{NBA}
\alias{NBA}
\alias{NBA_likelihood}
\alias{NBA_maxp}
\alias{NBA_table}
\alias{basketball}
\docType{data}
\title{Basketball dataset}
\description{
A point-by-point analysis of a basketball game
}
\usage{data(NBA)}
\details{

Dataset \code{NBA_table} is a dataframe contains a point-by-point
analysis of a basketball match.  Each row corresponds to a point scored.
The first column is the time of the score, the second is the number of
points scored, the third shows which team had possession at the start of
play, and the fourth shows which team scored.  The other columns show
the players.  Table entries show whether or not that particular player
was on the pitch when the point was scored.   

Likelihood function \code{NBA} is a \code{hyper2} object that gives the
log-likelihood function for this dataset.  There is a player named
\dQuote{\code{possession}} that is a reified entity representing the
effect of possession.

Object \code{NBA_maxp} is not the result of running \code{maxp(NBA)}; it
was obtained by repeatedly running \code{maxp_simplex()} on a
fault-tolerant system [it triggers a known \R bug, bugzilla id 17703,
giving a \dQuote{\code{wmmin not finite}} error].  It is not clear to me
that likelihood function \code{NBA} has a well-defined global maximum.

Object \code{NBA} poses difficulty for the numerical optimization
routines for some reason.

Note that function \code{volley()} is not applicable because we need to
include possession.

These objects can be generated by running script
\code{inst/NBA.Rmd}, which includes some further discussion and
technical documentation and creates file \code{NBA.rda} which
resides in the \code{data/} directory.

}
\references{
  \url{https://www.espn.com/nba/playbyplay?gameId=400954514}
}
\seealso{\code{\link{volleyball}}}
\examples{

data(NBA)
dotchart(NBA_maxp)

}
\keyword{datasets}

\name{baseball}
\alias{baseball}
\alias{baseball_maxp}
\alias{baseball_table}
\docType{data}
\title{Baseball results, following Agresti}
\description{Results from repeated games among seven baseball teams,
  following Agresti}
\usage{data(baseball)}
\format{A \code{hyper2} object that gives a likelihood function}
\details{

Agresti discusses results from seven baseball teams in the 1987 season
of the Eastern Division of the American League.  

A results table and likelihood function is given in the package as
\code{baseball_table} and \code{baseball} respectively.  The maximum
likelihood estimate is given as \code{baseball_maxp}, but can be
reproduced by \code{maxp(baseball)}.
  
These objects can be generated by running script
\code{inst/home_advantage.Rmd}, which includes some further discussion
and technical documentation, and creates file \code{baseball.rda} which
resides in the \code{data/} directory.

}
\references{
  A. Agresti 2002. \dQuote{Categorical data analysis}.  John Wiley and
  Sons;   p437
  }
  \seealso{\code{\link{hyper3}}}
\examples{
baseball_table
baseball_table[1:3,1:3] 
home_away3(baseball_table[1:3,1:3],1.3)

}

\name{ggol}
\alias{ggol}
\alias{ggrl}
\alias{goodbad}
\alias{choose_losers}
\alias{choose_winners}
\alias{general_grouped_rank_likelihood}
\alias{general_grouped_order_likelihood}
\alias{rank_likelihood}
\alias{rankvec_likelihood}
\alias{elimination}
\alias{race}
\alias{Plackett}
\alias{Plackett-Luce}
\title{Order statistics}
\description{
  Various functions for calculating the likelihood function for order
statistics
}
\usage{
ggrl(H, ...)
general_grouped_rank_likelihood(H, ...)
goodbad(winners, losers)
elimination(all_players)
rankvec_likelihood(v, nonfinishers)
race(v, nonfinishers)
}

\arguments{
  \item{H}{Object of class \code{hyper2}}
  \item{\dots}{Numeric or character vectors specifying groups of
    players with  equal rank, with higher-ranking groups coming earlier
    in the argument list}
  \item{all_players,winners,losers}{Numeric or character vectors
    specifying competitors.  See details}
  \item{v}{A character vector specifying ranks.  Thus
  \code{c("b","c","a")} means that \code{b} came first, \code{c} second,
  and \code{a} third} 
\item{nonfinishers}{A character vector with entries corresponding to
  competitors who did not finish.  Thus
  \code{race(c("a","b"),c("p","q"))} means that the field is
  \code{a,b,c,d}; \code{a} came first, \code{b} came second and \code{c}
  and \code{d} did not finish}
}
\details{

  These functions are designed to return likelihood functions, in the
  form of lists of \code{hyper2()} objects, for typical order statistics
  such as the results of rowing heats or MasterChef tournaments.

  Direct use of \code{rankvec_likelihood()} is discouraged: use
  \code{suppfun()} instead, for example \code{suppfun(letters)}.
  
  Function \code{ggrl()} is an easily-typed alias for
  \code{general_grouped_rank_likelihood()}.

  Function \code{goodbad()} is a convenience function for \code{ggrl()}
  in which a bunch of contestants is judged.  It returns a likelihood
  function for the observation that the members of one subset were
  better than those of another.  Thus \code{goodbad(letters[1:3],
  letters[4:5])} corresponds to the observation that \code{d} and
  \code{e} were put into a Masterchef-style elimination trial (and
  \code{a,b,c} were not).  The probability of this is the sum of 12
  Plackett-Luce probabilities, one for each order statistic preserving
  the fact that \code{a,b,c} were in the top 3 and \code{d,e} in the
  bottom 2.  Note that this is \emph{not} the same as
  \dQuote{competitors \code{a,b,c} had a competition against competitors
  \code{c,d} and \code{a,b,c} won}.
  
  Function \code{elimination()} gives a likelihood function for situations
  where the \emph{weakest} player is identified at each stage and
  subsequently eliminated from the competition.  It is intended for
  situations like the Great British Bake-off and Masterchef in which the
  observation is which player was chosen to leave the show.  In this
  function, argument \code{all_players} is sensitive to order, unlike
  \code{choose_winners()} and \code{choose_losers()} (an integer
  \code{n} is interpreted as \code{letters[seq_len(n)]}).  Element
  \code{i} of \code{all_players} is the \eqn{i^\mathrm{th}}{i-th} player
  to be eliminated.  Thus the first element of \code{all_players} is the
  first player to be eliminated (and would be expected to have the
  lowest strength).  The final element of \code{all_players} is the last
  player to be eliminated (or alternatively the only player not to be
  eliminated).

  Function \code{rank_likelihood()} is deprecated: use [\code{S3}
  generic] \code{supp.ranktable()} instead.  This takes a character
  vector of competitors with the order of elements corresponding to the
  finishing order; a Plackett-Luce likelihood function is returned.
  Thus \code{v=c("d","b","c","a")} corresponds to \code{d} coming first,
  \code{b} second, \code{c} third, and \code{a} fourth.  Function
  \code{race()} is an arguably more memorable synonym.

  An example of \code{race()} is given in \code{inst/rowing.Rmd}, and
  examples of \code{ggrl()} are given in \code{inst/loser.Rmd} and
  \code{inst/masterchef.Rmd}.

}
\author{Robin K. S. Hankin}
\seealso{\code{\link{rrank}},\code{\link{ordertable2supp}},\code{\link{race3}}}
\examples{

W <- hyper2(pnames=letters[1:5])
W1 <- ggrl(W, 'a', letters[2:4],'e')  # 6-element list
W2 <- ggrl(W, 'b', letters[3:5],'a')  # 6-element list

like_single_list(equalp(W1),W1)
like_series(equalp(W1),list(W1,W2))

if(FALSE){  # takes too long
# run 10 races:
r1 <- rrank(10,p=(7:1)/28)
colnames(r1) <- letters[1:7]

# Likelihood function for r1:
suppfun(r1)

# convert a rank table to a support function:
suppfun(wikitable_to_ranktable(volvo_table))

H <- hyper2()
for(i in 1:20){
  H <- H + race(sample(letters[1:5],sample(3,1),replace=FALSE))
}
equalp.test(H) # should not be significant (null is true)

H1 <- hyper2(pnames=letters[1:5])
H2 <- choose_losers(H1,letters[1:4],letters[1:2])   # {a,b} vs {c,d}; {a,b} lost
maxplist(H2,control=list(maxit=1))  # control set to save time
}

}

\name{increment}
\alias{increment}
\alias{decrement}
\alias{inc}
\alias{dec}
\title{Increment and decrement operators}
\description{
  Syntactic sugar for incrementing and decrementing likelihood functions.
  Useful for constructs such as \code{H[c("a", "b")] \%<>\% inc()}.
}
\usage{
inc(H, val = 1)
dec(H, val = 1)
}
\arguments{
  \item{H}{A hyper2 object}
  \item{val}{Numeric}
}
\details{

A very frequent operation is to increment a single term in a
\code{hyper2} object.  If

\preformatted{
>  H <- hyper2(list("a", c("a","b"), "c", c("a","b","c")), c(1:3,-6))
>  H
log( a * (a + b + c)^-6 * b^2 * c^3)
}

Suppose we wish to increment the power of \code{a+b}.  We could do:

\preformatted{
H[c("a","b")] <- H[c("a","b")] + 1
}

(see the discussion of \code{hyper2_sum_numeric} at
\code{Ops.hyper2.Rd}; also vignette \code{zeropower}).  Alternatively we
could use \code{magrittr} pipes:

\preformatted{
H[c("a","b")] \%<>\% add(1)
}

But \code{inc} and \code{dec} furnish convenient idiom to accomplish the
same thing:

\preformatted{
H[c("a","b")] \%<>\% inc
}

Functions \code{inc} and \code{dec} default to adding or subtracting 1,
but other values can  be supplied:

\preformatted{
H[c("a","b")] \%<>\% inc(3)
}

Or even 

\preformatted{
H[c("a","b")] \%<>\% inc(H["a"])
}

The deprecated convenience function \code{trial()} uses this idiom; see
\code{pick.Rd}.  Using \code{trial()} in this way ensures that the
powers sum to zero (unlike \code{inc()} and \code{dec()}.

The \code{inc} and \code{dec} operators and the \code{pick()} function
are used in \code{inst/kka.Rmd}.

Frankly \code{inc()} and \code{dec()} don't do anything that
\code{magrittr:add()} and \code{magrittr:subtract()} don't (except have
a default value of 1 (which is surprisingly useful)).

}
\author{Robin K. S. Hankin}
\examples{

(H <- hyper2(list("a",c("a","b"),c("a","b","c")),c(1,1,-2)))
H["a"] %<>% inc
H[c("a","x")] %<>% dec
H

## Better: H %<>% inc(beats("a","x"))


}

\name{karate}
\alias{karate}
\alias{karate_table}
\alias{karate_maxp}
\alias{karate_zermelo}
\docType{data}
\title{Karate dataset}
\description{
  Dataset from the 2018 World Karate Championships, men's 67kg.  It is
  an example of a dataset with too many degrees of freedom to be
  analysed easily by the package.
}
\usage{data(karate)}

\details{

Object \code{karate_table} is a dataframe of results showing results
from the 2018 World Karate Championships, men's 67kg; \code{karate} is
the associated likelihood function.  There are two maximum likelihood
estimates given; \code{karate_maxp}, the evaluate as returned by
\code{maxp()}, and \code{karate_maxp}, returned by \code{zermelo()} [the
value given by \code{maxp()} itself is less likely].

These objects can be generated by running script \code{inst/karate.Rmd},
which includes some further discussion and technical documentation and
creates file \code{karate.rda} which resides in the \code{data/}
directory.
}
\references{
\url{https://en.wikipedia.org/wiki/2018_World_Karate_Championships}
}
\note{
Table \code{karate_table} misses uninformative matches, that is,
competitions with 0-0 results.

}
\seealso{\code{\link{zapweak}}}
\examples{
summary(karate)
}

\name{dirichlet}
\alias{dirichlet}
\alias{dirichlet3}
\alias{rdirichlet}
\alias{is.dirichlet}
\alias{rp_unif}
\alias{gd}
\alias{Dirichlet}
\alias{gd}
\alias{GD}
\alias{GD_wong}
\alias{Connor}
\alias{Mosimann}
\alias{multinom}
\title{
Dirichlet distribution and generalizations
}
\description{
The Dirichlet distribution in likelihood (for \eqn{p}) form, including the
generalized Dirichlet distribution due to Connor and Mosimann
}
\usage{
dirichlet(powers, alpha)
dirichlet3(powers, lambda=NULL)
GD(alpha, beta, beta0=0)
GD_wong(alpha, beta)
rdirichlet(n, H)
is.dirichlet(H)
rp_unif(n, H)
multinom(x)
}
\arguments{
  \item{powers, x}{In functions \code{dirichlet()} and \code{multinom()},
    a (named) vector of powers}
  \item{alpha, beta}{A vector of parameters for the Dirichlet or
    generalized Dirichlet distribution}
  \item{beta0}{In function \code{GD()}, an arbitrary parameter}
  \item{H}{Object of class \code{hyper2}}
  \item{lambda}{Vector of weights in \code{dirichlet3()}}
  \item{n}{Number of observations}
}
\details{

  The \dfn{Dirichlet distribution} of order \eqn{K\geqslant 2}{omitted}
  with parameters \eqn{\alpha_1,\ldots,\alpha_K>0}{omitted} has a
  probability density function \eqn{\propto\prod_{i=1}^K
  p_i^{\alpha_{i}-1}}{omitted} where \eqn{p_i\geqslant 0}{omitted} and
  \eqn{\sum p_i=1}{omitted}.

  Function \code{rdirichlet()} returns random samples drawn from a
  Dirichlet distribution using the gamma distribution.  If second
  argument \code{H} is a \code{hyper2} object, it is tested [with
  \code{is.dirichlet()}] for being a Dirichlet distribution.  If so,
  samples from it are returned.  If not, (e.g. \code{icons}), an error
  is given.  If \code{H} is not a \code{hyper2} object, it is
  interpreted as a (possibly named) vector of parameters
  \eqn{\alpha}{alpha} [\strong{not} a vector of powers].
  
  Function \code{rp_unif()} returns uniformly distributed vectors,
  effectively using \code{H*0}; but note that this uses Dirichlet
  sampling which is much faster and better than the Metropolis-Hastings
  functionality documented at \code{rp.Rd}.

  Functions \code{GD()} and \code{GD_wong()} return a likelihood
  function corresponding to the Generalized Dirichlet distribution as
  presented by Connor and Mosimann, and Wong, respectively.  In
  \code{GD_wong()}, \code{alpha} and \code{beta} must be named vectors;
  the names of \code{alpha} give the names of
  \eqn{x_1,\ldots,x_k}{x_1,...,x_k} and the last element of \code{beta}
  gives the name of \eqn{x_{k+1}}.

  Function \code{dirichlet3()} returns a \code{hyper3} object with
  weights \code{lambda}.  If \code{lambda} is length less than that of
  \code{powers}, it is padded with \code{1}s [so default \code{NULL}
  corresponds to unit weights, that is, a \code{hyper2} object].  A
  use-case is given in \code{inst/rock_paper_scissors_monster.Rmd}.

  Experimental function \code{multinom()} also takes a named vector and
  interprets it in almost the same way as \code{dirichlet()}, but it
  allows for repeated names.  It is interpreted as the support function
  for multinomial trials.  If \code{x} is a named vector with no
  repeats, \code{dirichlet(x) == multinom(x)}.  But \code{multinom()}
  uses \CRANpkg{frab} formalism to deal with multiple entries with the
  same name, which are summed.  This is the Right Way (tm) to do this.
  Negative numbers, having no natural interpretation in this context,
  are not allowed.  The numbers do not need to be integers.  One
  use-case is given in the examples (which should be faster than looping
  over the named vector and accumulating a \code{hyper2} object, element
  by element; but I have no timings on this as yet).
   
}

\references{
  \itemize{
    \item R. J. Connor and J. E. Mosimann 1969.  \dQuote{Concepts of
    independence for proportions with a generalization of the Dirichlet
    distribution}.  \emph{Journal of the American Statistical
    Association}, 64:194--206
    \item T.-T. Wong 1998. \dQuote{Generalized Dirichlet distribution in
    Bayesian Analysis}.  \emph{Applied Mathematics and Computation},
    97:165--181 }
}

\author{Robin K. S. Hankin} 

\note{
  A dirichlet distribution can have a term with zero power.  But
  this poses problems for \code{hyper2} objects as zero power brackets
  are dropped.

  Function \code{dirichlet3()} is a replacement for now removed function
  \code{pair3()}.

  Function \code{rdirichlet()} commits a very mild (but necessary in the
  absence of a working \code{dismat} package) violation of \code{disordR}
  discipline, as the columns of the returned matrix have the same order
  as \code{pnames(H)}

}

\seealso{\code{\link{hyper2}},\code{\link{rp}}}

\examples{

x1 <- dirichlet(c(a=1, b=2, c=3))
x2 <- dirichlet(c(c=3, d=4))

x1+x2

multinom(c(x=2, a=3, b=3, x=1))  # NB repeated 'x'

x <- setNames(rpois(40,2.2), sample(letters[1:4],40,replace=TRUE))
multinom(x)

H <- dirichlet(c(a=1, b=2, c=3, d=4))
rdirichlet(10, H)
colMeans(rdirichlet(1e4, H))

dirichlet3(c(fish=3, chips=2), lambda=1.8)
dirichlet3(c(x=6, y=5, z=2), 1:3)

}

\name{zapweak}
\alias{zapweak}
\title{Zap weak competitors}
\description{
Given a \code{hyper2} object, discard competitors with a small estimated
strength.}
\usage{
zapweak(H, minstrength = 1e-05, maxit, ...)
}
\arguments{
  \item{H}{Object of class \code{hyper2}}
  \item{minstrength}{Strength below which to discard competitors}
  \item{maxit}{Maximum number of iterations; if missing, use
    \code{size(H)-1}}
  \item{\dots}{Further arguments, passed to \code{maxp()}}
}
\details{
Iteratively discards the weakest player (if the estimated strength is
less than \code{minstrength}) using \code{discard_flawed()}.
\code{maxp(..,n=1)} for efficiency.  
}
\value{Returns a slimmed-down \code{hyper2} object with weak players
  removed.
}
\note{

  This function is experimental and appears to be overly aggressive.
  For some likelihood functions \code{zapweak()} removes \emph{all} the
  players.

  I now think that there is no consistent way to remove weaker players
  from a likelihood function.  I think the only way to do it is to look
  at the dataset that generates the likelihood function, somehow weed
  out the players with the poorest performance, and generate a new
  likelihood function without them.

}

\seealso{\code{\link{discard_flawed}},\code{\link{maxp}}}
\author{Robin K. S. Hankin}
\examples{
 zapweak(icons)        # removes noone
# zapweak(rowing)      # removes everyone...

}

\name{suplist}
\alias{suplist}
\alias{supplist}
\alias{Ops.suplist}
\alias{suplist_add}
\alias{as.suplist}
\alias{sum.suplist}
\alias{suplist_times_scalar}
\alias{lsl}
\alias{loglik_lsl}
\alias{Ops.lsl}
\alias{lsl_add}
\title{Methods for suplist objects}
\description{

  Basic functionality for lists of \code{hyper2} objects, allowing the
  user to concatenate independent observations which are themselves
  composite objects such as returned by \code{ggrl()}.

}
\usage{
\method{Ops}{suplist}(e1, e2)
\method{sum}{suplist}(x,...,na.rm=FALSE)
suplist_add(e1,e2)
suplist_times_scalar(e1,e2)
as.suplist(L)
}
\arguments{
  \item{e1,e2}{Objects of class \code{suplist}, here interpreted as
    a list of possible likelihood functions (who should be added)}
  \item{x,...,na.rm}{In the \code{sum()} method, objects to be summed;
    \code{na.rm} is currently ignored}
  \item{L}{A list of \code{hyper2} objects}
}
\details{

  A \code{suplist} object is a list of \code{hyper2} objects.  Each
  element is a \code{hyper2} object that is consistent with an
  incomplete rank observation \eqn{R}; the list elements are exclusive
  and exhaustive for \eqn{R}.  If \code{S} is a \code{suplist} object,
  and \code{S=list(H1,H2,...,Hn)} where the \code{Hi} are \code{hyper2}
  objects, then
  \eqn{\mbox{Prob}(p|H_1)+\cdots+\mbox{Prob}(p|H_n)}{Prob(p|H1)+...+Prob(p|Hn)}.
  This is because the elements of a \code{suplist} object are disjoint
  alternatives.

 It is \strong{incorrect} to say that a likelihood function
  \eqn{\mathcal{L}_S(p)}{L_S(p)} for \eqn{p} is the sum of separate
  likelihood functions.  This is incorrect because the arbitrary
  multiplicative constant messes up the math, for example we might have
  \eqn{\mathcal{L}_{H_1}(p)=C_1\mathrm{Prob}(p|H_1)}{?} and
  \eqn{\mathcal{L}_{H_2}(p)=C_2\mathrm{Prob}(p|H_2)}{?} and indeed
  \eqn{\mathcal{L}_{{H_1}\cup H_2}(p)=C_{12}\left(\mathrm{Prob}(p|H_1)+\mathrm{Prob}(p|H_2)\right)}{?}  but

 \deqn{\mathcal{L}_{H_1}(p)+\mathcal{L}_{H_2}(p) \neq
   C_1\mathrm{Prob}(p|H_1)+C_2\mathrm{Prob}(p|H_2)}{see PDF}
    
 (the right hand side is meaningless).

 Functions \code{suplist_add()} and \code{sum.suplist()} implement
  \dQuote{\code{S1+S2}} as the support function for independent
  observations \code{S1} and \code{S2}.  The idea is that the support
  functions \dQuote{add} in the following sense.  If
  \code{S1=list(H1,...,Hr)} and \code{S2=list(I1,...,Is)} where
  \code{Hx,Ix} are \code{hyper2} objects, then the likelihood function
  for \dQuote{\code{S1+S2}} is the likelihood function for \code{S1}
  followed by (independent) \code{S2}.  Formally

  \deqn{
    \mbox{Prob}(p|S_1+S_2) =
    \left(
    \mbox{Prob}(p|H_1)
    +\cdots+
    \mbox{Prob}(p|H_r)
    \right)\cdot\left(
    \mbox{Prob}(p|I_1)
    +\cdots+
    \mbox{Prob}(p|I_s)
    \right)}{omitted; see PDF}

  \deqn{
    \log\mbox{Prob}(p|S_1+S_2) =
    \log\left(
    \mbox{Prob}(p|H_1)
    +\cdots+
    \mbox{Prob}(p|H_r)
    \right)+\log\left(
    \mbox{Prob}(p|I_1)
    +\cdots+
    \mbox{Prob}(p|I_s)
    \right)}{omitted; see PDF}
  
  However, \code{S1+S2} is typically a large and unwieldy object, and
  can be very slow to evaluate.  These functions are here because they
  provide slick package idiom.

  The experimental \code{lsl} mechanism furnishes an alternative
  methodology which is more computationally efficient at the expense of
  a non-explicit likelihood function.  It is not clear at present (2022)
  which of the two systems is better.
  
}
\value{
Returns a \code{suplist} object.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{Ops.hyper2}},\code{\link{Extract}},\code{\link{loglik}}}
\examples{

W <- hyper2(pnames=letters[1:5])
W1 <- ggrl(W, 'a', letters[2:3],'d')  # 2-element list
W2 <- ggrl(W, 'e', letters[1:3],'d')  # 6-element list
W3 <- ggrl(W, 'c', letters[4:5],'a')  # 2-element list

# likelihood function for independent observations  W1,W2,W3:

W1+W2+W3 # A 2*6*2=24-element list

like_single_list(equalp(W),W1+W2+W3)
\dontrun{dotchart(maxplist(W1+W1+W3),pch=16)} # takes a long time

a <- lsl(list(W1,W2,W3),4:6)  # observe W1 four times, W2 five times and W3 six times
loglik_lsl(equalp(W),a,log=TRUE)

}
\keyword{symbolmath}

\name{hyper2}
\alias{hyper2}
\alias{brackets}
\alias{brackets.hyper2}
\alias{powers}
\alias{powers.hyper2}
\alias{powers<-}
\alias{powers<-.hyper2}
\alias{pnames}
\alias{pnames.hyper2}
\alias{pnames.suplist}
\alias{pnames<-}
\alias{pnames<-.hyper2}
\alias{size}
\alias{size.hyper2}
\alias{is.hyper2}
\alias{as.hyper2}
\alias{is_valid_hyper2}
\alias{is_constant}
\title{
Basic functions in the hyper2 package
}
\description{
Basic functions in the hyper2 package
}
\usage{
hyper2(L=list(), d=0, pnames)
\method{brackets}{hyper2}(H)
\method{powers}{hyper2}(H)
\method{pnames}{hyper2}(H)
\method{pnames}{suplist}(H)
size(H)
as.hyper2(L,d,pnames)
is.hyper2(H)
is_valid_hyper2(L,d,pnames)
is_constant(H)
}
\arguments{
  \item{H}{A hyper2 object}
  \item{L}{A list of character vectors whose elements specify
    the brackets of a hyper2 object}
  \item{d}{A vector of powers; \code{hyper2()} recycles \emph{only if}
    \code{d} is of length 1}
  \item{pnames}{
    A character vector specifying the names of \eqn{p_1} through \eqn{p_n}.}
}
\details{
These are the basic functions of the hyper2 package.  Function
\code{hyper()} is the low-level creator function; \code{as.hyper2()} is
a bit more user-friendly and attempts to coerce its arguments into a
suitable form; for example, a matrix is interpreted as rows of brackets.

Functions \code{pnames()} and \code{pnames<-()} are the accessor and
setter methods for the player names.  Length-zero character strings are
acceptable player names.  The setter method \code{pnames<-()} can be
confusing.  Idiom such as \code{pnames(H) <- value} does not change the
likelihood function of \code{H} (except possibly its domain).  When
called, it changes the \code{pnames} internal vector, and will throw an
error if any element of \code{c(brackets(H))} is not present in
\code{value}.  It has two uses: firstly, to add players who do not
appear in the brackets; and secondly to rearrange the \code{pnames}
vector (the canonical use-case is \code{pnames(H) <- rev(pnames(H))}).
If you want to change the player names, use \code{psubs()} to substitute
players for other players.

Function \code{is_valid_hyper2()} tests for valid input, returning a
Boolean.  This function returns an error if a bracket contains a
repeated element, as in \code{hyper2(list(c("a","a")),1)}.

Note that it is perfectly acceptable to have an element of
\code{pnames} that is not present in the likelihood function (this would
correspond to having no information about that particular player).

Function \code{size()} returns the (nominal) length \eqn{n} of
nonnegative vector \eqn{p=\left(p_1,\ldots,p_n\right)}{p=(p_1,...,p_n)}
where \eqn{p_1+\cdots+p_n=1}{p_1+...+p_n=1}.

}
\author{Robin K. S. Hankin}
\seealso{
  \code{\link{Ops.hyper2}},
  \code{\link{Extract.hyper2}},
  \code{\link{loglik}},
  \code{\link{hyper2-package}}
  \code{\link{psubs}}
}
\examples{
(o <- hyper2(list("a","b","c",c("a","b"),letters[1:3]),c(1:3,-1,-5)))
(p <- hyper2(list("a",c("a","d")),c(1,-1)))
o+p


# Verify that the MLE is invariant under reordering
pnames(icons) <- rev(pnames(icons))
maxp(icons) - icons_maxp # should be small

}

\name{soling}
\alias{soling}
\alias{soling_after}
\alias{soling_maxp}
\alias{soling_after_maxp}
\alias{soling2000}
\alias{soling_table_2000}
\alias{soling_table}
\alias{soling2000_rr1}
\alias{soling2000_rr2}
\alias{soling2000_qf}
\alias{soling_table}
\alias{soling_rr1}
\alias{soling_rr2}
\alias{soling_qf}
\alias{soling_after}
\docType{data}
\title{Sailing at the 2000 Summer Olympics - soling}
\description{Race results from the 2000 Summer Olympics: soling}
\usage{data(soling)}
\format{A \code{hyper2} object that gives a likelihood function}
\details{

The Soling three person keelboat event at the 2000 Summer Olympic games
furnishes a rich dataset.  An order table and likelihood function is
given in the package as \code{soling_table} and \code{soling}
respectively.  Data from the round robins and the quarter final is given
in matrices \code{soling_rr1}, \code{soling_rr2}, \code{soling_qf}
respectively.

These objects can be generated by running script \code{inst/soling.Rmd},
which includes some further discussion and technical documentation, and
creates file \code{soling.rda} which resides in the \code{data/}
directory.

}
\references{

  Wikipedia contributors, \dQuote{Sailing at the 2000 Summer Olympics -
  Soling,} Wikipedia, The Free Encyclopedia,
  \url{https://en.wikipedia.org/w/index.php?title=Sailing_at_the_2000_Summer_Olympics_\%E2\%80\%93_Soling&oldid=945362535}
  (accessed March 23, 2020).

  }
\seealso{\code{\link{ordertable2supp}}}
\examples{
data(soling)
ordertable_to_ranktable(soling_table)

\dontrun{soling == suppfun(ordertable_to_ranktable(soling_table))}
pie(soling_maxp)
}

\name{consistency}
\alias{consistency}
\alias{consistencyplot}
\title{Consistency check for hyper2 objects}
\description{
Given a \code{hyper2} object, calculate the maximum likelihood point in
two ways and plot one against the other to check for consistency.
}
\usage{
consistency(H, plot=TRUE, ...)
}
\arguments{
  \item{H}{A \code{hyper2} object}
  \item{plot}{If \code{TRUE} (default), plot a comparison and return a
    matrix invisibly, and if \code{FALSE} return the matrix.  Modelled
    on argument \code{plot} of \code{hist}}
  \item{...}{Further arguments, passed to \code{points()}}
}

\details{

  Given a \code{hyper2} object, calculate the maximum likelihood
  estimate of the players' strengths using \code{maxp()}; then reverse
  the \code{pnames} attribute and calculate the players' strengths
  again.  These two estimates should be identical but small differences
  highlight numerical problems.  Typically, the differences are small if
  there are fewer than about 25 players.

  Reversing the \code{pnames()} is cosmetic in theory but is a
  non-trivial operation: for example, it changes the identity of the
  fillup from the last player to the first.
    
}

\value{Returns a named three-row matrix with first row being the direct
  evaluate, second row being the reverse of the reversed evaluate, and
  the third being the difference
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{ordertrans}}}
\examples{

# consistency(icons)

x <- icons
y <- icons
pnames(y) <- rev(pnames(y))
gradient(x,indep(equalp(x)))
gradient(y,indep(equalp(y)))

}

\name{eurodance}
\alias{eurodance}
\alias{Eurodance}
\alias{eurodance_table}
\alias{eurodance_maxp}
\docType{data}
\title{Eurovision Dance contest dataset}
\description{Voting patterns from Eurovision Dance Contest 2008}
\usage{data(eurovision)}
\format{
A \code{hyper2} object that gives a likelihood
function.
}
\details{

Object \code{eurodance} is a \code{hyper2} object that gives a
likelihood function for the skills of the 14 competitor countries in
2008 Eurovision Dance contest.  Object \code{eurodance_table} gives the
original dataset and \code{eurodance_maxp} the evaluate of the
competitors' Plackett-Luce strengths.

The dataset is interesting because, in addition to the regular votes by
each nation, there is an Expert jury vote as well.  We may use
Plackett-Luce likelihoods to compare the performance of the Expert jury
with the national votes.

These objects can be generated by running script
\code{inst/eurodance.Rmd}, which includes some further discussion and
technical documentation and creates file \code{eurodance.rda} which
resides in the \code{data/} directory.

}
\references{
  \itemize{
    \item 
  Wikipedia contributors, \dQuote{Eurovision Song Contest
    2009---Wikipedia, The Free Encyclopedia}, 2018,
    \url{https://en.wikipedia.org/w/index.php?title=Eurovision_Song_Contest_2009&oldid=838723921}  [Online; accessed 13-May-2018].
    \item P. M. E. Altham, personal communication
  }
}
\seealso{\code{\link{eurodance}}}
\examples{
data(eurodance)
dotchart(eurodance_maxp)



}

\name{counterstrike}
\alias{counterstrike}
\alias{counterstrike_likelihood}
\alias{counterstrike_maxp}
\alias{zacslist}
\docType{data}
\title{Counterstrike}
\description{
A kill-by-kill analysis of a counterstrike game.
}
\usage{data(counterstrike)}
\details{

E-sports are a form of competition using video games.  E-sports are
becoming increasingly popular, with high-profile tournaments attracting
over 400 million viewers, and prize pools exceeding US$20m.

Counter Strike: Global Offensive (CS-GO) is a multiplayer first-person
shooter game in which two teams of five compete in an immersive virtual
reality combat environment.  CS-GO is distinguished by the ability to
download detailed gamefiles in which every aspect of an entire match is
recorded, it being possible to replay the match at will.

Statistical analysis of such gamefiles is extremely difficult, primarily
due to complex gameplay features such as cooperative teamwork,
within-team communication, and real-time strategic fluidity.

It is the task of the statistician to make robust inferences from such
complex datasets, and here I discuss data from an influential match
between \dQuote{FaZe Clan} and \dQuote{Cloud9}, two of the most
successful E-sports syndicates of all time, when they competed at Boston
2018.
  
Dataset \code{counterstrike} is a loglikelihood function for the
strengths of ten counterstrike players; \code{counterstrike_maxp} is a
precomputed evaluate, and \code{zacslist} the observations used to
calculate the loglikelihood function.

The probability model is similar to that of \code{NBA}: when a player
kills (scores), this is taken to be a success of the whole team rather
than the shooter.

File \code{inst/counterstrike.R} and \code{inst/counterstrike_random.R}
include some further randomisation tests and discussion.

The objects documented here can be generated by running script
\code{inst/counterstrike.Rmd}, which includes some further discussion
and technical documentation and creates file \code{counterstrike.rda}
which resides in the \code{data/} directory.

Counterstrike dataset kindly supplied by Zachary Hankin.

}
\references{
  \itemize{
    \item \url{https://www.youtube.com/watch?v=XKWzlG4jDnI}
    \item \url{https://en.wikipedia.org/wiki/FaZe_Clan}
    \item \url{https://en.wikipedia.org/wiki/Cloud9}
  }
}
\examples{
dotchart(counterstrike_maxp)
}
\keyword{datasets}

\name{table_tennis}
\alias{table_tennis}
\alias{table_tennis_serve}
\alias{table_tennis_maxp}
\alias{ping_pong}
\docType{data}
\title{Match outcomes from repeated table tennis matches}
\description{Match outcomes from repeated singles table tennis matches}
\usage{data(table_tennis)}
\format{
A likelihood function corresponding to the match outcomes listed below.
}
\details{

There are four players, A, B, and C, who play singles table tennis
matches with the following results:

\itemize{
  \item A vs B, A serves, 5-1
  \item A vs B, B serves, 1-3
  \item A vs C, A serves, 4-1
  \item A vs C, C serves, 1-2
}

Object \code{table_tennis} is a \code{hyper2} log-likelihood function
for these observations; \code{S} is the strength of the serving ghost.
Probability vector \code{table_tennis_maxp} is the maximum likelihood
estimate.

These objects can be generated by running script
\code{inst/table_tennis_serve.Rmd}, which includes some further
discussion and technical documentation and creates file
\code{table_tennnis.rda} which resides in the \code{data/} directory.
  
}

\examples{
data(table_tennis)
dotchart(table_tennis_maxp)

specificp.gt.test(table_tennis, "S", 0)

}
\keyword{datasets}

\name{cplusplus}
\alias{cplusplus}
\alias{overwrite}
\alias{overwrite3}
\alias{accessor}
\alias{accessor3}
\alias{assigner}
\alias{assigner3}
\alias{addL}
\alias{addL3}
\alias{equal}
\alias{equality}
\alias{equality3}
\alias{identityL}
\alias{identityL3}
\alias{evaluate}
\alias{evaluate3}
\alias{differentiate}
\alias{differentiate3}
\alias{differentiate_n}
\alias{hyper2_accessor}
\alias{hyper2_equal}
\alias{hyper2_addL}
\alias{hyper2_assigner}
\alias{hyper2_differentiate}
\alias{hyper2_evaluate}
\alias{hyper2_identityL}
\alias{hyper2_overwrite}
\title{Wrappers to c calls}
\description{
Various low-level wrappers to \proglang{C} functions, courtesy of Rcpp
}
\usage{
overwrite(L1, powers1, L2, powers2)
accessor(L,powers,Lwanted)
assigner(L, powers, L2, value)
addL(L1, powers1, L2, powers2)
identityL(L,powers)
evaluate(L, powers, probs, pnames)
differentiate(L, powers, probs, pnames, n)
differentiate_n(L, powers, probs, pnames, n)
}

\arguments{
  \item{L, L1, L2, Lwanted}{
    Lists with character vector elements, used to specify the brackets of
    the hyperdirichlet distribution}
  \item{powers, powers1, powers2}{
    A numeric vector specifying the powers to which the brackets are raised
  }
  \item{value}{RHS in assignment, a numeric vector}
  \item{probs}{Vector of probabilities for evaluation of log-likelihood}
  \item{pnames}{Character vector of names}
  \item{n}{Integer specifying component to differentiate with respect to}
}
\details{
These functions are not really intended for the end-user, as
out-of-scope calls may cause crashes.

}
\value{These functions return a named List}
\author{Robin K. S. Hankin}

\name{balance}
\alias{balance}
\title{Enforce the zero power sum condition}
\description{

Sometimes a \code{hyper2} object is unbalanced in the sense that its
powers do not sum to zero.  This is rectified by \code{balance()}, which
modifies the power of the bracket corresponding to the sum of all pnames
accordingly.

}
\usage{
balance(H)
}
\arguments{
  \item{H}{object of class \code{hyper2} or \code{hyper3}}
}
\details{

  This is just a convenience function, all it does is

  \preformatted{
    H[pnames(H)] <- 0
    H[pnames(H)] <- -sum(pnames(H))
    H
  }


(the first line ensures that \code{H[pnames(H)]} is over-written
correctly by the second).  Package vignette \code{zeropower} discusses
the zero power sum condition: in particular, note that \code{loglik()}
throws an error if its likelihood function does not satisfy the
zero-sum-power condition.
  
}
\value{Returns a balanced \code{hyper2} object}
\author{Robin K. S. Hankin}
\seealso{\code{\link{print.hyper2}}, \code{\link{balance}}}
\examples{


H <- hyper2()
H["a"] <- 6
H["b"] <- 3
H[c("a","c")] <- 7
H <- balance(H)
maxp(H)
}

\name{Ops.hyper2}
\alias{Ops.hyper2}
\alias{Ops}
\alias{hyper2_sum_hyper2}
\alias{hyper2_add}
\alias{hyper2_prod}
\alias{hyper2_sum_numeric}
\alias{sum.hyper2}
\title{Arithmetic Ops Group Methods for hyper2 objects}
\description{

  Allows arithmetic operators \dQuote{\code{+}}, \dQuote{\code{*}} and
  comparison operators \dQuote{\code{==}} and \dQuote{\code{!=}}, to be
  used for hyper2 objects.
}
\usage{
\method{Ops}{hyper2}(e1, e2 = NULL)
\method{sum}{hyper2}(x, ..., na.rm=FALSE)
hyper2_sum_hyper2(e1, e2)
hyper2_sum_numeric(H, r)
}
\arguments{
  \item{e1, e2}{Objects of class \code{hyper2}, here interpreted as
    hyperdirichlet distributions}
  \item{x, ..., na.rm}{In the \code{sum()} method, objects to be summed;
    \code{na.rm} is currently ignored}
  \item{H, r}{In function \code{hyper2_sum_numeric()}, object \code{H} is
    a \code{hyper2} object and \code{r} is a length-one real vector (a
    number)}
}
\details{

  If two independent datasets have \code{hyper2} objects \code{H1} and
  \code{H2}, then package idiom for combining these would be \code{H1+H2};
  the additive notation \dQuote{\code{+}} corresponds to addition of the
  support (or multiplication of the likelihood).  So \code{hyper2}
  objects are better thought of as support functions than likelihood
  functions; this is reflected in the print method which explicitly
  wraps the likelihood function in a \dQuote{\code{log()}}.

  Scalar multiplication is defined: \code{n*H1} returns
  \code{H1+H1+...+H1}, corresponding to \code{n} repeated independent
  observations of the same data.

  Idiom \code{H1-H1} returns \code{H1 + (-1)*H2}, useful for
  investigating the difference between likelihood functions arising from
  two different observations, or different probability models.  An
  example is given in \code{inst/soling.Rmd}.
      
  Testing for equality is not straightforward for two implementation
  reasons.  Firstly, the object itself is stored internally as a
  \code{stl} \code{map}, which does not store keys in any particular
  order; and secondly, the \code{stl} \code{set} class is used for the
  brackets.  A set does not include information about the order of its
  elements; neither does it admit repeated elements.  See examples.

  A \code{hyper2} object may be added to a numeric vector under some
  circumstances.  If \code{H} is a \code{hyper2} object and \code{n} a
  numeric vector, then \dQuote{\code{H+n}} returns a \code{disord}
  object corresponding to \code{powers(H) + n}, if this is allowed.
  Function \code{hyper2_sum_numeric()} is defined so that idiom like
  \code{icons["L"] <- icons["L"] + 3} and \code{icons["L"] \%<>\%inc(3)}
  work as expected (without this, one has to type \code{icons["L"] <-
  powers(icons["L"]) + 3}, which sucks).

  Formerly, this operation returned an error if the powers have length
  exceeding one, so \dQuote{\code{icons[list("NB","L")] + 5}} failed.
  But because the sum is now defined to be a \code{disord} object,
  errors are flagged, correctly, as \CRANpkg{disordR} discipline
  violations.

  Raising a \code{hyper2} object to a power returns an error.  There are
  no unary operations for this class.
}
\value{
Returns a \code{hyper2} object or a Boolean.
}
\author{
Robin K. S. Hankin
}
\examples{

chess2 <- hyper2(list("Kasparov","Karpov",c("Kasparov","Karpov")),c(2,3,-5))

chess + chess2

maxp(chess + chess2)


}
\keyword{symbolmath}

\name{weights}
\alias{weight}
\alias{weights}
\alias{weights.hyper3}
\alias{setweight}
\alias{as.weight}
\alias{swait}
\title{Getting and setting the weights of symbols in \code{hyper3} objects}
\description{

Brackets of \code{hyper3} objects contain weighted probabilities; the
functionality documented here gets and sets the weights.

}
\usage{
weights.hyper3(object, ...)
as.weight(x)
setweight(x, index, value)
swait(x, ...)
}
\arguments{
  \item{x, object}{Objects of class \code{hyper3}}
  \item{index,value}{Vector of index names and values}
  \item{\dots}{named arguments corresponding to players to change weights}
}
\details{

  Suppose we have a \code{hyper3} object:

  \preformatted{
    > H <- hyper3_nv(list(c(a=1.2), c(b=1), c(a=1.2,b=1,c=1)), powers=c(3,4,-7))
    > H
    log( (a=1.2)^3 * (a=1.2, b=1, c=1)^-7 * (b=1)^4)
}


And we would like to change the weights of one of the players.  We can
do this in several ways:

Function \code{setweight()} alters the weight of every occurrence of a
set of players.  It is vectorised, so
\code{setweight(H,c("a","b"),88:89)} sets the weight of \code{a} to 88
and \code{b} to 89.  Example:

  \preformatted{
    > H <- hyper3_nv(list(c(a=1.2), c(b=1), c(a=1.2,b=1,c=1)), powers=c(3,4,-7))  # as before
    > setweight(H, "a",1.88)
    log( (a=1.88)^3 * (a=1.88, b=1, c=1)^-7 * (b=1)^4)
}

Experimental function \code{swait()} admits idiom such as \code{swait(H,
a=1.88)} to accomplish the same thing.

Replacement methods are defined, so \dQuote{\code{H["a"] <-
as.weight(3)}} will set the weight of every occurrence of player
\code{a} to 3.  If \code{H} is a \code{hyper2} object, it will be
coerced to \code{hyper3}.  

}
\value{
Generally return or deal with \code{hyper3} objects
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{hyper2}}}
\examples{

H <- hyper3_nv(list(c(a=1.2), c(b=1), c(a=1.2,b=1,c=1)), powers=c(3,4,-7))
H

setweight(H, "b", 1.88)


swait(H, b=1.88) # same thing


swait(H, a=1.1, b=2.2)

H |> swait(b=1.88)


race(letters[1:6]) |> swait(a=1.1, b=1.88)

}

\name{T20}
\alias{T20}
\alias{T20_table}
\alias{T20_toss}
\alias{T20_maxp}
\alias{T20_toss_maxp}
\docType{data}
\title{Indian Premier League T20 cricket}
  \description{
    Cricket dataset, T20 Indian Premier League  2008-2017
}
\usage{data(T20)}
\details{

  Dataframe \code{T20_table} has one row for each T20 \acronym{IPL}
  match in the period 2008-2017 with the exception of seven drawn
  matches and three no-result matches which were removed.  Object
  \code{T20} is a likelihood function for the strengths of the 13 teams,
  and \code{T20_toss} is a likelihood function that also includes a toss
  strength term.

  These objects can be generated by running script \code{inst/T20.Rmd},
  which is based on Chandel and Hankin 2019.  This includes some further
  discussion and technical documentation and creates file \code{T20.rda}
  which resides in the \code{data/} directory.

}
\references{
  \itemize{
    \item T. Chandel and R. K. S. Hankin 2019.  \dQuote{Analysing the
      impact of winning a coin toss in the Indian Premier League}.
    Auckland University of Technology.
  }
}
\examples{
summary(T20)
dotchart(T20_maxp)

}
\keyword{datasets}

\name{hyper3}
\alias{hyper3}
\alias{as.hyper3}
\alias{hyper3_to_hyper2}
\alias{as.namedvectorlist}
\alias{assign_lowlevel3}
\alias{hyper3_bw}
\alias{hyper3_nv}
\alias{hyper3_m}
\alias{is.hyper3}
\alias{is_ok_weightedplayers}
\alias{is_valid_hyper3}
\alias{loglik_single_redundant}
\alias{maxp3}
\alias{list2nv}
\alias{overwrite_lowlevel3}
\alias{powers<-.hyper3}
\alias{pnames<-.hyper3}
\title{Weighted probability vectors: \code{hyper3} objects}
\description{

Objects of class \code{hyper3} are a generalization of \code{hyper2} objects
that allow the brackets to contain weighted probabilities.

}
\usage{
hyper3(B = list(), W = list(), powers = 0, pnames)
hyper3_bw(B = list(), W = list(), powers = 0, pnames)
hyper3_nv(L=list(), powers=0, pnames)
hyper3_m(M, p, stripzeros=TRUE)
}
\arguments{
  \item{B}{A list of brackets}
  \item{W}{A list of weights}
  \item{L}{A list of named vectors}
  \item{powers}{Numeric vector of powers}
  \item{pnames}{Character vector of player names}
  \item{M}{Matrix of weights, column names being player names}
  \item{p}{Vector of powers, length equal to \code{ncol(M)}}
  \item{stripzeros}{Boolean with default \code{TRUE} meaning to silently
    remove all-zero rows of \code{M}}
}
\details{


As a motivating example, suppose two players with Bradley-Terry
strengths \eqn{p_1,p_2} play chess where we quantify the first-mover
advantage with a term \eqn{\lambda}{omitted}.  Suppose \eqn{p_1} plays
white \eqn{a+b} times with \eqn{a} wins and \eqn{b} losses, and plays
black \eqn{c+d} times with \eqn{c} wins and \eqn{d} losses.  Then a
sensible likelihood function might be

\deqn{
  \left(\frac{\lambda p_1}{\lambda p_1 + p_2}\right)^{a}
  \left(\frac{p_2        }{\lambda p_1 + p_2}\right)^{b}
  \left(\frac{p_1        }{p_1 + \lambda p_2}\right)^{c}
  \left(\frac{\lambda p_2}{p_1 + \lambda p_2}\right)^{d}
  }{omitted: see PDF}

If \eqn{a=1,b=2,c=3,d=4} and \eqn{\lambda=1.3}{omitted}, appropriate
package idiom might be:

\preformatted{

H <- hyper3()
H[c(p1=1.3)]      \%<>\% inc(1) # a=1
H[c(p2=1)]        \%<>\% inc(2) # b=2
H[c(p1=1.3,p2=1)] \%<>\% dec(3) # a+b=1+2=3
H[c(p1=1)]        \%<>\% inc(3) # c=3
H[c(p2=1.3)]      \%<>\% inc(4) # d=4
H[c(p1=1,p2=1.3)] \%<>\% dec(7) # c+d=3+4=7
H
> log( (p1=1)^3 * (p1=1, p2=1.3)^-7 * (p1=1.3)^1 * (p1=1.3, p2=1)^-3 *
(p2=1)^2 * (p2=1.3)^4)
}

The general form of terms of a \code{hyper3} object would be
\eqn{\left(w_1p_1+\cdots+w_rp_r\right)^{\alpha}}{omitted}; the
complete object would be

\deqn{
  \mathcal{L}\left(p_1,\ldots,p_n\right)=
  \prod_{j=1}^N\left(\sum_{i=1}^n
  w_{ij}p_i\right)^{\alpha_i}
  }{omitted: see PDF}

where we understand that \eqn{p_n=1-\sum_{i=1}^{n-1}p_i}{omitted};
many of the weights might be zero.  We see that the weights
\eqn{w_{ij}}{w_ij} may be arranged as a matrix and this form is taken
by function \code{hyper3_m()}.

  \itemize{

    \item Function \code{hyper3()} is the user-friendly creation method,
    which dispatches to a helper function depending on its arguments.

    \item Function \code{hyper3_bw()} takes a list of brackets
    (character vectors) and a list of weights (numeric vectors)  and
    returns a \code{hyper3} object.

    \item Function \code{hyper3_nv()} takes a list of named vectors and
    returns a \code{hyper3} object.

    \item Function \code{hyper3_m()} takes a matrix with rows being the
    brackets (entries are weights) and a numeric vector of powers.

    \item Function \code{evaluate3()} is a low-level helper function
    that evaluates a log-likelihood at a point in probability space.
    Don't use this: use the user-friendly \code{loglik()} instead, which
    dispatches to \code{evaluate3()}.

    \item Function \code{maxp3()} is a placeholder (it is not yet
    written).  But the intention is that it will maximize the
    log-likelihood of a \code{hyper3} object over the Bradley Terry
    strengths \emph{and} any weights given.  This might not be possible
    as currently envisaged; I present some thoughts in
    \code{inst/kka.Rmd}.

    \item Function \code{list2nv()} converts a list of character vectors
    into a named vector suitable for use as argument \code{e} of
    function \code{cheering3()}.  It is used in
    \code{inst/global_liveability_ranking.Rmd}.

    \item Function \code{as.namedvectorlist()} takes a \code{hyper3}
    object and returns a disoRdered list of named vectors corresponding
    to the brackets and their weights.

    \item Function \code{setweight()} alters the weight of every
    occurrence of a set of players, documented at \code{weights.Rd}. 

    \item Function \code{pnames<-()} operates as expected on
    \code{hyper3} objects: use \code{pnames(H3) <- new_pnames} to add
    players.  This idiom can only add players, so \code{new_pnames} must
    be a superset of \code{pnames(H3)}; also note that the order of the
    players cannot be changed in this way.

  }
}
\value{
Generally return or deal with \code{hyper3} objects
}
\author{Robin K. S. Hankin}
\note{
Functionality for \code{hyper3} objects is generally indicated by adding
a \dQuote{\code{3}} to function names, e.g. \code{gradient()} goes to
\code{gradient3()}.

Vignette \code{hyper3_creation} discusses the different creation
methods for \code{hyper3} objects.

}
\seealso{\code{\link{hyper2}}, \code{\link{weights}}}
\examples{


hyper3(B=list("a", c("a","b"),"b"), W=list(1.2,c(1.2,1),1), powers=c(3,4,-7))
hyper3(list(c(a=1.2), c(b=1), c(a=1.2, b=1)), powers=c(3,4,-7))
## Above two objects should be identical.

## Third method, send a matrix:
M <- matrix(rpois(15,3), 5, 3)
colnames(M) <- letters[1:3]
hyper3(M, c(2, 3, -1, -5, 1))   # second argument interpreted as powers



## Standard way to generate a hyper3 object is to create an empty object
## and populate it using the replacement methods:

a <- hyper3()  # default creation method [empty object]

a[c(p1=1.3)] <- 5
a[c(p2=1  )] <- 2
a[c(p1=1.3, p2=1)] <- -7
a

chess3  # representative simple hyper3 object

H1 <- rankvec_likelihood(letters[sample(6)])
H2 <- rankvec_likelihood(letters[sample(6)])
H1["a"] <- as.weight(1.2)          # "a" has some disadvantage in H1
H1[c("b", "c")] <- as.weight(2:3)  # "b" and "c" have some advantage in H1
H2[c("c", "d")] <- as.weight(1.5)  # "c" and "d" have some advantage in H2
H1 + H2

}

\name{hepatitis}
\alias{hepatitis}
\alias{hepatitis_table}
\alias{hepatitis_maxp}
\alias{hepatitis_count}
\docType{data}
\title{Hepatitis dataset discussed by Agresti}
\description{A dataset considered by Agresti}
\usage{data(hepatitis)}
\format{A \code{hyper2} object that gives a likelihood function}
\details{

Object \code{hepatitis_table} is drawn from Agresti, table 12.16, page
533.  Object \code{hepatitis} is a likelihood function of class
\code{lsl} and \code{hepatitis_maxp} a pre-calculated evaluate.

These objects can be generated by running script \code{inst/hepatitis.Rmd},
which includes some further discussion and technical documentation, and
creates file \code{hepatitis.rda} which resides in the \code{data/}
directory.

}
\references{
A. Agresti, 2002.  "Categorical data analysis".  John Wiley and
Sons. Table 13.1, p542.
 }
\seealso{\code{\link{race3}},\code{\link{hepatitis}}}
\examples{
pie(hepatitis_maxp)
}

\name{attemptstable}
\alias{attemptstable}
\alias{as.attemptstable}
\alias{attemptstable2supp}
\alias{attemptstable2supp3}
\alias{rattemptstable}
\title{Translate attempts tables to hyper3 support functions}
\description{
Attempts tables
}
\usage{
attemptstable2supp3(x, decreasing=TRUE, give.supp=TRUE, nothrow_loses=FALSE)
rattemptstable(ncompetitors=8, nthrows=6, prob=0.23)
}
\arguments{
  \item{x}{Data frame, see details}
  \item{decreasing}{Boolean, with \code{TRUE} meaning that the highest
      score wins [e.g. javelin distances] and \code{FALSE} meaning that the
      lowest score wins [e.g. times for a race]}
  \item{give.supp}{Boolean, return the support function or the order statistic}
  \item{nothrow_loses}{Boolean, should NA entries count as coming last (TRUE)
    or be ignored (FALSE)}
  \item{ncompetitors, nthrows, prob}{Parameters for \code{rattemptstable()}}
}
\details{


An \dfn{attemptstable} object is a data frame with each row being a
competitor and each column being an attempt.  The canonical example is
\code{javelin}:

\preformatted{
> javelin_table
          throw1 throw2 throw3 throw4 throw5 throw6
Chopra     87.03  87.58  76.79      X      X  84.24
Vadlejch   83.98      X      X  82.86  86.67      X
Vesely     79.73  80.30  85.44      X  84.98      X
Weber      85.30  77.90  78.00  83.10  85.15  75.72
Nadeem     82.40      X  84.62  82.91  81.98      X
Katkavets  82.49  81.03  83.71  79.24      X      X
Mardare    81.16  81.73  82.84  81.90  83.30  81.09
Etelatalo  78.43  76.59  83.28  79.20  79.99  83.05
}

No-throws are stored internally as \code{NA} entries [see, e.g., file
\code{inst/javelin.txt}], but they are displayed as an \code{X}.  The
print method, by default, emboldens the best attempt by each competitor.
This does not work in Rmarkdown documents; to suppress it, set option
\code{bold_personal_best} to \code{FALSE}:

\preformatted{options("bold_personal_best" = FALSE)}

To translate an \code{attemptstable} object to a \code{hyper3} support
function, use \code{suppfun()}; function \code{attemptstable2supp3()} is
a low-level helper function.

File \code{inst/javelin.Rmd} shows a use-case for
\code{attemptstable2supp3().}

Function \code{rattemptstable()} produces a random \code{attemptstable}
object.  The defaults give a table of the same size and same total
number of no-throws as \code{javelin_table}.
}
\value{Returns a \code{hyper3} object}
\note{

Compare argument \code{nothrow_loses} with argument \code{uninformative} of
function \code{ordertable2supp()}.

Argument \code{nothrow_loses} is Boolean, with default \code{TRUE}
meaning to interpret \code{NA} (no-throw) entries as losing attempts, or
alternatively to view as though they never happened and are thus
uninformative.  In contrast, argument \code{uninformative} of function
\code{ordertable2supp()} gives a list of entry types that are
interpreted as uninformative [the other entry types meaning \dQuote{came
last}].   A simple example is given in \code{inst/javelin.Rmd}.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{ordertable2supp}},\code{\link{javelin}}}
\examples{

jj <- javelin_table[1:3,]
jj
attemptstable2supp3(jj)

}

\name{masterchef}
\Rdversion{1.1}
\alias{masterchef}
\alias{MasterChef}
\alias{masterchef_maxp}
\alias{masterchef_constrained_maxp}
\docType{data}
\title{Masterchef series 6}
\description{
Data from Australian Masterchef Series 6
}
\usage{data(masterchef)}

\format{

Object \code{masterchef} is a list of \code{hyper2} objects;
\code{masterchef_pmax} and \code{masterchef_constrained_pmax} are named
vectors with unit sum.
  
}
\details{

The object is created using the code in \code{inst/masterchef.Rmd},
which is heavily documented.  Not all the information available is
included in the likelihood function as some of the early rounds result
in an unmanageably large list.  Inclusion is controlled by Boolean
vector \code{doo}.
 
The definitive source is the coloured table on the wiki page.
}
\references{

  Wikipedia contributors, \dQuote{MasterChef Australia (series 6),}
  Wikipedia, The Free Encyclopedia,
  \url{https://en.wikipedia.org/w/index.php?title=MasterChef_Australia_(series_6)&oldid=758432561}
  (accessed January 5, 2017).
  
}
\seealso{\code{\link{ggrl}}}
\examples{

a1 <- indep(equalp(masterchef[[1]]))            # equal strengths
a2 <- indep(masterchef_maxp)               # MLE
a3 <- indep(masterchef_constrained_maxp)   # constrained MLE

\dontrun{ # takes too long
like_series(a1, masterchef)
like_series(a2, masterchef)
like_series(a3, masterchef)
}
}
\keyword{datasets}

\name{summary.hyper2}
\alias{summary.hyper2}
\alias{print.summary.hyper2}
\title{Summary method for hyper2 objects}
\description{Give a summary of a hyper2 object, and a print method}
\usage{
\method{summary}{hyper2}(object, ...)
\method{print}{summary.hyper2}(x, ...)
}
\arguments{
  \item{object,x}{Object of class hyper2}
  \item{...}{Further arguments, currently ignored}
}
\details{
Mostly self-explanatory, based on the equivalent in the \code{untb} package.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{hyper2}}}
\examples{
summary(icons)
}

\name{karpov_kasparov_anand}
\alias{karpov_kasparov_anand}
\alias{kka}
\alias{chess3}
\alias{chess3_maxp}
\alias{kka_3draws}
\alias{kka_3whites}
\alias{white_wins}
\alias{drawn_games}
\alias{black_wins}
\alias{plays_white_wins}
\alias{plays_white_draws}
\alias{plays_white_loses}
\alias{kka_array}
\docType{data}
\title{Karpov, Kasparov, Anand}
\description{
  Data of three chess players: Karpov, Kasparov, and Anand.  Includes
  two likelihood functions for the strengths of the players, and an
  array of game results
}

\details{

  (there are three chess datasets in the package, documented at
  \code{interzonal.Rd} [the 1963 World championship], \code{kka.Rd}
  [Karpov-Kasparov-Anand dataset], and \code{chess.Rd}
  [rock-paper-scissors using Topalov-Anand-Karpov])

  The strengths of chess players may be assessed using the generalized
  Bradley-Terry model.  The \code{karpov_kasparov_anand} \code{hyper2}
  likelihood function allows one to estimate the players' strengths,
  propensity to draw, and also the additional strength conferred by
  playing white as personified by a draw monster and a white monster
  \code{draw} and \code{white} respectively.

  Object \code{karpov_kasparov_anand} assumes that the draw potential
  is the same for all three players; likelihood function
  \code{kka_3draws} allows the propensity to draw to differ between
  the three players.

  The reason that the players are different from those in the
  \code{chess} dataset is that the original data does not seem to be
  available any more.

  Dataset \code{kka} refers to scorelines of matches between three
  chess players (Kasparov, Karpov, Anand).  It is a named numeric
  vector with names such as
  \sQuote{\code{karpov_plays_white_beats_kasparov}} which has value
  18: we have a total of 18 games between Karpov and Kasparov in which
  Karpov played white and beat Kasparov.

  Object \code{chess3} is a simple \code{hyper3} object corresponding
  to pairwise comparison with draws; \code{chess3_maxp} is the
  evaluate, conditional on the estimated white-player advantage and
  draw proclivity.  This object is created and discussed in
  \code{inst/kka.Rmd}.  Array \code{kka_array} presents the same
  information in a 3D array.

  All data drawn from \code{https://www.chessgames.com} (search for
  \dQuote{Kasparov vs Karpov}, etc).  Note that the database allows one
  to sort by white wins or black wins (there is a \sQuote{refine search}
  tab at the bottom).  Some searches have more than one page of results.
  Numbers here downloaded 17 February 2019.  Note that only
  \sQuote{classical games} are considered here (rapid and exhibition
  games being ignored).

  These objects can be generated by running script
  \code{inst/kka.Rmd}, which includes some further discussion and
  technical documentation and creates file \code{kka.rda} which
  resides in the \code{data/} directory.

}
\seealso{\code{\link{chess}}}
\examples{
karpov_kasparov_anand
# pie(maxp(karpov_kasparov_anand))  # takes ~10s

M <- kka_array[,,1] + 1i*kka_array[,,3]
home_away(M)
home_away3(M,lambda=1.2)


}
\keyword{datasets}

\name{keep}
\alias{keep}
\alias{retain}
\alias{discard}
\alias{drop}
\alias{keep_flawed}
\alias{retain_flawed}
\alias{discard_flawed}
\alias{drop_flawed}
\alias{keep_flawed2}
\alias{discard_flawed2}
\title{Keep or discard players}
\description{Flawed functionality to keep or discard subsets of the players in
  a \code{hyper2} object or order table.}
\usage{
discard_flawed2(x, unwanted,...)
keep_flawed(H, wanted)
discard_flawed(H, unwanted)
}
\arguments{
  \item{H}{A \code{hyper2} object}
  \item{x}{An order table}
  \item{wanted, unwanted}{Players to keep or discard.  May be character
    or integer or logical}
  \item{...}{Further arguments passed to
    \code{wikitable_to_ranktable()}, notably \code{points}}
}
\details{

  \strong{Do not use these functions.  They are here as object lessons
  in poor thinking.  To work with a subset of competitors, see the
  example at \link{as.ordertable}.}

  Functions \code{keep_flawed2()} and \code{discard_flawed2()} take an
  order table and keep or discard specified rows, returning a reduced
  order table.  This is not a trivial operation.
  
  Functions \code{keep_flawed()} and \code{discard_flawed()} will either
  keep or discard players specified in the second argument.  It is not
  clear to me that these functions have any reasonable probabilistic
  interpretation and file \code{inst/retain.Rmd} gives a discussion.

  Given a wikitable or ordertable, it is possible to create a likelihood
  function based on a subset of rows using the \code{incomplete=TRUE}
  argument; see the example at \code{?ordertable2supp}.  But this method
  is flawed too because it treats non-finishers as if they finished in
  the order of their rows.

  Function \code{as.ordertable()} is the correct way to consider a
  subset of players in a wikitable.
  
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{ordertable2supp}},\code{\link{tidy}}}
\examples{

maxp(icons)
discard_flawed(icons, c("OA", "WAIS"))

\dontrun{# (takes too long)
data("skating")
maxp(skating)[1:4]      # numbers work, keep the first four skaters
maxp(keep_flawed(skating, pnames(skating)[1:4])) # differs!
}


}

\name{interzonal}
\alias{interzonal}
\alias{interzonal_maxp}
\alias{interzonal_table}
\alias{interzonal_collusion}
\alias{interzonal_collusion_maxp}
\alias{curacao}
\alias{curacao3}
\alias{stockholm1962}
\alias{collusion}
\alias{interzonal_collusion}
\docType{data}
\title{1963 World Chess Championships}
\description{
Likelihood functions for players' strengths in the fifth Interzonal
tournament which occurred as part of the 1963 Chess world Championships
in Stockholm, 1962.
}
\details{

  (there are three chess datasets in the package, documented at
  \code{interzonal.Rd} [the 1963 World championship], \code{kka.Rd}
  [Karpov-Kasparov-Anand dataset], and \code{chess.Rd}
  [rock-paper-scissors using Topalov-Anand-Karpov])

The 1963 World Chess Championship was notable for allegations of Soviet
collusion.  Specifically, Fischer publicly alleged that certain Soviet
players had agreed in advance to draw all their games.  The championship
included an \dQuote{interzonal} tournament in which 23 players competed
in Stockholm; and a \dQuote{Candidates} tournament in which 8 players
competed in Curacao.

Likelihood functions \code{interzonal} and \code{interzonal_collusion}
are created by files \file{inst/interzonal.Rmd}, which is heavily
documented and include some analysis.  Object \code{interzonal} includes
a term for drawing, (\dQuote{\code{draw}}), assumed to be the same for
all players; object \code{interzonal_collusion} includes in addition to
\code{draw}, a term for the drawing in Soviet-Soviet matches,
\dQuote{\code{coll}}.

Some other analysis is given in files
\code{inst/curacao11962_threeplayers.R} and
\code{inst/curacao1962_threeplayers_rest_monster.Rmd}.

}
\seealso{\code{\link{chess}},\code{\link{karpov_kasparov_anand}}}
\examples{
pie(interzonal_maxp)

# samep.test(interzonal,c("Fischer","Geller")) # takes too long
}
\keyword{datasets}

\name{Ops.hyper3}
\alias{Ops.hyper3}
\alias{hyper3_add}
\alias{hyper3_prod}
\alias{hyper3_sum_numeric}
\alias{hyper3_add}
\alias{sum.hyper3}
\alias{hyper3_equal}
\title{Arithmetic Ops Group Methods for hyper3 objects}
\description{

  Allows arithmetic operators \dQuote{\code{+}}, \dQuote{\code{*}} and
  comparison operators \dQuote{\code{==}} and \dQuote{\code{!=}}, to be
  used for \code{hyper3} objects.

}
\usage{
\method{Ops}{hyper3}(e1, e2 = NULL)
hyper3_add(e1,e2)
hyper3_prod(e1,n)
}
\arguments{
  \item{e1,e2}{Objects of class \code{hyper3}}
  \item{n}{Numeric vector of length 1}
}
\details{

Pretty much everything documented here is a straightforward translation
of the corresponding \code{hyper2} functionality.
  
}
\value{Returns a \code{hyper3} object or a Boolean.}
\author{Robin K. S. Hankin}
\examples{

H1 <- hyper3(list(c(a=1.2),c(b=1),c(a=1.2,b=1)),powers=c(3,4,-7))
H2 <- hyper3(list(c(a=1.2),c(b=1.2),c(a=2.2,b=1.2)),powers=c(2,3,-5))

H1
H2



}
\keyword{symbolmath}

\name{ordervec2supp3}
\alias{race3}
\alias{num3}
\alias{den3}
\alias{char2nv}
\alias{ordervec2supp3}
\alias{ordervec2supp3a}
\alias{ordertable2supp3}
\alias{args2ordervec}
\alias{race_to_hyper3}
\alias{cheering}
\alias{cheering3}
\alias{rankvec_likelihood3}
\title{Various functionality for races and hyper3 likelihood functions}
\description{
Various functions for calculating the likelihood function for order
statistics in the context of hyper3 likelihood functions.  Compare
\code{ggol()} for \code{hyper2} objects.  Used in the
\code{constructor()} suite of analysis.
}
\usage{
num3(v, helped=NULL, lambda=1)
den3(v, helped=NULL, lambda=1)
char2nv(x)
ordervec2supp3(v, nonfinishers=NULL)
ordervec2supp3a(v, nonfinishers=NULL, helped=NULL, lambda=1)
rankvec_likelihood3(v, nonfinishers=NULL)
ordertable2supp3(a, noninformative=NULL)
cheering3(v, e, help, nonfinishers=NULL)
args2ordervec(...)
}
\arguments{
  \item{v}{Ranks in the form of a character vector.  Element \code{v[1]}
    is the first-placed competitor, element \code{v[2]} the second, and
    so on.  For example, \code{ordervec2supp3(c('b','b','a','c','a'))}}
  \item{nonfinishers}{Character vector (a set) showing players that did
    not finish.  See details section and examples}
  \item{a}{An ordertable}
  \item{helped}{vector of entities being helped}
  \item{e, help, lambda}{Parameters controlling non-independence with
    \code{e} a named integer vector specifying equivalence classes of
    the competitors: names correspond to the competitors, values to
    their equivalence class, and \code{help} a numeric vector with
    entries corresponding to the equivalence classes of \code{e} and
    values the strength of the support}
  \item{x}{A character vector of competitors}
  \item{noninformative}{Character vector of entries in an ordertable
    that are interpreted as uninformative.  See
    \code{inst/noninformative.Rmd}}
  \item{...}{Arguments passed to \code{args2ordervec()}}
}
\details{

  Function \code{args2ordervec()} takes arguments with names
  corresponding to players, and entries corresponding to performances
  (e.g. distances thrown by a javelin, or times for completing a
  race).  It returns a character vector indicating the rank statistic.
  See examples, and also the javelin vignette.

  Function \code{ordervec2supp3()} takes character vector showing the
  order of finishing [i.e. a rank statistic], and returns a generalized
  Plackett-Luce support function in the form of a \code{hyper3} object.
  It can take the output of \code{args2ordervec()} or \code{rrace3()}.
  For example:

  \code{ordervec2supp3(c("a","b"), nonfinishers=c("a","b"))}

  corresponds to a race between two twins of strength \code{a} and two
  twins of strength \code{b}, with only one of each pair finishing;
  \code{a} comes first and \code{b} comes second; symbolically

  \deqn{a\succ b\succ\left\lbrace a,b\right\rbrace\longrightarrow
    \mathcal{L}(a,b\left|a+b=1\right.)=\frac{a}{2a+2b}\cdot\frac{b}{a+2b}}{omitted}

  Further,

  \code{ordervec2supp3(c("a","b"), c("a","b","c"))}

  corresponds to adding a singleton competitor of strength \code{c} who
  did not finish:

  \deqn{a\succ b\succ\left\lbrace a,b,c\right\rbrace\longrightarrow
    \mathcal{L}(a,b,c\left|a+b+c=1\right.)=\frac{a}{2a+2b+c}\cdot\frac{b}{a+2b+c}}{omitted}

  (observe that this likelihood function is informative about
  \eqn{c}).  See the examples section below.  Experimental function
  \code{ordervec2supp3a()} is a generalized version of
  \code{ordervec2supp3()} that allows for cheering effects.

  Functions \code{num3()} and \code{den3()} are low-level helper
  functions that calculate the numerator and denominator for
  Plackett-Luce likelihood functions with clones; used in
  \code{ordervec2supp3()} and \code{ordervec2supp3a()}.

  Function \code{ordertable2supp3()} takes an order table (the canonical
  example is the constructors' formula 1 grand prix results, see
  \code{constructor.Rd} and returns a generalized Plackett-Luce support
  function in the form of a \code{hyper3} object.  Argument
  \code{noninformative} is a character vector of entries that are
  interpreted as the competitor's placing is not informative about their
  PL strength for some reason (e.g., they elected not to compete because
  they had accumulated enough points to guarantee victory.  See
  \code{inst/noninformative.Rmd}.

  Function \code{char2nv()} takes a character vector and returns a named
  vector with entries corresponding to their names' counts.  It is used
  in the extraction and replacement methods for \code{hyper3} objects.

  Function \code{cheering3()} is a generalization of
  \code{ordervec2supp3()}.  Competitors who are not mentioned in
  argument \code{e} are assumed to be in an equivalence class of size 1,
  that is, they are not supported (or indeed suppressed) by anyone else:
  they are singletons in the terminology of Hankin (2006).  Extensive
  discussions are presented at \code{inst/plackett_luce_monster.Rmd} and
  \code{inst/eurovision.Rmd}.
  
  File \code{inst/javelin.Rmd} and \code{inst/race3.Rmd} show some
  use-cases for these functions.

}
\author{Robin K. S. Hankin}
\note{
  Function \code{ordervec2supp3()} is mis-named [it takes a \emph{rank}
  vector, not an \emph{order} vector]; it will be renamed
  \code{rankvec_likelihood3()}, eventually.
}  
\seealso{\code{\link{ordertable2supp}},\code{\link{ordertrans}}}
\examples{


ordervec2supp3(c("a", "a", "b", "c", "a", "b", "c"))
ordervec2supp3(rrace3())
ordervec2supp3(c("a","b"), nonfinishers=c("a","b"))  # a > b >> {a,b}



(o <- args2ordervec(a=c(1,6,9), b=c(2,3,4), c=c(1.1,11.1)))
H <- ordervec2supp3(o)
H
# equalp.test(H)   # takes too long for here


## Race: six competitors a-f finishing in alphabetical order.  Mutually
## supporting groups: (acd), (bf), (e).  Competitor "e" is not
## suppported by anyone else (he is a singleton) so does not need to be
## mentioned in argument 'e' and there are only two helpfulnesses to be
## considered: that of (acd) and that of (bf), which we will take to be
## 1.88 and 1.1111 respectively:

cheering3(v=letters[1:6], e=c(a=1,c=1,b=2,d=1,e=2), help=c(1.88,1.1111))



## Another race: four competitors, including two clones of "a", and two
## singletons "b" and "c".  Here "a" helps his clone at 1.88; and "b"
## and "c" help one another at 1.111:

cheering3(v=c("a","b","a","c"), e=c(a=1,b=2,c=2), help=c(1.8,1.111))


## Same race as above but this time there are two clones of "b", one of
## whom did not finish:

cheering3(v=c("a","b","a","c"), e=c(a=1,b=2,c=2), help=c(1.8,1.111), "b")


## Most common case would be that the clones help each other but noone
## else:

cheering3(v=c("a","b","a","c"), e=c(a=1,b=2,c=3), help=c(1.8,1.111,1), "b")

}

\name{skating}
\alias{skating}
\alias{skating_table}
\alias{skating_maxp}
\alias{skeleton}
\docType{data}
\title{Figure skating at the 2002 Winter Olympics}
\description{
  A likelihood function for the competitors at the Ladies' Free Skate
  at the 2002 Winter Olympics
}
\usage{skating}
\details{

  Three objects: \code{skating}, a log-likelihood function for the
  competitors' strengths, \code{skating_table}, an order table for each
  of the 9 judges, and \code{skating_maxp}, the result of
  \code{maxp(skating)}, which is included to save time in the examples.
    
  These objects can be generated by running script
  \code{inst/skating.Rmd}, which includes some further discussion and
  technical documentation.  The dataset is interesting because it has
  been analysed by many workers, including Lock and Lock, for
  consistency between the judges.

  Note that \code{skating_table} is an order table: it is structured so
  that each competitor is a row, and each judge is a column.  Function
  \code{suppfun()} requires a rank table [use \code{as.ranktable()} to
  coerce; see the examples].  Object \code{skating_table} is taken from
  Lock and Lock.  It corrects what appears to be an error in which judge
  5 ranked both Butyrskaya and Kettunen 12; there is no 13.  Using
  \acronym{EM}, I reckon that Butyrskaya should be ranked twelfth and
  Kettunen thirteenth.

}
\references{
  \itemize{
    \item
    \url{https://en.wikipedia.org/wiki/Figure_skating_at_the_2002_Winter_Olympics#Full_results_2}
    \item Robin Lock and Kari Frazer Lock, Winter
    2003. \dQuote{Judging Figure Skating Judges}.  \emph{STATS 36}, ASA
  }
}

\note{There is an (\code{Rbuildignore}-d) discussion of a
  \code{skeleton} dataset in the \code{inst/} directory of the repo,
  it's easy to confuse this with \code{skating}.
}
\author{Robin K. S. Hankin}
\examples{
data(skating)
dotchart(skating_maxp)

as.ranktable(skating_table)
suppfun(as.ranktable(skating_table))

rL <- sort(skating_maxp, decreasing=TRUE)
rL[] <- seq_along(rL)
rO <- seq_len(nrow(skating_table))
names(rO) <- rownames(skating_table)
ordertransplot(rO,rL,
   xlab="official rank",ylab="likelihood rank",
   main="Ladies free skating, 2002 Winter Olympics")

}
\keyword{datasets}

\name{rp}
\alias{rp}
\title{Random samples from the prior of a \code{hyper2} object}
\description{
Uses Metropolis-Hastings to return random samples from the prior of a
\code{hyper2} object
}
\usage{
rp(n, H, startp = NULL, fcm = NULL, fcv = NULL, SMALL = 1e-06, l=loglik, fillup=TRUE, ...)
}
\arguments{
  \item{H}{Object of class \code{hyper2}}
  \item{n}{Number of samples}
  \item{startp}{Starting value for the Markov chain, with default
    \code{NULL} being interpreted as starting from the evaluate}
  \item{fcm,fcv}{Constraints as for \code{maxp()}}
  \item{SMALL}{Notional small value for numerical stability}
  \item{l}{Log-likelihood function with default \code{loglik()}}
  \item{fillup}{Boolean, with default \code{TRUE} meaning to return a
    matrix with the fillup value added, and column names matching the
    \code{pnames()} of argument \code{H}}
  \item{...}{Further arguments, currently ignored}
}
\details{

  Uses the implementation of Metropolis-Hastings from the
  \CRANpkg{MCE} package to sample from the posterior PDF of a
  \code{hyper2} object.

  If the distribution is Dirichlet, use \code{rdirichlet()} to generate
  random observations: it is much faster, and produces serially
  independent samples.  To return \emph{uniform} samples, use
  \code{rp_unif()} (documented at \code{dirichlet.Rd}).
  
}
\value{Returns a matrix, each row being a unit-sum observation.}
\author{Robin K. S. Hankin}
\note{

  Function \code{rp()} a random sample from a given normalized
  likelihood function.  To return a random likelihood function, use
  \code{rhyper2()}.

  File \code{inst/ternaryplot_hyper2.Rmd} shows how to use
  \code{Ternary::ternaryPlot()} with \code{rp()}.

}
\seealso{\code{\link{maxp}},\code{\link{loglik}},\code{\link{dirichlet}},\code{\link{rhyper2}}}
\examples{
x <- rp(50, icons)  # try more than 50
matplot(x, type = "l")
matplot(t(x), type = "l")
plot(loglik(x, icons), type='b')
}

\name{pentathlon}
\alias{pentathlon}
\alias{pentathlon_table}
\alias{pentathlon_maxp}
\alias{pentathlon_ordertable}
\docType{data}
\title{Pentathlon}
\description{Results from the Men's pentathlon at the 2004 Summer Olympics}
\usage{data(pentathlon)}
\format{A \code{hyper2} object that gives a likelihood function}
\details{

Object \code{pentathlon} is a \code{hyper2} object that gives a
likelihood function for the strengths of the top seven competitors at
the Modern Men's Pentathlon, 2004 Summer Olympics.

Object \code{pentathlon_table} is an order table: a data frame with rows
being competitors, columns being disciplines, and entries being places.
Thus looking at the first row, first column we see that Moiseev placed
fifth at shooting.

These objects can be generated by running script
\code{inst/pentathlon.Rmd}, which includes some further discussion and
technical documentation and creates file \code{pentathlon.rda} which
resides in the \code{data/} directory.

}
\references{
\dQuote{Wikipedia contributors}, \emph{Modern pentathlon at the 2004
    Summer Olympics - Men's}.  Wikipedia, The Free Encyclopedia,
\url{https://en.wikipedia.org/w/index.php?title=Modern_pentathlon_at_the_2004_Summer_Olympics_\%E2\%80\%93_Men\%27s&oldid=833081611},
[Online; accessed 5-March-2020]
}
\note{Many of the competitors' names have diacritics, which I have removed.}
\seealso{\code{\link{ordertable}}}
\examples{
data(pentathlon)
pie(pentathlon_maxp)

as.ranktable(pentathlon_table)
suppfun(as.ranktable(pentathlon_table))
}

\name{curling}
\alias{curling_table}
\alias{curling}
\alias{Curling}
\alias{curling1}
\alias{curling2}
\alias{curling_maxp}
\alias{curling1_maxp}
\alias{curling2_maxp}
\docType{data}
\title{Curling at the Winter Olympics, 1998-2018}
\description{
Data for women's Olympic Curling  at the 2002 Winter Olympics.
}
\usage{data(curling)}
\details{

  There are five datasets loaded by \code{data("curling")}:

  \itemize{

    \item \code{curling_table}, an order table for Winter Olympics years
    1998,2002,2006,2010,2014, and 2018 for 13 countries.

    \item \code{curling1}, a log likelihood function on the assumption
    that not attending (indicated by \code{NA}) is equivalent to a
    \code{DNS} in Formula 1

    \item \code{curling2}, a log likelihood function on the assumption
    that not attending is noninformative

    \item \code{curling1_maxp} and \code{curling2_maxp}, corresponding
    evaluates
  }


These objects can be generated by running script
\code{inst/curling.Rmd}, which includes some further discussion and
technical documentation and creates file \code{curling.rda} which
resides in the \code{data/} directory.

}
\references{
  \itemize{
    \item

Wikipedia contributors. Curling at the Winter Olympics [Internet].
Wikipedia, The Free Encyclopedia; 2021 Jan 7, 14:23 UTC [cited 2021 Jan
21]. Available from:
\url{https://en.wikipedia.org/w/index.php?title=Curling_at_the_Winter_Olympics&oldid=998891075}
  }
}
\author{Robin K. S. Hankin}
\examples{
data(curling)
dotchart(curling1_maxp)

}
\keyword{datasets}

\name{ordertable}
\alias{ordertable}
\alias{order_table}
\alias{print.ordertable}
\alias{[.ordertable}
\alias{as.ordertable}
\title{Order tables}
\description{Manipulate, print and coerce to and from \code{ordertable} form}
\usage{
as.ordertable(w)
ordertable(x)
}
\arguments{

  \item{w}{In \code{as.ordertable()}, a generalized ordertable, a
    wikitable}
  \item{x}{In \code{ordertable()}, a matrix with rows corresponding to
    competitors and columns to judges}
}

\details{

The package makes extensive use of order tables and these are discussed
here together with a list of order tables available in the package as
data.  See also \code{ranktable.Rd}.

The prototypical ordertable would be \code{pentathlon_table}:

\preformatted{
> pentathlon_table
An ordertable:
              shooting fencing swimming riding running
Moiseev              5       1        1      6       5
Zadneprovskis        6       2        5      5       1
Capalini             4       6        2      3       4
Cerkovskis           3       3        7      7       2
Meliakh              1       7        4      1       6
Michalik             2       4        6      2       7
Walther              7       5        3      4       3
}

Although \code{pentathlon_table} is a dataset in the package, the source
dataset is also included in the \code{inst/} directory as file
\code{pentathlon.txt}; use idiom like
\code{read.table("inst/pentathlon.txt")} to load the order table.

Object \code{pentathlon_table} is a representative example of an
ordertable.  Each row is a competitor, each column an event (venue,
judge, \ldots).  The first row shows Moiseev's ranking in shooting
(5th), fencing (1st), and so on.  The first column shows the ranks of
the competitors in shooting.  Thus Moiseev came fifth, Zadneprovskis
came 6th, and so on.

However, to create a likelihood function we need ranks, not orders.  We
need to know, for a given event, who came first, who came second, and so
on (an extended discussion on the difference between rank and order is
given at \link{rrank}).  We can convert from an order table to a rank
table using \code{ordertable_to_ranktable()} (see also
\code{ranktable.Rd}):

\preformatted{
> ordertable_to_ranktable(pentathlon_table)
         c1            c2            c3         c4       c5           
shooting Meliakh       Michalik      Cerkovskis Capalini Moiseev      
fencing  Moiseev       Zadneprovskis Cerkovskis Michalik Walther      
swimming Moiseev       Capalini      Walther    Meliakh  Zadneprovskis
riding   Meliakh       Michalik      Capalini   Walther  Zadneprovskis
running  Zadneprovskis Cerkovskis    Walther    Capalini Moiseev      
         c6            c7        
shooting Zadneprovskis Walther   
fencing  Capalini      Meliakh   
swimming Michalik      Cerkovskis
riding   Moiseev       Cerkovskis
running  Meliakh       Michalik  
}

Above, we see the same data in a different format (an extended
discussion on the difference between rank and order is given in
\link{rrank}).

Many of the order tables in the package include entries that correspond
to some variation on \dQuote{did not finish}.  Consider the \link{volvo}
dataset, an ordertable:

\preformatted{
> volvo_table
An ordertable:  
              leg1 leg2 leg3 leg4 leg5 leg6 leg7 leg8 leg9
AbuDhabi      1    3    2    2    1    2    5    3    5
Brunel        3    1    5    5    4    3    1    5    2
Dongfeng      2    2    1    3  DNF    1    4    7    4
MAPFRE        7    4    4    1    2    4    2    4    3
Alvimedica    5    5    3    4    3    5    3    6    1
SCA           6    6    6    6    5    6    6    1    7
Vestas        4  DNF  DNS  DNS  DNS  DNS  DNS    2    6
}

In the above order table, we have \code{DNF} for \dQuote{did not finish}
and \code{DNS} for \dQuote{did not start}.  The \code{formula1} order
table has other similar entries such as \code{DSQ} for
\dQuote{disqualified} and a discussion is given at
\code{ordertable2supp.Rd}.

Links are given below to all the order tables in the package.  Note that
the table in \code{inst/eurovision.Rmd} (\code{wiki_matrix}) is not an
order table because no country is allowed to vote for itself.

To coerce a table like the Volvo dataset shown above into an order table
[that is, replace \code{DNS} with zeros, and also force nonzero entries
to be contiguous], use \code{as.ordertable()}.

Given an ordertable such as \code{F1_table_2017} which is a
\dQuote{wikitable} object, function \code{as.ordertable()} returns a
nicified version in which entries such as \code{DNS} are replaced with
zeros.  Finishing competitors are assigned numbers \eqn{1-n} with no
gaps; the function can be used to extract a subset of competitors.
Function \code{ordertable2supp()} offers similar functionality but
returns a \code{hyper2} object directly.

There is an experimental extraction method which extracts certain rows
of an ordertable; this is used in \code{inst/skating.Rmd}.

}
\author{Robin K. S Hankin}
\seealso{
  \code{\link{ordertable2supp}},
  \code{\link{rrank}},
  \code{\link{ranktable}}
}
\examples{

ordertable_to_ranktable(soling_table)
suppfun(soling_table) == soling  # should be TRUE

as.ordertable(F1_table_2017)
ordertable2supp(as.ordertable(F1_table_2017[1:4,]))

x <- rrank()
as.ordertable(x)

skating_table

## just the first three competitors:
skating_table[1:3,]
unclass(skating_table)[1:3,]

## above, look at hughes under J2 for both extraction methods

suppfun(skating_table[1:3,])

## just J1 and J2,  first five competitors:
skating_table[1:5,1:2]

}

\name{carcinoma}
\alias{carcinoma}
\alias{carcinoma_table}
\alias{carcinoma_maxp}
\alias{carcinoma_count}
\docType{data}
\title{Carcinoma dataset discussed by Agresti}
\description{A dataset considered by Agresti.  Seven clinicians are
  asked whether they see evidence for carcinoma on different patients.}
\usage{data(carcinoma)}
\format{A \code{hyper2} object that gives a likelihood function}
\details{

Object \code{carcinoma_table} is drawn from Agresti.  The first seven
columns correspond to the seven clinicians A-G, the next is the count
of observations, and the remaining columns are fitted values according
to different models discussed by Agresti.

Object \code{carcinoma} is a likelihood function (of class \code{lsl})
on the Bradley-Terry strengths of the seven clinicians.  The
clinicians diagnosed the presence or absence of carcinoma on a total
of 118 patients in a blind rating scheme.  The maximum likelihood
estimator for the clinicians' Bradley-Terry strengths is given as
\code{carcinoma_maxp}, which is computationally expensive to find.
The package also includes \code{carcinoma_count}, which is a different
estimator for the Clinicians' \acronym{BT} strengths.

These objects can be generated by running script
\code{inst/carcinoma.Rmd}, which includes some further discussion and
technical documentation, and creates file \code{carcinoma.rda} which
resides in the \code{data/} directory.  See
\url{https://robinhankin.github.io/hyper2/carcinoma.html}.


}
\references{

A. Agresti, 2002.  "Categorical data analysis".  John Wiley and
Sons. Table 13.1, p542.
 }
\seealso{\code{\link{race3}},\code{\link{hepatitis}}}
\examples{
pie(carcinoma_maxp)
}

\name{rowing}
\Rdversion{1.1}
\alias{rowing}
\alias{sculling}
\alias{sculls2016}
\alias{allrowers}
\alias{rowing_minimal}
\alias{rowing_table}
\alias{rowing_minimal_table}
\alias{rowing_maxp}
\alias{rowing_minimal_maxp}
\docType{data}
\title{Rowing dataset, sculling}
\description{Data from Men's single sculls, 2016 Summer Olympics}
\usage{data(rowing)}
\format{

Object \code{rowing} is a  \code{hyper2} object that gives a likelihood
function for the 2016 men's sculls.

}
\details{

Object \code{rowing} is created by the code in \code{inst/rowing.Rmd}.
This reads file \code{inst/rowing.txt}, each line of which is a heat
showing the finishing order.  Object \code{rowing_table} is the
corresponding \R list.

File \code{inst/rowing_minimal.txt} has the same data but with dominated
players (that is, any group of players none of whom have beaten any
player not in the group) have been removed.  This is because dominated
players have a ML strength of zero.

}
\references{

  Wikipedia contributors, \dQuote{Rowing at the 2016 Summer
  Olympics---Men's single sculls}, \emph{Wikipedia, The Free
  Encyclopedia},
  \url{https://en.wikipedia.org/w/index.php?title=Rowing_at_the_2016_Summer_Olympics_\%E2\%80\%93_Men\%27s_single_sculls&oldid=753517240}
  (accessed December 7, 2016).
  
}
\seealso{\code{\link{ggrl}}}
\examples{
dotchart(rowing_maxp)
}
\keyword{datasets}

\name{sushi}
\alias{sushi}
\alias{sushi_table}
\alias{sushi_maxp}
\alias{sushi_eq_classes}
\docType{data}
\title{Sushi dataset}
\description{
  Dataset from the \emph{preflib} website, discussing sushi preferences.
}
\usage{data(sushi)}

\details{

Sushi preference dataset.

\itemize{
  \item Object \code{sushi_table} is a database of sushi preferences;
  each row is a judge and the rows are rank vectors of the different
  sushi types.
  \item Object \code{sushi} is a \code{hyper3} likelihood function at the estimated
  value of lambda
  \item Object \code{sushi_maxp} is the maximum likelihood estimate for
  the Bradley-Terry strengths of the sushis, at the estimate for lambda.
  \item \code{sushi_eq_classes} is a named vector giving the
  equivalences classes of sushi.  Class 1 is tuna, class 2 is everything
  else.
  }

These objects can be generated by running script \code{inst/sushi.Rmd},
which includes some further discussion and technical documentation and
creates file \code{sushi.rda} which resides in the \code{data/}
directory.
}

\examples{
sushi
sushi_maxp
}

\name{pwa}
\alias{pwa}
\alias{pwa3}
\alias{advantage}
\alias{pwa3_single}
\alias{pwa23}
\alias{pwa23_single}
\alias{pwa23_multiple}
\alias{chameleon}
\alias{extra_strength}
\alias{additional_strength}
\alias{training_strength}
\alias{wet_strength}
\alias{white_strength}
\title{Player with advantage}
\description{

Confer additional \acronym{BT} strength on specified players, either by
introducing a reified entity (\code{pwa()} for \code{hyper2} objects) or
an increased weight (\code{pwa3()} for \code{hyper3} objects).
  
}
\usage{
pwa(H, pwa, chameleon = "S", hyper3=FALSE)
advantage(H, ...)
pwa3_single(H3, pwa, lambda)
pwa3(H3, ...)
pwa23_single(H, pwa, chameleon="S")
pwa23_multiple(H, pwa, chameleon="S")
pwa23(H, ...)
}

\arguments{
  \item{H}{A hyper2 object}
  \item{H3}{A hyper3 object}
  \item{pwa}{A list of the players with the supposed advantage; may be
    character in the case of a named hyper2 object, or an integer
    vector}
  \item{chameleon}{String representing the advantage, defaulting to
  \dQuote{\code{S}}, for support or strength}
  \item{lambda}{Value of person with advantage}
  \item{hyper3}{In function \code{pwa()}, Boolean with \code{TRUE}
    meaning to coerce to \code{hyper3} form and apply \code{pwa23()},
    and default \code{FALSE} meaning to return an error if a
    \code{hyper3} object would be returned}
  \item{...}{Further named arguments; see details}
}

\details{

Commonly, when considering competitive situations we suspect that one
player has an advantage of some type which we would like to quantify in
terms of an additional strength.  Examples might include racing at pole
position, playing white in chess, or playing soccer at one's home
ground.  

Function \code{pwa()} (\dQuote{player with advantage}) returns a
modified \code{hyper2} object with the additional strength represented
as a reified entity.

Given \code{H} of class \code{hyper2} and a competitor \code{a},
\code{pwa(H, "a")} replaces every occurrence of \code{a} in \code{H}
with \code{a+S}, with \code{S} representing the extra strength
conferred.

Experimental function \code{advantage()} allows one to use idiom such as
\code{advantage(H, a="steroids")}, which is sometimes more convenient.

Argument \code{a} must be of length 1.  To give more than one competitor
an advantage, use \code{pwa23()}.

Function \code{pwa3()} is the \code{hyper3} equivalent; it modifies the
weight of specified players.  The named argument mechanism
[e.g. \code{pwa3(H, e=1.2, f=3.4)}] is the same as that of \code{subs()}
in the \CRANpkg{mvp} package.  If there is exactly one ellipsis
argument, and it is a named vector, the elements are interpreted as
name/value pairs, so \code{pwa3(H, c(e=1.2, f=3.4))} works.  Function
\code{pwa3_single()} is a lower-level helper function that takes a
single player name and its lambda value as second and third arguments.

Functions in the \code{pwa23()} series coerce their first argument to a
\code{hyper3()} object.  They allow multiple entities to have multiple
reified help, so for example we may say
\code{pwa23(suppfun(letters[1:5]), b="cheat", b="steroids",
c="steroids", d="cheat")}.  For giving multiple players the same
advantage [or to give a single player multiple advantages], use
\code{pwa23_multiple()}, for example
\code{pwa23_multiple(suppfun(letters[1:5]), letters[2:4], "steroids")};
the recycling is inherited from \code{paste()}.  Note that if the player
with advantage does not exist, no change is made to \code{H}.

Nice examples of \code{pwa()} are given in \file{inst/cook.Rmd} and
\file{inst/universities.Rmd}, and some discussion of \code{pwa3()} is
given in \code{inst/pwa3.Rmd}.

}
\value{Returns an object of class \code{hyper2} or \code{hyper3}.}
\author{Robin K. S. Hankin}
\note{

Earlier versions of this package gave a contrived sequence of
observations, presented as an example of \code{pwa()} with multiple
advantaged competitors.  I removed it because the logic was flawed, but
it featured a chameleon who could impersonate (and indeed eat) certain
competitors, which is why the third argument is so named.

The aliases commemorate some uses of the function in the vignettes and
markdown files in the \file{inst/} directory.

}

\seealso{\code{\link{ordervec2supp}}}
\examples{

summary(formula1 |> pwa("Hamilton","pole"))

H <- ordervec2supp(c(a = 2, b = 3, c = 1, d = 5, e = 4))
pwa(H, 'a')
pwa3_single(H, "a", 1.888)

pwa3(H, b=1.8, c=0.7)

## Four races between a,b,c,d:
H1 <- ordervec2supp(c(a = 1, b = 3, c = 4, d = 2))  # a: 1st, b: 3rd, c: 4th, d: 2nd
H2 <- ordervec2supp(c(a = 0, b = 1, c = 3, d = 2))  # a: DNF, b: 1st, c: 3rd, d: 2nd
H3 <- ordervec2supp(c(a = 4, b = 2, c = 1, d = 3))  # a: 4th, b: 2nd, c: 1st, d: 3rd
H4 <- ordervec2supp(c(a = 3, b = 4, c = 1, d = 2))  # a: 3rd, b: 4th, c: 1st, d: 2nd

## Now it is revealed that a,b,c had some advantage in races 1,2,3
## respectively.  Is there evidence that this advantage exists?

\dontrun{  # takes ~10 seconds, too long for here
specificp.gt.test(pwa(H1,'a') + pwa(H2,'b') + pwa(H3,'c') + H4, "S", 0)
}

}

\name{hyper2-package}
\alias{hyper2-package}
\docType{package}
\title{
\packageTitle{hyper2}
}
\description{
  \if{html}{\figure{hyper2.png}{options: style='float: right' alt='logo' width='120'}}
\packageDescription{hyper2}
}
\details{

The DESCRIPTION file:
\packageDESCRIPTION{hyper2}
\packageIndices{hyper2}

  A generalization of the Dirichlet distribution, using a more
  computationally efficient method than the \pkg{hyperdirichlet}
  package.  The software is designed for the analysis of order
  statistics and team games.

  There are several vignettes that come with the package: see
  \href{https://robinhankin.github.io/hyper2/}{the io readme} (under
  \sQuote{articles}), or listed by \code{vignette(package = "hyper2")}.

  The hex sticker features the \dQuote{draw monster}, a concept
  introduced in Hankin (2020) as a reified entity, representing the
  winner of a drawn chess game.  The character was visualized and
  sketched by my daughter Annabel.  In the chess world, draws are often
  seen as dull and undesirable, and the draw monster embodies this
  sentiment: he is a dampening presence, a metaphorical wet blanket.
  The drawing conveys this lack of fighting spirit well, capturing the
  essence of an uninspired and anticlimactic outcome.
  
}
\author{
\packageAuthor{hyper2}

Maintainer: \packageMaintainer{hyper2}
}
\references{
  \itemize{
    \item 
    R. K. S. Hankin (2010). \dQuote{A Generalization of the Dirichlet Distribution},
    \emph{Journal of Statistical Software}, 33(11), 1-18,
    \doi{10.18637/jss.v033.i11}
    \item R. K. S. Hankin 2017. \dQuote{Partial Rank Data with the hyper2
      Package: Likelihood Functions for Generalized Bradley-Terry Models}.
    \emph{The \R Journal} 9:2, pages 429-439, \doi{10.32614/rj-2017-061}
    \item R. K. S. Hankin 2024.  \dQuote{Generalized Plackett-Luce
      Likelihoods}, \emph{Journal of Statistical Software}, 109(8), 1-17,
    \doi{10.18637/jss.v109.i08}
  }
}
\keyword{ package }
\examples{
icons
maxp(icons)
}

\name{volleyball}
\alias{volleyball}
\alias{vb}
\alias{volleyball_results}
\alias{volleyball_maxp}
\alias{volleyball_table}
\alias{volleyball_matrix}
\alias{vb_synthetic}
\docType{data}
\title{Results from the NOCS volleyball league}
\description{
  
Results from the \acronym{NOCS} volleyball league.

}
\usage{data(volleyball)}

\details{

A volleyball \dfn{set} is a Bernoulli trial between two disjoint subsets of
the players.  The two subsets are denoted (after the game) as the
\dQuote{winners} and the \dQuote{losers}: these are denoted by \code{1}
and \code{0} respectively.

Thus the first line reads of \code{volleyball_results} reads:
\preformatted{
 p1  p2  p3  p4  p5  p6  p7  p8  p9 
 1    0  NA   1   0   0  NA   1  NA
}

showing that the teams were \code{p1}, \code{p4} and \code{p8} against
\code{p2}, \code{p5} and \code{p6}; players \code{p3}, \code{p7} and
\code{p9} did not play.

Object \code{volleyball_table} is a matrix in which each column
corresponds to a player and each row corresponds to a volleyball set;
\code{volleyball} is the corresponding likelihood function in the form
of a \code{hyper2} distribution.

These datasets illustrate the fact that such Bernoulli trials are only
weakly informative.  

These objects can be generated by running script
\code{inst/volleyball.Rmd}, which includes some further discussion and
technical documentation and creates file \code{volleyball.rda} which
resides in the \code{data/} directory.

The dataset is used in an example at \code{zipf.Rd}: the players'
strengths are not Zipf.

}
\source{
 Volleyball games at \acronym{NOCS}, 2006-2008
}
\references{
  Robin K. S. Hankin (2010).
  \dQuote{A Generalization of the Dirichlet Distribution},
  \emph{Journal of Statistical Software}, 33(11), 1-18
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{zipf}}}
\examples{

volleyball == volley(volleyball_table)  # should be TRUE


}
\keyword{datasets}

\name{formula1}
\alias{formula1}
\alias{formula_1}
\alias{formula_one}
\alias{formula1_points_systems}
\alias{F1}
\alias{F1_2014}
\alias{F1_table_2016}
\alias{F1_table_2017}
\alias{F1_table_2018}
\alias{F1_table_2019}
\alias{formula1_table_2017}
\alias{F1_points_2017}
\alias{formula1_points_2017}
\alias{formula1_2017_table}
\alias{F1_2015}
\alias{F1_2016}
\alias{F1_2017}
\alias{F1_2018}
\alias{F1_2019}
\docType{data}
\title{Formula 1 dataset}
\description{Race results from 2017 Formula One World Championship}
\usage{
data(formula1)
formula1_points_systems(top=11)
}
\arguments{
  \item{top}{Number of drivers to retain in
    \code{formula1_points_systems()}}
}
\format{A \code{hyper2} object that gives a likelihood function}
\details{

Object \code{formula1} is a \code{hyper2} object that gives a likelihood
function for the strengths of the competitors of the 2017 Formula One
(Drivers') World Championship.  Object \code{F1_table_2017} is an order table: a
data frame with rows being drivers, columns being venues, and entries
being places.  Thus looking at the first row, first column we see that
Hamilton placed second in Austria.

The package uses files like \code{inst/formula1_2017.txt} as primary
sources.  These are generally copied from wikipedia, converted into
tab-separated clean seven bit ascii, and tidied up a little.  I have
removed diacritics from names, so we see \dQuote{\code{Raikkonen}},
\dQuote{\code{Perez}}, etc.  Also where distinct drivers with the same
surname compete, I have indicated this, e.g. \code{schumacher_R} is Ralf
Schumacher, \code{schumacher_M} is Michael, and \code{schumacher_Mick}
is Mick; the underscore device means that quoting should not be needed
in \R idiom.  I have not been entirely consistent here, with Bruno Senna
appearing as \dQuote{\code{Senna_B}} and Nelson Piquet Junior appearing
as \dQuote{\code{PiquetJ}} [on the grounds that in these cases the
fathers, being more eminent, should be the primary eponym] although this might
change in the future.

Object \code{F1_table_2017} is simply the first 20 columns of
\code{read.table(inst/formula1_2017.txt)} and object
\code{F1_points_2017} is column 21.  The final column of all the text
files is the points and it is easy to mistake this for a venue with
results (doing so will give a \dQuote{\code{Error in ordervec2supp(o) :
nonzero elements of d should be 1,2,3,4,...,n}} error).

The likelihood function \code{formula1} is
\code{ordertable2supp(F1_table_2017)} [NB: \code{suppfun(F1_table_2017)}
fails: \code{suppfun()} will not try to guess whether its argument is a
ranktable or an ordertable].  The datasets in the package are derived
from text files in the \code{inst/} directory
(e.g. \code{formula1_2017.txt}) by script file
\code{inst/f1points_Omaker.R}.  Executing this script creates files like
\code{formula1_results_2017.rda}.

The text files can be converted directly into \code{ranktable} objects
and support functions as follows:

\preformatted{
a <- read.table("formula1_2022.txt",header=TRUE)
a <- a[,seq_len(ncol(a)-1)]  # strips out the points column
wikitable_to_ranktable(a)
ordertable2supp(a)      # works fine
suppfun(ordertable(a))  # Same as previous line, but suppfun() needs to know what its argument is
}

To convert to a numeric matrix with \code{DNS} etc converted to
\code{NA}:

\preformatted{
a <- read.table("formula1_2024.txt",header=TRUE)
a <- a[, -ncol(a)]
a <- as.matrix(a)
storage.mode(a) <- "numeric"
}

[this is used in file \code{inst/test_formula1.R}, which provides a
consistency check for files \code{inst/formula1_???.txt}].

Function \code{formula1_points_system()} gives various possible points
systems for the winner, second, third, etc, placing drivers.  

The constructors' championship is discussed at \code{constructor.Rd}.

There is a large amount of documentation in the \code{inst/} directory
in the form of Rmd files.




}
\references{

  \dQuote{Wikipedia contributors}, \emph{2017 Formula One World
    Championship---Wikipedia, The Free Encyclopedia}, 2018.
  \url{https://en.wikipedia.org/w/index.php?title=2017_Formula_One_World_Championship&oldid=839923210}
  [Online; accessed 14-May-2018]
  }
\seealso{\code{\link{ordertable2supp}},\code{\link{constructor}}}
\examples{
summary(formula1)
\dontrun{#Takes too long
dotchart(maxp(formula1))
}

}

\name{volvo}
\alias{volvo}
\alias{volvo_ocean_race}
\alias{volvo_table_2014}
\alias{volvo2014}
\alias{volvo_table}
\alias{volvo_maxp}
\docType{data}
\title{Race results from the 2014-2015 Volvo Ocean Race}
\description{Race results from the twelfth edition of the
  round-the-world Volvo Ocean Race.}
\usage{data(volvo)}
\format{A \code{hyper2} object that gives a likelihood function}
\details{

Object \code{volvo} is a \code{hyper2} object that gives a likelihood
function for the strengths of the competitors of the 2014-2015 Volvo
Ocean Race; \code{volvo_maxp} is a precomputed maximum likelihood
estimate of the competitors' strengths.  Object \code{volvo_table} is a
data frame with rows being teams and columns being legs.

These objects can be generated by running script \code{inst/volvo.Rmd},
which includes some further discussion and technical documentation and
creates file \code{volvo.rda} which resides in the \code{data/}
directory.

}
\references{
Wikipedia contributors, 2019.
\dQuote{2014-2015 Volvo Ocean Race}.  In \emph{Wikipedia, the free encyclopedia}.
Retrieved 22:21, February 28, 2020.
\url{https://en.wikipedia.org/w/index.php?title=2014\%E2\%80\%932015_Volvo_Ocean_Race&oldid=914916131},

}
\seealso{\code{\link{ordertable2supp}}}
\examples{
pie(volvo_maxp)
# equalp.test(volvo)   # takes ~10 seconds to run


as.ordertable(volvo_table) # DNF and DNS -> 0

# convert table to a support function:
suppfun(volvo_table, noscore = c("DNF", "DNS"))

}

\name{pick}
\alias{trial}
\alias{pick}
\alias{pass}
\alias{beats}
\alias{loses}
\title{Individual pairwise trials, teams or individuals}
\description{

  A consistent suite of functions that give likelihood functions for a
  single pairwise trial between specified sets of competitors;
  supercedes \code{trial()}.

}
\usage{
pick(winners, allplayers)
pass(losers,  allplayers)
beats(winners, losers)
loses(losers, winners)
trial(winners, players, val=1)
}
\arguments{
  \item{winners}{Players on the winning team}
  \item{losers}{Players on the losing team}
  \item{allplayers}{All players, winning and losing}
  \item{players, val}{Inputs to (soon to be deprecated) function
    \code{trial()}}
}
\details{


Consider a competition between two disjoint sets of players,
\eqn{A=\left\lbrace a_1,\ldots, a_n\right\rbrace}{omitted} and
\eqn{B=\left\lbrace b_1,\ldots, b_m\right\rbrace}{omitted}.

If \eqn{A} wins the likelihood function will be
\eqn{\frac{a_1+\cdots+a_n}{a_1+\ldots+a_n+b_1+\ldots+b_m}} and if
\eqn{B} wins the likelihood function will be
\eqn{\frac{b_1+\cdots+b_m}{a_1+\ldots+a_n+b_1+\ldots+b_m}}.

We can represent a win for \eqn{A} with either \code{pick(A, c(A,B))} or
\code{beats(A,B)}; or \code{pass(B, c(A,B))} or \code{loses(B,A)}

Function \code{trial()} will be deprecated soon. It is an inefficient
synonym for \code{pick()}.

The \code{trial()} function is used in \code{inst/kka.Rmd} (but will
shortly be changed).

}
\author{Robin K. S. Hankin}
\examples{

pick(letters[1:2], letters[1:5])   # field abcde; ab beats cde
pass(letters[1:2], letters[1:5])   # field abcde; ab loses to cde

beats(letters[1:2], letters[3:5])  # field abcde; ab beats cde
loses(letters[1:2], letters[3:5])  # field abcde; ab loses to cde

monster <- "home"
players <- letters[1:4]

H <- hyper2()
H <- H + pick(c("a", "b", monster), c(players, monster))
H <- H + pick(c("a", "d", monster), c(players, monster))
H <- H + pick(c("a",      monster), c(players, monster))
H <- H + pass(c("c", "d", monster), c(players, monster))

## above see how pick() / pass() is better.

monster <- "cheating"

## Now new players e and f join the chat:
H <- H + beats(c("a", "e"), c("b", "c"))
H <- H + beats(c("a", "c", monster), c("b", "d"))
H <- H + beats(c("e", "f"), c("a", "b", monster))
H <- H + loses(c("a", "f", monster), c("d", "e"))
## above, see beats/loses are more convenient than pick/pass


H
}

\name{gradient}
\alias{gradient}
\alias{gradientn}
\alias{hessian}
\alias{hessian_lowlevel}
\alias{is_ok_hessian}
\alias{bordered_hessian}
\alias{hessian_bordered}
\title{Differential calculus}
\description{

Given a \code{hyper2} object and a point in probability space,
function \code{gradient()} returns the gradient of the log-likelihood;
function \code{hessian()} returns the bordered Hessian matrix.  By
default, both functions are evaluated at the maximum likelihood estimate
for \eqn{p}, as given by \code{maxp()}.

}
\usage{
gradient(H, probs=indep(maxp(H)))
hessian(H, probs=indep(maxp(H)), border=TRUE)
hessian_lowlevel(L, powers, probs, pnames,n) 
is_ok_hessian(M, give=TRUE)
}
\arguments{
  \item{H}{A \code{hyper2} object}
  \item{L,powers,n}{Components of a \code{hyper2} object}
  \item{probs}{A vector of probabilities}
  \item{pnames}{Character vector of names}
  \item{border}{Boolean, with default \code{TRUE} meaning to return the
    bordered Hessian and \code{FALSE} meaning to return the Hessian
    (warning: this option does not respect the unit sum constraint)}
  \item{M}{A bordered Hessian matrix, understood to have a single
    constraint (the unit sum) at the last row and column; the output of
    \code{hessian(border=TRUE)}}
  \item{give}{Boolean with default \code{FALSE} meaning for function
    \code{is_ok_hessian()} to return whether or not \code{M}
    corresponds to a negative-definite  matrix, and \code{TRUE} meaning
    to return more details}
}

\details{

Function \code{gradient()} returns the gradient of the log-likelihood
function.  If the \code{hyper2} object is of size \eqn{n}, then argument
\code{probs} may be a vector of length \eqn{n-1} or \eqn{n}; in the
former case it is interpreted as \code{indep(p)}.  In both cases, the
returned gradient is a vector of length \eqn{n-1}.
The function returns the derivative of the loglikelihood with respect to
the \eqn{n-1} independent components of
\eqn{\left(p_1,\ldots,p_n\right)}{(p_1,...,p_n)}, namely
\eqn{\left(p_1,\ldots,p_{n-1}\right)}{(p_1,...,p_n-1)}.  The fillup
value \eqn{p_n} is calculated as
\eqn{1-\left(p_1+\cdots + p_{n-1}\right)}{1-(p_1+...+p_n-1)}.

Function \code{gradientn()} returns the gradient of the loglikelihood
function but ignores the unit sum constraint.  If the \code{hyper2}
object is of size \eqn{n}, then argument \code{probs} must be a vector
of length \eqn{n}, and the function returns a named vector of length
\eqn{n}.  The last element of the vector is not treated differently from
the others; all \eqn{n} elements are treated as independent.  The sum
need not equal one.

Function \code{hessian()} returns the \dfn{bordered Hessian}, a matrix
of size \eqn{n+1\times n+1}{(n+1)*(n+1)}, which is useful when using
Lagrange's method of undetermined multipliers.  The first row and column
correspond to the unit sum constraint, \eqn{\sum p_1=1}{p_1+...+p_n=1}.
Row and column names of the matrix are the \code{pnames()} of the
\code{hyper2} object, plus \dQuote{\code{usc}} for \dQuote{Unit Sum
Constraint}.

The unit sum constraint borders could have been added with idiom
\code{magic::adiag(0,pad=1,hess)}, which might be preferable.

Function \code{is_ok_hessian()} returns the result of the second
derivative test for the maximum likelihood estimate being a local
maximum on the constraint hypersurface.  This is a generalization of the
usual unconstrained problem, for which the test is the Hessian's being
negative-definite.

Function \code{hessian_lowlevel()} is a low-level helper function that
calls the \proglang{C++} routine.

Further examples and discussion is given in file
\code{inst/gradient.Rmd}.  See also the discussion at \link{maxp} on the
different optimization routines available.
}

\value{

Function \code{gradient()} returns a vector of length \eqn{n-1} with
entries being the gradient of the log-likelihood with respect to the
\eqn{n-1} independent components of
\eqn{\left(p_1,\ldots,p_n\right)}{(p_1,...,p_n)}, namely
\eqn{\left(p_1,\ldots,p_{n-1}\right)}{(p_1,...,p_n-1)}.  The fillup
value \eqn{p_n} is calculated as
\eqn{1-\left(p_1,\ldots,p_{n-1}\right)}{1-(p_1,...,p_n-1)}.

If argument \code{border} is \code{TRUE}, function \code{hessian()}
returns an \eqn{n}-by-\eqn{n} matrix of second derivatives; the borders
are as returned by \code{gradient()}.  If \code{border} is \code{FALSE},
ignore the fillup value and return an \eqn{n-1}-by-\eqn{n-1} matrix.

Calling \code{hessian()} at the evaluate will not return exact zeros for
the constraint on the fillup value; \code{gradient()} is used and this
does not return exact zeros at the evaluate.
}

\author{Robin K. S. Hankin}

\note{
  There is a \code{NAMESPACE} clash with \code{pracma::gradient()}.
}
\examples{

data(chess)
p <- c(1/2, 1/3)
delta <- rnorm(2)/1e5  # delta needs to be quite small

deltaL  <- loglik(p+delta,chess) - loglik(p,chess)
deltaLn <- sum(delta*gradient(chess, p + delta/2))   # numeric

deltaL - deltaLn  # should be small [zero to first order]

H <- hessian(icons)
is_ok_hessian(H)

}

\name{RCLF}
\alias{RCLF}
\alias{RCLF3}
\alias{RCLF3_maxp}
\alias{RCLF3_lambda_max}
\alias{RCLF3_table}
\alias{Rangers}
\alias{Celtic}
\alias{Livingston}
\alias{Falkirk}
\docType{data}
\title{Dataset from four Scottish football clubs}
\description{
Datasets for match results for four Scottish clubs: Rangers, Celtic,
  Livingston, Falkirk.
  }

\usage{data(RCLF)}
\details{
  These objects refer to results from football matches among four
  Scottish clubs: Rangers, Celtic, Livingston, Falkirk.  A detailed
  analysis is presented in \code{inst/home_advantage.Rmd}, which creates
  the objects documented here from scratch.

  \enumerate{
    \item  Object \code{RCLF3_table} is a table of results for the four
    clubs: home wins, draws, and away wins.
    \code{RCLF3_lambda_max} for the maximum likelihood estimate for
    \eqn{\lambda}.
    \item Object \code{RCLF3} is a \code{hyper3} likelihood function for
    the table of won games only [that is, excluding draws], using
    \code{RCLF3_lambda_max} for the value of \eqn{\lambda}.
    \item Object \code{RCLF3_maxp} is its evaluate at the MLE for
    lambda}
The objects are created by \code{inst/home_advantage.Rmd}.
}
\source{
  Data obtained from \code{www.worldfootball.net}
}
\author{Robin K. S. Hankin}
\examples{
data(RCLF)
RCLF3_table
RCLF3_maxp
RCLF3_lambda_max

loglik(RCLF3_maxp,RCLF3)
equalp.test(RCLF3)
}
\keyword{datasets}

\name{matrix2supp}
\alias{matrix2supp}
\alias{matrix_to_HD}
\alias{saffy}
\alias{volley}
\title{Convert a matrix to a likelihood function}
\description{

  Functions to convert matrix observations to likelihood functions.
  Each row is an observation of some kind, and each column a player.

  Function \code{ordertable2supp()} is documented separately at
  \code{ordertable2supp}.
}
\usage{
saffy(M)
volley(M)
}

\arguments{\item{M}{A matrix of observations}}

\details{

Two functions are documented here:

\itemize{

  \item \code{saffy()}, which converts a matrix of restricted choices
   into a likelihood function; it is named for Saffron O'Neill.  The
   canonical example would be Saffron's climate change dataset,
   documented at \code{icons}.  Function \code{saffy()} returns the
   appropriate likelihood function for the dataset.


  \item \code{volley()}, which converts a matrix of winning and losing
  team members to a likelihood function.  The canonical example is the
  volleyball dataset.  Each row is a volleyball game; each column is a
  player.  An entry of 0 means \dQuote{on the losing side}, an entry of
  1 means \dQuote{on the winning side}, and an entry of \code{NA} means
  \dQuote{did not play}.
  }

}

\author{Robin K. S. Hankin}
\seealso{\code{\link{icons}},\code{\link{volleyball}}}
\examples{

icons == saffy(icons_table)  # should be TRUE

volley(volleyball_table) == volleyball # also should be TRUE

}

\name{jester}
\alias{jester}
\alias{maxjest}
\alias{jokes}
\alias{humour}
\alias{humor}
\alias{jester_maxp}
\alias{jester_table}
\docType{data}
\title{Jester dataset}
\description{A likelihood function for the Jester datasets}
\usage{data(jester)}
\details{

Object \code{jester} is a likelihood function for the 91 jokes rated by
the first 150 respondents in file \file{jester_dataset_1_3.zip}, taken
from Goldberg et al.  Object \code{jester_maxp} is the result of running
\code{maxp(jester)}.  The results table of (nearly) all jokes and
respondents is given as \code{jester_table} in which each row is a joke
and each column a respondent.

The dataset is interesting because it has been analysed by many workers,
including Goldberg, for patterns; here I assume that all the respondents
behave identically (but randomly).  It is included here because it is a
very severe numerical challenge in the context of the \code{hyper2}
package.  I am not convinced that \code{maxjest} is even close to the
true evaluate.

Objects \code{jester}, \code{jester_table}, and \code{jester_maxp} can
be generated by running script \file{inst/jester.Rmd}, which includes
some further technical documentation.  This file takes about 10 minutes
to run.

}
\references{
Eigentaste: A Constant Time Collaborative Filtering Algorithm.  Ken
Goldberg, Theresa Roeder, Dhruv Gupta, and Chris
Perkins. \emph{Information Retrieval}, 4(2), 133-151. July 2001.
}

\examples{

# maxp(jester)  # takes too long

# Note that the possibly poor identification of the evaluate
#  nevertheless allows us to reject the null of equality:

(LAM <- -2*(loglik(equalp(jester),jester)-loglik(jester_maxp,jester)))
pval <- pchisq(LAM,df=size(jester),lower.tail=FALSE)


}
\keyword{datasets}

\name{constructor}
\alias{constructor}
\alias{constructors}
\alias{constructor_2020}
\alias{constructor_2020_maxp}
\alias{constructor_table_2020}
\alias{constructor_2020_table}
\alias{constructor_2021}
\alias{constructor_2021_maxp}
\alias{constructor_table_2021}
\alias{constructor_2021_table}
\docType{data}
\title{Formula 1 dataset: the constructors' championship}
\description{Race results from 2017 Formula One constructors' Championship}
\usage{
data(constructor)
}
\format{A \code{hyper3} object that gives a likelihood function}
\details{

The Constructors championship runs parallel to the Formula 1 drivers'
championship.  The package currently includes data from 2020 and 2021; the 
following text applies to both years.  I will add more years eventually.

Object \code{constructor_2021_table} is a dataframe, taken from
Wikipedia, with rows corresponding to performance of a constructor.
Each constructor fields two cars in each race; the identity of the
driver is not important in this context (and indeed may change as the
season progresses).  The first column is the name of the constructor,
the next 22 columns show the ranks of the constructors' cars, and the
final one is the points awarded.  At each venue, the constructor's best
performance is listed first.  The constructors' names change quite
frequently (e.g. \dQuote{Red Bull Racing-TAG Heuer} raced 2016,2017, and
2018; \dQuote{Red Bull Racing-Honda} raced 2019, 2020, and 2021.  I am
not sure whether to treat these as separate entities or not; file
\code{inst/constructor_names.txt} gives a dataframe of team names and
years they competed (not currently part of the package).  The row names
of the dataframe cannot be the constructors because these are not
unique.

Object \code{constructor_2021_maxp} gives the maximum likelihood
estimate for the constructors' strengths. 
The corresponding \code{hyper3} likelihood function
\code{constructor_2021} is produced by \code{ordertable2supp3()}.

These objects can be generated by running script \code{inst/race3.Rmd},
which includes some further discussion and technical documentation, and
creates file \code{constructor.rda} which resides in the \code{data/}
directory.

}
\references{

  Wikipedia contributors. (2022, April 14). 2021 Formula One World
  Championship. In _Wikipedia, The Free Encyclopedia_. Retrieved 05:16,
  April 17, 2022, from
  \url{https://en.wikipedia.org/w/index.php?title=2021_Formula_One_World_Championship&oldid=1082745216}
 
  }
\seealso{\code{\link{formula1}}}
\examples{
dotchart(constructor_2021_maxp)
}

\name{eurovision}
\alias{euro}
\alias{euro2009}
\alias{eurovision}
\alias{Eurovision}
\alias{Eurovision2009}
\alias{eurovision2009}
\alias{eurovision2009_votingtable}
\alias{Eurovision_song_contest}
\alias{eurovision_table}
\alias{eurovision_maxp}
\docType{data}
\title{Eurovision Song contest dataset}
\description{Voting patterns from Eurovision 2009}
\usage{data(eurovision)}
\format{
A \code{hyper2} object that gives a likelihood function.
}
\details{

Object \code{eurovision} is a \code{hyper2} likelihood function for the
skills of the 18 competitor countries in semi-final 1 of the 2009
Eurovision Song contest.  Object \code{eurovision_table} gives the
original dataset and \code{eurovision_maxp} the evaluate of the
competitors' Plackett-Luce strengths.

The motivation for choosing this particular dataset is that Pat Altham
(Statistical Laboratory, Cambridge) considered it with a view to
discover similarities between voters.  In the current analysis, the
likelihood function \code{eurovision} assumes their independence.

These objects can be generated by running script
\code{inst/eurovision.Rmd}, which includes some further discussion and
technical documentation and creates file \code{eurovision.rda} which
resides in the \code{data/} directory.


}
\references{
  \itemize{
    \item 
  Wikipedia contributors, \dQuote{Eurovision Song Contest
    2009---Wikipedia, The Free Encyclopedia}, 2018,
    \url{https://en.wikipedia.org/w/index.php?title=Eurovision_Song_Contest_2009&oldid=838723921}  [Online; accessed 13-May-2018].
    \item P. M. E. Altham, personal communication
  }
}
\seealso{\code{\link{eurodance}}}
\examples{
data(eurovision)
dotchart(eurovision_maxp)

}

\name{universities}
\alias{universities}
\alias{universities_table}
\alias{universities_maxp}
\docType{data}
\title{New Zealand University ranking data}
\description{Times Higher Education World University Rankings}
\usage{data(universities)}
\format{A \code{hyper2} object that gives a likelihood function for
  ranking of \acronym{NZ} universities}
\details{
The data is taken directly from the \acronym{THE} website, specifying \dQuote{New Zealand}:

  \url{https://www.timeshighereducation.com/world-university-rankings/2020/world-ranking#!/page/0/length/25/locations/NZ/sort_by/rank/sort_order/asc/cols/stats}

Object \code{universities} is a \code{hyper2} support function and
\code{universities_table} a data frame.  

These objects can be generated by running script
\code{inst/universities.Rmd}, which includes some further discussion and
technical documentation, and creates file \code{universities.rda} which
resides in the \code{data/} directory.
  
}
\seealso{\code{\link{ordertable}}}
\examples{

summary(universities)

psubs(universities,c("AUT","UoA"),c("University of Auckland","Auckland University of Technology"))

pie(universities_maxp)

}

\name{maxp}
\alias{maxp}
\alias{maxplist}
\alias{maxp_lsl}
\alias{maxp_single}
\alias{maxp_single2}
\alias{maxp_simplex}
\alias{equalp}
\alias{allequal}
\alias{equalprobs}
\title{Maximum likelihood estimation}
\description{
  Find the maximum likelihood estimate for p, also equal probabilities.

  TLDR summary: if \code{H} is a \code{hyper2} or \code{hyper3}
  likelihood function, \code{maxp(H)} will find the maximum likelihood
  estimate for the \acronym{BT} strengths.  Canonical example:
  \code{maxp(icons)}.

}
\usage{
maxp(H, startp=NULL, give=FALSE, fcm=NULL, fcv=NULL, SMALL=1e-6, n=1,
   show=FALSE, justlikes=FALSE, ...)
maxplist(Hlist, startp=NULL, give=FALSE, fcm=NULL, fcv=NULL, SMALL=1e-6, ...)
maxp_single(H, startp=NULL, give=FALSE, fcm=NULL, fcv=NULL, SMALL=1e-6,
   maxtry=100, ...)
maxp_single2(H, startp=NULL, give=FALSE, fcm=NULL, fcv=NULL, SMALL=1e-6,
   maxtry=100, ...)
maxp_simplex(H, n=100, show=FALSE, give=FALSE, ...)
maxp_lsl(HLSL, startp = NULL, give = FALSE, fcm = NULL, fcv = NULL, SMALL=1e-6, ...)
equalp(H)
}
\arguments{
  \item{H}{A \code{hyper2} or \code{hyper3} object}
  \item{Hlist}{A list with elements all \code{hyper2} objects}
  \item{HLSL}{An \code{lsl} object}
  \item{startp}{A vector of probabilities specifying the start-point for
    optimization; if a full unit-sum vector, then the fill-up value will
    be removed by \code{indep()} (except for \code{maxp_lsl()})}
  \item{give}{Boolean, with default \code{FALSE} meaning to return just
    the evaluate (including fillup), and \code{TRUE} meaning to return
    the entire formal output of the optimization routine.  In function
    \code{maxp()}, a non-logical value [such as \code{0}] gives only the
    two most important pieces of information, viz the evaluate and a
    log-likelihood}
  \item{fcm,fcv}{Further problem-specific constraints}
  \item{n}{Number of start points to use}
  \item{show}{Boolean, with \code{TRUE} meaning to show successive
    estimates}
  \item{justlikes}{Boolean, with \code{TRUE} meaning to return just a
    vector of estimated likelihoods}
  \item{SMALL}{Numerical minimum for probabilities}
  \item{maxtry}{Integer specifying maximum number of times to try
    \code{constrOptim()} with slightly differing start points, to avoid
    a known \R bug which reports \code{wmmin is not finite}, bugzilla
    id 17703}
  \item{...}{Further arguments which \code{maxp()} passes to
    \code{constrOptim()}}
}
\details{
  Function \code{maxp()} returns the maximum likelihood estimate for
  \code{p}, which has the unit sum constraint implemented.

  Function \code{maxplist()} does the same but takes a list of
  \code{hyper2} objects (for example, the output of \code{ggrl()}).
  Note that \code{maxplist()} does not have access to the gradient of
  the objective function, which makes it slow.

  If function \code{maxp()} is given a \code{suplist} object it
  dispatches to \code{maxplist()}.

  Functions \code{maxp_single()} and \code{maxp_single2()} are helper
  functions which perform a single constrained optimization using
  \code{base::constrOptim()} or \code{alabama::constrOptim.nl()}
  respectively.  The functions should produce identical (or at least
  very similar) results.  They are used by \code{maxp()} and
  \code{maxp_simplex()} which dispatch to either \code{maxp_single()} or
  \code{maxp_single2()} depending on the value of option
  \code{use_alabama}.  If \code{TRUE}, they will use (experimental)
  \code{maxp_single2()}, otherwise (default) \code{maxp_single()}.
  Function \code{maxp_single()} is prone to the \dQuote{wmmin not
  finite} bug [bugzilla id 17703] but on the other hand is a bit
  slower.  I am not sure which one is better at this time.
 
  Function \code{maxp_simplex()} is intended for complicated or flat
  likelihood functions where finding local maxima might be a problem.
  It repeatedly calls \code{maxp_single()}, starting from a different
  randomly chosen point in the simplex each time.  This function does
  not take \code{fcm} or \code{fcv} arguments, it operates over the
  whole simplex (hence the name).  Further arguments, \code{...}, are
  passed to \code{maxp_single()}.
  
  The functions do not work for the \code{masterchef_series6} likelihood
  function.  These require a bespoke optimization as shown in the
  vignette.

  Function \code{equalp()} returns the value of \eqn{p} for which all
  elements are the same.

  In functions \code{maxp()} etc, arguments \code{fcm} and \code{fcv}
  implement linear constraints to be passed to \code{constrOptim()}.
  These constraints are in addition to the usual nonnegativity
  constraints and unit-sum constraint, and are added to the \code{ui}
  and \code{ci} arguments of \code{constrOptim()} with \code{rbind()}
  and \code{c()} respectively.  The operative lines are in
  \code{maxp_single()}:

  \preformatted{
    UI <- rbind(diag(nrow = n - 1), -1, fcm)
    CI <- c(rep(SMALL, n - 1), -1 + SMALL, fcv)
  }

  where in \code{UI}, the first \eqn{n-1} rows enforce nonnegativity of
  \eqn{p_i}, \eqn{1\leq p < n}{1<=i<n}; row \eqn{n} enforces
  nonnegativity of the fillup value \eqn{p_n}; and the remaining
  (optional) rows enforce additional linear constraints.  Argument
  \code{CI} is a vector with corresponding elements.

  Examples of their use are given in the \dQuote{icons} vignette.

}
\author{Robin K. S. Hankin}
\note{

  In manpages elsewhere, \code{n=2} is sometimes used.  Previous
  advice was to use \code{n=10} or greater in production work, but I
  now think this is overly cautious and \code{n=1} is perfectly
  adequate unless the dimension of the problem is large.

  The (bordered) Hessian is given by function \code{hessian()},
  documented at \code{gradient.Rd}; use this to assess the
  \dQuote{sharpness} of the maximum.

  Function \code{maxp()} takes \code{hyper2} or \code{hyper3} objects
  but it does not currently work with \code{lsl} objects; use
  \code{maxp_lsl()}.

  The built-in datasets generally include a pre-calculated result of
  running \code{maxp()}; thus hyper2 object \code{icons} and
  \code{icons_maxp} are included in the same \code{.rda} file.
  
  Function \code{maxp()} can trigger a known \R bug (bugzilla id 17703)
  which reports \dQuote{\code{wmmin is not finite}}.  Setting option
  \code{use_alabama} to \code{TRUE} makes the package use a different
  optimization routine.

}
\seealso{\code{\link{gradient}},\code{\link{fillup}}}
\examples{

maxp(icons)

W <- hyper2(pnames=letters[1:5])
W1 <- ggrl(W, 'a', letters[2:3],'d')  # W1 is a suplist object
\dontrun{maxp(W1)}  # takes a long time to maximize a suplist

}

\name{tennis}
\alias{tennis}
\alias{tennis_maxp}
\alias{tennis_ghost_maxp}
\alias{doubles}
\alias{doubles_noghost}
\alias{tennis_noghost}
\alias{doubles_ghost}
\alias{tennis_ghost}
\docType{data}
\title{Match outcomes from repeated doubles tennis matches}
\description{Match outcomes from repeated doubles tennis matches}
\usage{data(tennis)}
\format{
A hyper2 object corresponding to the match outcomes listed below.
}
\details{

There are four players, \eqn{p_1}{p1} to \eqn{p_4}{p4}.  These players
play doubles tennis matches with the following results:

\tabular{cc}{
  match \tab score\cr
  \eqn{\lbrace p_1,p_2\rbrace}{{p1,p2}} vs \eqn{\lbrace p_3,p_4\rbrace}{{p3,p4}} \tab 9-2\cr
  \eqn{\lbrace p_1,p_3\rbrace}{{p1,p3}} vs \eqn{\lbrace p_2,p_4\rbrace}{{p2,p4}} \tab 4-4\cr
  \eqn{\lbrace p_1,p_4\rbrace}{{p1,p4}} vs \eqn{\lbrace p_2,p_3\rbrace}{{p2,p3}} \tab 6-7\cr
  \eqn{\lbrace p_1\rbrace}{{p1}} vs \eqn{\lbrace p_3\rbrace}{{p3}} \tab 10-14\cr
  \eqn{\lbrace p_2\rbrace}{{p2}} vs \eqn{\lbrace p_3\rbrace}{{p3}} \tab 12-14\cr
  \eqn{\lbrace p_1\rbrace}{{p1}} vs \eqn{\lbrace p_4\rbrace}{{p4}} \tab 10-14\cr
  \eqn{\lbrace p_2\rbrace}{{p2}} vs \eqn{\lbrace p_4\rbrace}{{p4}} \tab 11-10\cr
  \eqn{\lbrace p_3\rbrace}{{p3}} vs \eqn{\lbrace p_4\rbrace}{{p4}} \tab 13-13\cr
}

It is suspected that \eqn{p_1}{p1} and \eqn{p_2}{p2} have some form of
team cohesion and play better when paired than when either solo or with
other players.  As the scores show, each player and, apart from p1-p2,
each doubles partnership, is of approximately the same strength.

Dataset \code{tennis} gives the appropriate likelihood function for the
players' strengths; and dataset \code{tennis_ghost} gives the
appropriate likelihood function if the extra strength due to team
cohesion of \eqn{\lbrace p_1,p_2\rbrace}{{p1,p2}} is represented by a
ghost player.

These objects can be generated by running script
\code{inst/tennis.Rmd}, which includes some further discussion and
technical documentation and creates file \code{tennis.rda} which
resides in the \code{data/} directory.

}
\source{Doubles tennis matches at \acronym{NOCS}, Jan-May 2008}
\references{
  Robin K. S. Hankin (2010).
  \dQuote{A Generalization of the Dirichlet Distribution},
  \emph{Journal of Statistical Software}, 33(11), 1-18
}
\examples{
summary(tennis)

tennis |> psubs(c("Federer","Laver","Graf","Navratilova"))

## Following line commented out because it takes too long:
# specificp.gt.test(tennis_ghost,"G",0)

}
\keyword{datasets}

\name{tests}
\alias{equalp.test}
\alias{knownp.test}
\alias{samep.test}
\alias{specificp.test}
\alias{print.equalptest}
\alias{print.hyper2test}
\alias{specificp.ne.test}
\alias{specificp.gt.test}
\alias{specificp.lt.test}
\alias{specificp.ge.test}
\alias{specificp.le.test}
\title{Hypothesis testing}
\description{
  Tests different nulls against a free alternative
}
\usage{
equalp.test(H, startp=NULL, ...)
knownp.test(H, p, ...)
samep.test(H, i, give=FALSE, startp=NULL, ...)
specificp.test(H, i, specificp=1/size(H),
         alternative = c("two.sided","less","greater"),  ...)
specificp.ne.test(H, i, specificp=1/size(H), ...)
specificp.gt.test(H, i, specificp=1/size(H), delta=1e-5, ...)
specificp.lt.test(H, i, specificp=1/size(H), ...)
\method{print}{hyper2test}(x, ...)
}
\arguments{
  \item{H}{A likelihood function, an object of class \code{hyper2}}
  \item{p}{In \code{knownp.test()}, putative strength vector to be
    tested}
  \item{...}{Further arguments passed by \code{equalp.test()} to
    \code{maxp()} and ignored by \code{print.hyper2test()}}
  \item{startp}{Starting value for optimization}
  \item{i}{A character vector of names}
  \item{specificp}{Strength, real number between 0 and 1}
  \item{alternative}{a character string specifying the alternative
    hypothesis, must be one of \code{two.sided} (default),
    \code{greater} or \code{less}.  You can specify just the initial
    letter (taken from \code{t.test.Rd})}
  \item{give}{Boolean, with \code{TRUE} meaning to return more
    detailed debugging information, and default \code{FALSE} meaning to
    return a more user-friendly object of class \code{equalp.test},
    which has its own print method}
  \item{x}{Object of class \code{equalp.test}, the result of
    \code{equalp.test()}}
  \item{delta}{Small value for numerical stability}
}
\details{

  Given a \code{hyper2} likelihood function, there are a number of
  natural questions to ask about the strengths of the players; see
  Hankin 2010 (JSS) for examples.  An extended discussion is presented
  in vignette \dQuote{\code{hyper2}} and the functions documented here
  cover most of the tests used in the vignette.

  The tests return an object with class \code{hyper2test}, which has its
  own print method.

\itemize{
  \item Function \code{equalp.test(H,p)} tests the null that all
  strengths are equal to vector \code{p}.  If \code{p} is missing, it
  tests \eqn{H_0\colon p_1=p_2=\cdots=p_n=\frac{1}{n}}{H0:
    p1=p2=...=pn=1/n}, for example \code{equalp.test(icons)}
  \item Function \code{knownp.test()} tests the null that the strengths
  are equal to the elements of named vector \code{p}; it is a
  generalization of \code{equalp.test()}.  Example:
  \code{knownp.test(icons,zipf(6))}. 
  \item Function \code{specificp.test(H,i,p)} tests
  \eqn{H_0\colon p_i=p}{H0: p_i=p}, for example
  \code{specificp.test(icons,"NB",0.1)}
  \item Function \code{samep.test()} tests \eqn{H_0\colon
  p_{i_1}=p_{i_2}=\cdots=p_{i_k}}{omitted}, for example
  \code{samep.test(icons,c("NB","L"))} tests that \code{NB} has the same
  strength as \code{L}. 
  \item Functions \code{specificp.ne.test(H,i,p)},
  \code{specificp.gt.test(H,i,p)}, and \code{specificp.lt.test(H,i,p)}
  are low-level helper functions that implement one- or two-sided versions
  of \code{specificp.test()} via the \code{alternative} argument,
  following \code{t.test()}
}
}

\value{
  The test functions return a list with class \code{"hyper2test"}
  containing the following components:
  \item{statistic}{the difference in support between the null and
    alternative}
  \item{p.value}{the (asymptotic) p-value for the test, based on Wilks's
    theorem} 
  \item{estimate}{the maximum likelihood estimate for \eqn{p}}
  \item{method}{a character string indicating what type of test was
    performed}
  \item{data.name}{a character string giving the name(s) of the data.}
}
\seealso{\code{\link{maxp}}}

\note{

Function \code{specificp.gt.test()} includes quite a bit of messing
about to ensure that frequently-used idiom like
\code{specificp.gt.test(icons,"NB",0)} works as expected, testing a null
of \code{p_NB=0} (observe that \code{specificp.ne.test(icons,"NB",0)}
will (correctly) throw an
error).  In the case of testing a strength's being zero, the support
function is often quite badly-behaved near the constraint [think tossing
a coin with probability \eqn{p} twice, observing one head and one tail,
and testing \eqn{p=0}; at the constraint, the likelihood is zero, the
support negative infinity, and the gradient of the support is infinite].
Numerically, the code tests \code{p_NB=delta}.  Note that similar
machinations are not required in \code{specificp.lt.test()} because a
null of \code{p_NB=1} is unrealistic.

Function \code{samep.test()} does not have access to gradient
information so it is slow, inaccurate, and may fail completely for
high-dimensional datasets.  If \code{any(i==n)}, this constrains the
fillup value; this makes no difference mathematically but the function
idiom is involved.

In functions \code{specificp.??.test(H,i,...)}, if \code{i} is not
present in \code{H}, an error is returned although technically the
result should be \dQuote{not enough evidence to reject}, as \code{H} is
uninformative about \code{i}.

}

\examples{

equalp.test(chess)

# samep.test(icons,c("NB","L"))
# knownp.test(icons,zipf(icons))

}

\name{pairwise}
\alias{pairwise}
\alias{zermelo}
\alias{home_away}
\alias{home_away3}
\alias{home_draw_away3}
\alias{home_away_table}
\alias{white_draw}
\alias{white_draw3}
\title{Pairwise comparisons}
\description{
  Convenience functions to generate likelihoods for pairwise
  comparisons.
}
\usage{
pairwise(M)
zermelo(M, maxit = 100, start, tol = 0, give = FALSE)
home_away(home_games_won, away_games_won, monster="home")
home_away3(home_games_won, away_games_won, lambda)
home_away_table(a, give=FALSE, teams)
white_draw3(A, lambda, D)
}
\arguments{
  \item{M}{Matrix of pairwise comparison results}
  \item{maxit}{Maximum number of iterations}
  \item{start}{Starting value for iteration; if missing, use
    \code{equalp()}}
  \item{tol}{Numerical tolerance for stopping criterion}
  \item{give}{In \code{zermelo()}, Boolean with default \code{FALSE}
    meaning to return the evaluate and \code{TRUE} meaning to return all
    iterations; in \code{home_away_table()} governs output form}
  \item{home_games_won, away_games_won}{Matrices showing home games won
    and away games won}
  \item{monster}{The monster, with default \code{home} representing
    the home ground advantage}
  \item{lambda}{The home ground advantage
    (or white advantage in chess)}
  \item{D}{Weight of draw}
  \item{A}{Array of dimension \code{n*n*3}, with \code{A[,,i]}
  corresponding to white wins, white draws, and white losses for
  \code{i=1,2,3}.  The canonical example would be \code{kka_array},
  see \code{inst/kka.Rmd} for details}
  \item{a, teams}{In function \code{home_away_table()}, argument
    \code{a} is a data frame (typically of football results),
    \code{give} a boolean governing output form, and \code{teams} a list
    of football teams.  See details}
}
\details{

  Function \code{pairwise(M)} takes a \code{M}, a matrix of pairwise
  comparisons and returns a \code{hyper2} likelihood function.  If there
  are only two competitors, so \code{M} is 2-by-2, it might be better to
  use \code{dirichlet()} or \code{race()} with 2 players, or one of 
  the \code{pick()} quartet.

  Function \code{home_away()} takes two matrices, one for home wins and
  one for away wins.  It returns a \code{hyper2} support function that
  includes a home advantage monster [or indeed any systematic
  advantage].  Set the monster to \code{NULL} if the home advantage is
  not present.  Function \code{home_away3()} is the same, but returns a
  \code{hyper3} object.  A complex matrix is interpreted as real parts
  being the home wins and imaginary parts away wins.

  Function \code{home_away_table()} takes a dataframe of results (each
  row being a single match) and returns a table amenable to analysis by
  \code{home_away()} or \code{home_away3()}.  If \code{give} takes its
  default value of \code{FALSE}, draws are discarded and a complex
  matrix of wins and losses is returned.  Files
  \code{inst/monster_vs_lambda.Rmd} and \code{inst/home_advantage.Rmd}
  show some use-cases.  Argument \code{teams} is a character vector that
  specifies the teams to be tabulated (useful if one wishes to change
  the default ordering of the teams).

  Function \code{white_draw3()} returns a \code{hyper3} likelihood
  function for pairwise comparisons, one of whom has a home team-type
  advantage (white player in the case of chess).  It is designed to
  work with an array of dimensions \eqn{n\times n\times 3}{n*n*3},
  where \eqn{n} is the number of players.  It is used in
  \code{inst/kka.Rmd} to create \code{chess3} likelihood function.

  If \code{home_games_won} is complex, then the real parts of the
  entries are interpreted as home games won, and the imaginary parts as
  away games won.

  Function \code{zermelo()} implements a standard iterative procedure
  for maximization of pairwise Bradley-Terry likelihoods (such as those
  produced by function \code{pairwise()}).  It takes a matrix of
  pairwise comparisons; the diagonal is disregarded.  The process keeps
  going until the difference between successive estimates does not
  exceed argumet \code{tol}; note that \code{tol=0} is acceptable.  If
  \code{give=TRUE}, a matrix with rows corresponding to successive
  approximations is returned; if so, argument \code{maxit} corresponds
  to the maximum number of rows.
}
\references{
  \itemize{
    \item D. R. Hunter 2004. \dQuote{MM algorithms for generalized
  Bradley-Terry models}. \emph{The Annals of Statistics}, volume 32,
number 1, pages 384--406
\item S. Borozki and others 2016.  \dQuote{An application of incomplete pairwise
  comparison matrices for ranking top tennis
  players}. \code{arXiv:1611.00538v1} \code{10.1016/j.ejor.2015.06.069}

\item R. R. Davidson and R. J. Beaver 1977. \dQuote{On extending the
  Bradley-Terry model to incorporate within-pair order effects}.
  \emph{Biometrics}, 33:693--702
}
}
\author{Robin K. S. Hankin}
\note{

An extended discussion of \code{pairwise()} is given in
\code{inst/zermelo.Rmd} and also \code{inst/karate.Rmd}.  Functions
\code{home_away()} and \code{home_away3()} are described and used in
\code{inst/home_advantage.Rmd}; see Davidson and Beaver 1977.

Experimental function \code{pair3()} is now removed as
  \code{dirichlet3()} is more general and has nicer idiom;
  \code{pair3(a=4, b=3, lambda=1.88)} and \code{dirichlet3(c(a=4, b=3),
  1.88)} give identical output.

}
\seealso{\code{\link{maxp}}}
\examples{ #Data is the top 5 players from Borozki's table 1

M <- matrix(c(
0,10,0, 2,5,
4, 0,0, 6,6,
0, 0,0,15,0,
0, 8,0, 0,7,
1 ,0,3, 0,0
),5,5,byrow=TRUE) 
players <-  c("Agassi","Becker","Borg","Connors","Courier")
dimnames(M) <- list(winner=players,loser=players)
M
# e.g. Agassi beats Becker 10 times and loses 4 times
pairwise(M)
zermelo(M)
# maxp(pairwise(M))  # should be identical (takes ~10s to run)

M2 <- matrix(c(NA,19+2i,17,11+2i,16+5i,NA,12+4i,12+6i,12+2i,19+10i,
NA,12+4i,11+2i,16+2i,11+7i,NA),4,4)
teams <- LETTERS[1:4]
dimnames(M2) <- list("@home" = teams,"@away"=teams)
home_away(M2)
home_away(M2, monster = NULL) # nullify the home ground monster
# home_away3(M2,lambda=1.2)  # works but takes too long (~3s)
home_away3(M2[1:3,1:3],lambda=1.2) 

M <- kka_array[,,1] + 1i*kka_array[,,3] # ignore draws
home_away(M)
# home_away3(M,lambda=1.3)  # works but takes too long (~3s)

white_draw3(kka_array,1.88,1.11)
}

\name{loglik}
\alias{loglik}
\alias{loglik_single}
\alias{like_single_list}
\alias{like_series}
\title{Log likelihood functions}
\description{
  Returns a log-likelihood for a given \code{hyper2} or \code{hyper3}
  object at a specific point in probability space
}
\usage{
loglik(p, H, log = TRUE)
loglik_single(p,H,log=TRUE)
like_single_list(p,Lsub)
like_series(p,L,log=TRUE)

}
\arguments{
  \item{H}{An object of class \code{hyper2} or \code{hyper3}}
  \item{p}{A probability point.  See details}
  \item{log}{Boolean with default \code{TRUE} meaning to return the
    log-likelihood and \code{FALSE} meaning to return the likelihood}
  \item{L,Lsub}{A list of \code{hyper2} objects, or a list of list of
    \code{loglik} objects}
}
\details{

  Function \code{loglik()} is a straightforward likelihood function.  It
  can take a vector of length \code{n=size(H)} or \code{size(H)-1}.  If
  given the vector
  \eqn{p=\left(p_1,\ldots,p_{n-1}\right)}{p=(p_1,...,p_{n-1})} it
  appends the fillup value, and then returns the (log)
  likelihood (names are discarded in this case).  If given a vector
  \eqn{p=\left(p_1,\ldots,p_{n}\right)}{p=(p_1,...,p_{n})} [notionally
  summing to 1] it requires a named vector, and names must match those
  of \code{H}.  The vector is reordered if necessary.

  If \code{p} is a matrix, the rows are interpreted as probability
  points.

  Function \code{loglik_single()} is a helper function that evaluates a
  likelihood function single point in probability space.  Functions
  \code{like_single_list()} and \code{like_series()} are intended for
  use with \code{ggrl()}.

}
\note{

  \emph{Likelihood} is defined up to an arbitrary multiplicative
  constant.  Log-likelihood (also known as \emph{support}) is defined up
  to an arbitrary additive constant.

  If function \code{loglik()} is given a probability vector of length
  \code{n}, the vector must satisfy the unit sum constraint (up to a
  small tolerance).  Also, it must be a named vector with names
  matching the \code{pnames} of argument \code{H}.

  \preformatted{
  > pnames(chess)
  [1] "Topalov" "Anand"   "Karpov"  
  > loglik(c(Topalov=0.7,Anand=0.2,Karpov=0.1),chess)
  [1] -69.45364
  > loglik(c(Karpov=0.1,Topalov=0.7,Anand=0.2),chess)  # identical, just a different order
  [1] -69.45364
  }

  But if given a vector of length \code{n-1} [e.g. the value of
  \code{indep()}], then the names are ignored and the entries are
  interpreted as the \acronym{BT} strengths of
  \code{pnames(H)[seq_len(n-1)]}:

  \preformatted{
  > loglik(c(0.7,0.2),chess)
  [1] -69.45364
  > loglik(c(foo=0.7,bar=0.2),chess)  # names are ignored 
  [1] -69.45364
  }

  (the above applies for \code{H} a \code{hyper2} or \code{hyper3}
  object).

  \code{loglik(p, H)} will throw an error if \code{H} does satisfy the
  zero-sum power requirement; see \code{?balance} and the
  \code{zeropower} package vignette.
  
  Empty brackets are interpreted consistently: that is, zero whatever
  the probability vector (although the print method is not perfect).
  
  }
\author{Robin K. S. Hankin}
\seealso{\code{\link{maxp}}, \code{\link{balance}}}
\examples{
data(chess)
loglik(c(1/3,1/3),chess)

loglik(rp(14,icons),icons)

\dontrun{ # takes too long
like_series(masterchef_maxp,masterchef)
like_series(indep(equalp(masterchef)),masterchef)
}

W <- hyper2(pnames=letters[1:6])
W1 <- ggrl(W, 'a', letters[2:5],'f')              # 24-element list
W2 <- ggrl(W, c('a','b'), c('c','d'),c('e','f'))  # 2^3=8 element list

like_single_list(rep(1/6,5),W1)      # information from first observation
like_series(rep(1/6,5),list(W1,W2))  # information from both observations

# hyper3 objects:
H3 <- ordervec2supp3(letters[c(1,2,3,3,2,1,2)])
loglik(c(a=1,b=2,c=3)/6,H3)
loglik(c(a=1,c=3,b=2)/6,H3) # identical
}

\name{Print}
\alias{print.hyper2}
\alias{print.hyper3}
\alias{print}
\alias{Print}
\alias{pnv}
\alias{give_warning_on_nonzero_power_sum}
\alias{nonzero_power_sum}
\alias{power_sum}
\title{Print methods}
\description{
Print methods for \code{hyper2} and \code{hyper3} objects
}
\usage{
\method{print}{hyper2}(x, ...)
\method{print}{hyper3}(x, ...)
}
\arguments{
  \item{x}{An object of class \code{hyper2} or \code{hyper3}}
  \item{\dots}{Further arguments, currently ignored}
}
\value{
  Returns the \code{hyper2} or \code{hyper3} object it was sent,
  invisibly.

  Function \code{pnv()} (\dQuote{\bold{p}rint \bold{n}amed
  \bold{v}ector}) takes a named vector and returns a character string
  that is used in the \code{hyper3} print method.  It is sensitive to
  base R print options such as \code{digits} and \code{scipen}.
  Currently there is no space around the \dQuote{=} symbol but this is
  easy to change.
  
}
\details{

Used mainly for their side-effect of printing the log-likelihood
function.  In the print method, a natural logarithm is indicated with
\dQuote{\code{log()}}---not \dQuote{\code{ln()}}---consistent with \R
builtin terminology \code{base::log()}.

The \code{hyper2} print method is sensitive to option
\code{give_warning_on_nonzero_power_sum}.  If \code{TRUE}, a warning is
issued if the powers have nonzero sum.  This is usually what you want
because observations are typically multinomial; a warning indicates
nonzero sum of powers, which should prompt us to check the coding.
Vignette \code{zeropower} gives a discussion of this issue.

The print method for \code{attemptstable} objects is given at
\code{attemptstable.Rd}.

}
\author{Robin K. S. Hankin}
\seealso{\code{\link{attemptstable}}}
\note{
  Sometimes the use of \code{pnv()} can be confusing, as distinct
  brackets can appear to be identical, as per the example.  See how the
  two terms with power \eqn{-1} appear to be identical but actually
  differ by \code{1e-12}, invisible to the print method which only shows
  seven significant figures.
}
\examples{
data(chess)
chess

getOption("digits")
hyper3(list(c(a=1), c(a=1, b=pi),c(a=1, b=pi+1e-12)), powers = c(2, -1, -1))

}

\name{tidy}
\alias{tidy}
\title{Tidy up a hyper2 object}
\description{Tidy up a hyper2 object by removing players about which we
  have no information}
\usage{tidy(H)}
\arguments{\item{H}{A \code{hyper2} object}}
\details{

  Function \code{tidy(H)} returns a hyper2 object mathematically
  identical to \code{H} but with unused players (that is, players that
  do not appear in any bracket) removed.  Players about which \code{H}
  is uninformative are removed from the \code{pnames} attribute.

  Note that idiom \code{pnames(H) <- foo} can also be used to manipulate
  the \code{pnames} attribute.


}
\author{Robin K. S. Hankin}
\examples{


H <- hyper2(pnames=letters)
H["a"] <- 1
H["b"] <- 2
H[c("a", "b")] <- -3

pnames(H)
pnames(tidy(H))

H == tidy(H)  # should be TRUE
}

\name{chess}
\alias{chess}
\alias{chess_table}
\alias{chess_maxp}
\alias{rock_paper_scissors}
\docType{data}
\title{Chess playing dataset}
\description{
A tally of wins and losses for games between three chess players:
Topalov, Anand, Karpov.
}
\usage{data(chess)}
\details{

  (there are three chess datasets in the package, documented at
  \code{interzonal.Rd} [the 1963 World championship], \code{kka.Rd}
  [Karpov-Kasparov-Anand dataset], and \code{chess.Rd}
  [rock-paper-scissors using Topalov-Anand-Karpov])
  
  This is a very simple dataset that can be used for illustration of
  \code{hyper2} idiom.
  
  The players are:
  \itemize{
    \item Grandmaster Veselin Topalov.  \acronym{FIDE} world
    champion 2005-2006; peak rating 2813
    \item Grandmaster Viswanathan Anand.  \acronym{FIDE} world
    champion 2000-2002, 2008; peak rating 2799
    \item Grandmaster Anatoly Karpov.  \acronym{FIDE} world
    champion 1993-1999; peak rating 2780
  }

  Observe that Topalov beats Anand, Anand beats Karpov, and Karpov beats
  Topalov (where \dQuote{beats} means \dQuote{wins more games than}).

  The games thus resemble a noisy version of \dQuote{rock paper scissors}.

  The likelihood function does not record who played white; see
  \code{karpov_kasparov_anand} for such a dataset.

These objects can be generated by running script
\code{inst/rock_paper_scissors.Rmd}, which includes some further
discussion and technical documentation and creates file \code{chess.rda}
which resides in the \code{data/} directory.


File \code{inst/ternaryplot_hyper2.Rmd} gives an example showing the
\code{chess} likelihood function that uses
\code{Ternary::ternaryPlot()}.


}
\references{
  \itemize{
    \item \url{https://en.chessbase.com/}
  }
}
\seealso{\code{\link{karpov_kasparov_anand}}}
\examples{
data(chess)
maxp(chess)

 mgf(chess,c(Anand=2),tol = 0.1)  # tolerance for speed

}
\keyword{datasets}

\name{ranktable}
\alias{ranktable}
\alias{as.ranktable}
\alias{is.ranktable}
\alias{ranktable_to_ordertable}
\alias{ordertable_to_ranktable}
\alias{wikitable_to_ranktable}
\alias{print.ranktablesummary}
\alias{summary.ranktable}
\title{Convert rank tables to and from order tables}
\description{
  Convert rank tables (as generated by \code{rrank()}, for example) to
  order tables like the formula 1 tables; and convert back.  Print and
  summary methods for rank tables are documented here.  See also
  \code{ordertable.Rd}. 
}
\usage{
ranktable_to_ordertable(xrank)
ordertable_to_ranktable(xorder)
wikitable_to_ranktable(wikitable, strict=FALSE)
\method{summary}{ranktable}(object, ...)
\method{print}{ranktablesummary}(x,...)
}
\arguments{
  \item{x, xrank, object}{A rank table, an object with class
    \code{ranktable}, for example the value of \code{rrank()}}
  \item{xorder, wikitable}{Order tables.  Argument \code{wikitable}
    refers to a generalized order table which can include entries such
    as DNF signifying did not finish.}
  \item{strict}{Controls for \code{wikitable_to_ranktable()}}
  \item{...}{Further arguments (currently ignored)}  
}
\details{

  Function \code{ranktable_to_ordertable()} uses \code{order()} to
  convert a ranktable object to an ordertable. Note that the rows of of
  the resulting ordertable will be in alphabetic order.  The prototype
  for order tables would be \code{skating_table}; to convert to
  ranktable form and back to ordertable, use:

  \preformatted{ranktable_to_ordertable(ordertable_to_ranktable(skating_table))}

  or

  \preformatted{as.ordertable(as.ranktable(skating_table))}

Function \code{ordertable_to_ranktable(x)} checks for each column being
a permutation of \code{seq_len(nrow(x))} and, if not, it stops.  In
particular, \acronym{DNF} entries are out of scope.  To convert order
tables such as \code{F1_table_2017}, which include \acronym{DNF}
entries, use \code{wikitable_to_ranktable()} or \code{ordertable2supp()}
to produce a likelihood function.

Function \code{ranktable_to_printable_object()} is a helper function
that coerces a \code{ranktable} object to a matrix that prints nicely.

File \code{inst/ordertable_to_ranktable.Rmd} discusses the
\code{ranktable} print method and also sets out a common gotcha for
interpretation of the internal structure of \code{ranktable} objects.

}
\value{An order table or rank table}
\author{Robin K. S. Hankin}
\seealso{
  \code{\link{rrank}},
  \code{\link{ordertable2supp}}
}
\examples{

p <- (5:1)/15
names(p) <- letters[1:5]
xrank <- rrank(12, p, rnames=month.abb)

pentathlon_table  # an ordertable
as.ranktable(pentathlon_table) # coerce to a ranktable

}

\name{length.hyper2}
\alias{length.hyper2}
\alias{length}
\title{Length method for hyper2 objects}
\description{
Length method for hyper2 objects, being the number of different brackets
in the expression}
\usage{\method{length}{hyper2}(x)}
\arguments{\item{x}{hyper2 object}}
\author{Robin K. S. Hankin}
\examples{

data("oneill")
length(icons)
seq_along(icons)
}

\name{ordertrans}
\alias{ordertrans}
\alias{ordertransplot}
\title{Order transformation}
\description{
Given an order vector, shuffle so that the players appear in a specified
order. 
}
\usage{
ordertrans(x, players)
ordertransplot(ox, oy, plotlims, ...)
}
\arguments{
  \item{x}{A (generalized) order vector}
  \item{players}{A character vector specifying the order in which the
    players will be listed; if missing, use \code{sort(names(x))}}
  \item{ox, oy}{Rank vectors}
  \item{plotlims}{Length two numeric vector giving x and y plot limits.
    If missing, use sensible default}
  \item{...}{Further arguments, passed to \code{plot()}}
}
\details{
  
  The best way to describe this function is with an example:

  \preformatted{
> x <- c(d=2,a=3,b=1,c=4)
> x
d a b c 
2 3 1 4 
}

In the above, we see \code{x} is an order vector showing that \code{d}
came second, \code{a} came third, \code{b} came first, and \code{c} came
fourth.  This is difficult to deal with because one has to search
through the vector to find a particular competitor, or a particular
rank.  This would be harder if the vector was longer.
If we wish to answer the question \dQuote{where did competitor \code{a}
  come?  where did \code{b} come?} we would want an \emph{order} vector
in which the competitors are in alphabetical order.  This is
accomplished by \code{ordertrans()}:

\preformatted{
> o <- ordertrans(x)
> o
a b c d 
3 1 4 2 
}

(this is equivalent to \code{o <- x[order(names(x))]}).  Object \code{o}
contains the same information as \code{x}, but presented differently.
This says that \code{a} came third, \code{b} came first, \code{c} came
fourth, and \code{d} came second.  In particular, the Plackett-Luce
order statistic is identical:

\preformatted{
> ordervec2supp(x) == ordervec2supp(o)
> [1] TRUE
}

There are nice examples of \code{ordertrans()} in
\code{inst/eurovision.Rmd} and \code{inst/carcinoma.Rmd}.  Vignette
\code{ordertrans} provides further discussion and examples.

Function \code{ordertrans()} takes a second argument which allows the
user to arrange an order vector into the order specified.

Function \code{ordertrans()} also works in the context of \code{hyper3}
objects:

\preformatted{
x <- c(d=2, a=3, b=1, a=4)
x
d a b a 
2 3 1 4 
ordertrans(x)
a a b d 
3 4 1 2 
}
  
Object \code{x} shows that \code{d} came second, \code{a} came third and
fourth, and \code{b} came first.  We can see that \code{ordertrans()}
gives the same information in a more intelligible format.  This
functionality is useful in the context of \code{hyper3} likelihood
functions.
}
\value{Returns a named vector}
\author{Robin K. S. Hankin}
\note{

  The argument to \code{ordertrans()} is technically an order vector
  because it answers the question \dQuote{where did the first-named
  competitor come?} (see the discussion at \link{rrank}).  But it is
  not a helpful order vector because you have to go searching through
  the names---which can appear in any order---for the competitor you are
  interested in.  I guess \dQuote{generalised order vector} might be a
  better description of the argument.


  
}
\seealso{\code{\link{rrank}}}
\examples{

x <- c(e=4L, a=7L, c=6L, b=1L, f=2L, g=3L, h=5L, i=8L, d=9L)
x
ordertrans(x, letters[1:9])

o <- unclass(skating_table)[,1]
names(o) <- rownames(skating_table)
o
ordertrans(o)

ordertrans(sample(icons_maxp), icons)


rL <- volvo_maxp   # rL is "ranks Likelihood"
rL[] <- rank(-volvo_maxp)

r1 <- as.numeric(unclass(volvo_table)[,1])  # ranks race 1
names(r1) <- rownames(volvo_table)
ordertransplot(rL, r1, xlab="likelihood rank, all races", ylab="rank, race 1")

}

\name{mult_grid}
\alias{mult_grid}
\alias{pair_grid}
\title{Kronecker matrix product functionality}
\description{
Peculiar version of \code{expand.grid()} for matrices
}
\usage{
mult_grid(L)
pair_grid(a,b)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{L}{List of matrices}
  \item{a,b}{Matrices}
}
\details{
Function \code{pair_grid(a,b)} returns a matrix with each column of
\code{a} \code{cbind()}-ed to each column of \code{b}.

Function \code{mult_grid()} takes a list of matrices; it is designed for
use by \code{ggrl()}.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{ggrl}}}
\examples{
pair_grid(diag(2),diag(3))
mult_grid(lapply(1:4,diag))
}

\name{fillup}
\alias{fillup}
\alias{indep}
\title{Fillup function}
\description{
  Function \code{fillup()} concatenates a vector with a \sQuote{fillup}
  value to ensure a unit sum; if given a matrix, attaches a column so
  the rowsums are 1.

  Function \code{indep()} is the inverse: it removes the final element
  of a vector, leaving only an independent set.
  
}
\usage{
fillup(x, H=NULL, total=1)
indep(x)
}
\arguments{
  \item{x}{Numeric vector}
  \item{H}{Object with \code{pnames()} attribute, typically of class
    \code{hyper2} or \code{hyper3}, used for names if supplied}
    \item{total}{Total value for probability}
}
\details{
  Usually you want the total to be one, to enforce the unit sum
  constraint.  Passing \code{total=0} constrains the sum to be
  zero.  This is useful when considering \eqn{\delta p}{dp}; see the
  example at \code{gradient.Rd}.
}
  
\author{Robin K. S. Hankin}
\seealso{\code{\link{equalp}},\code{\link{gradient}}}
\examples{
fillup(c(1/2, 1/3))

indep(c(1/2, 1/3, 1/6))

fillup(indep(icons_maxp))  
fillup(indep(icons_maxp), icons)

}

\name{powerboat}
\alias{powerboat}
\alias{powerboat_maxp}
\alias{powerboat2018}
\alias{powerboat_2018}
\alias{powerboat_table}
\docType{data}
\title{Powerboat dataset}
\description{Race results from the 2018 F1 Powerboat World Championship}
\usage{data(powerboat)}

\details{

  Object \code{powerboat_table} is a dataframe of results showing ranks
  of 21 drivers in the 2018 F1 Powerboat World Championship.  The format
  is standard, that is, can be interpreted by function
  \code{ordertable2supp()} and indeed
  \code{ordertable2supp(powerboat_table[,1:7])} gives the corresponding
  support function, \code{powerboat}.

  File \code{inst/powerboat.txt} is the source text file; to create
  \code{powerboat_table} use

  \code{read.table(system.file("powerboat.txt",package="hyper2"))}

  The dataset used here corrects an apparent typo in the wikipedia table
  (see github issue 37).

These objects can be generated by running script
\code{inst/powerboat.Rmd}, which includes some further discussion and
technical documentation and creates file \code{powerboat.rda} which
resides in the \code{data/} directory.
  
}
\references{

Wikipedia contributors. (2019, October 9). 2018 F1 Powerboat World
Championship. In \emph{Wikipedia, The Free Encyclopedia}.  Retrieved
00:45, February 21, 2020, from
\url{https://en.wikipedia.org/w/index.php?title=2018_F1_Powerboat_World_Championship&oldid=920386507}

}

\note{
  Many drivers have names with diacritics, which have been removed from
  the dataframe.
}
\seealso{\code{\link{ordertable2supp}}}
\examples{
suppfun(powerboat_table)
pie(powerboat_maxp)

#    equalp.test(powerboat)
#    samep.test(powerboat, c("Torrente", "Stark"))  
 
}

\name{suppfun}
\alias{suppfun}
\alias{suppfun.ordertable}
\alias{suppfun.ranktable}
\alias{suppfun.character}
\alias{suppfun.numeric}
\alias{suppfun.attemptstable}
\title{
Convert various datasets to support functions.
}
\description{

Function \code{suppfun()} is an \proglang{S3} generic that returns a
support function, dispatching on the class of its primary argument.
  
}
\usage{
\method{suppfun}{ordertable}(x, ...)
\method{suppfun}{ranktable}(x, times, ...)
\method{suppfun}{character}(x, nonfinishers, ...)
\method{suppfun}{attemptstable}(x, ...)
suppfun(x, ...)
}
\arguments{
  \item{x}{Dataset to be converted to a support function}
  \item{times}{Vector corresponding to rows of \code{x} giving how many
    times that observation was obtained (cf \code{preflib} datasets);
    recycled if necessary}
  \item{nonfinishers}{Character vector of competitors who did not
    finish, passed to \code{rankvec_likelihood()}}
  \item{...}{Further arguments}
}
\details{

Function \code{suppfun()} is intended as a consistent \code{S3} generic
approach to converting datasets to support functions.  I am gradually
going to switch out use of \code{ordertable2supp()} in favour of
\code{suppfun()} in the docs.

If given a data frame, \code{suppfun()} will not guess whether it is to
be interpreted as an ordertable or a ranktable or an ordertable, and
return an error.

}
\value{
Generally, return a support function
}
\author{Robin K. S. Hankin}
\examples{

suppfun(pentathlon_table)
suppfun(rrank(10, p = (7:1)/28))
suppfun(letters)
suppfun(c(c = 2, b = 1, a = 3))



}

\name{surfing}
\alias{surfing}
\alias{surfing_table}
\alias{surfing_maxp}
\alias{surfing_venuetypes}
\docType{data}
\title{Surfing dataset}
\description{Data from the 2019 World Surf League (WSL) tour}
\usage{data(surfing)}
\details{

  The package contains four datasets from WSL 2019:

  \itemize{
    \item \code{surfing}, a log likelihood function for the strengths of
    the competitors
    \item \code{surfing_maxp}, corresponding precalculated evaluate
    \item \code{surfing_venuetypes}, a dataframe showing the beach types
    at the different venues of the tour
  }

These objects can be generated by running script
\code{inst/surfing.Rmd}, which includes some further discussion and
technical documentation and creates file \code{surfing.rda} which
resides in the \code{data/} directory.

}
\references{

T. C. Driver and R. K. S. Hankin 2023.  \dQuote{Analysis of competitive
surfing tournaments with generalized {B}radley-{T}erry likelihoods},
\emph{Journal of Sports Analytics}, 9:133--140.
\doi{10.3233/JSA-220596}
}

  
\author{Robin K. S. Hankin}
\examples{
dotchart(surfing_maxp)

}
\keyword{datasets}

\name{handover}
\alias{handover}
\alias{handover_table}
\alias{handover_maxp}
\alias{handoff}
\alias{malpractice}
\docType{data}
\title{Dataset on communication breakdown in handover between physicians}
\description{

Likelihoood function for a dataset concerning medical malpractice, due
to Lin et al. (2009)

}
\usage{data(handover)}
\details{
  Object \code{handover} is a likelihood function corresponding to a
  dataset arising from 69 medical malpractice claims and concerns
  handover (or hand-off) between physicians.  This dataset was analysed
  by Lin et al. (2009), and further analysed by Altham and Hankin
  (2010).  The computational methods are presented in the (unmaintained)
  \pkg{hyperdirichlet} and \pkg{aylmer} packages and a further
  discussion is given in the \dQuote{integration} vignette of the
  \pkg{hyper2} package.  The original dataset is \code{handover_table},
  a three-by-three matrix of counts.

These objects can be generated by running script
\code{inst/handover.Rmd}, which includes some further discussion and
technical documentation, and creates file \code{handover.rda} which
resides in the \code{data/} directory.
}

\references{
  \itemize{
    \item Y. Lin and S. Lipsitz and D. Sinha and A. A. Gawande and
    S. E. Regenbogen and C. C. Greenberg, 2009. \dQuote{Using {B}ayesian
      \eqn{p}-values in a \eqn{2\times 2}{2x2} table of matched pairs with
      incompletely classified data}. \emph{Journal of the Royal Statistical
      Society, Series C}, 58:2
    \item P. M. E. Altham and R. K. S. Hankin, 2010. \dQuote{Using
      recently developed software on a \eqn{2\times 2}{2x2} table of
      matched pairs with incompletely classified data}.  \emph{Journal of
      the Royal Statistical Society, series C}, 59(2): 377-379
    \item R. K. S. Hankin 2010. \dQuote{A generalization of the Dirichlet
      distribution}.   \emph{Journal of Statistical software}, 33:11
    \item L. J. West and R. K. S. Hankin 2008. \dQuote{Exact tests for
      two-way contingency tables with structural zeros}.   \emph{Journal
      of Statistical software}, 28:11
  }
}

\examples{
data(handover)
maxp(handover)
}
\keyword{datasets}

\name{psubs}
\alias{psubs}
\alias{psubs_single}
\alias{psubs_names}
\alias{psubs_pnames}
\title{Substitute players of a hyper2 object}
\description{Given a hyper2 object, substitute some players}
\usage{
psubs(H, from, to)
psubs_single(H, from, to)
}
\arguments{
  \item{H}{hyper2 object}
  \item{from,to}{Character vector of players to substitute and their
    substitutes}
}
\details{

Function \code{psubs()} substitutes one or more player names, replacing
player \code{from[i]} with \code{to[i]}.  If argument \code{to} is
missing, all players are substituted, the second argument taken to be
the replacement: interpret \code{psubs(H,vec)} as
\code{psubs(H,from=pnames(H),to=vec)}.

Compare \code{pnames<-()}, which can only add players, or reorder
existing players.

Function \code{psubs_single()} is a low-level helper function that takes
a single player and its substitute; it is not intended for direct use.
}

\value{Returns a hyper2 object}
\author{Robin K. S. Hankin}
\examples{

psubs(icons,c("L","NB"),c("London","Norfolk Broads"))

rhyper2() |> psubs(letters,LETTERS)   # ignore i,j,k,...,z

psubs(icons,tolower(pnames(icons)))

}

\name{rrace}
\alias{rrace}
\title{A random race with given BT strengths}
\description{
Returns a rank vector suitable for interpretation with \code{race()}.
}
\usage{rrace(strengths)}
\arguments{
  \item{strengths}{
    Named vector with names being players and values being their
    Bradley-Terry strengths}
}
\details{
  Uses a simple recursive system to generate the ranks.
}
\value{
Returns a character vector with entries corresponding to the competitor.
The first element is the winner, the second the runner-up, and so on,
until the final element is the last to cross the finishing line.
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{rrace3}},\code{\link{hyper2}}}
\examples{

o <- c(a=0.4, b=0.3, c=0.2, d=0.1)
rrace(o)

suppfun(rrace(o))

table(replicate(100,rrace(o)[1]))  # should be about 40% "a"

as.ranktable(t(replicate(10, rrace(o)))) # same as rrank(10,o)

}

\name{rhyper2}
\alias{rhyper2}
\title{Random \code{hyper2} objects}
\description{
Random \code{hyper2} loglikelihood functions, intended as quick
  \dQuote{get you going} examples
}
\usage{
rhyper2(n = 8, s = 5, pairs = TRUE, teams = TRUE, race = TRUE, pnames)
}
\arguments{
  \item{n}{Number of competitors, treated as even}
  \item{s}{Integer, Measure of the complexity of the log likelihood function}
  \item{pairs,teams,race}{Boolean, indicating whether or not to include
    different observations}
  \item{pnames}{Character vector of names, if missing interpret as
    \code{letters}; set to \code{NA} meaning no names}
}
\author{Robin K. S. Hankin}
\note{
  Function \code{rhyper2()} returns a likelihood function based on
  random observations.  To return a random probability vector drawn from
  a from a given (normalized) likelihood function, use \code{rp()}.
}
\seealso{\code{\link{rp}},\code{\link{rrace}}}
\examples{

rhyper2()
rp(2,icons)

}

\name{icons}
\alias{icons}
\alias{icons_matrix}
\alias{icons_table}
\alias{icons_maxp}
\alias{oneill}
\docType{data}
\title{Dataset on climate change due to O'Neill}
\description{

  Object \code{icons_matrix} is a matrix of nine rows and six columns,
  one column for each of six icons relevant to climate change.  The
  matrix entries show the number of respondents who indicated which icon
  they found most concerning.  The nine rows show different classes of
  respondents who were exposed to different subsets (of size four) of
  the six icons.

  The columns correspond to the different stimulus icons used, detailed
  below.  An extensive discussion is given in West and Hankin 2008, and
  Hankin 2010; an updated analysis is given in the \code{icons}
  vignette.  

  Object \code{icons} is the corresponding likelihood function, which
  can be created with \code{saffy(icons_table)}.  It is used in
  \code{inst/ternaryplot_hyper2.Rmd}, which shows a ternary plot of
  random samples.

  File \code{inst/icons.Rmd} creates the objects documented here and
  includes some technical discussion.  In addition, there is a detailed
  vignette, \code{icons}, provided with the package: to see it, type
  \code{vignette("icons")} at the command line.
}
\usage{data(icons)}
\details{
  The six icons were used in this study were:
  \describe{
    \item{PB}{polar bears, which face extinction through loss of ice
    floe hunting grounds}
    \item{NB}{The Norfolk Broads, which flood due to intense rainfall
    events}
    \item{L}{London flooding, as a result of sea level rise}
    \item{THC}{The Thermo-haline circulation, which may slow or stop as
      a result of anthropogenic modification of the hydrological cycle}
    \item{OA}{Oceanic acidification as a result of anthropogenic emissions
    of carbon dioxide}
    \item{WAIS}{The West Antarctic Ice Sheet, which is calving into the sea
    as a result of climate change}
  }
}
\source{
  Data kindly supplied by Saffron O'Neill of the University of East Anglia
}
\references{
  \itemize{
    \item S. J. O'Neill and M. Hulme 2009. \emph{An iconic approach for
    representing climate change}.  Global Environmental Change,
    19:402-410
    \item I. Lorenzoni and N. Pidgeon 2005. \emph{Defining Dangers of
      Climate Change and Individual Behaviour: Closing the Gap}. 
    In  \emph{Avoiding Dangerous Climate Change} (conference
    proceedings), UK Met Office, Exeter, 1-3 February
    \item R. K. S. Hankin 2010. \dQuote{A generalization of the Dirichlet
      distribution}.   \emph{Journal of Statistical software}, 33:11
  }
}
\author{Robin K. S. Hankin}
\seealso{\code{\link{matrix2supp}}}
\examples{
data(icons)
pie(icons_maxp)
equalp.test(icons)

}
\keyword{datasets}

\name{character_to_number}
\alias{character_to_number}
\alias{char2num}
\title{
Convert a character vector to a numeric vector
}
\description{
Convert string descriptions of competitors into their number
}
\usage{
character_to_number(char, pnames)
char2num(char, pnames)
}
\arguments{
  \item{char}{Character vector to be converted}
  \item{pnames}{Names vector (usually \code{pnames(H)})}
}
\details{

In earlier versions of this package, the internal mechanism of functions
such as \code{ggrl()}, and all the \code{C++} code, operated with the
competitors labelled with a non-negative integer; it is then natural to
refer to the competitors as \code{p1}, \code{p2}, etc.

However, sometimes the competitors have names (as in, for example, the
\code{rowing} dataset).  If so, it is more natural to refer to the
competitors using their names rather than an arbitrary integer.

Function \code{character_to_number()} converts the names to numbers.  If
an element of \code{char} is not present in \code{pnames}, an error is
returned (function \code{char2num()} is an easy-to-type synonym).  The
function is here because it is used in \code{ggrl()}.

}
\author{Robin K. S. Hankin}
\seealso{\code{\link{suppfun}}}
\examples{

 x <- sample(9)
 names(x) <- sample(letters[1:9])
 H <- ordervec2supp(x)
 character_to_number(letters[1:3],pnames(H))

 char2num(c("PB","L"),pnames(icons))
}

\name{ordertable2points}
\alias{ordertable2points}
\title{Calculate points from an order table}
\description{

  Given an order table and a schedule of points, calculate the points
  awarded to each competitor.
}

\usage{
ordertable2points(o, points,totals=TRUE)
}
\arguments{
  \item{o}{Order table}
  \item{points}{A numeric vector indicating number of points awarded for
    first, second, third, etc placing}
  \item{totals}{Boolean, with default \code{TRUE} meaning to return the
    points for each player (row) and \code{FALSE} meaning to return the
    entire table but with orders replaced with points scored}
}
\value{Returns either an order table or a named numeric vector}
\author{Robin K. S. Hankin}
\seealso{\code{\link{ordertable}}}
\examples{

points <- c(25, 18, 15, 12, 10, 8, 6, 4, 2, 1, 0, 0)
o <- as.ordertable(F1_table_2017)
ordertable2points(o,points)


}

\name{head.hyper2}
\alias{head.hyper2}
\title{First few terms of a distribution: DEPRECATED}
\description{First few terms in a hyperdirichlet distribution}
\usage{\method{head}{hyper2}(x, ...)}
\arguments{
  \item{x}{Object of class \code{hyper2}}
  \item{\dots}{Further arguments, passed to \code{head()}}
}
\details{Function is  \code{x[head(brackets(x), ...)]}}
\value{Returns a \code{hyper2} object}
\author{Robin K. S. Hankin}
\examples{

p <- zipf(5)
names(p) <- letters[1:5]
H <- suppfun(rrank(20,p))
head(H)


}
